//
// Fitting with Voigt function
//
// The code below is after Ida et al.
// http://www.crl.nitech.ac.jp/~ida/software/IgorMacro/index-j.html
//
//

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>

#if defined(_MSC_VER) //MSVC Compiler
#define MY_EXPORT __declspec(dllexport)
#define round( x ) ((int)(x+0.5))
#define M_PI 3.14159265358979323846
#else
#define MY_EXPORT
#endif

#define  FACTOR   1.12837916709551257388
#define  RMAXREAL 0.5e154
#define  RMAXEXP  708.503061461606
#define  RMAXGONI 3.53711887601422e15

double wofz(double XI, double YI)
{
  double XABS,YABS,X,Y;
  double QRHO,XABSQ,XQUAD,YQUAD;
  int N,J,I;
  double XSUM,YSUM;
  double U1,V1,DAUX,U2,V2,U,V,C;
  double H,H2,RX,RY,SX,SY,XAUX,QLAMBDA,TX,TY,W1;
  int KAPN,NU,NP1;
//
	
//  int FLAG = 0;

  XABS = fabs(XI);
  YABS = fabs(YI);
  X    = XABS/6.3;
  Y    = YABS/4.4;
  
//
//     THE FOLLOWING if-STATEMENT PROTECTS
//     QRHO = (X*X + Y*Y) AGAINST OVERFLOW
//
  if ((XABS > RMAXREAL) || (YABS > RMAXREAL))
    return RMAXREAL;

  QRHO = X*X + Y*Y;

  XABSQ = XABS*XABS;
  XQUAD = XABSQ - YABS*YABS;
  YQUAD = 2*XABS*YABS;

  if (QRHO < 0.085264)
    {
//
//  IF (QRHO.LT.0.085264D0) THEN THE FADDEEVA-FUNCTION IS EVALUATED
//  USING A POWER-SERIES (ABRAMOWITZ/STEGUN, EQUATION (7.1.5), P.297)
//  N IS THE MINIMUM NUMBER OF TERMS NEEDED TO OBTAIN THE REQUIRED
//  ACCURACY
//
      QRHO  = (1-0.85*Y)*sqrt(QRHO);
      N     = (int)round(6 + 72*QRHO);
      J     = 2*N+1;
      XSUM  = 1.0/J;
      YSUM  = 0.0;
      I=N;
      do
	{
	  J    = J - 2;
	  XAUX = (XSUM*XQUAD - YSUM*YQUAD)/I;
	  YSUM = (XSUM*YQUAD + YSUM*XQUAD)/I;
	  XSUM = XAUX + 1.0/J;
	  I = I-1;
	}while (I>0);
      U1   = -FACTOR*(XSUM*YABS + YSUM*XABS) + 1.0;
      V1   =  FACTOR*(XSUM*XABS - YSUM*YABS);
      DAUX =  exp(-XQUAD);
      U2   =  DAUX*cos(YQUAD);
      V2   = -DAUX*sin(YQUAD);

      U    = U1*U2 - V1*V2;
      V    = U1*V2 + V1*U2;
    }
  else
    {
//
//  IF (QRHO.GT.1.O) THEN W(Z) IS EVALUATED USING THE LAPLACE
//  CONTINUED FRACTION
//  NU IS THE MINIMUM NUMBER OF TERMS NEEDED TO OBTAIN THE REQUIRED
//  ACCURACY
//
//  IF ((QRHO.GT.0.085264D0).AND.(QRHO.LT.1.0)) THEN W(Z) IS EVALUATED
//  BY A TRUNCATED TAYLOR EXPANSION, WHERE THE LAPLACE CONTINUED FRACTION
//  IS USED TO CALCULATE THE DERIVATIVES OF W(Z)
//  KAPN IS THE MINIMUM NUMBER OF TERMS IN THE TAYLOR EXPANSION NEEDED
//  TO OBTAIN THE REQUIRED ACCURACY
//  NU IS THE MINIMUM NUMBER OF TERMS OF THE CONTINUED FRACTION NEEDED
//  TO CALCULATE THE DERIVATIVES WITH THE REQUIRED ACCURACY
//
      if (QRHO > 1.0) 
	{
	  H    = 0.0;
	  KAPN = 0;
	  QRHO = sqrt(QRHO);
	  NU   = round(3 + (1442/(26*QRHO+77)));
	}
      else
	{
	  QRHO = (1-Y)*sqrt(1-QRHO);
	  H    = 1.88*QRHO;
	  H2   = 2*H;
	  KAPN = round(7  + 34*QRHO);
	  NU   = round(16 + 26*QRHO);
	}

      
      if (H > 0.0) 
	QLAMBDA = pow(H2,KAPN);

      RX = 0.0;
      RY = 0.0;
      SX = 0.0;
      SY = 0.0;

      N = NU;
      do
	{
	  NP1 = N + 1;
	  TX  = YABS + H + NP1*RX;
	  TY  = XABS - NP1*RY;
	  C   = 0.5/(TX*TX + TY*TY);
	  RX  = C*TX;
	  RY  = C*TY;
	  if ((H > 0.0) && (N <= KAPN))
	    {
	      TX = QLAMBDA + SX;
	      SX = RX*TX - RY*SY;
	      SY = RY*TX + RX*SY;
	      QLAMBDA = QLAMBDA/H2;
	    }
	  N = N-1;
	} while (N>=0);

      if (H == 0.0)
	{
	  U = FACTOR*RX;
	  V = FACTOR*RY;
	}
      else
	{
	  U = FACTOR*SX;
	  V = FACTOR*SY;
	}

      if (YABS == 0.0) 
	U = exp(-XABS*XABS);

    }
//
//  EVALUATION OF W(Z) IN THE OTHER QUADRANTS
//
  if (YI < 0.0)
    {
      if (QRHO < 0.085264)
	{
	  U2    = 2*U2;
	  V2    = 2*V2;
	}
      else
	{
	  XQUAD =  -XQUAD;
//
//         THE FOLLOWING IF-STATEMENT PROTECTS 2*EXP(-Z*Z)
//         AGAINST OVERFLOW
//
	  if ((YQUAD > RMAXGONI) || (XQUAD > RMAXEXP)) 
	    return RMAXREAL;

	  W1 =  2*exp(XQUAD);
	  U2  =  W1*cos(YQUAD);
	  V2  = -W1*sin(YQUAD);
	}

      U = U2 - U;
      V = V2 - V;
      if (XI > 0.0) 
	V = -V;
      else if (XI < 0.0) 
	V = -V;
    }
  return U;
}

double Voigt(double x, double *p)
{
  double xc = p[0];
  double a  = p[1];
  double gL = 0.5 * p[2];
  double gG = 1.0/(2*sqrt(log(2.0))) * p[3];

  if (gG==0.0)
    return a / ( M_PI * gL * ( 1 + pow( x/gL, 2 ) ) );
  else
    {
      double re = fabs(x-xc)/gG;
      double im = gL/gG;
      return a / sqrt(M_PI) * wofz( re, im ) / gG;
    }
}

double dVoigt(double x, double *p, int n)
{
  double fEpsilon;
  int j;
  double p2[4];
  double f[4];

  if(p[n]==0)
    fEpsilon = 1e-10;
  else
    fEpsilon = p[n] * 1e-7;

  for(j=0;j<4;j++) p2[j]=p[j];

  p2[n] = p[n] + 2 * fEpsilon; f[0] = Voigt( x, p2 );
  p2[n] = p[n] + 1 * fEpsilon; f[1] = Voigt( x, p2 );
  p2[n] = p[n] - 1 * fEpsilon; f[2] = Voigt( x, p2 );
  p2[n] = p[n] - 2 * fEpsilon; f[3] = Voigt( x, p2 );

  return  (-f[0] + 8*f[1] - 8*f[2] + f[3]) / (12*fEpsilon);
}

struct data {
  size_t n;
  size_t p;
  double * X;
  double * Y;
  double * weight;//weighting data
};

extern "C" MY_EXPORT char *name()
{
  return "Voigt";
}

extern "C" MY_EXPORT char *function()
{
  return "Voigt(x,xc,a,wl,wg)";
}

extern "C" MY_EXPORT char *parameters()
{
  return "xc,a,wl,wg";
}

extern "C" MY_EXPORT double function_eval(double x, double *params)
{
  return Voigt( x, params );
}

extern "C" MY_EXPORT int function_f (const gsl_vector * x, void *params,
        gsl_vector * f)
{
  size_t n = ((struct data *)params)->n;
  double *X = ((struct data *)params)->X;
  double *Y = ((struct data *)params)->Y;
  double *weight = ((struct data *)params)->weight;

  double p[4];
  for(int j=0;j<4;j++)
    p[j] = gsl_vector_get (x, j);

  size_t i;
  for (i = 0; i < n; i++)
    {
      double Yi = Voigt( X[i], p );
      gsl_vector_set (f, i, (Yi - Y[i])*sqrt(weight[i]));
    }

  return GSL_SUCCESS;
}
extern "C" MY_EXPORT double function_d (const gsl_vector * x, void *params)
{
  size_t n = ((struct data *)params)->n;
  double *X = ((struct data *)params)->X;
  double *Y = ((struct data *)params)->Y;
  double *weight = ((struct data *)params)->weight;

  double p[4];
  for(int j=0;j<4;j++)
    p[j] = gsl_vector_get (x, j);

  size_t i;
  double val=0;
  for (i = 0; i < n; i++)
    {
      double Yi = Voigt( X[i], p ); 
      double dYi = (Yi-Y[i])*sqrt(weight[i]);
      val += dYi*dYi;
    }

  return val;
}

extern "C" MY_EXPORT int function_df (const gsl_vector * x, void *params,
         gsl_matrix * J)
{
  size_t n = ((struct data *)params)->n;
  double *X = ((struct data *)params)->X;
  double *weight = ((struct data *)params)->weight;

  int j;
  double p[4];
  for(j=0;j<4;j++)
    p[j] = gsl_vector_get (x, j);

  size_t i;
  for (i = 0; i < n; i++)
    {
      /* Jacobian matrix J(i,j) = dfi / dxj, 
         where fi = (Yi - Y[i])/sigma[i],				    
         Yi = (b + c*X[i])/(1 + a*X[i])         
         and the xj are the parameters (a, b, c) */

      for(j=0;j<4;j++)
	gsl_matrix_set (J, i, j, dVoigt(X[i], p, j)*sqrt(weight[i]) );

    }
  return GSL_SUCCESS;
}

extern "C" MY_EXPORT int function_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J)
{
  function_f (x, params, f);
  function_df (x, params, J);

  return GSL_SUCCESS;
}
