/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/

#ifndef IMAGEMARKER_H
#define IMAGEMARKER_H

#include <qpaintdevicemetrics.h>
#include <qpixmap.h>
#include <qwt_plot.h>
#include <qwt_plot_marker.h>

class ImageMarker:public QwtPlotMarker
{
public:
  ImageMarker (const QPixmap & p);
  virtual void draw (QPainter * p, const QwtScaleMap & xMap,
		     const QwtScaleMap & yMap, const QRect & r) const;

  QRect rect ()
  {
    return transform (plot ()->canvasMap (xAxis ()),
		      plot ()->canvasMap (yAxis ()), d_rect);
  };
  void setRect (int x, int y, int w, int h);

  QSize size ()
  {
    return rect ().size ();
  };
  void setSize (const QSize & size);

  void setFileName (const QString & fn)
  {
    d_fileName = fn;
  };
  QString getFileName ()
  {
    return d_fileName;
  };

  QPixmap image ()
  {
    return d_pic;
  };

  void setOrigin (const QPoint & p);
  QPoint getOrigin ()
  {
    return rect ().topLeft ();
  };

  QwtDoubleRect boundingRect () const
  {
    return d_rect;
  };
  void setBoundingRect (const QwtDoubleRect & rect);

  void updateBoundingRect ();

private:
  QPoint d_pos;
  QPixmap d_pic;
  QSize d_size;
  QString d_fileName;
  QwtDoubleRect d_rect;
};

#endif
