/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/

#include "ScriptingLangDialog.h"
#include "application.h"

#include <qlistbox.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qobjectlist.h>

ScriptingLangDialog::ScriptingLangDialog (ScriptingEnv * env,
					  ApplicationWindow * parent,
					  const char *name, bool modal,
					  WFlags fl):
QDialog (parent, name, modal, fl),
scripted (env)
{
  setCaption (tr ("QtiPlot - Select scripting language"));

  langList = new QListBox (this, "langList");

  QHBox *box1 = new QHBox (this, "box1");
  btnOK = new QPushButton (box1, "btnOK");
  btnOK->setText (tr ("OK"));
  btnOK->setDefault (true);
  btnCancel = new QPushButton (box1, "btnCancel");
  btnCancel->setText (tr ("Cancel"));

  QVBoxLayout *layout = new QVBoxLayout (this, 5, 5, "layout");
  layout->addWidget (langList);
  layout->addWidget (box1);

  connect (btnOK, SIGNAL (clicked ()), this, SLOT (accept ()));
  connect (btnCancel, SIGNAL (clicked ()), this, SLOT (close ()));

  updateLangList ();
}

void
ScriptingLangDialog::updateLangList ()
{
  langList->clear ();
  langList->insertStringList (ScriptingLangManager::languages ());
  QListBoxItem *current = langList->findItem (scriptEnv->name ());
  if (current)
    langList->setCurrentItem (current);
}

void
ScriptingLangDialog::accept ()
{
  ApplicationWindow *app = (ApplicationWindow *) parent ();
  if (app->setScriptingLang (langList->currentText ()))
    close ();
  else
    QMessageBox::critical (this, tr ("QtiPlot - Scripting Error"),
			   tr
			   ("Scripting language \"%1\" failed to initialize.").
			   arg (langList->currentText ()));
}
