/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/

#include "associationsDialog.h"
#include "worksheet.h"
#include "FunctionCurve.h"

#include <qlabel.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qvbox.h>
#include <qtable.h>
#include <qheader.h>
#include <qwidgetlist.h>
#include <qcheckbox.h>
#include <qlineedit.h>

#include <qmessagebox.h>

associationsDialog::associationsDialog (QWidget * parent, const char *name,
					bool modal, WFlags fl):
QDialog (parent, name, modal, fl)
{
  if (!name)
    setName ("associationsDialog");

  setCaption (tr ("QtiPlot - Plot Associations"));
  setFocus ();

  QVBox *box1 = new QVBox (this, "box1");
  box1->setSpacing (5);
  box1->setMargin (5);

  QHBox *hbox1 = new QHBox (box1, "hbox1");
  box1->setSpacing (5);

  new QLabel (tr ("Spreadsheet: "), hbox1, "table");
  tableCaptionLabel = new QLabel (hbox1, "table");

  table = new QTable (3, 5, box1, "table");
  table->setLeftMargin (0);
  table->setColumnReadOnly (0, true);
  table->setCurrentCell (-1, -1);
  table->setMaximumHeight (8 * table->rowHeight (0));
  for (int i = 0; i < table->numCols (); i++)
    table->setColumnStretchable (0, true);

  associations = new QListBox (box1, "available");
  associations->setMinimumWidth(250);
  associations->setMinimumHeight(64);
  associations->setSelectionMode (QListBox::Single);

  QHeader *header = table->horizontalHeader ();
  header->setLabel (0, tr ("Column"));
  header->setLabel (1, tr ("X"));
  header->setLabel (2, tr ("Y"));
  header->setLabel (3, tr ("xErr")); // to adjust width
  header->setLabel (4, tr ("yErr")); // to adjust width
  header->setClickEnabled (false);

  QHBox *box2 = new QHBox (this,"box2");
  box2->setMargin (5);
  box2->setSpacing (5);

  btnClose = new QPushButton (box2);
  btnClose->setText (tr ("&Close"));
  btnClose->setFixedWidth(100);
  btnClose->setDefault (TRUE);

  QVBoxLayout *layout = new QVBoxLayout (this, 5, 5, "hlayout3");
  layout->addWidget (box1);
  layout->addWidget (box2);

  connect (associations, SIGNAL (highlighted (int)), this,
	   SLOT (updateTable (int)));
  connect (btnClose, SIGNAL (clicked ()), this, SLOT (accept()));
}

void
associationsDialog::accept ()
{
  // updateCurve ();
  close ();
}

void
associationsDialog::updateCurve ()
{
  int index = associations->currentItem ();
  Table *t = findTable (index);
  if (t && graph)
    {
      // curve type must be set here
      int ctype = -1;
      QString s = associations->currentText ();
      QStringList lst = QStringList::split (": ", s, false);
      QStringList cols = QStringList::split (',', lst[1], false);
      if(cols.count()==3)
	{
	  if(cols[2].endsWith("(xErr)"))
	    ctype = Graph::XErrorBars;
	  else if(cols[2].endsWith("(yErr)"))
	    ctype = Graph::YErrorBars;
	  else 
	    qDebug("Error 1 in associationsDialog::updateCurve");
	}
      else if(cols.count()==4)
	{
	  if(cols[2].endsWith("(X)"))
	    ctype = Graph::VectXYXY;
	  else if(cols[2].endsWith("(A)"))
	    ctype = Graph::VectXYAM;
	  else 
	    qDebug("Error 2 in associationDialog::updateCurve");
	}

      for (int i = 0; i < (int) cols.count (); i++)
	{
	  int pos = cols[i].findRev('(');
	  cols[i] = lst[0] + "_" + cols[i].left(pos);
	}
      s = cols.join(",");
      //qDebug("assoc = %s",s.ascii());
      graph->changePlotAssociation (t, index, ctype, s);
      graph->updatePlot ();
    }
}

void
associationsDialog::initTablesList (QWidgetList *lst, int curve)
{
  tables = lst;
  active_table = 0;

  if (curve < 0 || curve >= (int) associations->count ())
    curve = 0;

  associations->setCurrentItem (curve);
}

Table *
associationsDialog::findTable (int index)
{
  QString text = associations->text (index);
  QString name = text.section(':',0,0);
  for(Table *t = (Table *)tables->first(); t; t = (Table *)tables->next())
     if ( t->getName() == name )
	return t;
    
  return 0;
}

void
associationsDialog::updateTable (int index)
{
  Table *t = findTable (index);
  if (!t)
    return;

  if (active_table != t)
    {
      active_table = t;
      tableCaptionLabel->setText (t->getName());
      table->setNumRows (t->tableCols ());
      QStringList colNames = t->colNames ();
      int i;
      for (i = 0; i < table->numRows (); i++)
	{
	  //table->setText (i, 0, colNames[i]);
	  MyTableItem *item = new MyTableItem( table,QTableItem::Never, colNames[i]);
	  table->setItem(i,0,item );
	}
      for (int j = 1; j < table->numCols (); j++)
	{
	  for (i = 0; i < table->numRows (); i++)
	    {
	      QCheckBox *cb = new QCheckBox (table, 0);
	      cb->installEventFilter (this);
	      table->setCellWidget (i, j, cb);
	    }
	  table->adjustColumn (j);
	}
      table->adjustColumn (0);
    }
  updateColumnTypes ();
}

void
associationsDialog::updateColumnTypes ()
{
  QString text = associations->currentText ();

  //fprintf(stderr,"assoc = %s\n",text.ascii());

  QStringList lst = QStringList::split (": ", text, false);
  QStringList cols = QStringList::split (',', lst[1], false);

  int pos = cols[0].findRev('(');
  QString xColName = cols[0].left(pos);
  pos = cols[1].findRev('(');
  QString yColName = cols[1].left(pos);

  QHeader *header = table->horizontalHeader ();
  int i, n = (int) cols.count ();

  if (n < 3) table->hideColumn (3);
  if (n < 4) table->hideColumn (4);

  QCheckBox *it = 0;
  for (i = 0; i < table->numRows (); i++)
    {
      it = (QCheckBox *) table->cellWidget (i, 1);
      if (table->text (i, 0) == xColName)
	it->setChecked (true);
      else
	it->setChecked (false);

      it = (QCheckBox *) table->cellWidget (i, 2);
      if (table->text (i, 0) == yColName)
	it->setChecked (true);
      else
	it->setChecked (false);
    }

  QString errColName, xEndColName, yEndColName;

  if (n >= 3)
    {
      table->showColumn (3);
      if (n==4) table->showColumn (4);
      if (cols[2].contains ("(xErr)"))
	{
	  errColName = cols[2].remove ("(xErr)");
	  header->setLabel (3, tr ("xErr"));
	}
      else if(cols[2].contains ("(yErr)"))
	{
	  errColName = cols[2].remove ("(yErr)");
	  header->setLabel (3, tr ("yErr"));
	}
      else if (cols[2].contains ("(X)") && cols[3].contains ("(Y)"))
	{
	  xEndColName = cols[2].remove ("(X)");
	  yEndColName = cols[3].remove ("(Y)");
	  header->setLabel (3, tr ("xEnd"));
	  header->setLabel (4, tr ("yEnd"));
	}
      else if (cols[2].contains ("(A)") && cols[3].contains ("(M)"))
	{
	  xEndColName = cols[2].remove ("(A)");
	  yEndColName = cols[3].remove ("(M)");
	  header->setLabel (3, tr ("Ang"));
	  header->setLabel (4, tr ("Mag"));
	}
      else
	fprintf(stderr,"Invalid Association\n");

      for (i = 0; i < table->numRows (); i++)
	{
	  it = (QCheckBox *) table->cellWidget (i, 3);
	  if (table->text (i, 0) == errColName
	      || table->text (i, 0) == xEndColName)
	    it->setChecked (true);
	  else
	    it->setChecked (false);
	}
      if(n==4)
	for (i = 0; i < table->numRows (); i++)
	  {
	    it = (QCheckBox *) table->cellWidget (i, 4);
	    if (table->text (i, 0) == yEndColName)
	      it->setChecked (true);
	    else
	      it->setChecked (false);
	  }
    }
}

void
associationsDialog::uncheckCol (int col)
{
  for (int i = 0; i < table->numRows (); i++)
    {
      QCheckBox *it = (QCheckBox *) table->cellWidget (i, col);
      if (it)
	it->setChecked (false);
    }
}

void
associationsDialog::setGraph (Graph * g)
{
  graph = g;

  QStringList assoc = g->plotAssociations ();
  QStringList newNames;
  for (int i = 0; i < (int) assoc.count (); i++)
    {
      const QwtPlotItem *c = (QwtPlotItem *) graph->curve (i);
      if (!c || c->rtti () == FunctionCurve::RTTI)
	continue;
      //qDebug("assoc = %s",assoc[i].ascii());
      if (assoc[i].contains('_') && c->rtti () == QwtPlotItem::Rtti_PlotCurve)
	{
	  int ctype = g->curveType(i);
	  QStringList cols = QStringList::split(',',assoc[i],false);
	  QString table = cols[0].section('_',0,0);
	  cols.gres( table+"_", "", true );
          cols[0] += "(X)";
          cols[1] += "(Y)";
          if(ctype==Graph::XErrorBars)
            cols[2] += "(xErr)";
          else if(ctype==Graph::YErrorBars)
            cols[2] += "(yErr)";
          else if (ctype == Graph::VectXYXY)
            {
              cols[2] += "(X)";
              cols[3] += "(Y)";
            }
          else if (ctype == Graph::VectXYAM)
            {
              cols[2] += "(A)";
              cols[3] += "(M)";
            }
          newNames << table + ": " + cols.join(",");
	}
      else if (c->rtti () != QwtPlotItem::Rtti_PlotSpectrogram)
	newNames << assoc[i];
    }
  associations->insertStringList (newNames, -1);
  plotAssociationsList = newNames;
}

void
associationsDialog::updatePlotAssociation (int row, int col)
{
  //fprintf(stderr,"updatePlotAssociation %d, %d\n",row,col);

  int index = associations->currentItem ();
  QString text = associations->currentText ();
  QStringList lst = QStringList::split (": ", text, false);
  QStringList cols = QStringList::split (',', lst[1], false);

  QHeader *header = table->horizontalHeader ();
  QString label = header->label(col);
  if(label=="xEnd") label = "X";
  else if(label=="yEnd") label = "Y";
  else if(label=="Ang") label = "A";
  else if(label=="Mag") label = "M";
  cols[col-1] = table->text(row,0) + "(" + label + ")";
  text = lst[0] + ": " + cols.join (",");

  plotAssociationsList[index] = text;
  associations->clear ();
  associations->insertStringList (plotAssociationsList, -1);
  associations->setCurrentItem (index);
  updateCurve();
}

bool
associationsDialog::eventFilter (QObject * object, QEvent * e)
{
  QTableItem *it = (QTableItem *) object;
  if (!it)
    return false;

  if (e->type () == QEvent::MouseButtonPress)
    {
      if (((QCheckBox *) it)->isChecked ())
	return true;

      int col = 0, row = 0;
      for (int j = 1; j < table->numCols (); j++)
	{
	  for (int i = 0; i < table->numRows (); i++)
	    {
	      QCheckBox *cb = (QCheckBox *) table->cellWidget (i, j);
	      if (cb == (QCheckBox *) object)
		{
		  row = i;
		  col = j;
		  break;
		}
	    }
	}

      for (int i = 0; i < table->numRows (); i++)
	{
	  QCheckBox *it2 = (QCheckBox *) table->cellWidget (i, col);
	  if(i==row)
	    it2->setChecked (true);
	  else
	    it2->setChecked (false);
	}

      //uncheckCol (col);
      //((QCheckBox *) it)->setChecked (true);

      updatePlotAssociation (row, col);
      return true;
    }
  else if (e->type () == QEvent::MouseButtonDblClick)
    return true;
  else
    return false;
}

associationsDialog::~associationsDialog ()
{
  delete tables;
}
