/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "colFormulaDialog.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qhbuttongroup.h>
#include <qlineedit.h>

ColFormulaDialog::ColFormulaDialog (QWidget * parent, const char *name, 
				    bool modal, WFlags fl):
  QDialog (parent, name, modal, fl)
{
  if (!name)
    setName ("ColFormulaDialog");

  setCaption (tr ("QtiPlot - Generate Formula"));

  GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, QString::null, this,
		      "GroupBox1");

  new QLabel (tr ("Method of Calulation"), GroupBox1, "TextLabel1", 0);
  boxMethod = new QComboBox (GroupBox1, "boxMethod");
  boxMethod->insertItem("Sum");
  boxMethod->insertItem("Mean");
  boxMethod->insertItem("Product");
  boxMethod->insertItem("Geometric Mean");

  new QLabel(tr("Mapping Function"), GroupBox1, "TextLabel2",0);
  mapFuncEdit = new QLineEdit(GroupBox1, "mapFuncEdit");
  mapFuncEdit->setText("%c");

  new QLabel (tr ("Export Formula to"), GroupBox1, "TextLabel3", 0);
  columnsList = new QComboBox (GroupBox1, "listBox");
  columnsList->setEnabled (true);

  GroupBox2 = new QHButtonGroup (this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonOk = new QPushButton (GroupBox2, "buttonOk");
  buttonOk->setDefault (TRUE);
  buttonOk->setText (tr ("&OK"));
  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setText (tr ("&Cancel"));

  QVBoxLayout *vlayout = new QVBoxLayout (this, 5, 5, "vlayout");
  vlayout->addWidget (GroupBox1);
  vlayout->addWidget (GroupBox2);

  // signals and slots connections
  connect (buttonOk, SIGNAL (clicked ()), this, SLOT (accept ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
}

void
ColFormulaDialog::insertColumnsList (const QStringList & cols)
{
  int i, n = cols.count ();
  for (i = 0; i < n; i++)
    columnsList->insertItem (cols[i], i);

  columnsList->setCurrentItem (0);
}

ColFormulaDialog::~ColFormulaDialog ()
{
}

void
ColFormulaDialog::accept ()
{
  emit makeFormula( boxMethod->currentItem(),
		    mapFuncEdit->text(),
		    columnsList->currentText()); 
  close ();
}
