/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "convolutionDialog.h"
#include "graph.h"
#include "colorBox.h"
#include "application.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcheckbox.h>

convolutionDialog::convolutionDialog (int method, 
				      QWidget * parent, const char *name,
				      bool modal, WFlags fl):
QDialog (parent, name, modal, fl)
{
  if (!name)
    setName ("convolutionDialog");
  
  d_method = method;

  if(d_method==0)
    setCaption (tr ("QtiPlot - Convolution Options"));
  else
    setCaption (tr ("QtiPlot - Deconvolution Options"));

  QButtonGroup *GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox1");
 
  new QLabel (tr ("Signal"), GroupBox1, "TextLabel1", 0);
  boxSigName = new QComboBox (GroupBox1, "boxSigName");

  new QLabel (tr ("Response"), GroupBox1, "TextLabel2", 0);
  boxResName = new QComboBox (GroupBox1, "boxResName");

  new QLabel (tr ("Normalize Response"), GroupBox1, "TextLabel2c", 0);
  boxNorm = new QCheckBox(GroupBox1);
  boxNorm->setChecked(true);

  if(d_method==DeconvFFT)
    {
      new QLabel (tr ("Threshold"), GroupBox1, "TextLabel2a", 0);
      boxThres = new QLineEdit (GroupBox1, "boxThres" );
      boxThres->setText("0.001");
    }
  if(d_method==DeconvFFT || d_method==DeconvAndre)
    {
      new QLabel (tr ("Cut off (Hz)"), GroupBox1, "TextLabel3", 0);
      boxCutoff = new QLineEdit (GroupBox1, "boxCutoff");
      boxCutoff->setText("0.1");
    }
  if (d_method==DeconvGS)
    {
      new QLabel (tr ("Smoothing Point"), GroupBox1, "TextLabel2a", 0);
      boxSmooth = new QSpinBox(1,101,2,GroupBox1);
      boxSmooth->setValue(1);

      new QLabel (tr ("Damping"), GroupBox1, "TextLabel2b", 0);
      boxDamping = new QLineEdit (GroupBox1);
      boxDamping->setText("1");

      new QLabel (tr ("Iteration"), GroupBox1, "TextLabel3a", 0);
      boxIteration = new QSpinBox (1,1000,1,GroupBox1);
      boxIteration->setValue(5);

      new QLabel (tr("Constraint"), GroupBox1, "TextLabel4a", 0);
      boxConstraint = new QComboBox(GroupBox1);
      boxConstraint->insertItem("Positive");
      boxConstraint->insertItem("Negative");
      boxConstraint->insertItem("None");
    }

  new QLabel (tr ("Color"), GroupBox1, "TextLabel52", 0);
  boxColor = new ColorBox (FALSE, GroupBox1);
  boxColor->setColor (QColor (red));

  QButtonGroup *GroupBox2 =
    new QButtonGroup (1, QGroupBox::Horizontal, tr (""), this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonApply = new QPushButton (GroupBox2, "buttonApply");
  buttonApply->setAutoDefault (TRUE);
  buttonApply->setDefault (TRUE);

  if(method > 0)
  {
    buttonDone = new QPushButton (GroupBox2, "buttonDone");
    buttonDone->setAutoDefault (TRUE);
  }
  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setAutoDefault (TRUE);

  buttonHelp = new QPushButton (GroupBox2, "buttonHelp");
  buttonHelp->setAutoDefault (TRUE);

  QHBoxLayout *hlayout = new QHBoxLayout (this, 5, 5, "hlayout");
  hlayout->addWidget (GroupBox1);
  hlayout->addWidget (GroupBox2);

  languageChange ();

  curve_index = -1;

  // signals and slots connections
  connect (buttonApply, SIGNAL (clicked ()), this, SLOT (convolute ()));
  if(method>0)
    connect (buttonDone, SIGNAL (clicked ()), this, SLOT (close ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (cancel ()));
  connect (buttonHelp, SIGNAL (clicked ()), this, SLOT (help ()));
}

convolutionDialog::~convolutionDialog ()
{
}

void
convolutionDialog::languageChange ()
{
  buttonApply->setText (tr ("&Apply"));
  if(d_method>0) buttonDone->setText( tr("Done"));
  buttonCancel->setText (tr ("&Cancel"));
  buttonHelp->setText (tr ("&Help"));
}

void
convolutionDialog::cancel()
{
  ApplicationWindow *app = (ApplicationWindow *) this->parent();

  if(curve_index!=-1)
    {
      QwtPlotCurve *curve = d_graph->curve(curve_index);
      QString title = curve->title().text();
      QString tableName = title.section( '_', 0, 0 );
      Table *w = app->table(tableName);
      w->askOnCloseEvent (false);
      w->close ();
    }
  reject();
}

void
convolutionDialog::convolute ()
{
  QString sig = boxSigName->currentText ();
  QString res = boxResName->currentText ();
  QStringList clist = d_graph->analyzableCurvesList ();

  if (clist.contains (sig) <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Warning"),
			     tr
			     ("The curve <b> %1 </b> doesn't exist anymore! Operation aborted!").
			     arg (sig));
      boxSigName->clear ();
      boxSigName->insertStringList (clist);
      return;
    }

  if (clist.contains (res) <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Warning"),
			     tr
			     ("The curve <b> %1 </b> doesn't exist anymore! Operation aborted!").
			     arg (res));
      boxResName->clear ();
      boxResName->insertStringList (res);
      return;
    }

  int index = d_graph->curveIndex( res );
  QwtPlotCurve *c = d_graph->curve( index );
  if (c->minXValue() > 0 || c->maxXValue() < 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Warning"),
			     tr("The curve <b> %1 </b> must include x=0!\nin its X range").arg(res));
      return;
    }

  double cutoff = 0;
  double thres = 0;
  double damping = 0;
  int iteration = 0;
  if(d_method==DeconvFFT)
    {
      thres = boxThres->text().toDouble();
    }
  if(d_method==DeconvFFT || d_method==DeconvAndre)
    {
      cutoff = boxCutoff->text().toDouble();  
    }
  if(d_method==DeconvGS)
    {
      damping = boxDamping->text().toDouble();
      iteration = boxIteration->value();
    }

  ApplicationWindow *app = (ApplicationWindow *) this->parent();

  if(curve_index!=-1)
    {//remove old result
      QwtPlotCurve *curve = d_graph->curve(curve_index);
      QString title = curve->title().text();
      QString tableName = title.section( '_', 0, 0 );
      Table *w = app->table(tableName);
      w->askOnCloseEvent (false);
      w->close ();
    }

  switch(d_method)
    {
    case ConvFFT:
      d_graph->convoluteFFT(app, 0, sig, res, boxNorm->isChecked(), 
			    0, 0, boxColor->currentItem());
      break;
    case DeconvFFT:
      d_graph->convoluteFFT(app, 1, sig, res, boxNorm->isChecked(), 
			    thres, cutoff, boxColor->currentItem());
      break;
    case DeconvAndre:
      d_graph->convoluteFFT(app, 2, sig, res, boxNorm->isChecked(),
			    1.0, cutoff, boxColor->currentItem());
      break;
    case DeconvGS:
      d_graph->deconvoluteGS(app, sig, res, boxNorm->isChecked(),
			     boxSmooth->value(), damping, iteration,
			     boxConstraint->currentItem(),
			     boxColor->currentItem());
    }

  curve_index = d_graph->curves()-1; // newest curve

  if(d_method==0) close();
}

void
convolutionDialog::setGraph (Graph * g)
{
  d_graph = g;
  boxSigName->insertStringList (g->analyzableCurvesList(), -1);
  boxResName->insertStringList (g->analyzableCurvesList(), -1);
  boxSigName->setCurrentItem(0);
  boxColor->setCurrentItem(g->curves());
  if(boxResName->count()>1)
    boxResName->setCurrentItem(1);

  connect (d_graph, SIGNAL (closedGraph ()), this, SLOT (close ()));
}

void
convolutionDialog::help ()
{
  QString s1,s2;
  switch(d_method)
    {
    case 0: 
      s1 = "QtiPlot - Help for FFT Convolution";
      s2 = 
"The FFT convolution performs a numerical convolution of two curves.\n\
One curve is regarded as a signal function and the other a response\n\
function. The response curve must include x = 0 in its X range.\n\
Each length of the positive and negative part of the response curve must\n\
be shorter than the half of the signal curve. Otherwise the rest is ignored.";
      break;
    case 1:
      s1 = "QtiPlot - Help for FFT Deconvolution";
      s2 = 
"The FFT deconvolution performs a numerical deconvolution of a curve\n\
using another curve as a response function. The response curve must\n\
have x = 0 within its X range. Each length of the positive and negative\n\
part of the response curve must be shorter than the half of the signal\n\
curve. Otherwise the rest is ignored.\n\
If the response curve has very small values in frequency space the\n\
deconvoluted curve will have unphysically intense noises. To prevent\n\
this, user should specify a threshold value as a ratio to the peak\n\
intensity of the response function. This value usually works as the\n\
inverse of maximum enhancement factor for high frequency components.\n\
If you want to reduce other noises at the same time, specify a cut off\n\
frequency.";
      break;
    case 2:
      s1 = "QtiPlot - Help for FFT Deconvolution for decay curves";
      s2 = 
"The FFT deconvolution for decay curve is a numerical deconvolution under\n\
the assumption that the original curve is an exponential-like decay curve.\n\
The high frequency part of the spectrum is extrapolated by the fourier\n\
transform of an exponential curve.\n\
The response curve must have x = 0 within its X range. Each length of the\n\
positive and negative part of the response curve must be shorter than the\n\
half of the signal curve. Otherwise the rest is ignored.\n\
The origin of the signal curve must be defined precisely as it determines\n\
the accuracy of the deconvolution. The negative part of the signal curve will\n\
be trancated.";
      break;
    case 3:
      s1 = "QtiPlot - Help for Gauss-Seidel Deconvolution";
      s2 = 
"The GS deconvolution performs a numerical deconvolution of a curve by\n\
directly solving the deconvolution equation. The response curve must\n\
have x = 0 within its X range. Each length of the positive and negative\n\
part of the response curve must be shorter than the half of the signal\n\
curve. Otherwise the rest is ignored.\n\
The damping factor accelerate the convergence and usually takes a value\n\
between 1 and 4. The positive or negative constraint also accelerate the\n\
convergence in some cases.";
      break;
    }
  QMessageBox::about (0,s1,s2);
}

