/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/

#include "curveCalcDialog.h"
#include "application.h"
#include "graph.h"
#include "worksheet.h"
#include "FunctionCurve.h"
#include "parser.h"

#include <qwt_plot_curve.h>

#include <qvariant.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qhbuttongroup.h>
#include <qmessagebox.h>
#include <qtextedit.h>

curveCalcDialog::curveCalcDialog (QWidget * parent, const char *name, bool modal, WFlags fl):QDialog (parent, name, modal,fl)
{
  graph = 0;

  if (!name)
    setName ("curveCalcDialog");
  setCaption (tr ("QtiPlot - Two Curve Calculation"));

  GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox1");

  new QLabel (tr ("Curve A"), GroupBox1, "TextLabel1", 0);
  boxName1 = new QComboBox (GroupBox1, "boxName1");

  new QLabel (tr ("Curve B"), GroupBox1, "TextLabel2", 0);
  boxName2 = new QComboBox (GroupBox1, "boxName2");

  new QLabel (tr ("Operation"), GroupBox1, "TextLabel3", 0);
  boxMethod = new QComboBox (GroupBox1, "boxMethod");
  QStringList operation;
  operation << "A - B" << "A + B" << "A / B" << "A * B" << "formula";
  boxMethod->insertStringList(operation,-1);

  formulaLabel = new QLabel(tr("formula :"), GroupBox1, "TextLabel4", 0);
  formulaEdit = new QTextEdit (GroupBox1);
  formulaEdit->setTextFormat (Qt::PlainText);
  formulaEdit->setMaximumHeight (int (1.5 * formulaLabel->height ()));
  formulaLabel->hide();
  formulaEdit->hide ();

  GroupBox2 = new QHButtonGroup (this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonOk = new QPushButton (GroupBox2, "buttonOk");
  buttonOk->setAutoDefault (TRUE);
  buttonOk->setDefault (TRUE);

  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setAutoDefault (TRUE);

  QVBoxLayout *
    vlayout = new QVBoxLayout (this, 5, 5, "vlayout");
  vlayout->addWidget (GroupBox1);
  vlayout->addWidget (GroupBox2);

  languageChange ();

  // signals and slots connections
  connect (boxMethod, SIGNAL (activated(int)), 
	   this, SLOT(methodActivated(int)));
  connect (buttonOk, SIGNAL (clicked ()), this, SLOT (accept ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
}

curveCalcDialog::~curveCalcDialog ()
{
}

void
curveCalcDialog::languageChange ()
{
  buttonOk->setText (tr ("&OK"));
  buttonCancel->setText (tr ("&Cancel"));
}

void
curveCalcDialog::setGraph (Graph *g, const QString &name)
{
  graph = g;
  graphname = name;

  QStringList names1 = graph->analyzableCurvesList();
  QStringList names2 = graph->xyCurvesList();
  names1.prepend("All Curves");
  boxName1->insertStringList (names1, -1);
  names2.prepend("Previous One");
  boxName2->insertStringList (names2, -1);
}

void
curveCalcDialog::methodActivated(int index)
{
  if(index==4)
    {
      formulaLabel->show();
      formulaEdit->show ();
    }
  else
    {
      formulaLabel->hide();
      formulaEdit->hide ();      
      adjustSize();
    }
}

void
curveCalcDialog::accept ()
{
  ApplicationWindow *app = (ApplicationWindow *)this->parent();

  QStringList curves1,curves2;

  if (boxName1->currentItem() == 0)
    curves1 = graph->analyzableCurvesList();
  else
    curves1.append(boxName1->currentText());

  if (boxName2->currentItem() == 0) // Previous Ones
    {
      if(curves1.count()>1)
	{
	  curves2.append( curves1.last() );
	  for (int cv = 0; cv < (int)curves1.count()-1; cv++)
	    curves2.append( curves1[cv] );
	}
      else
	{
	  int item = boxName1->currentItem() - 1;
	  int count = boxName1->count() - 1;
	  item = ( (item - 1) + count ) % count;
	  curves2.append( boxName1->text(item+1) );
	}
    }
  else
    {
      int cindex2 = graph->curveIndex( boxName2->currentText() );
      QwtPlotCurve *c2 = graph->curve( cindex2 );
      if(c2->rtti () == FunctionCurve::RTTI)
	if(((FunctionCurve *)c2)->functionType()!=FunctionCurve::Normal)
	  {
	    QMessageBox::critical(this, tr("QtiPlot - Error"),
				  tr( "You can not perform two curve operations to parametric curves."));
	    return;
	  }
      for (int cv = 0; cv < (int)curves1.count(); cv++)
	curves2.append( boxName2->currentText() );
    }

  // check curves
  
  int maxpoints = 0;
  for (int cv = 0; cv < (int)curves1.count(); cv++)
    {
      int cindex1 = graph->curveIndex( curves1[cv] );
      QwtPlotCurve *c1 = graph->curve( cindex1 );
      int cindex2 = graph->curveIndex( curves2[cv] );
      QwtPlotCurve *c2 = graph->curve( cindex2 );

      double xmin = QMAX( c1->minXValue(), c2->minXValue() );
      double xmax = QMIN( c1->maxXValue(), c2->maxXValue() );

      int points=0;
      for (int j=0; j<c1->dataSize(); j++)
	if(c1->x(j) >= xmin && c1->x(j) <= xmax)
	  points++;
      if(points==0)
	{
	  QMessageBox::critical(this, tr("QtiPlot - Error"),
				tr( "There is no overlap between %1 and %2")
				.arg( curves1[cv]  )
				.arg( curves2[cv] ) );
	  return;
	}
      if (points > maxpoints) maxpoints = points;
    }

  // set formula

  QString formula;
  switch( boxMethod->currentItem() )
    {
    case 0 : formula = "A - B"; break;
    case 1 : formula = "A + B"; break;
    case 2 : formula = "A / B"; break;
    case 3 : formula = "A * B"; break;
    case 4 : formula = formulaEdit->text(); break;
    }
  myParser parser;
  double Avar,Bvar;
  parser.DefineVar("A",&Avar);
  parser.DefineVar("B",&Bvar);
  parser.SetExpr( formula.ascii() );
  
  // generate new Table

  QString tname = app->generateUniqueName("Calculated");
  QString legend = tr("Calculated Table : ") + formula + " ( A : ";
  if (curves1.count()==1)
    legend += curves1[0] + ", B ; " + curves2[0] + " )";
  else
    {
      legend += graphname + ", B : ";
      if (boxName2->currentItem()==0)
	legend += "Previous Ones )";
      else
	legend += boxName2->currentText() + " )";
    }
  int prec = app->tbl_output_precision;
  Table *calcTable = app->newHiddenTable( tname, legend, maxpoints,
					  (int)curves1.count()*2, prec);
  for(int i=0;i<(int)curves1.count();i++)
    {
      calcTable->setColPlotDesignation(i*2,  Table::X);
      calcTable->setColPlotDesignation(i*2+1,Table::Y);
    }
  calcTable->setHeaderColType();

  // calculate

  for (int cv = 0; cv < (int)curves1.count(); cv++)
    {
      int cindex1 = graph->curveIndex( curves1[cv] );
      int cindex2 = graph->curveIndex( curves2[cv] );

      QwtPlotCurve *c1 = graph->curve( cindex1 );
      QwtPlotCurve *c2 = graph->curve( cindex2 );

      double xmin = QMAX( c1->minXValue(), c2->minXValue() );
      double xmax = QMIN( c1->maxXValue(), c2->maxXValue() );

      int n = c1->dataSize();
      QMemArray <double> x(n), y1(n), y2(n);

      int points=0;
      for (int i=0; i<n; i++)
	if(c1->x(i) >= xmin && c1->x(i) <= xmax)
	  {
	    x[points] = c1->x(i);
	    y1[points] = c1->y(i);
	    points++;
	  }
      
      if(c2->rtti () == FunctionCurve::RTTI)
	{
	  QStringList f2 = ((FunctionCurve *)c2)->formulas();
	  myParser parser2;
	  double xvar;
	  parser2.DefineVar("x", &xvar);
	  parser2.SetExpr( f2[0].ascii() );
	  for (int i=0; i<points; i++)
	    {
	      xvar = x[i];
	      y2[i] = parser2.Eval();
	    }
	}
      else
	{
	  x.resize(points);
	  if(!graph->interpolate( cindex2, 0, x, y2 )) // linear interp.
	    {
	      for(int i=0; i<points; i++)
		y2[i] = 0;
	    }
	}
            
      for (int i = 0; i < points; i++)
	{
	  Avar = y1[i];
	  Bvar = y2[i];
	  double result;
	  try
	    {
	      result = parser.Eval();
	    }
	  catch(mu::ParserError &e)
	    {
	      int choice =
		QMessageBox::question(this,tr("QtiPlot - Calculation error"),
		              QString::fromLocal8Bit(e.GetMsg().c_str())+" : Continue?",
				      QMessageBox::Yes | QMessageBox::Default,
				      QMessageBox::No | QMessageBox::Escape);
	      if (choice == QMessageBox::Yes)
		result = 0;
	      else
		{
		  calcTable->askOnCloseEvent(false);
		  calcTable->close();
		  return;
		}
	    }
	  calcTable->setText( i, cv*2, QString::number(x[i],'g',prec) );
	  calcTable->setText( i, cv*2+1, QString::number(result,'g',prec) );
	}
    }

  // make graph

  MultiLayer *calcPlot = app->multilayerPlot(calcTable, 
				 calcTable->YColumns(),
				 Graph::Line );
 ((myWidget *)calcPlot)->setWindowLabel(legend);
  Graph *ag = app->activeGraph;
  for(int i = 0; i < (int)curves1.count(); i++)
    {
      int cindex = graph->curveIndex( curves1[i] );
      QwtPlotCurve *c = graph->curve(cindex);
      ag->setCurvePen(i,c->pen());
      ag->setCurveSymbol(i,c->symbol());
      int style = graph->curveType( cindex );
      if(style <= Graph::LineSymbols)
	ag->setCurveType(i,style);
      if(style==Graph::Scatter)
	ag->setCurveStyle(i,QwtPlotCurve::NoCurve);
    }

  ag->replot();
  
  close ();
}

