/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "expDecayDialog.h"
#include "graph.h"
#include "colorBox.h"
#include "application.h"
#include "Fitter.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qmessagebox.h>

expDecayDialog::expDecayDialog (int type, QWidget * parent, const char *name, bool modal, WFlags fl):QDialog (parent, name, modal,
	 fl)
{
  if (!name)
    setName ("expDecayDialog");

  slopes = type;

  setCaption (tr ("QtiPlot - Verify initial guesses"));
  GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox1");

  new
  QLabel (tr ("Exponential Fit of"), GroupBox1, "TextLabel1", 0);
  boxName = new QComboBox (GroupBox1, "boxName");

  if (type < 0)
    dampingLabel =
      new QLabel (tr ("Growth time"), GroupBox1, "TextLabel2", 0);
  else if (type == 1)
    dampingLabel = new QLabel (tr ("Decay time"), GroupBox1, "TextLabel2", 0);
  else
    dampingLabel =
      new QLabel (tr ("First decay time (t1)"), GroupBox1, "TextLabel2", 0);

  boxFirst = new QLineEdit (GroupBox1, "boxOrder");
  boxFirst->setText (tr ("1"));

  if (type > 1)
    {
      new
      QLabel (tr ("Second decay time (t2)"), GroupBox1, "TextLabel3", 0);
      boxSecond = new QLineEdit (GroupBox1, "boxPoints");
      boxSecond->setText (tr ("1"));

      thirdLabel =
	new QLabel (tr ("Third decay time (t3)"), GroupBox1, "TextLabel5", 0);

      boxThird = new QLineEdit (GroupBox1, "boxOrder");
      boxThird->setText (tr ("1"));

      if (type < 3)
	{
	  thirdLabel->hide ();
	  boxThird->hide ();
	}
    }

  new QLabel (tr ("Amplitude"), GroupBox1, "TextLabel4", 0);
  boxAmp = new QLineEdit (GroupBox1, "boxAmp");
  boxAmp->setText (tr ("1"));

  new QLabel (tr ("Fit curve Xmin"), GroupBox1, "TextLabel4", 0);
  boxStart = new QLineEdit (GroupBox1, "boxStart");
  boxStart->setText (tr ("0"));

  new QLabel (tr ("Fit curve Xmax"), GroupBox1, "TextLabel5", 0);
  boxEnd = new QLineEdit (GroupBox1, "boxEnd");

  new QLabel (tr ("Color"), GroupBox1, "TextLabel52", 0);
  boxColor = new ColorBox (FALSE, GroupBox1);
  boxColor->setColor (QColor (red));

  GroupBox2 =
    new QButtonGroup (1, QGroupBox::Horizontal, tr (""), this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonFit = new QPushButton (GroupBox2, "buttonFit");
  buttonFit->setAutoDefault (TRUE);
  buttonFit->setDefault (TRUE);

  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setAutoDefault (TRUE);

  QHBoxLayout *
    hlayout = new QHBoxLayout (this, 5, 5, "hlayout");
  hlayout->addWidget (GroupBox1);
  hlayout->addWidget (GroupBox2);

  languageChange ();

  // signals and slots connections
  connect (buttonFit, SIGNAL (clicked ()), this, SLOT (fit ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
  connect (boxName, SIGNAL (activated (int)), this,
           SLOT (activateCurve (int)));
}


expDecayDialog::~expDecayDialog ()
{
}


void
expDecayDialog::languageChange ()
{
  buttonFit->setText (tr ("&Fit"));
  buttonCancel->setText (tr ("&Close"));
}

void
expDecayDialog::setGraph (Graph * g)
{
  if (!g)
    return;

  d_graph = g;
  QStringList clist = d_graph->analyzableCurvesList();
  boxName->insertStringList (clist, -1);
  if (d_graph->selectorsEnabled ())
    {
      int index;
      QString selected = d_graph->selectedCurveTitle();
      if((index = clist.findIndex( selected )) >=0 )
	boxName->setCurrentItem( index );
      activateCurve(index);
    }
  else
    activateCurve(0);

  connect (d_graph, SIGNAL (closedGraph ()), this, SLOT (close ()));
  connect (d_graph, SIGNAL (dataRangeChanged ()), this,
           SLOT (changeDataRange ()));
}

void
expDecayDialog::activateCurve(int index)
{
  QString ctitle = boxName->text(index);
  int cindex = d_graph->curveIndex(ctitle);
  QwtPlotCurve *c = d_graph->curve(cindex);
  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve)
    return;

  double start,end;
  d_graph->range(cindex,&start,&end);
  boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
  boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));
}

void
expDecayDialog::changeDataRange ()
{
  double start = d_graph->selectedXStartValue ();
  double end = d_graph->selectedXEndValue ();
  boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
  boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));
}

void
expDecayDialog::fit ()
{
  QString curve = boxName->currentText ();
  QwtPlotCurve *c = d_graph->curve (curve);
  QStringList clist = d_graph->analyzableCurvesList ();
  if (!c || clist.contains (curve) <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Warning"),
			     tr
			     ("The curve <b> %1 </b> doesn't exist anymore! Operation aborted!").
			     arg (curve));
      boxName->clear ();
      boxName->insertStringList (clist);
      return;
    }

  ApplicationWindow *app = (ApplicationWindow *) this->parent ();
  Fit *fitter;
  if (slopes == 3)
    {
      QMemArray <double> x_init(6);
      x_init[0] = boxAmp->text ().toDouble ()/3; // amplitude 1
      x_init[1] = 1.0 / boxFirst->text ().toDouble (); // decaytime 1
      x_init[2] = boxAmp->text ().toDouble ()/3; // amplitude 2
      x_init[3] = 1.0 / boxSecond->text ().toDouble (); // decaytime 2
      x_init[4] = boxAmp->text ().toDouble ()/3; // amplitude 3
      x_init[5] = 1.0 / boxThird->text ().toDouble (); // decaytime 3
      //x_init[6] = boxYOffset->text ().toDouble (); // y offset
      fitter = new ThreeExpFit (app, d_graph);
      fitter->setInitialGuesses (x_init);
    }
  else if (slopes == 2)
    {
      QMemArray <double> x_init(4);
      x_init[0] = boxAmp->text ().toDouble ()/2; // amplitude 1
      x_init[1] = 1.0 / boxFirst->text ().toDouble (); // decaytime 1
      x_init[2] = boxAmp->text ().toDouble ()/2; // amplitude 2
      x_init[3] = 1.0 / boxSecond->text ().toDouble (); // decaytime 2
      //x_init[4] = boxYOffset->text ().toDouble (); // y offset
      fitter = new TwoExpFit (app, d_graph);
      fitter->setInitialGuesses (x_init);
    }
  else if (slopes == 1 || slopes == -1)
    {
      QMemArray <double> x_init(2);
      x_init[0] = boxAmp->text ().toDouble (); // amplitude
      x_init[1] = slopes / boxFirst->text ().toDouble (); // decay time
      // x_init[2] = boxYOffset->text ().toDouble (); // y offset
      fitter = new ExponentialFit (app, d_graph, slopes == -1);
      fitter->setInitialGuesses (x_init);
    }

  if (fitter->setDataFromCurve (boxName->currentText (),
				boxStart->text().toDouble (),
				boxEnd->text().toDouble ()))
    {
      fitter->setColor(boxColor->currentItem ());
      fitter->setGenerateFunction(app->generateFunction);
      fitter->setGenerateUniformFitPoints(app->generateUniformFitPoints);
      fitter->setFitPoints(app->fitPoints);
      QString name = QString(fitter->name())+tr("Fit");
      if(app->generateFunction)
	name = d_graph->generateUniqueName(name);
      else
	name = app->generateUniqueName(name);
      fitter->setCurveName(name);
      fitter->fit ();
      delete fitter;
    }

  close();
}
