/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "fftDialog.h"
#include "graph.h"
#include "parser.h"
#include "application.h"
#include "multilayer.h"
#include "worksheet.h"
#include "plot.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qvbox.h>
#include <qcheckbox.h>
#include <qmessagebox.h>

FFTDialog::FFTDialog (int type, QWidget * parent, const char *name,
		      bool modal, WFlags fl):
QDialog (parent, name, modal, fl)
{
  setCaption (tr ("QtiPlot - FFT Options"));

  d_table = 0;
  d_graph = 0;
  d_type = type;

  QVBox *box = new QVBox (this);
  box->setSpacing (5);
  box->setMargin (5);

  QButtonGroup *GroupFFT =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), box, "GroupBox3");
  GroupFFT->setRadioButtonExclusive (TRUE);

  forwardBtn = new QRadioButton (GroupFFT, "forwardBtn");
  forwardBtn->setChecked (TRUE);

  backwardBtn = new QRadioButton (GroupFFT, "backwardBtn");

  QButtonGroup *GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), box, "GroupBox1");

  if (d_type == onGraph)
    {
      new QLabel (tr ("Curve"), GroupBox1, "TextLabel1", 0);
      boxName = new QComboBox (GroupBox1, "boxShow");
    }
  else
    {
      new QLabel (tr ("Real"), GroupBox1, "TextLabel11", 0);
      boxReal = new QComboBox (GroupBox1, "boxReal");
      new QLabel (tr ("Imaginary"), GroupBox1, "TextLabel111", 0);
      boxImaginary = new QComboBox (GroupBox1, "Imaginary");
    }

  new QLabel (tr ("Sampling Interval"), GroupBox1, "TextLabel4", 0);
  boxSampling = new QLineEdit (GroupBox1, "boxStart");

  new QLabel (tr("Window Function"), GroupBox1, "TextLabel5", 0);
  boxWindow = new QComboBox(GroupBox1, "boxWindow");
  boxWindow->insertItem("NoWindow");
  boxWindow->insertItem("Hanning");
  boxWindow->insertItem("Hamming");
  boxWindow->setCurrentItem(0);

  boxResample = new QCheckBox (box, "boxResample");
  boxResample->setChecked (false);
  if(type==onTable)
    boxResample->setEnabled(false);

  boxNormalize = new QCheckBox (box, "boxNorm");
  boxNormalize->setChecked (false);

  boxOrder = new QCheckBox (box, "boxOrder");
  boxOrder->setChecked (true);


  QButtonGroup *GroupBox2 =
    new QButtonGroup (1, QGroupBox::Horizontal, tr (""), this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonOK = new QPushButton (GroupBox2, "buttonOK");
  buttonOK->setDefault (TRUE);

  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");

  QHBoxLayout *hlayout = new QHBoxLayout (this, 5, 5, "hlayout");
  hlayout->addWidget (box);
  hlayout->addWidget (GroupBox2);

  languageChange ();

  // signals and slots connections
  if(d_type == onGraph)
    {
      setFocusProxy (boxName);
      connect (boxName, SIGNAL (activated (int)), this,
	       SLOT (activateCurve (int)));
    }
  connect (buttonOK, SIGNAL (clicked ()), this, SLOT (accept ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
}

FFTDialog::~FFTDialog ()
{
}


void
FFTDialog::languageChange ()
{
  boxResample->setText (tr ("&Resampling"));
  boxNormalize->setText (tr ("&Normalize Amplitude"));
  boxOrder->setText (tr ("&Shift Results"));
  backwardBtn->setText (tr ("&Inverse"));
  forwardBtn->setText (tr ("&Forward"));
  buttonOK->setText (tr ("&OK"));
  buttonCancel->setText (tr ("&Close"));
}

void
FFTDialog::accept ()
{
  double sampling;
  try
  {
    myParser parser;
    parser.SetExpr (boxSampling->text ().ascii ());
    sampling = parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (this, tr ("QtiPlot - Sampling value error"),
			   e.GetMsg ().c_str());
    boxSampling->setFocus ();
    return;
  }

  ApplicationWindow *app = (ApplicationWindow *) this->parent ();

  if (d_graph)
    {
      int index = d_graph->curveIndex(boxName->currentText());
      if (index < 0)
	return;

      if(!d_graph->fft (app, index, forwardBtn->isChecked (), sampling,
			boxResample->isChecked(), boxNormalize->isChecked (), boxOrder->isChecked (), (Graph::FFTWindow)boxWindow->currentItem() ))
	return;

      Table *w = app->table (app->tableWindows.last ());
      QStringList list;
      list << w->getName() + "_Amplitude";
      
      MultiLayer *d = app->multilayerPlot (w, list, 0);
      if (!d)
	{
	  close ();
	  return;
	}

      Graph *g = d->activeGraph ();
      if (g)
	{
	  Plot *plot = g->plotWidget ();
	  plot->setTitle (QString::null);
	  if (forwardBtn->isChecked ())
	    plot->setAxisTitle (QwtPlot::xBottom, tr ("Frequency") + " (Hz)");
	  else
	    plot->setAxisTitle (QwtPlot::xBottom, tr ("Time") + " (s)");
	  
	  plot->setAxisTitle (QwtPlot::yLeft, tr ("Amplitude"));
	  plot->replot ();
	}
      d->show();
    }
  else
    {
      if (boxReal->currentText ().isEmpty ())
	{
	  QMessageBox::critical (this, tr ("QtiPlot - Error"),
				 tr
				 ("Please choose a column for the real part of the data!"));
	  boxReal->setFocus ();
	  return;
	}
      else
	d_table->fft (app, sampling, boxReal->currentText (),
		      boxImaginary->currentText (), forwardBtn->isChecked (),
		      boxNormalize->isChecked (), boxOrder->isChecked (),
		      (Table::FFTWindow)boxWindow->currentItem() );
    }

  close ();
}

void
FFTDialog::setGraph (Graph * g)
{
  d_graph = g;
  boxName->insertStringList (g->analyzableCurvesList (), -1);
  activateCurve (0);
};

void
FFTDialog::activateCurve (int index)
{
  QString ctitle = boxName->text(index);
  int cindex = d_graph->curveIndex(ctitle);
  QwtPlotCurve *c = d_graph->curve (cindex);
  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve)
    return;
  int n = c->dataSize ();
  double dx = ( c->maxXValue() - c->minXValue() ) / (n-1);
  boxSampling->setText (QString::number( dx ) );
  boxResample->setEnabled(true);
};

void
FFTDialog::setTable (Table * t)
{
  d_table = t;
  QStringList l = t->columnsList ();
  boxReal->insertStringList (l);
  boxImaginary->insertStringList (l);

  l = t->selectedColumns ();
  int selected = (int) l.size ();
  if(selected>0)
    {
      int rcol = t->colIndex(l[0]);
      boxReal->setCurrentItem (rcol);
      int xcol = t->colX( rcol );
      if (xcol >= 0)
	{
	  double x0 = t->text (0, xcol).toDouble ();
	  double x1 = t->text (1, xcol).toDouble ();
	  boxSampling->setText (QString::number (x1 - x0));
	}
      if(selected>1)
	boxImaginary->setCurrentItem (t->colIndex (l[1]));
      else
	boxImaginary->setCurrentText (QString::null);
    }
  else
    {
      boxReal->setCurrentText (QString::null);
      boxImaginary->setCurrentText (QString::null);
    }

  boxResample->setEnabled(false);
};

