#include "gslutils.h"
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_sf.h>

#include <stdio.h>
#include <time.h>
#include <math.h>

static const gsl_rng_type *T;
static gsl_rng *r;

void gsl_init()
{
  gsl_set_error_handler_off();

  gsl_rng_env_setup();
  T = gsl_rng_default;
  r = gsl_rng_alloc (T);
  gsl_rng_set(r,(unsigned long)time(NULL));
  atexit(gsl_end);
}

double rand_uniform()
{
  return gsl_rng_uniform(r);
}

int rand_poisson(double x)
{
  return gsl_ran_poisson(r,x);
}

double rand_gaussian(double x)
{
  return gsl_ran_gaussian(r,x);
}

double rand_exponential(double x)
{
  return gsl_ran_exponential(r,x);
}

int rand_binomial(double x, double y)
{
  return gsl_ran_binomial(r,x,int(y+0.5));
}

double rand_gamma(double a, double b)
{
  return gsl_ran_gamma(r,a,b);
}

void gsl_end()
{
  gsl_rng_free (r);
}
