/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "importOPJ.h"
#include "note.h"
#include "matrix.h"
#include <OPJFile.h>

#include <qregexp.h>


ImportOPJ::ImportOPJ (ApplicationWindow * app, const QString & filename):
mw (app)
{
  char buf[512];

  strcpy(buf,filename.local8Bit());
  OPJFile opj (buf);
  parse_error = opj.Parse ();
  importTables (opj);
  importNotes (opj);
  mw->cascade();
}

bool
ImportOPJ::importTables (OPJFile opj)
{
  //int QtiPlot_scaling_factor = 10; //in Origin width is measured in characters while in QtiPlot - pixels --- need to be accurate
  for (int s = 0; s < opj.numSpreads (); s++)
    {
      int nr_cols = opj.numCols (s);
      int maxrows = opj.maxRows (s);

      if (!nr_cols)             //remove tables without cols
        continue;

      Table *table = mw->newTable (opj.spreadName (s), maxrows, nr_cols);
      if (!table)
	return false;

      table->setWindowLabel (opj.spreadLabel (s));

      for (int j = 0; j < nr_cols; j++)
	{
	  QString name (opj.colName (s, j));
	  table->setColName (j, name.replace (QRegExp (".*_"), ""));
          table->setCommand (j, QString (opj.colCommand (s, j)));
          //table->setColComment (j, QString (opj.colComment (s, j)));
          //table->changeColWidth (opj.colWidth (s, j) * QtiPlot_scaling_factor, j);
	  
	  if (QString (opj.colType (s, j)) == "X")
	    table->setColPlotDesignation (j, Table::X);
	  else if (QString (opj.colType (s, j)) == "Y")
	    table->setColPlotDesignation (j, Table::Y);
	  else if (QString (opj.colType (s, j)) == "Z")
	    table->setColPlotDesignation (j, Table::Z);
	  else if (QString (opj.colType (s, j)) == "DX")
            table->setColPlotDesignation (j, Table::xErr);
          else if (QString (opj.colType (s, j)) == "DY")
            table->setColPlotDesignation (j, Table::yErr);
	  else
	    table->setColPlotDesignation (j, Table::None);

	  table->setHeaderColType();

	  for (int i = 0; i < opj.numRows (s, j); i++)
	    {
	      if (strcmp (opj.colType (s, j), "LABEL"))
		{		// number
		  double *val = (double *)opj.oData (s, j, i, true);
		  if (fabs(*val)>0 && fabs (*val) < 2.0e-300)
		    continue;
		  table->setText (i, j, QString::number (*val));
		}
	      else		// label? doesn't seem to work
		table->setText (i, j, QString ((char *)opj.oData (s, j, i)));
	    }
	}
      table->show();

    }

//Import matrices
  for (int s = 0; s < opj.numMatrices (); s++)
    {
      int nr_cols = opj.numMatrixCols (s);
      int nr_rows = opj.numMatrixRows (s);

      Matrix *matrix = mw->newMatrix (opj.matrixName (s), nr_rows, nr_cols);
      if (!matrix)
        return false;

      matrix->setWindowLabel (opj.matrixLabel (s));
      matrix->setFormula (opj.matrixFormula (s));
      //matrix->setColumnsWidth (opj.matrixWidth (s) * QtiPlot_scaling_factor);
      matrix->table ()->blockSignals (true);
      for (int j = 0; j < nr_cols; j++)
        {
          for (int i = 0; i < nr_rows; i++)
            {
              double val = opj.matrixData (s, j, i);
              if (fabs (val) > 0 && fabs (val) < 2.0e-300)      // empty entry
                continue;

	      //matrix->setCell (i, j, val);
	      matrix->setText( i, j, QString::number(val) );
            }
        }

      matrix->table ()->blockSignals (false);
      matrix->show();

    }

  return true;
}

bool
ImportOPJ::importNotes (const OPJFile & opj)
{
  for (int n = 0; n < opj.numNotes (); n++)
    {
      QString name = opj.noteName (n);
      //QRegExp rx ("^@\\((\\S+)\\)$");
      //if (rx.indexIn (name) == 0)
      //  name = rx.cap (1);

      Note *note = mw->newNote (name);
      if (!note)
        return false;
      note->setWindowLabel (opj.noteLabel (n));
      note->setText (opj.noteText (n));
    }

  return true;
}

