/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "intDialog.h"
#include "parser.h"
#include "graph.h"
#include "application.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qmessagebox.h>
#include <qhbuttongroup.h>

intDialog::intDialog (QWidget * parent, const char *name, bool modal,
		      WFlags fl):
QDialog (parent, name, modal, fl)
{
  if (!name)
    setName ("polynomFitDialog");
  setCaption (tr ("QtiPlot - Integration Options"));
  setSizeGripEnabled (true);
  setSizePolicy (QSizePolicy (QSizePolicy::Expanding, QSizePolicy::Fixed));

  GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox1");

  new QLabel (tr ("Integration of"), GroupBox1, "TextLabel1", 0);
  boxName = new QComboBox (GroupBox1, "boxShow");

  /*
  new QLabel (tr ("Order (1 - 5, 1 = Trapezoid Rule)"), GroupBox1,
	      "TextLabel2", 0);
  boxOrder = new QSpinBox (1, 5, 1, GroupBox1, "boxOrder");

  new QLabel (tr ("Number of iterations (Max=40)"), GroupBox1, "TextLabel3",
	      0);
  boxSteps = new QSpinBox (2, 40, 1, GroupBox1, "boxPoints");
  boxSteps->setValue (40);

  new QLabel (tr ("Tolerance"), GroupBox1, "TextLabel4", 0);
  boxTol = new QLineEdit (GroupBox1, "boxTol");
  boxTol->setText ("0.01");
  */

  new QLabel (tr ("Lower limit"), GroupBox1, "TextLabel5", 0);
  boxStart = new QLineEdit (GroupBox1, "boxStart");

  new QLabel (tr ("Upper limit"), GroupBox1, "TextLabel6", 0);
  boxEnd = new QLineEdit (GroupBox1, "boxEnd");

  GroupBox2 = new QHButtonGroup (this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonOk = new QPushButton (GroupBox2, "buttonOk");
  buttonOk->setAutoDefault (TRUE);
  buttonOk->setDefault (TRUE);

  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setAutoDefault (TRUE);

  buttonHelp = new QPushButton (GroupBox2, "buttonHelp");
  buttonHelp->setAutoDefault (TRUE);

  QVBoxLayout *vlayout = new QVBoxLayout (this, 5, 5, "vlayout");
  vlayout->addWidget (GroupBox1);
  vlayout->addWidget (GroupBox2);

  languageChange ();

  // signals and slots connections
  connect (buttonOk, SIGNAL (clicked ()), this, SLOT (accept ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
  connect (buttonHelp, SIGNAL (clicked ()), this, SLOT (help ()));
  connect (boxName, SIGNAL (activated (int)), this,
	   SLOT (activateCurve (int)));
}

intDialog::~intDialog ()
{
}

void
intDialog::languageChange ()
{
  buttonOk->setText (tr ("&Integrate"));
  buttonCancel->setText (tr ("&Close"));
  buttonHelp->setText (tr ("&Help"));
}

void
intDialog::accept ()
{
  QString curveTitle = boxName->currentText ();
  QStringList clist = d_graph->analyzableCurvesList ();
  if (clist.contains (curveTitle) <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Warning"),
			     tr
			     ("The curve <b> %1 </b> doesn't exist anymore! Operation aborted!").
			     arg (curveTitle));
      boxName->clear ();
      boxName->insertStringList (clist);
      return;
    }

  //int index = boxName->currentItem ();
  int index = d_graph->curveIndex(curveTitle);
  QwtPlotCurve *c = d_graph->curve (index);
  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve || c->dataSize () < 2)
    {
      QString s = tr ("You cannot fit index:");
      s += "<p><b>'" + curveTitle + "'</b><p>";
      s += tr ("because it has less than 2 points!");
      QMessageBox::warning (0, tr ("QtiPlot - Warning"), s);
      return;
    }

  /*
  try
  {
    mu::Parser parser;
    parser.SetExpr (boxTol->text ().ascii ());
    parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (0, tr ("QtiPlot - Tolerance value error"),
			   e.GetMsg ().c_str());
    boxTol->clear ();
    boxTol->setFocus ();
    return;
  }
  */

  double start, stop;
  double minx = c->minXValue ();
  double maxx = c->maxXValue ();
// Check the Xmin 
  QString from = boxStart->text ().lower ();
  if (from == "min")
    {
      boxStart->setText (QString::number (minx));
      return;
    }
  else if (from == "max")
    {
      boxStart->setText (QString::number (maxx));
      return;
    }
  else
    {
      try
      {
	myParser parser;
	parser.SetExpr ((boxStart->text ()).ascii ());
	start = parser.Eval ();

	if (start < minx)
	  {
	    QMessageBox::warning (0, tr ("QtiPlot - Input error"),
				  tr
				  ("Please give a number larger or equal to the minimum value of X, for the lower limit.\n If you do not know that value, type min in the box."));
	    boxStart->clear ();
	    boxStart->setFocus ();
	    return;
	  }
	if (start > maxx)
	  {
	    QMessageBox::warning (0, tr ("QtiPlot - Input error"),
				  tr
				  ("Please give a number smaller or equal to the maximum value of X, for the lower limit.\n If you do not know that value, type max in the box."));
	    boxStart->clear ();
	    boxStart->setFocus ();
	    return;
	  }
      }
      catch (mu::ParserError & e)
      {
	QMessageBox::critical (0, tr ("QtiPlot - Start limit error"),
			       e.GetMsg ().c_str());
	boxStart->clear ();
	boxStart->setFocus ();
	return;
      }
    }

// Check Xmax
  QString end = boxEnd->text ().lower ();
  if (end == "min")
    {
      boxEnd->setText (QString::number (minx));
      return;
    }
  else if (end == "max")
    {
      boxEnd->setText (QString::number (maxx));
      return;
    }
  else
    {
      try
      {
	myParser parser;
	parser.SetExpr ((boxEnd->text ()).ascii ());
	stop = parser.Eval ();
	if (stop > maxx)
	  {
	    QMessageBox::warning (0, tr ("QtiPlot - Input error"),
				  tr
				  ("Please give a number smaller or equal to the maximum value of X, for the upper limit.\n If you do not know that value, type max in the box."));
	    boxEnd->clear ();
	    boxEnd->setFocus ();
	    return;
	  }
	if (stop < minx)
	  {
	    QMessageBox::warning (0, tr ("QtiPlot - Input error"),
				  tr
				  ("Please give a number larger or equal to the minimum value of X, for the upper limit.\n If you do not know that value, type min in the box."));
	    boxEnd->clear ();
	    boxEnd->setFocus ();
	    return;
	  }
      }
      catch (mu::ParserError & e)
      {
	QMessageBox::critical (0, tr ("QtiPlot - End limit error"),
			       e.GetMsg ().c_str());
	boxEnd->clear ();
	boxEnd->setFocus ();
	return;
      }
    }

  start = boxStart->text ().toDouble ();
  stop = boxEnd->text ().toDouble ();

  QString res = d_graph->integrateCurve (curveTitle, start, stop);
  if (!res.isEmpty ())
    {
      ApplicationWindow *app = (ApplicationWindow *) this->parent ();
      app->logInfo += res;
      app->showResults (true);
      app->modifiedProject ();
    }
  close();
}

void
intDialog::setGraph (Graph * g)
{
  d_graph = g;
  QStringList clist = g->analyzableCurvesList();
  boxName->insertStringList (clist, -1);
  if (d_graph->selectorsEnabled ())
    {
      int index = 0;
      QString selected = d_graph->selectedCurveTitle();
      if((index = clist.findIndex( selected )) >=0 )
        boxName->setCurrentItem( index );
      activateCurve(index);
    }
  else
    activateCurve(0);


  connect (d_graph, SIGNAL (closedGraph ()), this, SLOT (close ()));
  connect (d_graph, SIGNAL (dataRangeChanged ()), this,
	   SLOT (changeDataRange ()));
};

void
intDialog::activateCurve (int index)
{
  QString ctitle = boxName->text(index);
  int cindex = d_graph->curveIndex(ctitle);
  QwtPlotCurve *c = d_graph->curve (cindex);

  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve)
    return;

  double start,end;
  d_graph->range(cindex, &start, &end);

  boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
  boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));
};

void
intDialog::changeDataRange ()
{
  double start = d_graph->selectedXStartValue ();
  double end = d_graph->selectedXEndValue ();
  boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
  boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));
}

void
intDialog::help ()
{
  QMessageBox::about (0, tr ("QtiPlot - Help for Integration"),
		      tr(
"Integration of a curve performs a numerical integration\n\
of an interpolated function over the specified range.\n\
The method of the interpolation is Akima-spline when the\n\
data number is greater than 5. Otherwise linear or cubic-spline\n\
interpolation is used. The details of the calculation can\n\
be found in the reference manual of the gsl library."));
}
