/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "layerOrderDialog.h"
#include "pixmaps.h"
#include "multilayer.h"

#include <qlabel.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <qvbox.h>

layerOrderDialog::layerOrderDialog (QWidget * parent, const char *name, bool modal,
			    WFlags fl):
QDialog (parent, name, modal, fl)
{
  if (!name)
    setName ("layerOrderDialog");
  setCaption (tr ("QtiPlot - layer position dialog"));
  setFocus ();

  QHBox *box0 = new QHBox (this, "box0");
  box0->setSpacing (5);

  QVBox *box3 = new QVBox (box0, "box3");
  box3->setMargin (5);
  box3->setSpacing (5);

  TextLabel2 = new QLabel (box3, "TextLabel2");
  TextLabel2->setText (tr ("Layer contents"));

  contents = new QListBox (box3, "contents");
  contents->setSelectionMode (QListBox::Single);

  QVBox *box6 = new QVBox (box0, "box6");
  box6->setMargin (5);
  box6->setSpacing (5);

  btnUp = new QPushButton (box6, "btnUp");
  btnUp->setPixmap (QPixmap (up_xpm));
  btnUp->setFixedWidth (35);
  btnUp->setFixedHeight (30);

  btnDown = new QPushButton (box6, "btnDown");
  btnDown->setPixmap (QPixmap (down_xpm));
  btnDown->setFixedWidth (35);
  btnDown->setFixedHeight (30);

  QVBox *box4 = new QVBox (box0, "box4");
  box4->setMargin (5);
  box4->setSpacing (5);

  btnOK = new QPushButton (box4, "btnOK");
  btnOK->setText (tr ("OK"));
  btnOK->setDefault (TRUE);

  btnCancel = new QPushButton (box4, "btnCancel");
  btnCancel->setText (tr ("Close"));

  QVBoxLayout *layout = new QVBoxLayout (this, 5, 5, "hlayout3");
  layout->addWidget (box0);

  connect (btnUp, SIGNAL (clicked ()), this, SLOT (layerUp ()));
  connect (btnDown, SIGNAL (clicked ()), this, SLOT (layerDown ()));

  connect (btnOK, SIGNAL (clicked ()), this, SLOT (accept ()));
  connect (btnCancel, SIGNAL (clicked ()), this, SLOT (close ()));
}

void
layerOrderDialog::setMultiLayer (MultiLayer *m)
{
  int n = m->layers();
  multi = m;

  QStringList sl;
  for (int i = 0; i < n; i++)
    sl.append( tr("Layer")+QString::number(i+1) );
  contents->insertStringList (sl);
}

void
layerOrderDialog::layerUp()
{
  int s = contents->currentItem();
  if( s > 0 )
    {
      QString dummy = contents->text(s-1);
      contents->changeItem(contents->text(s),s-1);
      contents->changeItem(dummy,s);
      contents->setSelected(s-1,true);
    }
}

void
layerOrderDialog::layerDown()
{
  int s = contents->currentItem();
  if( s > 0 && s < (int)contents->count() )
    {
      QString dummy = contents->text(s+1);
      contents->changeItem(contents->text(s),s+1);
      contents->changeItem(dummy,s);
      contents->setSelected(s+1,true);
    }
}

void
layerOrderDialog::accept()
{
  QValueList <int> order;
  for ( int i = 0; i < (int)contents->count(); i++ )
    order << contents->text(i).remove("Layer").toInt() - 1 ;
  multi->reorderLayers( order );

  close();
}

layerOrderDialog::~layerOrderDialog ()
{
}
