/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "lineDlg.h"
#include "colorButton.h"
#include "LineMarker.h"
#include "graph.h"
#include "application.h"

#include <qwt_plot.h>
#include <qwt_scale_engine.h>

#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qcolordialog.h>
#include <qtabwidget.h>
#include <qhbox.h>
#include <qlineedit.h>

lineDialog::lineDialog (LineMarker * line, QWidget * parent, const char *name,
			bool modal, WFlags fl):
QDialog (parent, name, modal, fl),
lm (line)
{
  setCaption (tr ("QtiPlot - Line options"));

  tw = new QTabWidget (this);
  options = new QWidget (tw);
  GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, QString::null, options,
		      "GroupBox1");

  new QLabel (tr ("Color"), GroupBox1);
  colorBox = new ColorButton (GroupBox1);
  colorBox->setColor (lm->color ());

  new QLabel (tr ("Line type"), GroupBox1);
  styleBox = new QComboBox (FALSE, GroupBox1);
  styleBox->insertItem ("_____");
  styleBox->insertItem ("- - -");
  styleBox->insertItem (".....");
  styleBox->insertItem ("_._._");
  styleBox->insertItem ("_.._..");

  setLineStyle (lm->style ());

  new QLabel (tr ("Line width"), GroupBox1);
  widthBox = new QSpinBox( GroupBox1, "widthBox" );
  widthBox->setMinValue(1);
  widthBox->setValue(lm->width());

  startBox = new QCheckBox (GroupBox1);
  startBox->setText (tr ("Arrow at &start"));
  startBox->setChecked (lm->hasStartArrow ());

  endBox = new QCheckBox (GroupBox1, "endBox");
  endBox->setText (tr ("Arrow at &end"));
  endBox->setChecked (lm->hasEndArrow ());

  QHBoxLayout *hl1 = new QHBoxLayout (options, 5, 5, "hl1");
  hl1->addWidget (GroupBox1);

  tw->insertTab (options, tr ("Opti&ons"));

  head = new QWidget (tw, "head");
  QButtonGroup *GroupBox4 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), head, "GroupBox4");

  new QLabel (tr ("Length"), GroupBox4, "TextLabel111", 0);
  boxHeadLength = new QSpinBox (0, 100, 1, GroupBox4, "boxHeadLength");
  boxHeadLength->setValue (lm->headLength ());

  new QLabel (tr ("Angle"), GroupBox4, "TextLabel1112", 0);
  boxHeadAngle = new QSpinBox (0, 85, 5, GroupBox4, "boxHeadAngle");
  boxHeadAngle->setValue (lm->headAngle ());

  filledBox = new QCheckBox (GroupBox4, "filledBox");
  filledBox->setText (tr ("&Filled"));
  filledBox->setChecked (lm->filledArrowHead ());

  QHBoxLayout *hl3 = new QHBoxLayout (head, 5, 5, "hl3");
  hl3->addWidget (GroupBox4);

  tw->insertTab (head, tr ("Arrow &Head"));

  initGeometryTab ();

  GroupBox2 =
    new QButtonGroup (4, QGroupBox::Horizontal, QString::null, this,
		      "GroupBox2");
  GroupBox2->setLineWidth (0);

  buttonDefault = new QPushButton (GroupBox2);
  buttonDefault->setText (tr ("Set &Default"));

  btnApply = new QPushButton (GroupBox2, "btnApply");
  btnApply->setText (tr ("&Apply"));

  btnOk = new QPushButton (GroupBox2, "btnOk");
  btnOk->setText (tr ("&Ok"));

  QVBoxLayout *hlayout = new QVBoxLayout (this, 5, 5, "hlayout");
  hlayout->addWidget (tw);
  hlayout->addWidget (GroupBox2);

  enableHeadTab ();

  connect (colorBox, SIGNAL (clicked ()), this, SLOT (pickColor ()));
  connect (btnOk, SIGNAL (clicked ()), this, SLOT (accept ()));
  connect (btnApply, SIGNAL (clicked ()), this, SLOT (apply ()));
  connect (tw, SIGNAL (currentChanged (QWidget *)), this,
	   SLOT (enableButtonDefault (QWidget *)));
  connect (buttonDefault, SIGNAL (clicked ()), this,
	   SLOT (setDefaultValues ()));

  connect(styleBox, SIGNAL( activated(int) ), this, SLOT( apply() ) );
  connect(widthBox, SIGNAL( valueChanged(int) ), this, SLOT( apply() ) );
  connect(startBox, SIGNAL( toggled(bool) ), this, SLOT( apply() ) );
  connect(endBox, SIGNAL( toggled(bool) ), this, SLOT( apply() ) );
  connect(boxHeadLength, SIGNAL( valueChanged(int) ), this, SLOT( apply() ) );
  connect(boxHeadAngle, SIGNAL( valueChanged(int) ), this, SLOT( apply() ) );
  connect(filledBox, SIGNAL( toggled(bool) ), this, SLOT( apply() ) );
}

void
lineDialog::initGeometryTab ()
{
  geometry = new QWidget (tw);

  QHBox *box1 = new QHBox (geometry);
  box1->setSpacing (5);
  box1->setMargin (5);

  new QLabel (tr ("Unit"), box1);
  unitBox = new QComboBox (box1);
  unitBox->insertItem (tr ("Scale Coordinates"));
  unitBox->insertItem (tr ("Pixels"));

  QHBox *box2 = new QHBox (geometry);
  box2->setSpacing (5);

  QButtonGroup *GroupBox2 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr ("Start Point"), box2);

  new QLabel (tr ("X"), GroupBox2, "TextLabel11", 0);
  xStartBox = new QLineEdit (GroupBox2);

  new QLabel (tr ("Y"), GroupBox2, "TextLabel111", 0);
  yStartBox = new QLineEdit (GroupBox2);

  QButtonGroup *GroupBox3 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr ("End Point"), box2);

  new QLabel (tr ("X"), GroupBox3, "TextLabel11", 0);
  xEndBox = new QLineEdit (GroupBox3);

  new QLabel (tr ("Y"), GroupBox3, "TextLabel111", 0);
  yEndBox = new QLineEdit (GroupBox3);

  QHBox *box3 = new QHBox (geometry);
  box3->setSpacing (5);
  new QLabel (tr("Slope : "),box3);
  slopeBox = new QLineEdit (box3);
  slopeBox->setReadOnly(true);

  QVBoxLayout *hl2 = new QVBoxLayout (geometry, 5, 5);
  hl2->addWidget (box1);
  hl2->addWidget (box2);
  hl2->addWidget (box3);

  tw->insertTab (geometry, tr ("&Geometry"));

  connect (unitBox, SIGNAL (activated (int)), this,
	   SLOT (displayCoordinates (int)));
  displayCoordinates (0);
}

void
lineDialog::displaySlope(int unit)
{
  if(unit==ScaleCoordinates)
  {
    slopeBox->setEnabled(true);

    bool inf=false;
    double slope = lm->slope(&inf);
    if(!inf)
      slopeBox->setText(QString::number(slope));
    else
      slopeBox->setText("inf");
  }
  else
    slopeBox->setEnabled(false);
}

void
lineDialog::displayCoordinates (int unit)
{
  double x1,y1,x2,y2;
  if (unit == ScaleCoordinates)
    {
      QwtDoublePoint sp = lm->startPointCoord ();
      QwtDoublePoint ep = lm->endPointCoord ();
      x1 = sp.x(); y1 = sp.y();
      x2 = ep.x(); y2 = ep.y();
    }
  else
    {
      QPoint startPoint = lm->startPoint ();
      QPoint endPoint = lm->endPoint ();
      x1 = startPoint.x(); y1 = startPoint.y();
      x2 = endPoint.x();   y2 = endPoint.y();
    }
  xStartBox->setText (QString::number(x1));
  yStartBox->setText (QString::number(y1));
  xEndBox->setText (QString::number(x2));
  yEndBox->setText (QString::number(y2));
  displaySlope(unit);
}

void
lineDialog::setCoordinates (int unit)
{
  if (unit == ScaleCoordinates)
    {
      lm->setStartPoint (xStartBox->text ().replace (",", ".").toDouble (),
			 yStartBox->text ().replace (",", ".").toDouble ());
      lm->setEndPoint (xEndBox->text ().replace (",", ".").toDouble (),
		       yEndBox->text ().replace (",", ".").toDouble ());
    }
  else
    {
      lm->
	setStartPoint (QPoint
		       (xStartBox->text ().toInt (),
			yStartBox->text ().toInt ()));
      lm->
	setEndPoint (QPoint
		     (xEndBox->text ().toInt (), yEndBox->text ().toInt ()));
    }
  displaySlope(unit);
}

void
lineDialog::apply ()
{
  if (tw->currentPage () == (QWidget *) options)
    {
      lm->setStyle (Graph::getPenStyle (styleBox->currentItem ()));
      lm->setColor (colorBox->color ());
      lm->setWidth (widthBox->value());
      lm->drawEndArrow (endBox->isChecked ());
      lm->drawStartArrow (startBox->isChecked ());
    }
  else if (tw->currentPage () == (QWidget *) head)
    {
      if (lm->headLength () != boxHeadLength->value ())
	lm->setHeadLength (boxHeadLength->value ());

      if (lm->headAngle () != boxHeadAngle->value ())
	lm->setHeadAngle (boxHeadAngle->value ());

      if (lm->filledArrowHead () != filledBox->isChecked ())
	lm->fillArrowHead (filledBox->isChecked ());
    }
  else if (tw->currentPage () == (QWidget *) geometry)
    setCoordinates (unitBox->currentItem ());

  QwtPlot *plot = lm->plot ();
  Graph *g = (Graph *) plot->parent ();
  plot->replot ();
  g->emitModified ();

  enableHeadTab ();
}

void
lineDialog::accept ()
{
  apply ();
  close ();
}

void
lineDialog::setLineStyle (Qt::PenStyle style)
{
  if (style == Qt::SolidLine)
    styleBox->setCurrentItem (0);
  else if (style == Qt::DashLine)
    styleBox->setCurrentItem (1);
  else if (style == Qt::DotLine)
    styleBox->setCurrentItem (2);
  else if (style == Qt::DashDotLine)
    styleBox->setCurrentItem (3);
  else if (style == Qt::DashDotDotLine)
    styleBox->setCurrentItem (4);
}

void
lineDialog::enableHeadTab ()
{
  if (startBox->isChecked () || endBox->isChecked ())
    tw->setTabEnabled (head, true);
  else
    tw->setTabEnabled (head, false);
}

void
lineDialog::pickColor ()
{
  QColor c = QColorDialog::getColor (colorBox->color (), this);
  if (!c.isValid () || c == colorBox->color ())
    return;

  colorBox->setColor (c);
  apply();
}

void
lineDialog::setDefaultValues ()
{
  ApplicationWindow *app = (ApplicationWindow *) this->parent ();
  if (!app)
    return;

  app->setArrowDefaultSettings (widthBox->value(),
				colorBox->color (),
				Graph::getPenStyle (styleBox->currentItem ()),
				boxHeadLength->value (),
				boxHeadAngle->value (),
				filledBox->isChecked ());
}

void
lineDialog::enableButtonDefault (QWidget * w)
{
  if (w == geometry)
    buttonDefault->setEnabled (false);
  else
    buttonDefault->setEnabled (true);
}

lineDialog::~lineDialog ()
{
}
