/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "matrixInterpDialog.h"
#include "matrix.h"
#include "parser.h"
#include "application.h"
#include "nrutil.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qmessagebox.h>

matrixInterpDialog::matrixInterpDialog (QWidget * parent, const char *name,
					  bool modal, WFlags fl):
QDialog (parent, name, modal, fl)
{
  if (!name)
    setName ("matrixInterpDialog");
  setCaption (tr ("QtiPlot - Matrix Interpolation Options"));

  QButtonGroup *GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox1");

  new QLabel (tr ("2D Interpolation of"), GroupBox1, "TextLabel1", 0);
  boxName = new QComboBox (GroupBox1);

  new QLabel (tr ("Method"), GroupBox1, "TextLabel2", 0);
  boxMethod = new QComboBox (GroupBox1);

  new QLabel (tr ("X Min"), GroupBox1, "TextLabel4", 0);
  boxXMin = new QLineEdit (GroupBox1);
  boxXMin->setText (tr ("0"));

  new QLabel (tr ("X Max"), GroupBox1, "TextLabel5", 0);
  boxXMax = new QLineEdit (GroupBox1);
  boxXMax->setText (tr ("10"));

  new QLabel (tr ("X Step"), GroupBox1, "TextLabel3", 0);
  boxXStep = new QLineEdit (GroupBox1);
  boxXStep->setText(tr("1"));

  new QLabel (tr ("Y Min"), GroupBox1, "TextLabel41", 0);
  boxYMin = new QLineEdit (GroupBox1);
  boxYMin->setText (tr ("0"));

  new QLabel (tr ("Y Max"), GroupBox1, "TextLabel51", 0);
  boxYMax = new QLineEdit (GroupBox1);
  boxYMax->setText (tr ("10"));

  new QLabel (tr ("Y Step"), GroupBox1, "TextLabel31", 0);
  boxYStep = new QLineEdit (GroupBox1);
  boxYStep->setText(tr("1"));

  QButtonGroup *GroupBox2 =
    new QButtonGroup (1, QGroupBox::Horizontal, tr (""), this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonApply = new QPushButton (GroupBox2, "buttonApply");
  buttonApply->setAutoDefault (TRUE);
  buttonApply->setDefault (TRUE);

  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setAutoDefault (TRUE);

  QHBoxLayout *hlayout = new QHBoxLayout (this, 5, 5, "hlayout");
  hlayout->addWidget (GroupBox1);
  hlayout->addWidget (GroupBox2);

  languageChange ();

  // signals and slots connections
  connect (boxName, SIGNAL (activated (int)), this,
	   SLOT (activateMatrix (int)));
  connect (buttonApply, SIGNAL (clicked ()), this, SLOT (accept ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
}

matrixInterpDialog::~matrixInterpDialog ()
{
}


void
matrixInterpDialog::languageChange ()
{
  buttonApply->setText (tr ("&Make"));
  buttonCancel->setText (tr ("&Close"));

  boxMethod->clear();
  boxMethod->insertItem (tr ("Linear"));
  boxMethod->insertItem (tr ("Cubic Spline"));
  boxMethod->insertItem (tr ("Non-rounded Akima"));
}

void
matrixInterpDialog::accept ()
{
  double xmin,xmax,xstep,ymin,ymax,ystep;

  try
  {
    myParser parser;
    parser.SetExpr (boxXMin->text ().ascii ());
    xmin = parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (this, tr ("QtiPlot - X Min value error"),
			   e.GetMsg ().c_str());
    boxXMin->setFocus ();
    return;
  }

  try
  {
    myParser parser;
    parser.SetExpr (boxXMax->text ().ascii ());
    xmax = parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (this, tr ("QtiPlot - X Max value error"),
			   e.GetMsg ().c_str());
    boxXMax->setFocus ();
    return;
  }

  if (xmin > xmax)
    {
      QMessageBox::critical( this, tr("Value Error"),
			     tr("X Max must be greater than X Min"));
      return;
    }

  try
  {
    myParser parser;
    parser.SetExpr (boxXStep->text ().ascii ());
    xstep = parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (this, tr ("QtiPlot - X Step value error"),
			   e.GetMsg ().c_str());
    boxXStep->setFocus ();
    return;
  }

  if (xstep <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Input error"),
			     tr("X Step must be positive value."));
      boxXStep->setFocus ();
      return;
    }

  try
  {
    myParser parser;
    parser.SetExpr (boxYMin->text ().ascii ());
    ymin = parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (this, tr ("QtiPlot - Y Min value error"),
			   e.GetMsg ().c_str());
    boxYMin->setFocus ();
    return;
  }

  try
  {
    myParser parser;
    parser.SetExpr (boxYMax->text ().ascii ());
    ymax = parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (this, tr ("QtiPlot - Y Max value error"),
			   e.GetMsg ().c_str());
    boxYMax->setFocus ();
    return;
  }

  if (ymin > ymax)
    {
      QMessageBox::critical( this, tr("Value Error"),
			     tr("Y Max must be greater than Y Min"));
      return;
    }

  try
  {
    myParser parser;
    parser.SetExpr (boxYStep->text ().ascii ());
    ystep = parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (this, tr ("QtiPlot - Y Step value error"),
			   e.GetMsg ().c_str());
    boxYStep->setFocus ();
    return;
  }

  if (ystep <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Input error"),
			     tr("Y Step must be positive value."));
      boxYStep->setFocus ();
      return;
    }

  ApplicationWindow *app = (ApplicationWindow *)parent();

  int rows = qRound( (xmax-xmin)/xstep ) + 1;
  int cols = qRound( (ymax-ymin)/ystep ) + 1;
  xmax = xmin + xstep * (rows-1);
  ymax = ymin + ystep * (cols-1);

  Matrix *m = app->matrix(boxName->currentText());
  if(!m) return;

  QString caption = app->generateUniqueName (tr ("MatrixInt"));
  Matrix *w = app->newMatrix (caption, rows, cols);
  QString label;
  switch(boxMethod->currentItem())
    {
    case 0:label = "Linear"; break;
    case 1:label = "Cubic Spline"; break;
    case 2:label = "Akima Spline"; break;
    }
  label += " Interpolation of " + boxName->currentText();

  double **temp = matrix( 0, rows, 0, cols );
  m->interpolate( xmin, xmax, rows, ymin, ymax, cols, 
		  boxMethod->currentItem(), temp );
  for (int i = 0; i < rows; i++)
    for (int j = 0; j < cols; j++)
      w->setValue(i, j, temp[i][j] );

  free_matrix( temp, 0, rows, 0, cols );
  
  w->setCoordinates( xmin, xmax, ymin, ymax );
  w->setWindowLabel ( label );
  w->setCaptionPolicy ( myWidget::Both );
  w->show();

  close();
}

void
matrixInterpDialog::setMatrixNames( QStringList names )
{
  boxName->clear();
  boxName->insertStringList(names);
}

void
matrixInterpDialog::setMatrix (Matrix *m)
{
  if (!m) return;

  boxName->setCurrentText(m->getName());

  d_matrix = m;
  double xmin = QMIN(m->xStart(),m->xEnd());
  double xmax = QMAX(m->xStart(),m->xEnd());
  double xstep = (xmax - xmin)/(m->numRows()-1);
  double ymin = QMIN(m->yStart(),m->yEnd());
  double ymax = QMAX(m->yStart(),m->yEnd());
  double ystep = (ymax - ymin)/(m->numCols()-1);

  boxXMin->setText(QString::number(xmin));
  boxXMax->setText(QString::number(xmax));
  boxXStep->setText(QString::number(xstep));
  boxYMin->setText(QString::number(ymin));
  boxYMax->setText(QString::number(ymax));
  boxYStep->setText(QString::number(ystep));

  connect (d_matrix, SIGNAL (closedWindow (myWidget *)), this, SLOT (close ()));
}

void
matrixInterpDialog::activateMatrix(int index)
{
  ApplicationWindow *app = (ApplicationWindow *)parent();
  QString title = boxName->text(index);
  Matrix *m = app->matrix( title );
  if (m) setMatrix( m );
}
