/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#ifndef MUPARSER_SCRIPTING_H
#define MUPARSER_SCRIPTING_H

#include "Scripting.h"
#include "gslutils.h"

//#include "muParser.h"
#include "parser.h"

#include <gsl/gsl_sf.h>
#include <qasciidict.h>

class muParserScript:public Script
{
Q_OBJECT public:
  muParserScript (ScriptingEnv * env, const QString & code,
		  QObject * context = 0, const QString & name = "<input>");

  public slots:bool compile (bool asFunction = true);
  QVariant eval ();
  bool exec ();
  bool setQObject (QObject * val, const char *name);
  bool setInt (int val, const char *name);
  bool setDouble (double val, const char *name);

private:
  //double col (const QString & arg);
  double col (const char *arg);
  double cell (int row, int col);
  double *addVariable (const char *name);
  double *addVariableR (const char *name);
  static double mu_col (const char *arg)
  {
    return current->col (arg);
  }
  static double mu_cell (double row, double col)
  {
    return current->cell (qRound (row), qRound (col));
  }
  static double *mu_addVariable (const char *name, void *)
  {
    return current->addVariable (name);
  }
  static double *mu_addVariableR (const char *name, void *)
  {
    return current->addVariableR (name);
  }
  static QString compileColArg (const QString & in);

  //  mu::Parser parser, rparser;
  myParser parser, rparser;
  QAsciiDict < double >variables, rvariables;
  QStringList muCode;

public:
  static muParserScript *current;
};

class muParserScripting:public ScriptingEnv
{
Q_OBJECT public:
  static const char *langName;

  muParserScripting (ApplicationWindow * parent):
  ScriptingEnv (parent, langName)
  {
    initialized = true;
  }
  static ScriptingEnv *constructor (ApplicationWindow * parent)
  {
    return new muParserScripting (parent);
  }

  bool isRunning () const
  {
    return true;
  }
  Script *newScript (const QString & code, QObject * context,
		     const QString & name = "<input>")
  {
    return new muParserScript (this, code, context, name);
  }

  // we do not support global variables
  bool setQObject (QObject *, const char *)
  {
    return false;
  }
  bool setInt (int, const char *)
  {
    return false;
  }
  bool setDouble (double, const char *)
  {
    return false;
  }

  const QStringList mathFunctions () const;
  const QString mathFunctionDoc (const QString & name) const;

  static const myParser::mathFunction math_functions[];

private:
  static double myrand (double x)
  {
    if(x<=0) return 0;
    return x * rand_uniform();
  }
  static double poisson(double x)
  {
    return (double)rand_poisson(x);
  }
  static double nrand(double x)
  {
    return rand_gaussian(x);
  }
  static double erand(double x)
  {
    return rand_exponential(x);
  }
  static double binomial(double x, double y)
  {
    return rand_binomial(x,y);
  }
  static double grand(double x, double y)
  {
    return rand_gamma(x,y);
  }
};

class EmptySourceError:public
  mu::ParserError
{
public:
  EmptySourceError ()
  {
  }
};

#endif
