/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "normalizeDialog.h"
#include "plot.h"
#include "graph.h"
#include "parser.h"
#include "application.h"
#include "pixmaps.h"
#include "nrutil.h"

#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qapplication.h>
#include <qspinbox.h>

#include <qwt_plot.h>
#include <qwt_scale_engine.h>

normalizeDialog::normalizeDialog(QWidget *parent, const char *name, bool modal,
			     WFlags fl):
  QDialog(parent, name, modal, fl)
{
  if (!name)
    setName ("normalizeDialog");
  setCaption (tr ("QtiPlot - Normalize Graph"));

  QButtonGroup *GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr ("Search Maximum"), this, "GroupBox1");
  
  new QLabel (tr ("from"), GroupBox1, "TextLabel1", 0);
  boxMaxFrom = new QLineEdit (GroupBox1, "boxMaxFrom");
  boxMaxFrom->setText (tr ("0"));

  new QLabel (tr ("to"), GroupBox1, "TextLabel2", 0);
  boxMaxTo = new QLineEdit (GroupBox1, "boxMaxTo");
  boxMaxTo->setText (tr ("0"));

  new QLabel (tr ("Average"), GroupBox1, "TextLabel21", 0);
  boxAverage1 = new QCheckBox("",GroupBox1);
  boxAverage1->setChecked(false);

  new QLabel (tr ("Smoothing Points"), GroupBox1, "TextLabel3", 0);
  boxMaxSmooth = new QSpinBox(1, 99, 2, GroupBox1, "boxMaxSmooth");
  boxMaxSmooth->setValue (5);

  boxSubtract =
    new QButtonGroup (2, QGroupBox::Horizontal, tr ("Subtract Base"), this, "GroupBox2");
  boxSubtract->setCheckable(true);
  boxSubtract->setChecked(false);
  //  GroupBox2->setFlat (TRUE);
  // GroupBox2->setLineWidth (0);

  new QLabel (tr ("from"), boxSubtract, "TextLabel4", 0);
  boxMinFrom = new QLineEdit (boxSubtract, "boxMinFrom");
  boxMinFrom->setText (tr ("0"));

  new QLabel (tr ("to"), boxSubtract, "TextLabel5", 0);
  boxMinTo = new QLineEdit (boxSubtract, "boxMinTo");
  boxMinTo->setText (tr ("0"));

  new QLabel (tr ("Average"), boxSubtract, "TextLabel51", 0);
  boxAverage2 = new QCheckBox("",boxSubtract);
  boxAverage2->setChecked(false);

  new QLabel (tr ("Smoothing Points"), boxSubtract, "TextLabel6", 0);
  boxMinSmooth = new QSpinBox(1, 99, 5, boxSubtract, "boxMinSmooth");
  boxMinSmooth->setValue(5);

  QButtonGroup *GroupBox3 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox3");
  GroupBox3->setFlat (TRUE);
  GroupBox3->setLineWidth (0);

  buttonApply = new QPushButton (GroupBox3, "buttonApply");
  buttonApply->setAutoDefault (TRUE);
  buttonApply->setDefault (TRUE);
  buttonApply->setText (tr ("&Apply"));

  buttonClose = new QPushButton (GroupBox3, "buttonClose");
  buttonClose->setAutoDefault (TRUE);
  buttonClose->setText (tr ("&Close"));

  QVBoxLayout *vlayout = new QVBoxLayout (this, 5, 5, "vlayout");
  vlayout->addWidget (GroupBox1);
  vlayout->addWidget (boxSubtract);
  vlayout->addWidget (GroupBox3);

  // signals and slots connections
  connect (buttonApply, SIGNAL (clicked ()), this, SLOT (accept()));
  connect (buttonClose, SIGNAL (clicked ()), this, SLOT (close()));
  connect (boxAverage1, SIGNAL (clicked()), this, SLOT (showMaxPoints()));
  connect (boxAverage2, SIGNAL (clicked()), this, SLOT (showMinPoints()));

  firsttime = true;
}

void
normalizeDialog::showMaxPoints()
{
  boxMaxSmooth->setEnabled(!boxAverage1->isChecked());
}

void
normalizeDialog::showMinPoints()
{
  boxMinSmooth->setEnabled(!boxAverage2->isChecked());
}

normalizeDialog::~normalizeDialog()
{
}

void
normalizeDialog::setGraph(Graph *g, const QString &name)
{
  d_graph = g;
  graphname = name;

  connect (d_graph, SIGNAL (closedGraph ()), this, SLOT (close ()));

  Plot *d_plot = d_graph->plotWidget ();
  const QwtScaleDiv *scDiv = d_plot->axisScaleDiv (QwtPlot::xBottom);
  QString xmin = QString::number(QMIN (scDiv->lBound (), scDiv->hBound ()));
  QString xmax = QString::number (QMAX (scDiv->lBound (), scDiv->hBound ()));

  boxMaxFrom->setText(xmin);
  boxMaxTo->setText(xmax);
  boxMinFrom->setText(xmin);
  boxMinTo->setText(xmax);
}

double
normalizeDialog::curve_max(QwtPlotCurve *c, double xmin, double xmax, int np)
{
  int n = c->dataSize();
  QMemArray <double> s(n);

  if(np<4)
    {
      for(int i = 0; i < n; i++ )
	s[i] = c->y(i);
    }
  else
    {
      QMemArray <double> y(n);
      for(int i=0; i<n; i++) y[i] = c->y(i);
      smooth_vect(y.data(), n, np/2, np/2, 2, s.data());
    }

  double max=0;
  bool first = true;
  for(int i = 0; i < n; i++)
    if((c->x(i)>=xmin) && (c->x(i)<=xmax))
      if( first || (s[i]>max) )
	{
	  max = s[i];
	  first = false;
	}
  return max;
}

double
normalizeDialog::curve_min(QwtPlotCurve *c, double xmin, double xmax, int np)
{
  int n = c->dataSize();
  QMemArray <double> s(n);
  
  if(np<4)
    {
      for(int i = 0; i < n; i++ )
	s[i] = c->y(i);
    }
  else
    {
      QMemArray <double> y(n);
      for(int i=0; i<n; i++) y[i] = c->y(i);
      smooth_vect(y.data(), n, np/2, np/2, 2, s.data());
    }

  double min=0;
  bool first = true;
  for(int i = 0; i < n; i++)
    if( (c->x(i) >= xmin) && (c->x(i) <= xmax) )
      if( first || (s[i]<min) )
	{
	  min = s[i];
	  first = false;
	}
  return min;
}

double
normalizeDialog::curve_average(QwtPlotCurve *c, double xmin, double xmax)
{
  int n = c->dataSize();
  int num=0;
  double sum=0;
  for(int i=0; i<n; i++)
    {
      if(c->x(i)>=xmin && c->x(i)<=xmax)
	{
	  num++;
	  sum += c->y(i);
	}
    }
  if(num>0)
    return sum/num;
  else
    return 0;
}

void
normalizeDialog::accept()
{
  double maxfrom,maxto,minfrom,minto;
  int maxsmooth,minsmooth;

  try
  {
    myParser parser;
    parser.SetExpr (boxMaxFrom->text().ascii());
    maxfrom = parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (this, tr ("QtiPlot - from value error"),
                           e.GetMsg ().c_str());
    boxMaxFrom->setFocus ();
    return;
  }

  try
  {
    myParser parser;
    parser.SetExpr (boxMaxTo->text().ascii());
    maxto = parser.Eval ();
  }
  catch (mu::ParserError & e)
  {
    QMessageBox::critical (this, tr ("QtiPlot - to value error"),
                           e.GetMsg ().c_str());
    boxMaxTo->setFocus ();
    return;
  }

  maxsmooth = boxMaxSmooth->value();

  if(boxSubtract->isChecked())
    {
      try
	{
	  myParser parser;
	  parser.SetExpr (boxMinFrom->text().ascii());
	  minfrom = parser.Eval ();
	}
      catch (mu::ParserError & e)
	{
	  QMessageBox::critical (this, tr ("QtiPlot - from value error"),
				 e.GetMsg ().c_str());
	  boxMinFrom->setFocus ();
	  return;
	}

      try
	{
	  myParser parser;
	  parser.SetExpr (boxMinTo->text().ascii());
	  minto = parser.Eval ();
	}
      catch (mu::ParserError & e)
	{
	  QMessageBox::critical (this, tr ("QtiPlot - to value error"),
				 e.GetMsg ().c_str());
	  boxMinTo->setFocus ();
	  return;
	}

      minsmooth = boxMinSmooth->value();
    }

  ApplicationWindow *app = (ApplicationWindow *)this->parent();

  int prec = app->tbl_output_precision;

  if(firsttime)
    {
      //QStringList cl = d_graph->plotItemsList(); // all plots
      int maxrows = 0;
      for (int i = 0; i < d_graph->curves(); i++)
	{
	  QwtPlotCurve *c = d_graph->curve(i);
	  if (c && c->rtti () != QwtPlotItem::Rtti_PlotSpectrogram
	      && d_graph->curveType(i) != Graph::XErrorBars
	      && d_graph->curveType(i) != Graph::YErrorBars)
	    {
	      curves << i;
              int ds=c->dataSize();
	      if (maxrows < ds) maxrows = ds;
	      if(maxsmooth > ds) maxsmooth = ds;
              if(minsmooth > ds) minsmooth = ds;
	    }
	}
      QString tname = app->generateUniqueName("Normalized");
      QString legend = tr("Normalized data of ") + graphname;
      normalizeTable = app->newHiddenTable( tname, legend, maxrows,
					  curves.count()*2, prec);
      for(int i=0;i<(int)curves.count();i++)
	{
	  normalizeTable->setColPlotDesignation(i*2,  Table::X);
	  normalizeTable->setColPlotDesignation(i*2+1,Table::Y);
	}
      normalizeTable->setHeaderColType();
    }

  QApplication::setOverrideCursor (waitCursor);

  int nc = (int)curves.count();
  for (int i = 0; i < nc; i++)
    {
      QwtPlotCurve *c = d_graph->curve(curves[i]);
      double max = 0;
      if(boxAverage1->isChecked())
	max = curve_average( c, maxfrom, maxto );
      else
        max = curve_max( c, maxfrom, maxto, maxsmooth );
      double min = 0;
      if(boxSubtract->isChecked())
	{
	  if(boxAverage2->isChecked())
	    min = curve_average( c, minfrom, minto );
	  else
	    min = curve_min( c, minfrom, minto, minsmooth );
	}
      for (int r = 0; r < c->dataSize(); r++)
	{
	  double xv = c->x(r);
	  double yv = (c->y(r) - min) / (max - min);
	  normalizeTable->setText( r, i*2, QString::number(xv,'g',prec) );
	  normalizeTable->setText( r, i*2+1, QString::number(yv,'g',prec) );
	}
    }
  if(firsttime)
    {
      normalizePlot = app->multilayerPlot(normalizeTable,
					  normalizeTable->YColumns(),
					  Graph::Line );
      ((myWidget *)normalizePlot)->setWindowLabel(tr("Normalized curves of ")+graphname);
      Graph *ag = app->activeGraph;
      for(int i = 0; i < nc; i++)
	{
	  QwtPlotCurve *c = d_graph->curve(curves[i]);
	  ag->setCurvePen(i,c->pen());
	  ag->setCurveSymbol(i,c->symbol());
	  int style = d_graph->curveType(curves[i]);
	  if(style <= Graph::LineSymbols)
	    ag->setCurveType(i,style);
	  if(style==Graph::Scatter)
	    ag->setCurveStyle(i,QwtPlotCurve::NoCurve);
 	}
      app->activeGraph->replot();
      firsttime = false;
    }
  else
    normalizeTable->notifyChanges();

  QApplication::restoreOverrideCursor ();
}
