/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "parser.h"

const
  myParser::mathFunction myParser::math_functions[] = {
  {"abs", 1, NULL, NULL, NULL, NULL,
   "abs(x):\n Absolute value of x."},
  {"acos", 1, NULL, NULL, NULL, NULL,
   "acos(x):\n Inverse cos function."},
  {"acosh", 1, NULL, NULL, NULL, NULL,
   "acosh(x):\n Hyperbolic inverse cos function."},
  {"airyAi", 1, airyAi, NULL, NULL, NULL,
   "airyAi(x):\n Airy function Ai(x)."},
  {"airyBi", 1, airyBi, NULL, NULL, NULL,
   "airyBi(x):\n Airy function Bi(x)."},
  {"asin", 1, NULL, NULL, NULL, NULL,
   "asin(x):\n Inverse sin function."},
  {"asinh", 1, NULL, NULL, NULL, NULL,
   "asinh(x):\n Hyperbolic inverse sin function."},
  {"atan", 1, NULL, NULL, NULL, NULL,
   "atan(x):\n Inverse tan function."},
  {"atan2", 2, NULL, myatan2, NULL, NULL,
   "atan2(y,x):\n Inverse tan function same to atan(y/x)."},
  {"atanh", 1, NULL, NULL, NULL, NULL,
   "atanh(x):\n  Hyperbolic inverse tan function."},
  {"avg", -1, NULL, NULL, NULL, NULL,
   "avg(x,y,...):\n  Mean value of all arguments."},
  {"besselI", 2, NULL, bessel_I, NULL, NULL,
   "besselI(nu,x):\n Modified cylindrical Bessel function of order n, I_nu(x). nu can be fractional."},
  {"besselJ", 2, NULL, bessel_J, NULL, NULL,
   "besselJ(nu,x):\n Regular cylindrical Bessel function of order n, J_nu(x). nu can be fractional."},
  {"besselJzero", 2, NULL, bessel_Jn_zero, NULL, NULL,
   "besselJzero(nu,int s):\n s-th positive zero x_s of regular cylindrical Bessel function of order n, J_nu(x_s)=0. nu can be fractional."},
  {"besselK", 2, NULL, bessel_K, NULL, NULL,
   "besselK(nu,x):\n Modified cylindrical Bessel function of order n, K_nu(x). nu can be fractional."},
  {"besselY", 2, NULL, bessel_Y, NULL, NULL,
   "besselY(nu,x):\n Irregular cylindrical Bessel function of order n, Y_nu(x). nu can be fractional."},
  {"bessel_j0", 1, bessel_J0, NULL, NULL, NULL,
   "bessel_j0(x):\n  Regular cylindrical Bessel function of zeroth order, J_0(x). Obsolate. Use besselJ."},
  {"bessel_j1", 1, bessel_J1, NULL, NULL, NULL,
   "bessel_j1(x):\n  Regular cylindrical Bessel function of first order, J_1(x). Obsolate. Use besselJ."},
  {"bessel_jn", 2, NULL, bessel_Jn, NULL, NULL,
   "bessel_jn(x,int n):\n Regular cylindrical Bessel function of order n, J_n(x). Obsolate. Use besselJ."},
  {"bessel_jn_zero", 2, NULL, bessel_Jn_zero, NULL, NULL,
   "bessel_jn_zero(int n,int s):\n s-th positive zero x_s of regular cylindrical Bessel function of order n, J_n(x_s)=0. Obsolate. Use besselJzero."},
  {"bessel_y0", 1, bessel_Y0, NULL, NULL, NULL,
   "bessel_y0(x):\n Irregular cylindrical Bessel function of zeroth order, Y_0(x). Obsolate. Use besselY."},
  {"bessel_y1", 1, bessel_Y1, NULL, NULL, NULL,
   "bessel_y1(x):\n Irregular cylindrical Bessel function of first order, Y_1(x). Obsolate. Use besselY."},
  {"bessel_yn", 2, NULL, bessel_Yn, NULL, NULL,
   "bessel_yn(x,int n):\n Irregular cylindrical Bessel function of order n, Y_n(x). Obsolate. Use besselY."},
  {"beta", 2, NULL, beta, NULL, NULL,
   "beta(a,b):\n Computes the Beta Function, B(a,b) = Gamma(a)*Gamma(b)/Gamma(a+b) for a > 0, b > 0."},
  {"ceil", 1, myceil, NULL, NULL, NULL,
   "ceil(x):\n Round to the next larger integer,\n smallest integer larger or equal to x."},
  {"chebyshevT", 2, NULL, chebyshevT, NULL, NULL,
   "chebyshevT(int n,x):\n Computes the Chebyshev polynomial of first kind T_n(x)."},
  {"chebyshevU", 2, NULL, chebyshevU, NULL, NULL,
   "chebyshevU(int n,x):\n Computes the Chebyshev polynomial of second kind U_n(x)."},
  {"comb", 2, NULL, comb, NULL, NULL,
   "comb(int n,int k):\n Calculates the combinatorial factor nCk."},
  {"cos", 1, NULL, NULL, NULL, NULL,
   "cos(x):\n Calculate cosine."},
  {"cosh", 1, NULL, NULL, NULL, NULL,
   "cosh(x):\n Hyperbolic cos function."},
  {"debye", 2, NULL, debye, NULL, NULL,
   "debye(int n, x):\n Calculates Debye integral of (n/x^n)\\int^x_0 t^n/(exp(t)-1)dt. n upto 6 is supported."},
  {"dfact", 1, dfact, NULL, NULL, NULL,
   "dfact(int n):\n Double factorial of positive integer n!!=n(n-2)(n-4)... ."},
  {"ellintE", 2, NULL, ellintE, NULL, NULL,
   "ellintE(phi,k):\n Compute the incomplete elliptic integral E(phi,k)."},
  {"ellintF", 2, NULL, ellintF, NULL, NULL,
   "ellintF(phi,k):\n Compute the incomplete elliptic integral F(phi,k)."},
  {"ellintP", 3, NULL, NULL, ellintP, NULL,
   "ellintP(phi,k,n):\n Compute the incomplete elliptic integral Pi(phi,k,n)."},
  {"erf", 1, erf, NULL, NULL, NULL,
   "erf(x):\n  The error function."},
  {"erfc", 1, erfc, NULL, NULL, NULL,
   "erfc(x):\n Complementary error function erfc(x) = 1 - erf(x)."},
  {"erfz", 1, erfz, NULL, NULL, NULL,
   "erfz(x):\n The Gaussian probability density function Z(x)."},
  {"erfq", 1, erfq, NULL, NULL, NULL,
   "erfq(x):\n The upper tail of the Gaussian probability function Q(x)."},
  {"eta", 1, eta, NULL, NULL, NULL,
   "eta(x):\n Computes the Eta function."},
  {"exp", 1, NULL, NULL, NULL, NULL,
   "exp(x):\n Exponential function: e raised to the power of x."},
  {"fact", 1, fact, NULL, NULL, NULL,
   "fact(int n):\n Factorial of positive integer n!=n(n-1)(n-2)... ."},
  {"floor", 1, myfloor, NULL, NULL, NULL,
   "floor(x):\n Round to the next smaller integer,\n largest integer smaller or equal to x."},
  {"gamma", 1, gamma, NULL, NULL, NULL,
   "gamma(x):\n Computes the Gamma function, subject to x not being a negative integer."},
  {"gammaln", 1, gammaln, NULL, NULL, NULL,
   "gammaln(x):\n Computes the logarithm of the Gamma function, subject to x not a being negative integer. For x<0, log(|Gamma(x)|) is returned."},
  {"gegenpoly", 3, NULL, NULL, gegenpoly, NULL,
   "gegenpoly(int n,m,x):\n Computes the Gegenbauer polynomial C^m_n(x)."},
  {"hazard", 1, hazard, NULL, NULL, NULL,
   "hazard(x):\n Computes the hazard function for the normal distribution h(x) = erfz(x)/erfq(x)."},
  {"hermite", 2, NULL, hermite, NULL, NULL,
   "hermite(int n,x):\n Computes Hermite polynomial H_n(x)."},
  {"hyperg", 4, NULL, NULL, NULL, hyperg,
   "hyperg(a,b,c,x): Computes hypergeometric function 2F1(a,b,c;x)."},
  {"hypergM", 3, NULL, NULL, hypergM, NULL,
   "hypergM(a,b,x): Computes confluent hypergeometric function M(a,b;x)."},
  {"hypergU", 3, NULL, NULL, hypergU, NULL,
   "hypergU(a,b,x): Computes confluent hypergeometric function U(a,b;x)."},
  {"if", 3, NULL, NULL, NULL,  NULL,
   "if(e1, e2, e3):\n if e1 then e2 else e3."},
  {"jacobiP", 4, NULL, NULL, NULL, jacobiP,
   "jacobiP(int n, a, b, x):\n Computes Jacobi Polynomial P^(a,b)_n(x)."},
  {"laguerre", 3, NULL, NULL, laguerre, NULL,
   "laguerre(int n,k,x):\n Computes Laguerre Polynomial L^k_n(x)."},
  {"legendreP", 2, NULL, legendreP, NULL, NULL,
   "legendreP(int n,x):\n Computes Legendre Polynomial P_n(x)."},
  {"legendrePnm", 3, NULL, NULL, legendrePnm, NULL,
   "legendrePnm(int n,int m,x):\n Computes Associated Legendre Polynomial P^m_n(x)."},
  {"legendreQ", 2, NULL, legendreQ, NULL, NULL,
   "legendreQ(int n,x):\n Computes Legendre Polynomial of second kind Q_n(x)."},
  {"ln", 1, NULL, NULL, NULL, NULL,
   "ln(x):\n Calculate natural logarithm log_e."},
  {"log", 1, NULL, NULL, NULL, NULL,
   "log(x):\n Calculate decimal logarithm log_10."},
  {"log10", 1, NULL, NULL, NULL, NULL,
   "log10(x):\n Calculate decimal logarithm log_10."},
  {"log2", 1, NULL, NULL, NULL, NULL,
   "log2(x):\n Calculate binary logarithm log_2."},
  {"mathieuCE", 3, NULL, NULL, mathieuCE, NULL,
   "mathieuCE(int n,q,x):\n Computes the angular Mathieu functions ce_n(q,x)."},
  {"mathieuSE", 3, NULL, NULL, mathieuSE, NULL,
   "mathieuSE(int n,q,x):\n Computes the angular Mathieu functions se_n(q,x)."},
  {"max", -1, NULL, NULL, NULL, NULL,
   "max(x,y,...):\n Calculates maximum of all arguments."},
  {"min", -1, NULL, NULL, NULL, NULL,
   "min(x,y,...):\n Calculates minimum of all arguments."},
  {"mod", 2, NULL, mod, NULL, NULL,
   "mod(x,y):\n Calculates rest of integer division x/y,\n x modulo y."},
  {"perm", 2, NULL, perm, NULL, NULL,
   "perm(int n,int k):\n Calculates the permutation factor nPk."},
  {"poch", 2, NULL, poch, NULL, NULL,
   "poch(a,x):\n Calculates the Pochhammer symbol (a)_x = gamma(a+x)/gamma(a)."},
  {"polygamma", 2, NULL, polygamma, NULL, NULL,
   "polygamma(int n,x):\n Calculates the polygamma functions of order n."},
  {"pow", 2, NULL, mypow, NULL, NULL,
   "pow(x,y):\n Raise x to the power of y, x^y."},
  {"rint", 1, NULL, NULL, NULL, NULL,
   "rint(x):\n Round to nearest integer."},
  {"sign", 1, NULL, NULL, NULL, NULL,
   "sign(x):\n Sign function: -1 if x<0; 1 if x>0."},
  {"sin", 1, NULL, NULL, NULL, NULL,
   "sin(x):\n Calculate sine."},
  {"sinh", 1, NULL, NULL, NULL, NULL,
   "sinh(x):\n Hyperbolic sin function."},
  {"sqrt", 1, NULL, NULL, NULL, NULL,
   "sqrt(x):\n Square root function."},
  {"sum", -1, NULL, NULL, NULL, NULL,
   "sum(x,y,...):\n Calculate sum of all arguments."},
  {"tan", 1, NULL, NULL, NULL, NULL,
   "tan(x):\n Calculates tangent function."},
  {"tanh", 1, NULL, NULL, NULL, NULL,
   "tanh(x):\n Hyperbolic tan function."},
  {"zeta", 1, zeta, NULL, NULL, NULL,
   "zeta(x):\n Computes the Zeta function."},
  {0, 0, NULL, NULL, NULL, NULL, 0}
};

myParser::myParser ():Parser ()
{
  DefineConst ("pi", M_PI);
  DefineConst ("Pi", M_PI);
  DefineConst ("PI", M_PI);
  DefineConst ("e", M_E);
  DefineConst ("E", M_E);

  for(const mathFunction *i = math_functions; i->name; i++)
    if (i->numargs == 1 && i->fun1 != NULL)
      DefineFun (i->name, i->fun1);
    else if (i->numargs == 2 && i->fun2 != NULL)
      DefineFun (i->name, i->fun2);
    else if (i->numargs == 3 && i->fun3 != NULL)
      DefineFun (i->name, i->fun3);
    else if (i->numargs == 4 && i->fun4 != NULL)
      DefineFun (i->name, i->fun4);
}

QStringList
myParser::functionsList ()
{
  QStringList lst;
  for(const mathFunction *i = math_functions; i->name; i++)
    lst << QString(i->description).section(":",0,0); 

  return lst;
}

QString
myParser::explainFunction (int index)
{
  return QString(math_functions[index].description);
}
