/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#ifndef MYPARSER_H
#define MYPARSER_H

#include "muParser.h"
#include <gsl/gsl_version.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_sf.h>

#if GSL_MAJOR_VERSION==2
#define gsl_sf_mathieu_ce gsl_sf_mathieu_ce_e
#define gsl_sf_mathieu_se gsl_sf_mathieu_se_e
#endif

#include <qstringlist.h>

using namespace mu;

class myParser:public Parser
{
public:
  myParser ();

  static QStringList functionsList ();
  static QString explainFunction (int index);

  static double myatan2( double x, double y )
  {
    return atan2(y,x);
  }
  static double myceil (double x)
  {
    return ceil (x);
  }
  static double myfloor (double x)
  {
    return floor (x);
  }
  static double mod (double x, double y)
  {
    return fmod (x, y);
  }
  static double mypow (double x, double y)
  {
    return pow (x, y);
  }
  static double bessel_J0 (double x)
  {
    return gsl_sf_bessel_J0 (x);
  }
  static double bessel_J1 (double x)
  {
    return gsl_sf_bessel_J1 (x);
  }
  static double bessel_Jn (double x, double n)
  {
    return gsl_sf_bessel_Jn ((int) n, x);
  }
  static double bessel_Y0 (double x)
  {
    return gsl_sf_bessel_Y0 (x);
  }
  static double bessel_Y1 (double x)
  {
    return gsl_sf_bessel_Y1 (x);
  }
  static double bessel_Yn (double x, double n)
  {
    return gsl_sf_bessel_Yn ((int) n, x);
  }
  static double bessel_I(double n, double x)
  {
    if(n==0)
      return gsl_sf_bessel_I0( x );
    else if(n==1)
      return gsl_sf_bessel_I1( x );
    else if(n==int(n))
      return gsl_sf_bessel_In( (int)n, x );
    else
      return gsl_sf_bessel_Inu( n, x );
  }
  static double bessel_J(double n, double x)
  {
    if(n==0)
      return gsl_sf_bessel_J0( x );
    else if(n==1)
      return gsl_sf_bessel_J1( x );
    else if(n==int(n))
      return gsl_sf_bessel_Jn((int) n, x);
    else
      return gsl_sf_bessel_Jnu( n, x );
  }
   static double bessel_K(double n, double x)
  {
    if(n==0)
      return gsl_sf_bessel_K0( x );
    else if(n==1)
      return gsl_sf_bessel_K1( x );
    else if(n==int(n))
      return gsl_sf_bessel_Kn( (int)n, x );
    else
      return gsl_sf_bessel_Knu( n, x );
  }
   static double bessel_Y(double n, double x)
   {
     if(n==0)
       return gsl_sf_bessel_Y0( x );
     else if(n==1)
       return gsl_sf_bessel_Y1( x );
     else if(n==int(n))
       return gsl_sf_bessel_Yn((int) n, x);
     else
       return gsl_sf_bessel_Ynu( n, x );
   }
   static double bessel_Jn_zero (double n, double s)
   {
     if(n==0)
       return gsl_sf_bessel_zero_J0((unsigned int) s);
     else if(n==1)
       return gsl_sf_bessel_zero_J1((unsigned int) s);
     else
       return gsl_sf_bessel_zero_Jnu(n,(unsigned int)s);
   }
  static double beta (double a, double b)
  {
    return gsl_sf_beta (a, b);
  }
  static double erf (double x)
  {
    return gsl_sf_erf (x);
  }
  static double erfc (double x)
  {
    return gsl_sf_erfc (x);
  }
  static double erfz (double x)
  {
    return gsl_sf_erf_Z (x);
  }
  static double erfq (double x)
  {
    return gsl_sf_erf_Q (x);
  }
  static double gamma (double x)
  {
    return gsl_sf_gamma (x);
  }
  static double gammaln (double x)
  {
    return gsl_sf_lngamma (x);
  }
  static double hazard (double x)
  {
    return gsl_sf_hazard (x);
  }
  static double airyAi(double x)
  {
    return gsl_sf_airy_Ai( x, GSL_PREC_DOUBLE );
  }
  static double airyBi(double x)
  {
    return gsl_sf_airy_Bi( x, GSL_PREC_DOUBLE );
  }
  static double zeta(double x)
  {
    return gsl_sf_zeta( x );
  }
  static double eta(double x)
  {
    return gsl_sf_eta( x );
  }
  static double laguerre(double n, double k, double x)
  {
    return gsl_sf_laguerre_n(int(n), k, x);
  }
  static double legendreP(double n, double x)
  {
    return gsl_sf_legendre_Pl( (int)n, x );
  }
  static double legendrePnm(double n, double m, double x)
  {
    return gsl_sf_legendre_Plm( int(n), int(m), x );
  }
  static double legendreQ(double n, double x)
  {
    return gsl_sf_legendre_Ql( (int)n, x );
  }
  static double hyperg(double a, double b, double c, double x)
  {
    return  gsl_sf_hyperg_2F1( a, b, c, x );
  }
  static double hypergM(double a, double b, double x)
  {
    return gsl_sf_hyperg_1F1( a, b, x );
  }
  static double hypergU(double a, double b, double x)
  {
    return gsl_sf_hyperg_U( a, b, x );
  }
  static double fact(double n)
  {
    return gsl_sf_fact( (int)n );
  }
  static double dfact(double n)
  {
    return gsl_sf_doublefact( (int)n );
  }
  static double poch(double a, double x)
  {
    return gsl_sf_poch( a, x );
  }
  static double debye(double n, double x)
  {
    switch ((int)n)
      {
      case 1 : return gsl_sf_debye_1(x);
      case 2 : return gsl_sf_debye_2(x);
      case 3 : return gsl_sf_debye_3(x);
      case 4 : return gsl_sf_debye_4(x);
      case 5 : return gsl_sf_debye_5(x);
      case 6 : return gsl_sf_debye_6(x);
      }
    return 0;
  }
  static double gegenpoly(double n, double l, double x)
  {
    return gsl_sf_gegenpoly_n ((int)n, l, x);
  }
  static double polygamma(double n, double x)
  {
    return gsl_sf_psi_n((int)n, x);
  }
  static double comb( double n, double k )
  {
    return gsl_sf_choose( (int)n, (int)k ) ;
  }
  static double perm( double n, double k )
  {
    return fact(n)/fact(n-k);
  }
  static double mathieuCE( double n, double q, double x)
  {
    gsl_sf_result result;
    if(gsl_sf_mathieu_ce( (int)n, q, x, &result )==GSL_SUCCESS)
      return result.val;
    else
      return 0;
  }  
  static double mathieuSE( double n, double q, double x)
  {
    gsl_sf_result result;
    if(gsl_sf_mathieu_se( (int)n, q, x, &result )==GSL_SUCCESS)
      return result.val;
    else
      return 0;
  }
  static double ellintE( double phi, double k)
  {
    return gsl_sf_ellint_E( phi, k, GSL_PREC_DOUBLE);
  }
  static double ellintF( double phi, double k)
  {
    return gsl_sf_ellint_F( phi, k, GSL_PREC_DOUBLE);
  }
  static double ellintP( double phi, double k, double n)
  {
    return gsl_sf_ellint_P( phi, k, n, GSL_PREC_DOUBLE);
  }
  static double hermite( double n, double x )
  {
    int n1 = (int) n;
    int n2 = n1/2;
    double a = gsl_sf_fact(n1) / gsl_sf_fact(n2); 
    if(n2 % 2) a = -a;
    if(n1 % 2) 
      return a * 2 * x * hypergM( -n2, 1.5, x*x );
    else
      return a * hypergM( -n2, 0.5, x*x );
  }
  static double chebyshevT( double n, double x )
  {
    return gsl_sf_hyperg_2F1( -(int)n, (int)n, 0.5, (1-x)/2 );
  }
  static double chebyshevU( double n, double x )
  {
    return ((int)n+1) * gsl_sf_hyperg_2F1( -(int)n, (int)n+2, 1.5, (1-x)/2 );
  }
  static double jacobiP(double n, double a, double b, double x)
  {
    int n1 = (int)n;
    return gsl_sf_poch( a+1, n1 ) / gsl_sf_fact( n1 ) 
      * gsl_sf_hyperg_2F1(-n1, n1+a+b+1, a+1, (1-x)/2 );
  }

  struct mathFunction
  {
    char *name;
    int numargs;
    double (*fun1) (double);
    double (*fun2) (double, double);
    double (*fun3) (double, double, double);
    double (*fun4) (double, double, double, double);
    char *description;
  };
  static const mathFunction math_functions[];

};

#endif
