/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "polynomFitDialog.h"
#include "graph.h"
#include "colorBox.h"
#include "application.h"
#include "Fitter.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qmessagebox.h>

polynomFitDialog::polynomFitDialog (QWidget * parent, const char *name,
				    bool modal, WFlags fl):
QDialog (parent, name, modal, fl)
{
  if (!name)
    setName ("polynomFitDialog");
  setCaption (tr ("QtiPlot - Polynomial Fit Options"));
  setSizeGripEnabled (true);
  setSizePolicy (QSizePolicy (QSizePolicy::Expanding, QSizePolicy::Fixed));

  GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox1");
  GroupBox1->
    setSizePolicy (QSizePolicy (QSizePolicy::Expanding, QSizePolicy::Fixed));

  new QLabel (tr ("Polynomial Fit of"), GroupBox1, "TextLabel1", 0);
  boxName = new QComboBox (GroupBox1, "boxShow");

  new QLabel (tr ("Order (1 - 9, 1 = linear)"), GroupBox1, "TextLabel2", 0);
  boxOrder = new QSpinBox (1, 9, 1, GroupBox1, "boxOrder");
  boxOrder->setValue (2);

  new QLabel (tr ("Fit curve Xmin"), GroupBox1, "TextLabel4", 0);
  boxStart = new QLineEdit (GroupBox1, "boxStart");
  boxStart->setText (tr ("0"));

  new QLabel (tr ("Fit curve Xmax"), GroupBox1, "TextLabel5", 0);
  boxEnd = new QLineEdit (GroupBox1, "boxEnd");

  new QLabel (tr ("Color"), GroupBox1, "TextLabel52", 0);
  boxColor = new ColorBox (FALSE, GroupBox1);
  boxColor->setColor (QColor (red));

  new QLabel (tr ("Show Formula on Graph?"), GroupBox1, "TextLabel6", 0);
  boxShowFormula = new QCheckBox (GroupBox1, "boxShow");
  boxShowFormula->setChecked (FALSE);

  GroupBox2 =
    new QButtonGroup (1, QGroupBox::Horizontal, tr (""), this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonFit = new QPushButton (GroupBox2, "buttonFit");
  buttonFit->setAutoDefault (TRUE);
  buttonFit->setDefault (TRUE);

  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setAutoDefault (TRUE);

  QHBoxLayout *hlayout = new QHBoxLayout (this, 5, 5, "hlayout");
  hlayout->addWidget (GroupBox1);
  hlayout->addWidget (GroupBox2);

  languageChange ();
  setMaximumHeight (GroupBox1->height ());

  // signals and slots connections
  connect (buttonFit, SIGNAL (clicked ()), this, SLOT (fit ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
  connect (boxName, SIGNAL (activated (int)), this,
	   SLOT (activateCurve (int)));
}

polynomFitDialog::~polynomFitDialog ()
{
}

void
polynomFitDialog::languageChange ()
{
  buttonFit->setText (tr ("&Fit"));
  buttonCancel->setText (tr ("&Close"));
}

void
polynomFitDialog::fit ()
{
  QString curveTitle = boxName->currentText ();
  QStringList clist = d_graph->analyzableCurvesList ();
  if (clist.contains (curveTitle) <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Warning"),
			     tr
			     ("The curve <b> %1 </b> doesn't exist anymore! Operation aborted!").
			     arg (curveTitle));
      boxName->clear ();
      boxName->insertStringList (clist);
      return;
    }

  //int index = boxName->currentItem ();
  int index = d_graph->curveIndex(curveTitle);
  QwtPlotCurve *c = d_graph->curve (index);
  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve || c->dataSize () < 2)
    {
      QString s = tr ("You cannot fit curve:");
      s += "<p><b>'" + curveTitle + "'</b><p>";
      s += tr ("because it has less than 2 points!");
      QMessageBox::warning (0, tr ("QtiPlot - Warning"), s);
      return;
    }
  else
    {
      ApplicationWindow *app = (ApplicationWindow *) this->parent ();
      PolynomialFit *fitter =
	new PolynomialFit (app, d_graph, boxOrder->value (),
			   boxShowFormula->isChecked ());
      if (fitter->
	  setDataFromCurve (boxName->currentText (),
			    boxStart->text ().toDouble (),
			    boxEnd->text ().toDouble ()))
	{
          fitter->setColor (boxColor->currentItem ());
          fitter->setGenerateFunction(app->generateFunction);
          fitter->setGenerateUniformFitPoints(app->generateUniformFitPoints);
          fitter->setFitPoints(app->fitPoints);
	  QString name = QString(fitter->name())+tr("Fit");
	  if(app->generateFunction)
	    name = d_graph->generateUniqueName(name);
	  else
	    name = app->generateUniqueName(name);
	  fitter->setCurveName(name);
	  fitter->fit ();
	  delete fitter;
	}
    }
  close();
}

void
polynomFitDialog::setGraph (Graph * g)
{
  d_graph = g;
  QStringList clist = g->analyzableCurvesList();
  boxName->insertStringList (clist, -1);
  if (d_graph->selectorsEnabled ())
    {
      int index = 0;
      QString selected = d_graph->selectedCurveTitle();
      if((index = clist.findIndex( selected )) >=0 )
	boxName->setCurrentItem( index );
      activateCurve(index);
    }
  else
    activateCurve(0);

  connect (d_graph, SIGNAL (closedGraph ()), this, SLOT (close ()));
  connect (d_graph, SIGNAL (dataRangeChanged ()), this,
	   SLOT (changeDataRange ()));
};

void
polynomFitDialog::activateCurve (int index)
{
  QString ctitle = boxName->text(index);
  int cindex = d_graph->curveIndex(ctitle);
  QwtPlotCurve *c = d_graph->curve(cindex);
  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve)
    return;

  double start,end;
  d_graph->range(cindex,&start,&end);
  boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
  boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));
};

void
polynomFitDialog::changeDataRange ()
{
  double start = d_graph->selectedXStartValue ();
  double end = d_graph->selectedXEndValue ();
  boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
  boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));
}
