/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "removeSpikeDialog.h"
#include "graph.h"
#include "Fitter.h"
#include "parser.h"
#include "gslutils.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qcheckbox.h>

removeSpikeDialog::removeSpikeDialog (QWidget * parent, const char *name,
				    bool modal, WFlags fl):
QDialog (parent, name, modal, fl)
{
  if (!name)
    setName ("removeSpikeDialog");
  setCaption (tr ("QtiPlot - Remove Spike Dialog"));
  setSizeGripEnabled (false);
  setSizePolicy (QSizePolicy (QSizePolicy::Expanding, QSizePolicy::Maximum));

  GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox1");
  GroupBox1->
   setSizePolicy (QSizePolicy (QSizePolicy::Expanding, QSizePolicy::Fixed));

  new QLabel (tr ("Remove Spike of"), GroupBox1, "TextLabel1", 0);
  boxName = new QComboBox (GroupBox1, "boxShow");

  QLabel *label = new QLabel(tr("Fit Start"),GroupBox1, "TextLabelX1",0);
  boxX1 = new QLineEdit(GroupBox1,"boxX1");
  label = new QLabel(tr("Spike Start"),GroupBox1, "TextLabelX2",0);
  boxX2 = new QLineEdit(GroupBox1,"boxX2");
  label = new QLabel(tr("Spike End"),GroupBox1, "TextLabelX3",0);
  boxX3 = new QLineEdit(GroupBox1,"boxX3");
  label = new QLabel(tr("Fit End"),GroupBox1, "TextLabelX4",0);
  boxX4 = new QLineEdit(GroupBox1,"boxX4");
  label = new QLabel(tr("Add Noise?"),GroupBox1, "TextLabelX5",0);
  boxNoise = new QCheckBox(GroupBox1,"");
  boxNoise->setChecked(false);

  GroupBox2 =
    new QButtonGroup (1, QGroupBox::Horizontal, tr (""), this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonApply = new QPushButton(GroupBox2, "buttonApply");
  buttonApply->setDefault (TRUE);

  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setAutoDefault (TRUE);

  buttonHelp = new QPushButton(GroupBox2, "buttohHelp");
  
  QHBoxLayout *hlayout = new QHBoxLayout (this, 5, 5, "hlayout");
  hlayout->addWidget (GroupBox1);
  hlayout->addWidget (GroupBox2);

  languageChange ();

  // signals and slots connections
  connect (buttonApply, SIGNAL (clicked ()), this, SLOT (apply ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
  connect (buttonHelp, SIGNAL (clicked ()), this, SLOT (help ()));
  connect (boxName, SIGNAL (activated (int)), this,
	   SLOT (activateCurve (int)));
}

removeSpikeDialog::~removeSpikeDialog ()
{
}

void
removeSpikeDialog::languageChange ()
{
  buttonApply->setText (tr ("&Apply"));
  buttonCancel->setText (tr ("&Cancel"));
  buttonHelp->setText(tr("&Help"));
}

void
removeSpikeDialog::apply ()
{
  QString curveTitle = boxName->currentText ();
  QStringList clist = d_graph->analyzableCurvesList ();
  if (clist.contains (curveTitle) <= 0)
  {
    QMessageBox::critical (this, tr ("QtiPlot - Warning"),
      tr
      ("The curve <b> %1 </b> doesn't exist anymore! Operation aborted!").
      arg (curveTitle));
    boxName->clear ();
    boxName->insertStringList (clist);
    return;
  }

  bool finished=false;
  int index = d_graph->curveIndex(curveTitle);
  QwtPlotCurve *c = d_graph->curve (index);

  ApplicationWindow *app = (ApplicationWindow *) this->parent ();
  Fit *fitter;
  fitter = new PolynomialFit (app, d_graph, 3, false);
  QValueList <double> ranges;
  ranges.append(boxX1->text().toDouble());
  ranges.append(boxX2->text().toDouble());
  ranges.append(boxX3->text().toDouble());
  ranges.append(boxX4->text().toDouble());

  if(fitter->setDataFromCurve(index, ranges))
    {
      fitter->setGenerateFunction(true);
      fitter->setGenerateUniformFitPoints(true);
      fitter->setFitPoints(app->fitPoints);
      QString name = QString(fitter->name())+tr("Fit");
      name = d_graph->generateUniqueName(name);
      fitter->setCurveName(name);
      fitter->fit();
      double stddev = fitter->stddev();
      int choice =
	QMessageBox::question(this, tr("QtiPlot - Replace?"),
			   tr("Replace the spike region with the fit result?"),
			   QMessageBox::Yes, QMessageBox::No );
      if(choice == QMessageBox::Yes)
	{
	  double x2 = boxX2->text().toDouble();
	  double x3 = boxX3->text().toDouble();
	  QString formula = fitter->resultingFormula();
	  myParser parser;
	  double xvar;
	  parser.DefineVar("x", &xvar);
	  parser.SetExpr( formula.ascii() );
	  int n = c->dataSize();
	  QMemArray <double> dat(n);
	  for(int i = 0; i < n; i++)
	    {
	      xvar = c->x(i);
	      if(x2 <= xvar && xvar <= x3)
		{
		  dat[i] = parser.Eval();
		  if(boxNoise->isChecked())
		    dat[i] += rand_gaussian(stddev);
		}
	      else
		dat[i] = c->y(i);
	    }
	  emit updateTableColumn( c->title().text(), dat ); 
	  finished = true;
	}
      int id = d_graph->curveIndex(name);
      d_graph->removeCurve(id,true);      
    }
  
  delete fitter;
  if(finished) close();
}

void
removeSpikeDialog::setGraph (Graph * g)
{
  d_graph = g;
  QStringList clist = g->analyzableCurvesList();
  boxName->insertStringList (clist, -1);
  if (d_graph->selectorsEnabled ())
    {
      int index = 0;
      QString selected = d_graph->selectedCurveTitle();
      if((index = clist.findIndex( selected )) >=0 )
        boxName->setCurrentItem( index );
      activateCurve(index);
    }
  else
    activateCurve(0);

  connect (d_graph, SIGNAL (closedGraph ()), this, SLOT (close ()));
  connect (d_graph, SIGNAL (dataRangeChanged ()), this,
	   SLOT (changeDataRange ()));
};

void
removeSpikeDialog::activateCurve (int index)
{
  QString ctitle = boxName->text(index);
  int cindex = d_graph->curveIndex(ctitle);
  QwtPlotCurve *c = d_graph->curve(cindex);
  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve)
    return;

  double start,end;
  d_graph->range(cindex,&start,&end);
  double x1,x2,x3,x4;
  x2 = QMIN(start, end);
  x3 = QMAX(start, end);
  x1 = x2 - (x3-x2);
  x4 = x3 + (x3-x2);

  boxX1->setText (QString::number (x1, 'g', 15));
  boxX2->setText (QString::number (x2, 'g', 15));
  boxX3->setText (QString::number (x3, 'g', 15));
  boxX4->setText (QString::number (x4, 'g', 15));
};

void
removeSpikeDialog::changeDataRange ()
{
  double start = d_graph->selectedXStartValue ();
  double end = d_graph->selectedXEndValue ();
  double x1,x2,x3,x4;
  x2 = QMIN(start, end);
  x3 = QMAX(start, end);
  x1 = x2 - (x3-x2);
  x4 = x3 + (x3-x2);

  boxX1->setText (QString::number (x1, 'g', 15));
  boxX2->setText (QString::number (x2, 'g', 15));
  boxX3->setText (QString::number (x3, 'g', 15));
  boxX4->setText (QString::number (x4, 'g', 15));
}

void
removeSpikeDialog::help()
{
 QMessageBox::about (0, tr ("QtiPlot - Help for Spike Noise Removal"),
"The spike noise removal eliminates a spike-like peak from the curve\n\
by smoothly connecting the adjacent regions. Please specify X ranges\n\
for the spike peak and the adjacent region for fitting.");
}
