/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "sineFitDialog.h"
#include "graph.h"
#include "colorBox.h"
#include "application.h"
#include "Fitter.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <gsl/gsl_fft_halfcomplex.h>
#include <gsl/gsl_fft_complex.h>

sineFitDialog::sineFitDialog (QWidget * parent, const char *name, bool modal, WFlags fl):QDialog (parent, name, modal,
	 fl)
{
  if (!name)
    setName ("sineFitDialog");

  setCaption (tr ("QtiPlot - Verify initial guesses"));
  GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox1");

  new QLabel (tr ("Sine Fit of"), GroupBox1, "TextLabel1", 0);
  boxName = new QComboBox (GroupBox1, "boxName");

  new QLabel (tr ("Amplitude"), GroupBox1, "TextLabel2", 0);
  boxAmp = new QLineEdit (GroupBox1, "boxAmp");
  boxAmp->setText (tr ("1"));

  new QLabel (tr ("Frequency"), GroupBox1, "TextLabel3", 0);
  boxFreq = new QLineEdit (GroupBox1, "boxFreq");
  boxFreq->setText (tr ("1"));

  new QLabel (tr ("Phase"), GroupBox1, "TextLabel4", 0);
  boxPhase = new QLineEdit (GroupBox1, "boxPhase");
  boxPhase->setText (tr ("0"));

  new QLabel (tr ("Y Offset"), GroupBox1, "TextLabel5", 0);
  boxOfs = new QLineEdit (GroupBox1, "boxAmp");
  boxOfs->setText (tr ("0"));

  new QLabel (tr ("Fit curve Xmin"), GroupBox1, "TextLabel4", 0);
  boxStart = new QLineEdit (GroupBox1, "boxStart");
  boxStart->setText (tr ("0"));

  new QLabel (tr ("Fit curve Xmax"), GroupBox1, "TextLabel5", 0);
  boxEnd = new QLineEdit (GroupBox1, "boxEnd");

  new QLabel (tr ("Color"), GroupBox1, "TextLabel52", 0);
  boxColor = new ColorBox (FALSE, GroupBox1);
  boxColor->setColor (QColor (red));

  GroupBox2 =
    new QButtonGroup (1, QGroupBox::Horizontal, tr (""), this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonFit = new QPushButton (GroupBox2, "buttonFit");
  buttonFit->setAutoDefault (TRUE);
  buttonFit->setDefault (TRUE);

  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setAutoDefault (TRUE);

  QHBoxLayout *
    hlayout = new QHBoxLayout (this, 5, 5, "hlayout");
  hlayout->addWidget (GroupBox1);
  hlayout->addWidget (GroupBox2);

  languageChange ();

  // signals and slots connections
  connect (buttonFit, SIGNAL (clicked ()), this, SLOT (fit ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
  connect (boxName, SIGNAL (activated (int)), this,
           SLOT (activateCurve (int)));
}


sineFitDialog::~sineFitDialog ()
{
}


void
sineFitDialog::languageChange ()
{
  buttonFit->setText (tr ("&Fit"));
  buttonCancel->setText (tr ("&Close"));
}

void
sineFitDialog::setGraph (Graph * g)
{
  if (!g)
    return;

  d_graph = g;
  QStringList clist = d_graph->analyzableCurvesList();
  boxName->insertStringList (clist, -1);
  if (d_graph->selectorsEnabled ())
    {
      int index;
      QString selected = d_graph->selectedCurveTitle();
      if((index = clist.findIndex( selected )) >=0 )
	boxName->setCurrentItem( index );
      activateCurve(index);
    }
  else
    activateCurve(0);

  connect (d_graph, SIGNAL (closedGraph ()), this, SLOT (close ()));
  connect (d_graph, SIGNAL (dataRangeChanged ()), this,
           SLOT (changeDataRange ()));
}

void
sineFitDialog::activateCurve(int index)
{
  QString ctitle = boxName->text(index);
  int cindex = d_graph->curveIndex(ctitle);
  QwtPlotCurve *c = d_graph->curve(cindex);
  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve)
    return;

  double start,end;
  d_graph->range(cindex,&start,&end);
  boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
  boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));

  guessInitialParam();
}

void
sineFitDialog::changeDataRange ()
{
  double start = d_graph->selectedXStartValue ();
  double end = d_graph->selectedXEndValue ();
  boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
  boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));

  guessInitialParam();
}

void
sineFitDialog::guessInitialParam()
{
  double start = boxStart->text().toDouble();
  double end = boxEnd->text().toDouble();

  int cindex = d_graph->curveIndex( boxName->currentText() );
  QMemArray <double> x,y;
  int n = d_graph->sortedCurveData( cindex, start, end, x, y );

  double ymin=y[0],ymax=y[0];
  for(int i=1;i<n;i++){
    if(y[i]>ymax) ymax = y[i];
    if(y[i]<ymin) ymin = y[i];    
  }
  double amplitude = ( ymax - ymin ) / 2;
  double yoffset = ( ymax + ymin ) / 2;
  boxAmp->setText(QString::number(amplitude));
  boxOfs->setText(QString::number(yoffset));

  QMemArray <double> result(n*2);
  double sampling = x[1] - x[0];
  double df = 1.0 / ( sampling * n );
  
  gsl_fft_real_workspace *work = gsl_fft_real_workspace_alloc (n);
  gsl_fft_real_wavetable *real = gsl_fft_real_wavetable_alloc (n);

  if (!work || !real)
    {
      QMessageBox::warning (0, "QtiPlot",
			    tr("Could not allocate memory, operation aborted!"));
      return;
    }
  
  gsl_fft_real_transform (y.data(), 1, n, real, work);
  gsl_fft_halfcomplex_unpack (y.data(), result.data(), 1, n);
  
  gsl_fft_real_wavetable_free (real);
  gsl_fft_real_workspace_free (work);

  /* ΤߤǤԽʬնʿѤ롣*/
  double amax = 0;
  int imax = 1;
  for(int i=1;i<n/2;i++)
    {
      double yr = result[i*2];
      double yi = result[i*2+1];
      double amp = sqrt( yr * yr + yi * yi );
      if ( amp > amax ) {
	amax = amp;
	imax = i;
      }
    }
  double asum = 0;
  double fsum = 0;
  double psum = 0;
  for(int j=-2;j<=2;j++)
    if(imax+j>0){
      double freq = df * (imax+j);
      double yr = result[(imax+j)*2];
      double yi = result[(imax+j)*2+1];
      double amp = sqrt( yr * yr + yi * yi );
      double phase = atan2(yi,yr)*180/M_PI + 90;	
      if (phase>180) phase -= 360;
      asum += amp;
      fsum += amp * freq;
      psum += amp * phase;
    }  
  boxFreq->setText(QString::number(fsum / asum));
  boxPhase->setText(QString::number(psum / asum));
}


void
sineFitDialog::fit ()
{
  QString curve = boxName->currentText ();
  QwtPlotCurve *c = d_graph->curve (curve);
  QStringList clist = d_graph->analyzableCurvesList ();
  if (!c || clist.contains (curve) <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Warning"),
			     tr
			     ("The curve <b> %1 </b> doesn't exist anymore! Operation aborted!").
			     arg (curve));
      boxName->clear ();
      boxName->insertStringList (clist);
      return;
    }

  ApplicationWindow *app = (ApplicationWindow *) this->parent ();
  Fit *fitter;

  QMemArray <double> x_init(4);
  x_init[0] = boxAmp->text ().toDouble (); // amplitude
  x_init[1] = boxFreq->text ().toDouble (); // frequency
  x_init[2] = boxPhase->text ().toDouble (); // phase
  x_init[3] = boxOfs->text ().toDouble (); // offset
  fitter = new SineFit (app, d_graph);
  fitter->setInitialGuesses (x_init);

  if (fitter->setDataFromCurve (boxName->currentText (),
				boxStart->text().toDouble (),
				boxEnd->text().toDouble ()))
    {
      fitter->setColor(boxColor->currentItem ());
      fitter->setGenerateFunction(app->generateFunction);
      fitter->setGenerateUniformFitPoints(app->generateUniformFitPoints);
      fitter->setFitPoints(app->fitPoints);
      QString name = QString(fitter->name())+tr("Fit");
      if(app->generateFunction)
	name = d_graph->generateUniqueName(name);
      else
	name = app->generateUniqueName(name);
      fitter->setCurveName(name);
      fitter->fit ();
      delete fitter;
    }

  close();
}
