/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "subtBackDialog.h"
#include "graph.h"
#include "Fitter.h"
#include "parser.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qmessagebox.h>
#include <qinputdialog.h>

subtBackDialog::subtBackDialog (QWidget * parent, const char *name,
				    bool modal, WFlags fl):
QDialog (parent, name, modal, fl)
{
  if (!name)
    setName ("subtBackDialog");
  setCaption (tr ("QtiPlot - Background Subtraction Options"));
  setSizeGripEnabled (false);
  setSizePolicy (QSizePolicy (QSizePolicy::Expanding, QSizePolicy::Maximum));

  GroupBox1 =
    new QButtonGroup (2, QGroupBox::Horizontal, tr (""), this, "GroupBox1");
  GroupBox1->
   setSizePolicy (QSizePolicy (QSizePolicy::Expanding, QSizePolicy::Fixed));

  new QLabel (tr ("Subtract Background of"), GroupBox1, "TextLabel1", 0);
  boxName = new QComboBox (GroupBox1, "boxShow");

  new QLabel(tr("Fit Type"), GroupBox1, "TextLabel11", 0 );
  boxType = new QComboBox (GroupBox1, "boxType");
  QStringList methods;
  methods << "Polynomial" << "Gaussian" << "Lorentzian" << "Exponential" 
	  << "Oscillation";
  boxType->insertStringList(methods,-1);

  new QLabel (tr ("Fit Order (0 - 9, 0 = constant)"), GroupBox1, "TextLabel2", 0);
  boxOrder = new QSpinBox (0, 9, 1, GroupBox1, "boxOrder");
  boxOrder->setValue (0);

  new QLabel (tr("Number of Background Ranges"), GroupBox1, "TextLabel21", 0 );
  boxNumData = new QSpinBox (1,20,1,GroupBox1, "boxNumData" );
  boxNumData->setValue(1);
  d_ranges = 1;

  labels.clear();
  labels.setAutoDelete(true);
  values.clear();
  values.setAutoDelete(true);

  labels.append(new QLabel (tr ("Background Range1 Xmin"), GroupBox1, "TextLabel4", 0));
  boxStart = new QLineEdit (GroupBox1, "boxStart");
  values.append(boxStart);

  labels.append(new QLabel (tr ("Background Range1 Xmax"), GroupBox1, "TextLabel5", 0));
  boxEnd = new QLineEdit (GroupBox1, "boxEnd");
  values.append(boxEnd);

  GroupBox2 =
    new QButtonGroup (1, QGroupBox::Horizontal, tr (""), this, "GroupBox2");
  GroupBox2->setFlat (TRUE);
  GroupBox2->setLineWidth (0);

  buttonApply = new QPushButton (GroupBox2, "buttonApply");
  buttonApply->setAutoDefault (TRUE);
  buttonApply->setDefault (TRUE);

  buttonCancel = new QPushButton (GroupBox2, "buttonCancel");
  buttonCancel->setAutoDefault (TRUE);

  buttonRecall = new QPushButton (GroupBox2, "buttonRecall");

  buttonHelp = new QPushButton(GroupBox2, "buttohHelp");
  
  QHBoxLayout *hlayout = new QHBoxLayout (this, 5, 5, "hlayout");
  hlayout->addWidget (GroupBox1);
  hlayout->addWidget (GroupBox2);

  languageChange ();

  // signals and slots connections
  connect (buttonApply, SIGNAL (clicked ()), this, SLOT (apply ()));
  connect (buttonCancel, SIGNAL (clicked ()), this, SLOT (reject ()));
  connect (buttonRecall, SIGNAL (clicked ()), this, SLOT (recallParams()));
  connect (buttonHelp, SIGNAL (clicked ()), this, SLOT (help ()));
  //connect (boxName, SIGNAL(activated (int)), this, SLOT(activateCurve (int)));
  connect (boxType, SIGNAL (activated(int)), this, SLOT (changeType(int)));
  connect (boxNumData, SIGNAL (valueChanged(int)), this,
	   SLOT( updateDataNum(int) ) );
}

subtBackDialog::~subtBackDialog ()
{
}

void
subtBackDialog::recallParams()
{
  ApplicationWindow *app = (ApplicationWindow *)this->parent();
  QStringList list = QStringList::split(',',app->backGroundFunc,true);

  if (list.count() < 5) return;

  boxType->setCurrentText(list[0]);
  boxOrder->setValue(list[1].toInt());
  int n = list[2].toInt();
  boxNumData->setValue(n);
  updateDataNum(n);
  for(int i=0;i<n;i++)
    {
      ((QLineEdit *)values.at(i*2))->setText(list[i*2+3]);
      ((QLineEdit *)values.at(i*2+1))->setText(list[i*2+4]);
    }

  if(boxType->currentItem()==0)
    boxOrder->setEnabled(true);
  else
    boxOrder->setEnabled(false);
}

void
subtBackDialog::languageChange ()
{
  buttonApply->setText (tr ("&Apply"));
  buttonCancel->setText (tr ("&Close"));
  buttonRecall->setText (tr ("&Recall"));
  buttonHelp->setText(tr("&Help"));
}

void
subtBackDialog::updateDataNum(int dnum)
{
  int newnum = boxNumData->value();
  if (newnum<1) return;
  if (d_ranges < newnum)
    {
      for(int i = d_ranges; i<newnum; i++)
	{
	  QLabel *newlabel = new QLabel(tr("Background Range%1 Xmin").arg(i+1), GroupBox1);
	  newlabel->show();
	  labels.append( newlabel );	  
	  QLineEdit *newedit = new QLineEdit (GroupBox1);
	  newedit->show();
	  values.append( newedit );
	  newlabel = new QLabel(tr("Background Range%1 Xmax").arg(i+1), GroupBox1);
	  newlabel->show();
	  labels.append( newlabel );	  
	  newedit = new QLineEdit (GroupBox1);
	  newedit->show();
	  values.append( newedit );
	}
    }
  else if( d_ranges > newnum )
    {
      for(int i=d_ranges-1; i>=newnum; i--)
	{
	  values.remove(i*2+1);
	  labels.remove(i*2+1);
	  values.remove(i*2);
	  labels.remove(i*2);
	}
      adjustSize();
    }
  d_ranges = newnum;
}

void
subtBackDialog::guessInitialFitParam( QwtPlotCurve *c, int fittype, 
				      QValueList <double> &ranges,
				      QMemArray <double> &p )
{
  bool first = true;
  double ymin,ymax;
  int imax,imin;
  int n = c->dataSize();
  for(int i=0;i<n;i++)
    for(int j=0;j<(int)ranges.count()/2;j++)
      if(ranges[j*2] <= c->x(i) && c->x(i) <= ranges[j*2+1])
	{
	  if(first)
	    {
	      ymin = c->y(i);
	      ymax = c->y(i);
	      imax = i;
	      imin = i;
	      first = false;
	    }
	  else
	    {
	      if(c->y(i)>ymax)
		{
		  ymax = c->y(i);
		  imax = i;
		}
	      else if(c->y(i)<ymin)
		{
		  ymin = c->y(i);
		  imin = i;
		}
	    }
	  break;
	}

  p[0] = ymin; // initial offset
  p[1] = ymax-ymin; // initial amplitude
  switch(fittype)
    {
    case 1: // Gaussian
    case 2: // Lorentzian
      p[2] = c->x(imax); // initial center
      p[3] = fabs(c->x(n-1) - c->x(0)) / 5; // initial width
      break;
    case 3: // Exponential
      p[2] = fabs( c->x(n-1) - c->x(0) )/3; // initial decay rate
      break;
    case 4: // Oscillation
      // initial frequency
      double dummy = 3.1415 / fabs( c->x(imax) - c->x(imin) );
      double period = QInputDialog::getDouble("Input","Input Estimated Oscillation Period", dummy);
      if (period>0)
	p[2] = 3.1415 * 2 / period;
      else
	p[2] = dummy;
      p[3] = c->x(imax); // initial zero point
      break;
    }
}

void
subtBackDialog::apply ()
{
  QString curveTitle = boxName->currentText ();
  QStringList clist = d_graph->analyzableCurvesList ();
  if (clist.contains (curveTitle) <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Warning"),
			     tr
			     ("The curve <b> %1 </b> doesn't exist anymore! Operation aborted!").
			     arg (curveTitle));
      boxName->clear ();
      boxName->insertStringList (clist);
      return;
    }

  bool finished=false;
  int index = d_graph->curveIndex(curveTitle);
  QwtPlotCurve *c = d_graph->curve (index);

  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve || c->dataSize () < 2)
    {
      QString s = tr ("You cannot fit curve:");
      s += "<p><b>'" + curveTitle + "'</b><p>";
      s += tr ("because it has less than 2 points!");
      QMessageBox::warning (0, tr ("QtiPlot - Warning"), s);
      return;
    }

  ApplicationWindow *app = (ApplicationWindow *) this->parent ();
  Fit *fitter;
  int fittype = boxType->currentItem();
  QStringList plist;
  switch(fittype)
    {
    case 0:
      fitter = new PolynomialFit (app, d_graph, boxOrder->value (), false);
      break;
    case 1:
      fitter = new NonLinearFit( app, d_graph );
      plist.clear();
      plist << "y0" << "A" << "xc" << "w";
      ((NonLinearFit *)fitter)->setParametersList(plist);      
      ((NonLinearFit *)fitter)->setFormula("y0+A*exp(-(x-xc)^2/w^2)");
      break;
    case 2:
      fitter = new NonLinearFit( app, d_graph );
      plist.clear();
      plist << "y0" << "A" << "xc" << "w";
      ((NonLinearFit *)fitter)->setParametersList(plist);      
      ((NonLinearFit *)fitter)->setFormula("y0+A/((x-xc)^2+w^2)");
      break;
    case 3:
      fitter = new NonLinearFit( app, d_graph );
      plist.clear();
      plist << "y0" << "A" << "s";
      ((NonLinearFit *)fitter)->setParametersList(plist);      
      ((NonLinearFit *)fitter)->setFormula("y0+A*exp(-x*s)");
      break;
    case 4:
      fitter = new NonLinearFit( app, d_graph );
      plist.clear();
      plist << "y0" << "A" << "w" << "x0";
      ((NonLinearFit *)fitter)->setParametersList(plist);      
      ((NonLinearFit *)fitter)->setFormula("y0+A*cos(w*(x-x0))");
    }
  QValueList <double> ranges;
  for (int i=0; i<d_ranges; i++)
    {
      double xstart = ((QLineEdit *)values.at(i*2))->text().toDouble();
      double xend = ((QLineEdit *)values.at(i*2+1))->text().toDouble();
      ranges.append(xstart);
      ranges.append(xend);
    }
  if(fitter->setDataFromCurve(index, ranges))
    {
      if(fittype>0)
	{
	  QMemArray <double> p(4);
	  guessInitialFitParam(c, fittype, ranges, p);
	  fitter->setInitialGuesses(p);
	}
      fitter->setGenerateFunction(true);
      fitter->setGenerateUniformFitPoints(true);
      fitter->setFitPoints(app->fitPoints);
      QString name = QString(fitter->name())+tr("Fit");
      name = d_graph->generateUniqueName(name);
      fitter->setCurveName(name);
      fitter->fit();
      int choice =
	QMessageBox::question(this, tr("QtiPlot - Subtract Background"),
			      tr("Subtract fitted background?"),
			      QMessageBox::Yes, QMessageBox::No );
      if(choice== QMessageBox::Yes)
	{
	  QString formula = fitter->resultingFormula();
	  myParser parser;
	  double xvar;
	  parser.DefineVar("x", &xvar);
	  parser.SetExpr( formula.ascii() );
	  int n = c->dataSize();
	  QMemArray <double> dat(n);
	  for(int i = 0; i < n; i++)
	    {
	      xvar = c->x(i);
	      double back = parser.Eval();
	      dat[i] = c->y(i) - back;
	    }
	  emit updateTableColumn( c->title().text(), dat ); 
	  finished = true;
	}
      int id = d_graph->curveIndex(name);
      d_graph->removeCurve(id,true);      
    }

  delete fitter;
  if(finished)
    {
      QString text = boxType->currentText ()+","
	+ QString::number(boxOrder->value()) + ","
	+ QString::number(d_ranges);
      for(int i=0; i<d_ranges; i++)
	{
	  text += "," + ((QLineEdit *)values.at(i*2))->text();
	  text += "," + ((QLineEdit *)values.at(i*2+1))->text();
	}
      //qDebug("background = %s",text.ascii());
      app->backGroundFunc = text;
      //close();
    }
}

void
subtBackDialog::setGraph (Graph * g)
{
  d_graph = g;
  QStringList clist = g->analyzableCurvesList();
  boxName->insertStringList (clist, -1);
  if (d_graph->selectorsEnabled ())
    {
      int index = 0;
      QString selected = d_graph->selectedCurveTitle();
      if((index = clist.findIndex( selected )) >=0 )
        boxName->setCurrentItem( index );
      activateCurve(index);
    }
  else
    activateCurve(0);

  connect (d_graph, SIGNAL (closedGraph ()), this, SLOT (close ()));
  connect (d_graph, SIGNAL (dataRangeChanged ()), this,
	   SLOT (changeDataRange ()));
};

void
subtBackDialog::activateCurve (int index)
{
  QString ctitle = boxName->text(index);
  int cindex = d_graph->curveIndex(ctitle);
  QwtPlotCurve *c = d_graph->curve(cindex);
  if (!c || c->rtti () != QwtPlotItem::Rtti_PlotCurve)
    return;
  if(boxNumData->value()==1)
    {
      double start,end;
      d_graph->range(cindex,&start,&end);
      boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
      boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));
    }
};

void
subtBackDialog::changeDataRange ()
{
  if(boxNumData->value()==1)
    {
      double start = d_graph->selectedXStartValue ();
      double end = d_graph->selectedXEndValue ();
      boxStart->setText (QString::number (QMIN (start, end), 'g', 15));
      boxEnd->setText (QString::number (QMAX (start, end), 'g', 15));
    }
}

void
subtBackDialog::changeType(int t)
{
  if(t==0)
    boxOrder->setEnabled(true);
  else
    boxOrder->setEnabled(false);
}

void
subtBackDialog::help()
{
 QMessageBox::about (0, tr ("QtiPlot - Help for Background Subtraction"),
"The background subtraction performs a numerical fitting of a background\n\
region by a polynomial function, a peak function (Gaussian or Lorentzian)\n\
or an exponential function and subtract it from the data curve.");
}
