/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/
#include "widget.h"
#include "qurl.h"
#include <qmessagebox.h>

myWidget::myWidget (const QString & label, QWidget * parent, const char *name,
		    WFlags f):
QWidget (parent, name, f)
{
  d_max_size = QSize ();
  w_label = label;
  caption_policy = Both;
  askOnClose = true;
  w_status = Normal;
  titleBar = NULL;
}

void
myWidget::setName (const QString & newname)
{
  //qDebug( "name is set to : %s",newname.ascii() );
  myname = newname;
  QString dummy = newname;
  QUrl::encode(dummy);
  QWidget::setName(dummy);
  updateCaption ();
}

//QString
//myWidget::getName()
//{
//  QString dummy = name();
//  QUrl::decode( dummy );
//  return dummy;
//}

void
myWidget::updateCaption ()
{
  switch (caption_policy)
    {
    case Name:
      setCaption (myname);
      break;

    case Label:
      if (!w_label.isEmpty ())
	setCaption (w_label);
      else
	setCaption (myname);
      break;

    case Both:
      if (!w_label.isEmpty ())
	setCaption (myname + " - " + w_label);
      else
	setCaption (myname);
      break;
    }
}

void
myWidget::closeEvent (QCloseEvent * e)
{
  //fprintf(stderr,"closeEvent : %s\n",name());
  if (askOnClose)
    {
      switch (QMessageBox::information (0, tr ("QtiPlot"),
					tr ("Do you want to hide or delete") +
					"<p><b>'" + myname +
					"'</b> ?", tr ("Delete"), tr ("Hide"),
					tr ("Cancel"), 0, 2))
	{
	case 0:
	  //fprintf(stderr,"emitted closedwindow\n");
	  emit closedWindow (this);
	  e->accept ();
	  break;

	case 1:
	  e->ignore ();
	  emit hiddenWindow (this);
	  break;

	case 2:
	  e->ignore ();
	  break;
	}
    }
  else
    {
      //fprintf(stderr,"emitted closedwindow\n");
      emit closedWindow (this);
      e->accept ();
    }
}

QString
myWidget::aspect ()
{
  QString s = tr ("Normal");
  switch (w_status)
    {
    case Hidden:
      return tr ("Hidden");
      break;

    case Normal:
      break;

    case Minimized:
      return tr ("Minimized");
      break;

    case Maximized:
      return tr ("Maximized");
      break;
    }
  return s;
};

bool
myWidget::event (QEvent * e)
{
  // we do not need to handle QEvent::Hide because simple hide occurs when
  // folder is changed and it does not change the window status.
  if (e->type () == QEvent::ShowMinimized)
    {
      //fprintf(stderr,"show minimized event occur for %s\n",name());
      w_status = Minimized;
      emit statusChanged (this);
    }
  else if (e->type () == QEvent::ShowMaximized)
    {
      //fprintf(stderr,"show maximized event occur for %s\n",name());
      w_status = Maximized;
      d_max_size = size ();
      emit statusChanged (this);
    }
  else if (e->type () == QEvent::ShowNormal)
    {
      //fprintf(stderr,"show normal event occur for %s\n",name());
      w_status = Normal;
      emit statusChanged (this);
    }
  return QWidget::event (e);
}

void
myWidget::setHidden ()
{
  //fprintf(stderr,"%s is set hidden\n",name());
  w_status = Hidden;
  emit statusChanged (this);
  hide ();
}

void
myWidget::setNormal ()
{
  //fprintf(stderr,"%s is set normal\n",name());
  w_status = Normal;
  emit statusChanged (this);
}

void
myWidget::showMaximized ()
{
  //fprintf(stderr,"myWidget::showMaximized is called\n");
  QWidget::showMaximized ();
}

QString
myWidget::sizeToString ()
{
  return QString::number (8 * sizeof (this) / 1024.0, 'f',
			  1) + " " + tr ("kB");
}

void
myWidget::reparent (QWidget * parent, WFlags f, const QPoint & p, bool showIt)
{
  titleBar = (QWidget *) parent->child ("qt_ws_titlebar", "QWidget", false);
  if (titleBar)
    titleBar->installEventFilter (this);
  QWidget::reparent (parent, f, p, showIt);
}

bool
myWidget::eventFilter (QObject * object, QEvent * e)
{
  if (e->type () == QEvent::ContextMenu && object == titleBar)
    {
      emit showTitleBarMenu ();
      ((QContextMenuEvent *) e)->accept ();
      return true;
    }
  return QObject::eventFilter (object, e);
}


void
myWidget::activate()
{
  showNormal();
  setActiveWindow();
}
