/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/

#include "application.h"
#include "pixmaps.h"
#include "curvesDialog.h"
#include "plotDialog.h"
#include "axesDialog.h"
#include "lineDlg.h"
#include "textDialog.h"
#include "exportDialog.h"
#include "tableDialog.h"
#include "valuesDialog.h"
#include "errDlg.h"
#include "LegendMarker.h"
#include "LineMarker.h"
#include "ImageMarker.h"
#include "importDialog.h"
#include "graph.h"
#include "plot.h"
#include "pieDialog.h"
#include "plotWizard.h"
#include "polynomFitDialog.h"
#include "expDecayDialog.h"
#include "sineFitDialog.h"
#include "functionDialog.h"
#include "fitDialog.h"
#include "surfaceDialog.h"
#include "graph3D.h"
#include "plot3DDialog.h"
#include "imageDialog.h"
#include "multilayer.h"
#include "layerDialog.h"
#include "layerOrderDialog.h"
#include "analysisDialog.h"
#include "intDialog.h"
#include "fwhmDialog.h"
#include "momentDialog.h"
#include "configDialog.h"
#include "imageExportDialog.h"
#include "matrixDialog.h"
#include "matrixSizeDialog.h"
#include "matrixValuesDialog.h"
#include "shrinkMatrixDialog.h"
#include "importOPJ.h"
#include "associationsDialog.h"
#include "renameWindowDialog.h"
#include "ErrorBar.h"
#include "interpolationDialog.h"
#include "cascadeDialog.h"
#include "normalizeDialog.h"
#include "alignCurvesDialog.h"
#include "fileDialogs.h"
#include "smoothCurveDialog.h"
#include "filterDialog.h"
#include "fftDialog.h"
#include "epsExportDialog.h"
#include "note.h"
#include "folder.h"
#include "findDialog.h"
#include "Scripting.h"
#include "scales.h"
#include "Fitter.h"
#include "FunctionCurve.h"
#include "ScriptingLangDialog.h"
#include "ScriptWindow.h"
#include "TableStatistics.h"
#include "Spectrogram.h"
#include "subtBackDialog.h"
#include "curveCalcDialog.h"
#include "matrixInterpDialog.h"
#include "convolutionDialog.h"
#include "matrixFilterDialog.h"
#include "removeSpikeDialog.h"
#include "parser.h"

#ifndef Q_OS_WIN
#include <qassistantclient.h>
#endif
#include <qworkspace.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qinputdialog.h>
#include <qregexp.h>
#include <qtextview.h>
#include <qlistview.h>
#include <qcursor.h>
#include <qevent.h>
#include <qaction.h>
#include <qprogressdialog.h>
#include <qpixmapcache.h>
#include <qsettings.h>
#include <qstylefactory.h>
#include <qdragobject.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qprocess.h>
#include <qtranslator.h>
#include <qsplitter.h>
#include <qobjectlist.h>
#include <qeventloop.h>
#include <qsimplerichtext.h>
#include <qstringlist.h>
#include <zlib.h>

using namespace Qwt3D;

extern "C"
{
  // void file_compress (char *file, char *mode);
  // void file_uncompress (char *file);
  void gz_compress (FILE *in, gzFile out);
  void gz_uncompress (gzFile in, FILE *out);
}


ApplicationWindow::ApplicationWindow ():
QMainWindow (0, "main window", WDestructiveClose),
scripted (ScriptingLangManager::newEnv (this))
{
  init ();
}

ApplicationWindow::ApplicationWindow (const QStringList & l):
QMainWindow (0, "main window", WDestructiveClose),
scripted (ScriptingLangManager::newEnv (this))
{
  int args = (int) l.size ();
  if (args > 2)
    {
      ApplicationWindow *aux = new ApplicationWindow ();
      aux->hideActiveWindow ();

      QMessageBox::critical (aux, tr ("QtiPlot - Error"),
			     tr
			     ("Too many command line options (maximum accepted is 2)!"));
      delete aux;
      exit (1);
    }
  else if (args == 2)
    {
      if (QFile::exists (l[1]))
	{
	  ApplicationWindow *app = open (l[1]);
	  if (app)
	    app->parseCommandLineArgument (l[0], 2);
	  else
	    exit(1);
	}
      else
	{
	  ApplicationWindow *aux = new ApplicationWindow ();
	  aux->hideActiveWindow ();

	  QMessageBox::critical (aux, tr ("QtiPlot - Error"),
				 tr
				 ("<b> %1 </b>: Unknown command line option or the file doesn't exist!").
				 arg (l[1]));
	  delete aux;
	  exit (1);
	}
    }
  else if (args == 1)
    {
      if (QFile::exists(l[0]))
	{
	  ApplicationWindow *app = open(l[0]);
	  if(!app)
	    exit(1);
	}
      else
	parseCommandLineArgument (l[0], 1);
    }
}

ApplicationWindow::~ApplicationWindow ()
{
  if (lastCopiedLayer)
    delete lastCopiedLayer;

  delete hiddenWindows;
  delete outWindows;
  delete current_folder;

  if (scriptWindow)
    delete scriptWindow;

  QApplication::clipboard ()->clear (QClipboard::Clipboard);
}

void
ApplicationWindow::init ()
{
  initGlobalConstants ();

  // this must not be here!!
  // QPixmapCache::setCacheLimit (20 * QPixmapCache::cacheLimit ());
  //

  tablesDepend = new QPopupMenu (this, 0);

  createActions ();
  initToolBars ();
  initPlot3DToolBar ();

  initMainMenu ();

  explorerWindow = new QDockWindow (this);
  explorerWindow->setCaption (tr ("Project Explorer"));
  explorerWindow->setResizeEnabled (true);
  explorerWindow->setCloseMode (QDockWindow::Always);
  explorerWindow->setFixedExtentHeight (150);
  addDockWindow (explorerWindow, Qt::DockBottom);

  explorerSplitter = new QSplitter (Qt::Horizontal, explorerWindow);

  folderview = new FolderListView (explorerSplitter);
  folderview->header ()->setClickEnabled (FALSE);
  folderview->addColumn (tr ("Folder"));
  folderview->setRootIsDecorated (TRUE);
  folderview->setResizeMode (QListView::LastColumn);
  folderview->header ()->hide ();
  folderview->setSelectionMode (QListView::Single);

  connect (folderview, SIGNAL (currentChanged (QListViewItem *)),
	   this, SLOT (folderItemChanged (QListViewItem *)));
  connect (folderview,
	   SIGNAL (itemRenamed (QListViewItem *, int, const QString &)), this,
	   SLOT (renameFolder (QListViewItem *, int, const QString &)));
  connect (folderview,
	   SIGNAL (contextMenuRequested
		   (QListViewItem *, const QPoint &, int)), this,
	   SLOT (showFolderPopupMenu (QListViewItem *, const QPoint &, int)));
  connect (folderview, SIGNAL (dragItems (QPtrList < QListViewItem >)), this,
	   SLOT (dragFolderItems (QPtrList < QListViewItem >)));
  connect (folderview, SIGNAL (dropItems (QListViewItem *)), this,
	   SLOT (dropFolderItems (QListViewItem *)));
  connect (folderview, SIGNAL (renameItem (QListViewItem *)), this,
	   SLOT (startRenameFolder (QListViewItem *)));
  connect (folderview, SIGNAL (addFolderItem ()), this, SLOT (addSubFolder ()));
  connect (folderview, SIGNAL (deleteSelection ()), this,
	   SLOT (deleteSelectedItems ()));

  current_folder = new Folder (0, tr ("UNTITLED"));
  FolderListItem *fli = new FolderListItem (folderview, current_folder);
  current_folder->setFolderListItem (fli);
  fli->setOpen (TRUE);
  fli->setActive(true);

  listview = new FolderListView (explorerSplitter);
  listview->addColumn (tr ("Name"), -1);
  listview->addColumn (tr ("Type"), -1);
  listview->addColumn (tr ("View"), -1);
  listview->addColumn (tr ("Size"), -1);
  listview->addColumn (tr ("Created"), -1);
  listview->addColumn (tr ("Label"), -1);
  listview->setResizeMode (QListView::LastColumn);
  listview->setMinimumHeight (80);
  listview->setSelectionMode (QListView::Extended);

  explorerWindow->setWidget (explorerSplitter);
  explorerWindow->hide ();

  logWindow = new QDockWindow (this);
  logWindow->setCaption (tr ("Results Log"));
  logWindow->setResizeEnabled (true);
  logWindow->setCloseMode (QDockWindow::Always);
  addDockWindow (logWindow, Qt::DockTop, true);

  results = new QTextEdit (logWindow, "results");
  results->setReadOnly (TRUE);

  logWindow->setWidget (results);
  logWindow->hide ();

#ifdef SCRIPTING_CONSOLE
  consoleWindow = new QDockWindow (this);
  consoleWindow->setCaption (tr ("Scripting Console"));
  consoleWindow->setResizeEnabled (true);
  consoleWindow->setCloseMode (QDockWindow::Always);
  addDockWindow (consoleWindow, Qt::DockTop, true);
  console = new QTextEdit (consoleWindow, "console");
  console->setReadOnly (true);
  consoleWindow->setWidget (console);
  consoleWindow->hide ();
#endif

  ws = new QWorkspace (this);
  ws->setScrollBarsEnabled (TRUE);
  setCentralWidget (ws);
  setAcceptDrops (true);

  hiddenWindows = new QWidgetList ();
  outWindows = new QWidgetList ();

  window_to_close = 0;

  scriptWindow = 0;

  readSettings ();
  createLanguagesList ();
  insertTranslatedStrings ();

  hideToolbars();
  
  // qassistantclient crashes on Windows7
#ifndef Q_OS_WIN
  assistant = new QAssistantClient ("", this);
#endif

  QAccel *accel = new QAccel (this);
  accel->connectItem (accel->insertItem (Key_F5), ws,
		      SLOT (activateNextWindow ()));
  accel->connectItem (accel->insertItem (Key_F6), ws,
		      SLOT (activatePrevWindow ()));
  accel->connectItem (accel->insertItem (Key_Delete), this,
		      SLOT (clearSelection ()));

  connect (actionShowLog, SIGNAL (toggled (bool)), this,
	   SLOT (showResults (bool)));
  connect (logWindow, SIGNAL (visibilityChanged (bool)), actionShowLog,
	   SLOT (setOn (bool)));
  connect (explorerWindow, SIGNAL (visibilityChanged (bool)),
	   actionShowExplorer, SLOT (setOn (bool)));
#ifdef SCRIPTING_CONSOLE
  connect (actionShowConsole, SIGNAL (toggled (bool)), consoleWindow,
	   SLOT (setShown (bool)));
  connect (consoleWindow, SIGNAL (visibilityChanged (bool)),
	   actionShowConsole, SLOT (setOn (bool)));
#endif
  connect (tablesDepend, SIGNAL (activated (int)), this,
	   SLOT (showTable (int)));

  connect (this, SIGNAL (modified ()), this, SLOT (modifiedProject ()));
  connect (ws, SIGNAL (windowActivated (QWidget *)), this,
	   SLOT (windowActivated (QWidget *)));
  connect (listview, SIGNAL (doubleClicked (QListViewItem *)), this,
//	   SLOT (maximizeWindow (QListViewItem *)));
	   SLOT (activateWindow (QListViewItem *)));
  connect (listview, SIGNAL (doubleClicked (QListViewItem *)), this,
	   SLOT (folderItemDoubleClicked (QListViewItem *)));
  connect (listview,
	   SIGNAL (contextMenuRequested
		   (QListViewItem *, const QPoint &, int)), this,
	   SLOT (showWindowPopupMenu (QListViewItem *, const QPoint &, int)));
  connect (listview, SIGNAL (dragItems (QPtrList < QListViewItem >)), this,
	   SLOT (dragFolderItems (QPtrList < QListViewItem >)));
  connect (listview, SIGNAL (dropItems (QListViewItem *)), this,
	   SLOT (dropFolderItems (QListViewItem *)));
  connect (listview, SIGNAL (renameItem (QListViewItem *)), this,
//	   SLOT (startRenameFolder (QListViewItem *)));
	   SLOT (listViewItemRenamed (QListViewItem *)));
  connect (listview, SIGNAL (addFolderItem ()), this, SLOT (addSubFolder ()));
  connect (listview, SIGNAL (deleteSelection ()), this,
	   SLOT (deleteSelectedItems ()));
  connect (listview, SIGNAL (itemRenamed (QListViewItem *, int, const QString &)),
	   this, SLOT (renameWindow (QListViewItem *, int, const QString &)));
  connect (scriptEnv, SIGNAL (error (const QString &, const QString &, int)),
	   this, SLOT (scriptError (const QString &, const QString &, int)));

  connect (recentMenu, SIGNAL (activated (int)), this,
	   SLOT (openRecentProject (int)));
  //connect (&http, SIGNAL (done (bool)), this,
  //	   SLOT (receivedVersionFile (bool)));

}

void
ApplicationWindow::initGlobalConstants ()
{
#ifdef Q_OS_MACX		// Mac
  appStyle = "Macintosh (Aqua)";
#else
  appStyle = "Windows";
#endif

  majVersion = 0;
  minVersion = 8;
  patchVersion = 9;
  d_extra_version = "a";
  projectname = "untitled";
  lastModified = 0;
  activeGraph = 0;
  lastCopiedLayer = 0;
  copiedLayer = FALSE;
  copiedMarkerType = Graph::None;
  aw = 0;
  logInfo = QString::null;
  savingTimerId = 0;

  fwhmPeakFrac = 20;
  fwhmHalfFrac = 30;

  backGroundFunc = "";

  // autoSearchUpdatesRequest = false;

  show_windows_policy = ActiveFolder;

  appFont = QFont ();
  QString family = appFont.family ();
  int pointSize = appFont.pointSize ();
  tableTextFont = appFont;
  tableHeaderFont = appFont;
  plotAxesFont = QFont (family, pointSize, QFont::Bold, FALSE);
  plotNumbersFont = QFont (family, pointSize);
  plotLegendFont = appFont;
  plotTitleFont = QFont (family, pointSize + 2, QFont::Bold, FALSE);

  plot3DAxesFont = QFont (family, pointSize, QFont::Bold, FALSE);
  plot3DNumbersFont = QFont (family, pointSize);
  plot3DTitleFont = QFont (family, pointSize + 2, QFont::Bold, FALSE);
}

void
ApplicationWindow::applyUserSettings ()
{
//set user defined colors
  listview->setPaletteForegroundColor (panelsTextColor);
  listview->setPaletteBackgroundColor (panelsColor);

  results->setPaper (QBrush (panelsColor, SolidPattern));
  results->setPaletteForegroundColor (panelsTextColor);

  ws->setPaletteBackgroundColor (workspaceColor);

  QPalette pal = qApp->palette ();
  pal.setColor (QPalette::Active, QColorGroup::Base, QColor (panelsColor));
  qApp->setPalette (pal, true, 0);

  updateAppFonts ();

  QColorGroup cg;
  cg.setColor (QColorGroup::Text, QColor (green));
  cg.setColor (QColorGroup::HighlightedText, QColor (darkGreen));
  cg.setColor (QColorGroup::Background, QColor (black));
  info->setPalette (QPalette (cg, cg, cg));

  setScriptingLang (defaultScriptingLang);
}

void
ApplicationWindow::initToolBars ()
{
  setIcon (QPixmap (logo_xpm));
  QPixmap openIcon, saveIcon;

  fileTools = new QToolBar (this, "file operations");
  fileTools->setCloseMode (QDockWindow::Undocked);
  addToolBar (fileTools, tr ("File"), Top, TRUE);

  actionNewProject->addTo (fileTools);
  actionNewTable->addTo (fileTools);
  actionNewMatrix->addTo (fileTools);
  actionNewNote->addTo (fileTools);
  actionNewGraph->addTo (fileTools);
  actionNewFunctionPlot->addTo (fileTools);
  actionNewSurfacePlot->addTo (fileTools);

  fileTools->addSeparator ();

  actionOpen->addTo (fileTools);
  actionOpenTemplate->addTo (fileTools);
  actionSaveProject->addTo (fileTools);
  actionSaveTemplate->addTo (fileTools);

  fileTools->addSeparator ();

  actionLoad->addTo (fileTools);
  actionLoadMultiple->addTo (fileTools);

  fileTools->addSeparator ();

  actionCopyWindow->addTo (fileTools);
  actionPrint->addTo (fileTools);

  fileTools->addSeparator ();

  actionShowExplorer->addTo (fileTools);
  actionShowLog->addTo (fileTools);
#ifdef SCRIPTING_PYTHON
  actionShowScriptWindow->addTo (fileTools);
#endif

  editTools = new QToolBar (this, "edit operations");
  editTools->setCloseMode (QDockWindow::Undocked);
  addToolBar (editTools, tr ("Edit"));

  actionUndo->addTo (editTools);
  actionRedo->addTo (editTools);
  actionCutSelection->addTo (editTools);
  actionCopySelection->addTo (editTools);
  actionPasteSelection->addTo (editTools);
  actionClearSelection->addTo (editTools);

  plotTools = new QToolBar (this, "plot operations");
  plotTools->setCloseMode (QDockWindow::Undocked);
  addToolBar (plotTools, tr ("Plot"));

  actionAddLayer->addTo (plotTools);
  actionShowLayerDialog->addTo (plotTools);
  actionShowLayerOrderDialog->addTo (plotTools);
  actionAutomaticLayout->addTo (plotTools);

  plotTools->addSeparator ();

  actionShowCurvesDialog->addTo (plotTools);
  actionAddErrorBars->addTo (plotTools);
  actionAddFunctionCurve->addTo (plotTools);
  actionNewLegend->addTo (plotTools);

  plotTools->addSeparator ();

  actionUnzoom->addTo (plotTools);

  dataTools = new QActionGroup (this, "dataTools");
  dataTools->setExclusive (TRUE);
  dataTools->setUsesDropDown (FALSE);

  btnPointer =
    new QAction (tr ("Disable &tools"), QString::null, dataTools,
		 "btnPointer");
  btnPointer->setToggleAction (TRUE);
  btnPointer->setIconSet (QPixmap (pointer_xpm));
  btnPointer->setOn (TRUE);
  btnPointer->addTo (plotTools);

  btnZoomIn = new QAction (tr ("&Zoom In"), tr ("Ctrl++"), dataTools);
  btnZoomIn->setToggleAction (TRUE);
  btnZoomIn->setIconSet (QPixmap (zoom_xpm));
  btnZoomIn->addTo (plotTools);

  btnZoomOut = new QAction (tr ("Zoom &Out"), tr ("Ctrl+-"), dataTools);
  btnZoomOut->setToggleAction (TRUE);
  btnZoomOut->setIconSet (QPixmap (zoomOut_xpm));
  btnZoomOut->addTo (plotTools);

  btnCursor =
    new QAction (tr ("&Data reader"), tr ("CTRL+D"), dataTools, "btnCursor");
  btnCursor->setToggleAction (TRUE);
  btnCursor->setIconSet (QPixmap (select_xpm));
  btnCursor->addTo (plotTools);

  btnSelect =
    new QAction (tr ("&Select data range"), tr ("ALT+S"), dataTools,
		 "btnSelect");
  btnSelect->setToggleAction (TRUE);
  btnSelect->setIconSet (QPixmap (cursors_xpm));
  btnSelect->addTo (plotTools);

  btnPicker =
    new QAction (tr ("S&creen reader"), QString::null, dataTools,
		 "btnPicker");
  btnPicker->setToggleAction (TRUE);
  btnPicker->setIconSet (QPixmap (cursor_16));
  btnPicker->addTo (plotTools);

  btnMovePoints =
    new QAction (tr ("&Move Data Points"), tr ("Ctrl+ALT+M"), dataTools,
		 "btnMovePoints");
  btnMovePoints->setToggleAction (TRUE);
  btnMovePoints->setIconSet (QPixmap (hand_xpm));
  btnMovePoints->addTo (plotTools);

  btnRemovePoints =
    new QAction (tr ("Remove &Bad Data Points"), tr ("Alt+B"), dataTools,
		 "btnRemovePoints");
  btnRemovePoints->setToggleAction (TRUE);
  btnRemovePoints->setIconSet (QPixmap (gomme_xpm));
  btnRemovePoints->addTo (plotTools);

  connect (dataTools, SIGNAL (selected (QAction *)), this,
	   SLOT (pickDataTool (QAction *)));
  plotTools->addSeparator ();

  actionAddText =
    new QAction (QPixmap (text_xpm), tr ("Add &Text"), tr ("ALT+T"), this);
  actionAddText->setToggleAction (TRUE);
  connect (actionAddText, SIGNAL (activated ()), this, SLOT (addText ()));
  actionAddText->addTo (plotTools);

  btnArrow =
    new QAction (tr ("Draw &Arrow"), tr ("CTRL+ALT+A"), dataTools,
		 "btnArrow");
  btnArrow->setToggleAction (TRUE);
  btnArrow->setIconSet (QPixmap (arrow_xpm));
  btnArrow->addTo (plotTools);

  btnLine =
    new QAction (tr ("Draw &Line"), tr ("CTRL+ALT+L"), dataTools, "btnLine");
  btnLine->setToggleAction (TRUE);
  btnLine->setIconSet (QPixmap (lPlot_xpm));
  btnLine->addTo (plotTools);

  actionTimeStamp->addTo (plotTools);
  actionAddImage->addTo (plotTools);

  tableTools = new QToolBar (this, "table operations");
  tableTools->setCloseMode (QDockWindow::Undocked);
  addToolBar (tableTools, tr ("Table"), Top, FALSE);

  actionPlotL->addTo (tableTools);
  actionPlotP->addTo (tableTools);
  actionPlotLP->addTo (tableTools);
  actionPlotVerticalBars->addTo (tableTools);
  actionPlotHorizontalBars->addTo (tableTools);
  actionPlotArea->addTo (tableTools);
  actionPlotPie->addTo (tableTools);
  actionPlotHistogram->addTo (tableTools);
  actionBoxPlot->addTo (tableTools);
  actionPlotVectXYXY->addTo (tableTools);
  actionPlotVectXYAM->addTo (tableTools);

  tableTools->addSeparator ();

  actionPlot3DRibbon->addTo (tableTools);
  actionPlot3DBars->addTo (tableTools);
  actionPlot3DScatter->addTo (tableTools);
  actionPlot3DTrajectory->addTo (tableTools);

  tableTools->addSeparator ();

  actionAddColToTable->addTo (tableTools);
  actionAddRowToTable->addTo (tableTools);
  actionShowColStatistics->addTo (tableTools);
  actionShowRowStatistics->addTo (tableTools);

  plotTools->hide ();
  tableTools->hide ();

  displayBar = new QToolBar (this, "display operations");
  info = new QLineEdit (displayBar, "test");
  info->setReadOnly (TRUE);
  displayBar->setStretchableWidget (info);
  // displayBar->setResizeEnabled (true);
  // displayBar->setMaximumHeight (2 * displayBar->sizeHint ().height ());
  displayBar->setCloseMode (QDockWindow::Undocked);
  addToolBar (displayBar, tr ("Data Display"), Top, FALSE);
  setDockEnabled (displayBar, DockLeft, false);
  setDockEnabled (displayBar, DockRight, false);
  displayBar->hide ();
  
  plotMatrixBar = new QToolBar (this);
  plotMatrixBar->setCloseMode (QDockWindow::Undocked);
  addToolBar (plotMatrixBar, tr ("Matrix Plot"), Bottom, FALSE);

  actionPlot3DWireFrame->addTo (plotMatrixBar);
  actionPlot3DHiddenLine->addTo (plotMatrixBar);

  actionPlot3DPolygons->addTo (plotMatrixBar);
  actionPlot3DWireSurface->addTo (plotMatrixBar);

  plotMatrixBar->addSeparator ();

  actionPlot3DBars->addTo (plotMatrixBar);
  actionPlot3DScatter->addTo (plotMatrixBar);

  plotMatrixBar->addSeparator ();
  actionColorMap->addTo (plotMatrixBar);
  actionContourMap->addTo (plotMatrixBar);
  actionGrayMap->addTo (plotMatrixBar);

  plotMatrixBar->hide ();
}

void
ApplicationWindow::insertTranslatedStrings ()
{
  if (projectname == "untitled")
    setCaption (tr ("QtiPlot - untitled"));

  listview->setColumnText (0, tr ("Name"));
  listview->setColumnText (1, tr ("Type"));
  listview->setColumnText (2, tr ("View"));
  listview->setColumnText (3, tr ("Size"));
  listview->setColumnText (4, tr ("Created"));
  listview->setColumnText (5, tr ("Label"));

  if (scriptWindow)
    scriptWindow->setCaption (tr ("QtiPlot - Script Window"));
  explorerWindow->setCaption (tr ("Project Explorer"));
  logWindow->setCaption (tr ("Results Log"));
#ifdef SCRIPTING_CONSOLE
  consoleWindow->setCaption (tr ("Scripting Console"));
#endif
  displayBar->setLabel (tr ("Data Display"));
  tableTools->setLabel (tr ("Table"));
  plotTools->setLabel (tr ("Plot"));
  fileTools->setLabel (tr ("File"));
  editTools->setLabel (tr ("Edit"));
  plotMatrixBar->setLabel (tr ("Matrix Plot"));
  plot3DTools->setLabel (tr ("Surface 3D"));

  fileMenu->changeItem (newMenuID, tr ("&New"));
  fileMenu->changeItem (recentMenuID, tr ("&Recent projects"));
  fileMenu->changeItem (exportID, tr ("&Export Graph"));
  fileMenu->changeItem (importMenuID, tr ("&Import ASCII"));

  plot2DMenu->changeItem (specialPlotMenuID, tr ("Special Line/Symb&ol"));
  plot2DMenu->changeItem (statMenuID, tr ("Statistical &Graphs"));
  plot2DMenu->changeItem (panelMenuID, tr ("Pa&nel"));
  plot2DMenu->changeItem (plot3dID, tr ("Plot 3&D"));

  tableAnalysisMenu->changeItem (normMenuID, tr ("&Normalize"));

  tableMenu->changeItem (setAsMenuID, tr ("Set columns &as"));
  tableMenu->changeItem (fillMenuID, tr ("&Fill columns with"));

  graphAnalysisMenu->changeItem (translateMenuID, tr ("&Translate"));
  graphAnalysisMenu->changeItem (smoothMenuID, tr ("&Smooth"));
  graphAnalysisMenu->changeItem (filterMenuID, tr ("&FFT filter"));
  graphAnalysisMenu->changeItem (deconvMenuID, tr ("Deconvolute"));
  graphAnalysisMenu->changeItem (fitExpMenuID, tr ("Fit E&xponential Decay"));
  graphAnalysisMenu->changeItem (multiPeakMenuID, tr ("Fit &Multi-peak"));

  translateActionsStrings ();
  customMenu (ws->activeWindow ());
}

void
ApplicationWindow::initMainMenu ()
{
  fileMenu = new QPopupMenu (this);
  fileMenu->setFont (appFont);

  typeMenu = new QPopupMenu (this);
  typeMenu->setFont (appFont);
  actionNewProject->addTo (typeMenu);
  actionNewTable->addTo (typeMenu);
  actionNewMatrix->addTo (typeMenu);
  actionNewNote->addTo (typeMenu);
  actionNewGraph->addTo (typeMenu);
  actionNewFunctionPlot->addTo (typeMenu);
  actionNewSurfacePlot->addTo (typeMenu);

  newMenuID = fileMenu->insertItem (tr ("&New"), typeMenu);
  actionOpen->addTo (fileMenu);

  recentMenu = new QPopupMenu (this);
  recentMenu->setFont (appFont);
  recentMenuID = fileMenu->insertItem (tr ("&Recent projects"), recentMenu);

  fileMenu->insertSeparator ();

  actionLoadImage->addTo (fileMenu);
  actionImportImage->addTo (fileMenu);

  fileMenu->insertSeparator ();

  actionSaveProject->addTo (fileMenu);
  actionSaveProjectAs->addTo (fileMenu);

  fileMenu->insertSeparator ();
  actionOpenTemplate->addTo (fileMenu);
  actionSaveTemplate->addTo (fileMenu);
  fileMenu->insertSeparator ();

  exportPlot = new QPopupMenu (this);
  actionExportGraph->addTo (exportPlot);
  actionExportAllGraphs->addTo (exportPlot);
  exportID = fileMenu->insertItem (tr ("&Export Graph"), exportPlot);

  actionPrint->addTo (fileMenu);
  actionPrintAllPlots->addTo (fileMenu);

  fileMenu->insertSeparator ();

  actionShowExportASCIIDialog->addTo (fileMenu);

  importMenu = new QPopupMenu (this);
  importMenu->setFont (appFont);
  actionLoad->addTo (importMenu);
  actionLoadMultiple->addTo (importMenu);

  importMenu->insertSeparator ();

  actionShowImportDialog->addTo (importMenu);
  importMenuID = fileMenu->insertItem (tr ("&Import ASCII"), importMenu);

  fileMenu->insertSeparator ();

  actionCloseAllWindows->addTo (fileMenu);

  editMenu = new QPopupMenu (this);
  editMenu->setFont (appFont);
  actionUndo->addTo (editMenu);
  actionRedo->addTo (editMenu);

  editMenu->insertSeparator ();

  actionCutSelection->addTo (editMenu);
  actionCopySelection->addTo (editMenu);
  actionPasteSelection->addTo (editMenu);
  actionClearSelection->addTo (editMenu);

  editMenu->insertSeparator ();

  actionDeleteUselessTables->addTo (editMenu);
  actionClearLogInfo->addTo (editMenu);

  editMenu->insertSeparator ();

  actionShowConfigureDialog->addTo (editMenu);

  viewMenu = new QPopupMenu (this);
  viewMenu->setFont (appFont);
  viewMenu->setCheckable (TRUE);
  actionShowPlotWizard->addTo (viewMenu);
  actionShowExplorer->addTo (viewMenu);
  actionShowLog->addTo (viewMenu);
#ifdef SCRIPTING_CONSOLE
  actionShowConsole->addTo (viewMenu);
#endif

  graphMenu = new QPopupMenu (this);
  graphMenu->setFont (appFont);
  graphMenu->setCheckable (TRUE);
  actionShowCurvesDialog->addTo (graphMenu);
  actionAddErrorBars->addTo (graphMenu);
  actionAddFunctionCurve->addTo (graphMenu);
  actionNewLegend->addTo (graphMenu);

  graphMenu->insertSeparator ();

  actionResetColors->addTo (graphMenu);
  actionBlackenColors->addTo (graphMenu);
  actionResetSymbols->addTo (graphMenu);
  actionResetFonts->addTo (graphMenu);

  graphMenu->insertSeparator ();

  actionAddText->addTo (graphMenu);
  btnArrow->addTo (graphMenu);
  btnLine->addTo (graphMenu);
  actionTimeStamp->addTo (graphMenu);
  actionAddImage->addTo (graphMenu);

  graphMenu->insertSeparator ();	//layers section
  actionAddLayer->addTo (graphMenu);
  actionResizeLayer->addTo(graphMenu);
  actionDeleteLayer->addTo (graphMenu);
  actionShowLayerDialog->addTo (graphMenu);
  actionShowLayerOrderDialog->addTo (graphMenu);
  graphMenu->insertSeparator ();
  actionAlign->addTo(graphMenu);
  actionCascade->addTo(graphMenu);
  actionNormalizeGraph->addTo(graphMenu);

  matrixPlotMenu = new QPopupMenu (this);
  matrixPlotMenu->setFont (appFont);

  actionPlot3DWireFrame->addTo (matrixPlotMenu);
  actionPlot3DHiddenLine->addTo (matrixPlotMenu);

  actionPlot3DPolygons->addTo (matrixPlotMenu);
  actionPlot3DWireSurface->addTo (matrixPlotMenu);

  matrixPlotMenu->insertSeparator ();

  actionPlot3DBars->addTo (matrixPlotMenu);
  actionPlot3DScatter->addTo (matrixPlotMenu);

  matrixPlotMenu->insertSeparator ();
  actionColorMap->addTo (matrixPlotMenu);
  actionContourMap->addTo (matrixPlotMenu);
  actionGrayMap->addTo (matrixPlotMenu);

  matrixMenu = new QPopupMenu (this);
  matrixMenu->setFont (appFont);

  actionSetMatrixProperties->addTo (matrixMenu);
  actionSetMatrixDimensions->addTo (matrixMenu);
  actionSetMatrixValues->addTo (matrixMenu);

  matrixMenu->insertSeparator ();

  actionTransposeMatrix->addTo (matrixMenu);
  actionInvertMatrix->addTo (matrixMenu);
  actionShrinkMatrix->addTo (matrixMenu);

  matrixMenu->insertSeparator ();
  actionGoToRow->addTo (matrixMenu);
  matrixMenu->insertSeparator ();
  actionConvertMatrix->addTo (matrixMenu);

  initPlotMenu ();
  initTableAnalysisMenu ();
  initMatrixAnalysisMenu();
  initTableMenu ();
  initPlotDataMenu ();

  graphAnalysisMenu = new QPopupMenu (this);
  graphAnalysisMenu->setFont (appFont);

  translateMenu = new QPopupMenu (this);
  translateMenu->setFont (appFont);
  actionTranslateVert->addTo (translateMenu);
  actionTranslateHor->addTo (translateMenu);
  translateMenuID = graphAnalysisMenu->insertItem (tr ("&Translate"), translateMenu);

  actionSubtractLine->addTo(graphAnalysisMenu);
  actionSubtractBack->addTo(graphAnalysisMenu);
  actionRemoveSpike->addTo(graphAnalysisMenu);

  graphAnalysisMenu->insertSeparator ();

  actionCurveCalc->addTo(graphAnalysisMenu);

  graphAnalysisMenu->insertSeparator ();

  actionDifferentiate->addTo (graphAnalysisMenu);
  actionShowIntDialog->addTo (graphAnalysisMenu);
  actionShowFwhmDialog->addTo (graphAnalysisMenu);
  actionShowMomentDialog->addTo (graphAnalysisMenu);

  graphAnalysisMenu->insertSeparator ();

  smoothMenu = new QPopupMenu (this);
  smoothMenu->setFont (appFont);
  actionSmoothSavGol->addTo (smoothMenu);
  actionSmoothAverage->addTo (smoothMenu);
  actionGaussConv->addTo (smoothMenu);
  actionLorentzConv->addTo (smoothMenu);
  smoothMenuID = graphAnalysisMenu->insertItem (tr ("&Smooth"), smoothMenu);

  filterMenu = new QPopupMenu (this);
  filterMenu->setFont (appFont);
  actionLowPassFilter->addTo (filterMenu);
  actionHighPassFilter->addTo (filterMenu);
  actionBandPassFilter->addTo (filterMenu);
  actionBandBlockFilter->addTo (filterMenu);
  actionBandInterpFilter->addTo (filterMenu);
  filterMenuID = graphAnalysisMenu->insertItem (tr ("&FFT filter"), filterMenu);

  graphAnalysisMenu->insertSeparator ();
  actionInterpolate->addTo (graphAnalysisMenu);
  actionFFT->addTo (graphAnalysisMenu);
  actionConvoluteCurves->addTo (graphAnalysisMenu);
  deconvMenu = new QPopupMenu (this);
  deconvMenu->setFont(appFont);
  actionDeconvFFT->addTo(deconvMenu);
  actionDeconvAndre->addTo(deconvMenu);
  actionDeconvGS->addTo(deconvMenu);
  deconvMenuID = graphAnalysisMenu->insertItem(tr("Deconvolute"), deconvMenu);

  graphAnalysisMenu->insertSeparator ();
  actionFitLinear->addTo (graphAnalysisMenu);
  actionShowFitPolynomDialog->addTo (graphAnalysisMenu);
  actionShowSineFitDialog->addTo (graphAnalysisMenu);

  graphAnalysisMenu->insertSeparator ();

  decayMenu = new QPopupMenu (this);
  decayMenu->setFont (appFont);
  actionShowExpDecayDialog->addTo (decayMenu);
  actionShowTwoExpDecayDialog->addTo (decayMenu);
  actionShowExpDecay3Dialog->addTo (decayMenu);
  fitExpMenuID = graphAnalysisMenu->insertItem (tr ("Fit E&xponential Decay"),
						decayMenu);

  actionFitExpGrowth->addTo (graphAnalysisMenu);
  actionFitSigmoidal->addTo (graphAnalysisMenu);
  actionFitGauss->addTo (graphAnalysisMenu);
  actionFitLorentz->addTo (graphAnalysisMenu);

  multiPeakMenu = new QPopupMenu (this);
  multiPeakMenu->setFont (appFont);
  actionMultiPeakGauss->addTo (multiPeakMenu);
  actionMultiPeakLorentz->addTo (multiPeakMenu);
  multiPeakMenuID =
    graphAnalysisMenu->insertItem (tr ("Fit &Multi-peak"), multiPeakMenu);

  graphAnalysisMenu->insertSeparator ();

  actionShowFitDialog->addTo (graphAnalysisMenu);

  formatMenu = new QPopupMenu (this);
  formatMenu->setFont (appFont);

  scriptingMenu = new QPopupMenu (this);
  scriptingMenu->setFont (appFont);

  windowsMenu = new QPopupMenu (this);
  windowsMenu->setFont (appFont);
  windowsMenu->setCheckable (TRUE);
  connect (windowsMenu, SIGNAL (aboutToShow ()),
	   this, SLOT (windowsMenuAboutToShow ()));

  helpMenu = new QPopupMenu (this);
  helpMenu->setFont (appFont);

  actionShowHelp->addTo (helpMenu);
  actionChooseHelpFolder->addTo (helpMenu);
  helpMenu->insertSeparator ();
  actionHomePage->addTo (helpMenu);
  actionDonate->addTo (helpMenu);
  helpMenu->insertSeparator ();
  actionAbout->addTo (helpMenu);

  disableActions ();
}

void
ApplicationWindow::initTableMenu ()
{
  tableMenu = new QPopupMenu (this);
  tableMenu->setFont (appFont);

  setAsMenu = new QPopupMenu (this);
  setAsMenu->setFont (appFont);

  actionSetXCol->addTo (setAsMenu);
  actionSetYCol->addTo (setAsMenu);
  actionSetZCol->addTo (setAsMenu);
  setAsMenu->insertSeparator ();
  actionSetYErrCol->addTo (setAsMenu);
  actionSetXErrCol->addTo (setAsMenu);
  setAsMenu->insertSeparator ();

  actionDisregardCol->addTo (setAsMenu);
  setAsMenuID = tableMenu->insertItem (tr ("Set columns &as"), setAsMenu);

  actionShowColumnOptionsDialog->addTo (tableMenu);
  tableMenu->insertSeparator ();

  actionShowColumnValuesDialog->addTo (tableMenu);
  actionTableRecalculate->addTo (tableMenu);

  fillMenu = new QPopupMenu (this);
  fillMenu->setFont (appFont);
  actionSetAscValues->addTo (fillMenu);
  actionSetRandomValues->addTo (fillMenu);
  fillMenuID = tableMenu->insertItem (tr ("&Fill columns with"), fillMenu);

  actionClearTable->addTo (tableMenu);
  tableMenu->insertSeparator ();
  actionAddColToTable->addTo (tableMenu);
  actionShowColsDialog->addTo (tableMenu);
  tableMenu->insertSeparator ();
  actionAddRowToTable->addTo (tableMenu);
  actionShowRowsDialog->addTo (tableMenu);
  actionGoToRow->addTo (tableMenu);
  tableMenu->insertSeparator ();
  actionGenerateFormula->addTo (tableMenu);
  tableMenu->insertSeparator ();
  actionConvertTable->addTo (tableMenu);
}

void
ApplicationWindow::initPlotDataMenu ()
{
  plotDataMenu = new QPopupMenu (this);
  plotDataMenu->setFont (appFont);
  plotDataMenu->setCheckable (TRUE);

  btnPointer->addTo (plotDataMenu);
  btnZoomIn->addTo (plotDataMenu);
  btnZoomOut->addTo (plotDataMenu);
  actionUnzoom->addTo (plotDataMenu);
  plotDataMenu->insertSeparator ();

  btnCursor->addTo (plotDataMenu);
  btnSelect->addTo (plotDataMenu);
  btnPicker->addTo (plotDataMenu);

  plotDataMenu->insertSeparator ();

  btnMovePoints->addTo (plotDataMenu);
  btnRemovePoints->addTo (plotDataMenu);
}

void
ApplicationWindow::initPlotMenu ()
{
  plot2DMenu = new QPopupMenu (this);
  plot2DMenu->setFont (appFont);
  specialPlot = new QPopupMenu (this);
  specialPlot->setFont (appFont);
  panelsMenu = new QPopupMenu (this);
  panelsMenu->setFont (appFont);
  statMenu2 = new QPopupMenu (this);
  statMenu2->setFont (appFont);

  actionPlotL->addTo (plot2DMenu);
  actionPlotP->addTo (plot2DMenu);
  actionPlotLP->addTo (plot2DMenu);

  actionPlotVerticalDropLines->addTo (specialPlot);
  actionPlotSpline->addTo (specialPlot);
  actionPlotVertSteps->addTo (specialPlot);
  actionPlotHorSteps->addTo (specialPlot);
  specialPlotMenuID =
    plot2DMenu->insertItem (tr ("Special Line/Symb&ol"), specialPlot);

  plot2DMenu->insertSeparator ();

  actionPlotVerticalBars->addTo (plot2DMenu);
  actionPlotHorizontalBars->addTo (plot2DMenu);
  actionPlotArea->addTo (plot2DMenu);
  actionPlotPie->addTo (plot2DMenu);
  actionPlotVectXYXY->addTo (plot2DMenu);
  actionPlotVectXYAM->addTo (plot2DMenu);

  plot2DMenu->insertSeparator ();

  actionBoxPlot->addTo (statMenu2);
  actionPlotHistogram->addTo (statMenu2);
  actionPlotStackedHistograms->addTo (statMenu2);
  statMenuID = plot2DMenu->insertItem (tr ("Statistical &Graphs"), statMenu2);

  actionPlot2VerticalLayers->addTo (panelsMenu);
  actionPlot2HorizontalLayers->addTo (panelsMenu);
  actionPlot4Layers->addTo (panelsMenu);
  actionPlotStackedLayers->addTo (panelsMenu);
  panelMenuID = plot2DMenu->insertItem (tr ("Pa&nel"), panelsMenu);

  plot3DMenu = new QPopupMenu (this);
  plot3DMenu->setFont (appFont);
  actionPlot3DRibbon->addTo (plot3DMenu);
  actionPlot3DBars->addTo (plot3DMenu);
  actionPlot3DScatter->addTo (plot3DMenu);
  actionPlot3DTrajectory->addTo (plot3DMenu);

  plot2DMenu->insertSeparator ();
  plot3dID = plot2DMenu->insertItem (tr ("Plot 3&D"), plot3DMenu);
}

void
ApplicationWindow::initTableAnalysisMenu ()
{
  tableAnalysisMenu = new QPopupMenu (this);
  tableAnalysisMenu->setFont (appFont);

  statMenu = new QPopupMenu(this);
  tableAnalysisMenu->insertItem(tr("Statistics"), statMenu);

  actionShowColStatistics->addTo (statMenu);
  actionShowRowStatistics->addTo (statMenu);
  actionShowXYStatistics->addTo (statMenu);
  statMenu->insertItem ( tr ("Area Statistics"),
			this, SLOT (showTableAreaStatistics()));
  statMenu->insertItem ( tr ("Moment Analysis"),
			this, SLOT (showMomentAnalysis()));

  tableAnalysisMenu->insertSeparator ();

  actionSortSelection->addTo (tableAnalysisMenu);
  actionSortTable->addTo (tableAnalysisMenu);

  normMenu = new QPopupMenu (this);
  normMenu->setFont (appFont);
  actionNormalizeSelection->addTo (normMenu);
  actionNormalizeTable->addTo (normMenu);
  normMenuID = tableAnalysisMenu->insertItem (tr ("&Normalize"), normMenu);

  tableAnalysisMenu->insertSeparator ();

  actionFFT->addTo (tableAnalysisMenu);
  actionCorrelate->addTo (tableAnalysisMenu);
  actionConvolute->addTo (tableAnalysisMenu);
  actionDeconvolute->addTo (tableAnalysisMenu);

  //tableAnalysisMenu->insertSeparator ();
  //actionShowFitDialog->addTo (tableAnalysisMenu);
}

void
ApplicationWindow::initMatrixAnalysisMenu ()
{
  matrixAnalysisMenu = new QPopupMenu (this);
  matrixAnalysisMenu->setFont (appFont);

  matrixAnalysisMenu->insertItem( tr("Area Statistics"),
				 this, SLOT(showMatrixAreaStatistics()) );
  matrixAnalysisMenu->insertSeparator ();
  actionMatrixInterpolate->addTo (matrixAnalysisMenu);
  actionMatrixFFT->addTo(matrixAnalysisMenu);
  actionMatrixFilter->addTo(matrixAnalysisMenu);
  matrixAnalysisMenu->insertSeparator ();
  actionMatrixDeterminant->addTo (matrixAnalysisMenu);
}

void
ApplicationWindow::customMenu (QWidget * w)
{
  menuBar ()->clear ();
  menuBar ()->insertItem (tr ("&File"), fileMenu);
  menuBar ()->insertItem (tr ("&Edit"), editMenu);
  menuBar ()->insertItem (tr ("&View"), viewMenu);
  menuBar ()->insertItem (tr ("S&cripting"), scriptingMenu);
  scriptingMenu->clear ();
#ifdef SCRIPTING_DIALOG
  actionScriptingLang->addTo (scriptingMenu);
#endif
  actionRestartScripting->addTo (scriptingMenu);

// these use the same keyboard shortcut (Ctrl+Return) and should not be enabled at the same time
  actionNoteEvaluate->setEnabled (false);
  actionTableRecalculate->setEnabled (false);

  if (w)
    {
      actionPrintAllPlots->setEnabled (projectHas2DPlots ());
      actionPrint->setEnabled (true);
      actionCutSelection->setEnabled (true);
      actionCopySelection->setEnabled (true);
      actionPasteSelection->setEnabled (true);
      actionClearSelection->setEnabled (true);
      actionSaveTemplate->setEnabled (true);

      if (w->isA ("MultiLayer"))
	{
	  menuBar ()->insertItem (tr ("&Graph"), graphMenu);
	  menuBar ()->insertItem (tr ("&Data"), plotDataMenu);
	  menuBar ()->insertItem (tr ("&Analysis"), graphAnalysisMenu);
	  menuBar ()->insertItem (tr ("For&mat"), formatMenu);

	  fileMenu->setItemEnabled (exportID, TRUE);
	  actionShowExportASCIIDialog->setEnabled (FALSE);
	  //close menu does not exist
	  //fileMenu->setItemEnabled (closeID, TRUE);

	  formatMenu->clear ();
	  actionShowPlotDialog->addTo (formatMenu);
	  actionShowCurveFormatDialog->addTo (formatMenu);
	  Graph *g = ((MultiLayer *) w)->activeGraph ();
	  if (g && !g->isPiePlot ())
	    {
	      formatMenu->insertSeparator ();
	      actionShowScaleDialog->addTo (formatMenu);
	      actionShowAxisDialog->addTo (formatMenu);
	      actionShowAxisDialog->setEnabled (true);
	      formatMenu->insertSeparator ();
	      actionShowGridDialog->addTo (formatMenu);
	    }
	  actionShowTitleDialog->addTo (formatMenu);
	}
      else if (w->isA ("Graph3D"))
	{
	  disableActions ();

	  menuBar ()->insertItem (tr ("For&mat"), formatMenu);

	  actionPrint->setEnabled (TRUE);
	  actionSaveTemplate->setEnabled (true);
	  fileMenu->setItemEnabled (exportID, TRUE);
	  //fileMenu->setItemEnabled (closeID, TRUE);

	  formatMenu->clear ();
	  actionShowPlotDialog->addTo (formatMenu);
	  actionShowScaleDialog->addTo (formatMenu);
	  actionShowAxisDialog->addTo (formatMenu);
	  actionShowTitleDialog->addTo (formatMenu);
	  if (((Graph3D *) w)->coordStyle () == Qwt3D::NOCOORD)
	    actionShowAxisDialog->setEnabled (FALSE);
	}
      else if (w->inherits ("Table"))
	{
	  menuBar ()->insertItem (tr ("&Plot"), plot2DMenu);
	  if (w->isA ("Table"))
	    {
	      menuBar ()->insertItem (tr ("&Analysis"), tableAnalysisMenu);
	      menuBar ()->insertItem (tr ("&Table"), tableMenu);
	    }

	  actionShowExportASCIIDialog->setEnabled (TRUE);
	  actionTableRecalculate->setEnabled (true);
	  fileMenu->setItemEnabled (exportID, FALSE);
	  //fileMenu->setItemEnabled (closeID, TRUE);
	}
      else if (w->isA ("Matrix"))
	{
	  menuBar ()->insertItem (tr ("&Plot3D"), matrixPlotMenu);
	  menuBar ()->insertItem (tr ("&Analysis"), matrixAnalysisMenu);
	  menuBar ()->insertItem (tr ("&Matrix"), matrixMenu);
	}
      else if (w->isA ("Note"))
	{
	  actionSaveTemplate->setEnabled (false);
	  actionNoteEvaluate->setEnabled (true);
	  scriptingMenu->insertSeparator ();
	  actionNoteExecute->addTo (scriptingMenu);
	  actionNoteExecuteAll->addTo (scriptingMenu);
	  actionNoteEvaluate->addTo (scriptingMenu);

	  actionNoteExecute->disconnect (SIGNAL (activated ()));
	  actionNoteExecuteAll->disconnect (SIGNAL (activated ()));
	  actionNoteEvaluate->disconnect (SIGNAL (activated ()));
	  connect (actionNoteExecute, SIGNAL (activated ()), w,
		   SLOT (execute ()));
	  connect (actionNoteExecuteAll, SIGNAL (activated ()), w,
		   SLOT (executeAll ()));
	  connect (actionNoteEvaluate, SIGNAL (activated ()), w,
		   SLOT (evaluate ()));
	}
      else
	disableActions ();

      menuBar ()->insertItem (tr ("&Windows"), windowsMenu);
    }
  else
    disableActions ();

  menuBar ()->insertItem (tr ("&Help"), helpMenu);

}

void
ApplicationWindow::disableActions ()
{
  actionSaveTemplate->setEnabled (false);
  actionPrintAllPlots->setEnabled (false);
  actionPrint->setEnabled (false);
  actionShowExportASCIIDialog->setEnabled (false);
  fileMenu->setItemEnabled (exportID, false);
  //fileMenu->setItemEnabled (closeID, false);

  actionUndo->setEnabled (false);
  actionRedo->setEnabled (false);

  actionCutSelection->setEnabled (false);
  actionCopySelection->setEnabled (false);
  actionPasteSelection->setEnabled (false);
  actionClearSelection->setEnabled (false);
}

void
ApplicationWindow::customToolBars (QWidget * w)
{

  /*
  if (!projectHas3DPlots ())
    plot3DTools->hide ();
  if (!projectHas2DPlots ())
    plotTools->hide ();
  if (!projectHasMatrices ())
    plotMatrixBar->hide ();
  if ((int) tableWindows.count () <= 0)
    tableTools->hide ();
  */

  if (w)
    {
      if (w->isA ("MultiLayer"))
	{
	  if (plotTools->isHidden ())
	    plotTools->show ();

	  plotTools->setEnabled (true);
	  plot3DTools->setEnabled (false);
	  tableTools->setEnabled (false);
	  plotMatrixBar->setEnabled (false);
	}
      else if (w->inherits ("Table"))
	{
	  if (tableTools->isHidden ())
	    tableTools->show ();

	  plotTools->setEnabled (false);
	  plot3DTools->setEnabled (false);
	  tableTools->setEnabled (true);
	  plotMatrixBar->setEnabled (false);
	}
      else if (w->isA ("Matrix"))
	{
	  if (plotMatrixBar->isHidden ())
	    plotMatrixBar->show ();

	  plotTools->setEnabled (false);
	  plot3DTools->setEnabled (false);
	  tableTools->setEnabled (false);
	  plotMatrixBar->setEnabled (true);
	}
      else if (w->isA ("Graph3D"))
	{
	  plotTools->setEnabled (false);
	  tableTools->setEnabled (false);
	  plotMatrixBar->setEnabled (false);

	  if (plot3DTools->isHidden ())
	    plot3DTools->show ();

	  Graph3D *plot = (Graph3D *) w;
	  if (plot->plotStyle () == Qwt3D::NOPLOT)
	    plot3DTools->setEnabled (false);
	  else
	    plot3DTools->setEnabled (true);

	  custom3DActions (w);
	}
      else if (w->isA ("Note"))
	{
	  plotTools->setEnabled (false);
	  plot3DTools->setEnabled (false);
	  tableTools->setEnabled (false);
	  plotMatrixBar->setEnabled (false);
	}
    }

  lineUpDockWindows();
  //  else
  //  hideToolbars ();
}

void
ApplicationWindow::hideToolbars ()
{
  plot3DTools->hide ();
  plotTools->hide ();
  tableTools->hide ();
  plotMatrixBar->hide ();

  plotTools->setEnabled (false);
  tableTools->setEnabled (false);
  plot3DTools->setEnabled (false);
  plotMatrixBar->setEnabled (false);
}

void
ApplicationWindow::showExplorer ()
{
  if (!explorerWindow->isVisible ())
    explorerWindow->show ();
  else
    explorerWindow->hide ();
}

void
ApplicationWindow::plot3DRibbon ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  Table *w = (Table *) ws->activeWindow ();
  if (int (w->selectedColumns ().count ()) == 1)
    w->plot3DRibbon ();
  else
    QMessageBox::warning (0, tr ("QtiPlot - Plot error"),
			  tr
			  ("You must select exactly one column for plotting!"));
}

void
ApplicationWindow::plot3DWireframe ()
{
  plot3DMatrix (Qwt3D::WIREFRAME);
}

void
ApplicationWindow::plot3DHiddenLine ()
{
  plot3DMatrix (Qwt3D::HIDDENLINE);
}

void
ApplicationWindow::plot3DPolygons ()
{
  plot3DMatrix (Qwt3D::FILLED);
}

void
ApplicationWindow::plot3DWireSurface ()
{
  plot3DMatrix (Qwt3D::FILLEDMESH);
}

void
ApplicationWindow::plot3DBars ()
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return;

  if (w->inherits ("Table"))
    {
      if (int (((Table *) w)->selectedColumns ().count ()) == 1)
	((Table *) w)->plot3DBars ();
      else
	QMessageBox::warning (0, tr ("QtiPlot - Plot error"),
			      tr
			      ("You must select exactly one column for plotting!"));
    }
  else
    plot3DMatrix (Qwt3D::USER);
}

void
ApplicationWindow::plot3DScatter ()
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return;

  if (w->inherits ("Table"))
    {
      if (int (((Table *) w)->selectedColumns ().count ()) == 1)
	((Table *) w)->plot3DScatter ();
      else
	QMessageBox::warning (0, tr ("QtiPlot - Plot error"),
			      tr
			      ("You must select exactly one column for plotting!"));
    }
  else if (w->isA ("Matrix"))
    plot3DMatrix (Qwt3D::POINTS);
}

void
ApplicationWindow::plot3DTrajectory ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  Table *w = (Table *) ws->activeWindow ();
  if (int (w->selectedColumns ().count ()) == 1)
    w->plot3DTrajectory ();
  else
    QMessageBox::warning (0, tr ("QtiPlot - Plot error"),
			  tr
			  ("You must select exactly one column for plotting!"));
}

void
ApplicationWindow::plotVerticalBars ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotVB ();
}

void
ApplicationWindow::plotHorizontalBars ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotHB ();
}

void
ApplicationWindow::plotHistogram ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotHistogram ();
}

void
ApplicationWindow::plotArea ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotArea ();
}

void
ApplicationWindow::plotPie ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  if (int (((Table *) ws->activeWindow ())->selectedColumns ().count ()) == 1)
    ((Table *) ws->activeWindow ())->plotPie ();
  else
    QMessageBox::warning (0, tr ("QtiPlot - Plot error"),
			  tr
			  ("You must select exactly one column for plotting!"));
}

void
ApplicationWindow::plotL ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotL ();
}

void
ApplicationWindow::plotP ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotP ();
}

void
ApplicationWindow::plotLP ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotLP ();
}

void
ApplicationWindow::plotVerticalDropLines ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotVerticalDropLines ();
}

void
ApplicationWindow::plotSpline ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotSpline ();
}

void
ApplicationWindow::plotVertSteps ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotVertSteps ();
}

void
ApplicationWindow::plotHorSteps ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotHorSteps ();
}

void
ApplicationWindow::plotVectXYXY ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotVectXYXY ();
}

void
ApplicationWindow::plotVectXYAM ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotVectXYAM ();
}

void
ApplicationWindow::updateTable (const QString & caption, int row,
				const QString & text)
{// called by the signal updateTable in graph.cpp
  Table *w = table (caption);
  if (!w)
    return;

  QStringList cvs = QStringList::split (',', caption, FALSE);
  int xcol = w->colIndex(cvs[0]);
  int ycol = w->colIndex(cvs[1]);
  QString colName = cvs[1];

  if (w->columnType (xcol) == Table::Numeric
      && w->columnType (ycol) == Table::Numeric)
    {
      QStringList values = QStringList::split ('\t', text, FALSE);
      w->setText (row, xcol, values[0]);
      w->setText (row, ycol, values[1]);
      updateCurves (w, colName);
      //      emit modified ();
      emit modifiedProject(w);
    }
  else
    QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			  tr
			  ("This operation cannot be performed on curves plotted from columns having a non-numerical format."));
}

void
ApplicationWindow::updateTableColumn (const QString & colName,
				      QMemArray <double> & dat )
{
  Table *w = table (colName);
  if (!w)
    return;

  int col = w->colIndex (colName);
  if (w->columnType (col) == Table::Numeric)
    {
      int prec;
      char f;
      w->columnNumericFormat (col, f, prec);
      int i = 0, j = 0;
      int rows = dat.size();
      while (i < rows && j < w->tableRows ())
	{
	  if (!w->text (j, col).isEmpty ())
	    {
	      w->setText (j, col, QString::number (dat[i], f, prec));
	      i++;
	    }
	  j++;
	}

      updateCurves (w, colName);
      // emit modified ();
      emit modifiedProject(w);
    }
  else
    QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			  tr
			  ("This operation cannot be performed on curves plotted from columns having a non-numerical format."));
}

void
ApplicationWindow::clearCellFromTable (const QString & association,
				       int point)
{//called by the signal clearCell in graph.cpp

  QStringList cols = QStringList::split (',', association, false);

  Table *w = table (cols[1]);
  if (!w) return;

  int xcol = w->colIndex (cols[0]);
  int ycol = w->colIndex (cols[1]);
  int rows = w->tableRows ();

  if (xcol < 0 || ycol < 0)
    return;

  if (w->columnType (ycol) == Table::Numeric)
    {
      int it=-1;
      int r;
      for (r = 0; r < rows; r++)
	{
	  QString xval = w->text (r, xcol);
	  QString yval = w->text (r, ycol);
	  if (!xval.isEmpty () && !yval.isEmpty ())
	    {
	      it++;
	      if(it==point){
		//fprintf(stderr,"r=%d  it=%d  point=%d\n",r,it,point);
		break;
	      }
	    }
	}
      if(it==point) w->clearCell( r, ycol );
    }
  else
    QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			  tr
			  ("This operation cannot be performed on curves plotted from columns having a non-numerical format."));
}

void
ApplicationWindow::renameListViewItem (const QString & oldName,
				       const QString & newName)
{
  QListViewItem *it =
    listview->findItem (oldName, 0, Qt::ExactMatch | Qt::CaseSensitive);
  if (it)
    it->setText (0, newName);
}

void
ApplicationWindow::setListViewLabel (const QString & caption,
				     const QString & label)
{
  QListViewItem *it =
    listview->findItem (caption, 0, Qt::ExactMatch | Qt::CaseSensitive);
  if (it)
    it->setText (5, label);
}

void
ApplicationWindow::setListViewDate (const QString & caption,
				    const QString & date)
{
  QListViewItem *it =
    listview->findItem (caption, 0, Qt::ExactMatch | Qt::CaseSensitive);
  if (it)
    it->setText (4, date);
}

void
ApplicationWindow::setListView (const QString & caption, const QString & view)
{
  QListViewItem *it =
    listview->findItem (caption, 0, Qt::ExactMatch | Qt::CaseSensitive);
  if (it)
    it->setText (2, view);
}

void
ApplicationWindow::setListViewSize (const QString & caption,
				    const QString & size)
{
  QListViewItem *it =
    listview->findItem (caption, 0, Qt::ExactMatch | Qt::CaseSensitive);
  if (it)
    it->setText (3, size);
}

QString
ApplicationWindow::listViewDate (const QString & caption)
{
  QListViewItem *it = listview->findItem (caption, 0, ExactMatch | CaseSensitive);
  if (it)
    return it->text (4);
  else
    return "";
}

void
ApplicationWindow::updateTableNames (const QString & oldName,
				     const QString & newName)
{
  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      if (w->isA ("MultiLayer"))
	{
	  QWidgetList *gr_lst = ((MultiLayer *) w)->graphPtrs ();
	  for (Graph *g = (Graph *)gr_lst->first(); g; g = (Graph *)gr_lst->next())
	    g->updateTableName(oldName,newName);
	}
      else if (w->isA ("Graph3D"))
	{
	  ((Graph3D *) w)->updateTableName(oldName,newName);
	}
      else if (w->isA ("TableStatistics"))
	{
	  Table *t = (Table *) w;
	  QString name = t->getName();
	  QString label = t->windowLabel();
	  if(name.startsWith(oldName+"-"))
	    {
	      name.replace(oldName,newName);
	      label.replace(oldName,newName);
	      t->setName(name);
	      t->setWindowLabel(label);
	    }
	}
    }
  delete windows;
}

void
ApplicationWindow::updateColNames (const QString & oldName,
				   const QString & newName)
{
  //qDebug("updateColNames from %s to %s",oldName.ascii(),newName.ascii());

  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      if (w->isA ("MultiLayer"))
	{
	  QWidgetList *gr_lst = ((MultiLayer *) w)->graphPtrs ();
	  for (Graph *g = (Graph *)gr_lst->first(); g; g = (Graph *)gr_lst->next())
            g->updateColName(oldName, newName);
	}
      else if (w->isA ("Graph3D"))
	{
          ((Graph3D *) w)->updateColName(oldName,newName);
	}
      // no need to update TableStatistics.
      // It is done by the signal
    }
  delete windows;
}

void
ApplicationWindow::changeMatrixName (const QString & oldName,
				     const QString & newName)
{
  QWidget *w;
  QWidgetList *lst = windowsList ();
  for (w = lst->first (); w; w = lst->next ())
    {
      if (w->isA ("MultiLayer")) // Spectrogram
      {
        QWidgetList *g_lst = ((MultiLayer *) w)->graphPtrs ();
	for (Graph *g = (Graph *)g_lst->first(); g; g = (Graph *)g_lst->next())
	  for (int i = 0; i < g->curves (); i++)
	  {
	    Spectrogram *sp = (Spectrogram *) g->curve (i);
	    if (sp && sp->rtti () == QwtPlotItem::Rtti_PlotSpectrogram
	        && sp->title ().text () == oldName)
	      sp->setTitle( newName );
	  }
      }
      else if (w->isA ("Graph3D"))
	{
	  ((Graph3D *) w)->updateMatrixName (oldName, newName);
	}
    }
  delete lst;
}

void
ApplicationWindow::remove3DMatrixPlots (Matrix * m)
{
  if (!m)
    return;

  QApplication::setOverrideCursor (waitCursor);

  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      if (w->isA ("Graph3D") && ((Graph3D *) w)->getMatrix () == m)
	((Graph3D *) w)->clearData ();
      else if (w->isA ("MultiLayer"))
      {
	QWidgetList *lst = ((MultiLayer *) w)->graphPtrs ();
	for (Graph *g = (Graph *)lst->first(); g; g = (Graph *)lst->next ())
	  for (int i = 0; i < g->curves (); i++)
	    {
	    Spectrogram *sp = (Spectrogram *) g->curve (i);
	    if (sp && sp->rtti () == QwtPlotItem::Rtti_PlotSpectrogram
	      && sp->matrix () == m)
	      g->removeCurve (i, true);
	    }
      }
    }
  delete windows;
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::update3DMatrixPlots (QWidget * window)
{
  Matrix *m = (Matrix *) window;
  if (!m)
    return;

  QApplication::setOverrideCursor (waitCursor);

  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      if (w->isA ("Graph3D") && ((Graph3D *) w)->getMatrix () == m)
	((Graph3D *) w)->updateMatrixData ();
      else if (w->isA ("MultiLayer"))
	{
	  QWidgetList *lst = ((MultiLayer *) w)->graphPtrs ();
	  for(Graph *g = (Graph *)lst->first(); g; g = (Graph *)lst->next())
	    for (int i = 0; i < g->curves (); i++)
	      {
		Spectrogram *sp = (Spectrogram *) g->curve (i);
		if (sp && sp->rtti () == QwtPlotItem::Rtti_PlotSpectrogram
		    && sp->matrix () == m)
		  sp->updateData (m);
	      }
	}
    }

  delete windows;
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::add3DData ()
{
  if ((int) tableWindows.count () <= 0)
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no tables available in this project.</h4>"
			     "<p><h4>Please create a table and try again!</h4>"));
      return;
    }

  QStringList zColumns = columnsList (Table::Z);
  if ((int) zColumns.count () <= 0)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Warning"),
			     tr
			     ("There are no available columns with plot designation set to Z!"));
      return;
    }

  analysisDialog *ad =
    new analysisDialog (0, "Column :", "analysisDialog", TRUE,
			WDestructiveClose);
  connect (ad, SIGNAL (options (const QString &)), this,
	   SLOT (insertNew3DData (const QString &)));
  ad->setCaption (tr ("QtiPlot - Choose data set"));
  ad->setCurveNames (zColumns,-1);
  ad->showNormal ();
  ad->setActiveWindow ();
}

void
ApplicationWindow::change3DData ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      int index = -1;
      QStringList zColList = columnsList (Table::Z);
      QString assoc = ((Graph3D *)ws->activeWindow())->formula();
      QStringList lst =  QStringList::split (',', assoc, false);
      int pos;
      if(lst.count()==3 && (pos=lst[2].find("<Z>"))!=-1)
	{
	  QString zColName = lst[2].left( pos );
	  for(int i=0;i<(int)zColList.count();i++)
	    if(zColList[i]==zColName)
	      index = i;
	}

      analysisDialog *ad =
	new analysisDialog (0, "Column :", "analysisDialog", TRUE,
			    WDestructiveClose);
      connect (ad, SIGNAL (options (const QString &)), this,
	       SLOT (change3DData (const QString &)));

      ad->setCaption (tr ("QtiPlot - Choose data set"));
      ad->setCurveNames (zColList, index);
      ad->showNormal ();
      ad->setActiveWindow ();
    }
}

void
ApplicationWindow::change3DMatrix ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      int index = -1;
      QStringList matlst = matrixNames();
      Matrix *m = ((Graph3D *)ws->activeWindow())->getMatrix();
      if(m)
	{
	  QString matname = m->getName();
	  for(int i=0;i<(int)matlst.count();i++)
	    if(matlst[i]==matname)
	      index = i;
	}

      analysisDialog *ad =
	new analysisDialog (0, "Matrix :", "analysisDialog", TRUE,
			    WDestructiveClose);
      connect (ad, SIGNAL (options (const QString &)), this,
	       SLOT (change3DMatrix (const QString &)));
      ad->setCaption (tr ("QtiPlot - Choose matrix to plot"));
      ad->setCurveNames (matlst, index);
      ad->showNormal ();
      ad->setActiveWindow ();
    }
}

void
ApplicationWindow::change3DMatrix (const QString & matrix_name)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      ((Graph3D *) ws->activeWindow ())->changeMatrix (matrix (matrix_name));
      emit modified ();
    }
}

void
ApplicationWindow::add3DMatrixPlot ()
{
  QStringList matrices = matrixNames ();
  if ((int) matrices.count () <= 0)
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no matrices available in this project.</h4>"
			     "<p><h4>Please create a matrix and try again!</h4>"));
      return;
    }

  analysisDialog *ad =
    new analysisDialog (0, tr ("Matrix") + " :", "analysisDialog", TRUE,
			WDestructiveClose);
  connect (ad, SIGNAL (options (const QString &)), this,
	   SLOT (insert3DMatrixPlot (const QString &)));

  ad->setCaption (tr ("QtiPlot - Choose matrix to plot"));
  ad->setCurveNames (matrices,-1);
  ad->showNormal ();
  ad->setActiveWindow ();
}

void
ApplicationWindow::insert3DMatrixPlot (const QString & matrix_name)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      ((Graph3D *) ws->activeWindow ())->addMatrixData (matrix (matrix_name));
      emit modified ();
    }
}

void
ApplicationWindow::insertNew3DData (const QString & colName)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      ((Graph3D *) ws->activeWindow ())->insertNewData (table (colName),
							colName);
      emit modified ();
    }
}

void
ApplicationWindow::change3DData (const QString & colName)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      ((Graph3D *) ws->activeWindow ())->changeDataColumn (table (colName),
							   colName);
      emit modified ();
    }
}

void
ApplicationWindow::editSurfacePlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      Graph3D *g = (Graph3D *) ws->activeWindow ();

      sDialog *sd =
	new sDialog (this, "sDialog", TRUE, WStyle_Tool | WDestructiveClose);
      connect (sd,
	       SIGNAL (options
		       (const QString &, double, double, double, double,
			double, double)), g,
	       SLOT (insertFunction
		     (const QString &, double, double, double, double, double,
		      double)));
      connect (sd, SIGNAL (clearFunctionsList ()), this,
	       SLOT (clearSurfaceFunctionsList ()));

      sd->insertFunctionsList (surfaceFunc);
      if (g->hasData ())
	{
	  sd->setFunction (g->formula ());
	  sd->setLimits (g->xStart (), g->xStop (), g->yStart (),
			 g->yStop (), g->zStart (), g->zStop ());
	}
      sd->show ();
      sd->setActiveWindow ();
    }
}

void
ApplicationWindow::newSurfacePlot ()
{
  sDialog *sd =
    new sDialog (this, "sDialog", TRUE, WStyle_Tool | WDestructiveClose);
  connect (sd,
	   SIGNAL (options
		   (const QString &, double, double, double, double, double,
		    double)), this, SLOT (newPlot3D (const QString &, double,
						     double, double, double,
						     double, double)));
  connect (sd, SIGNAL (clearFunctionsList ()), this,
	   SLOT (clearSurfaceFunctionsList ()));

  sd->insertFunctionsList (surfaceFunc);
  sd->show ();
  sd->setActiveWindow ();
}

Graph3D *
ApplicationWindow::newPlot3D (const QString & formula, double xl, double xr,
			      double yl, double yr, double zl, double zr)
{
  QString label = generateUniqueName (tr ("Graph"));
  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  customPlot3D (plot);

  plot->addFunction (formula, xl, xr, yl, yr, zl, zr, 0);
  plot->resize (500, 450);
  //plot->setCaption (label);
  plot->setName (label);
  plot->update ();

  initPlot3D (plot);

  emit modified ();
  return plot;
}

void
ApplicationWindow::updateSurfaceFuncList (const QString & s)
{
  surfaceFunc.remove (s);
  surfaceFunc.push_front (s);
  while ((int) surfaceFunc.size () > 10)
    surfaceFunc.pop_back ();
}

Graph3D *
ApplicationWindow::newPlot3D (const QString & caption,
			      const QString & formula, double xl, double xr,
			      double yl, double yr, double zl, double zr,
			      int scaletype)
{
  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  plot->addFunction (formula, xl, xr, yl, yr, zl, zr, scaletype);
  plot->update ();

  QString label = caption;
  if (alreadyUsedName (label))
    label = generateUniqueName (tr ("Graph"));

  //plot->setCaption (label);
  plot->setName (label);
  initPlot3D (plot);
  return plot;
}

Graph3D *
ApplicationWindow::dataPlot3D (Table * table, const QString & colName)
{
  QApplication::setOverrideCursor (waitCursor);

  QString label = generateUniqueName (tr ("Graph"));
  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  customPlot3D (plot);

  plot->addData (table, colName);
  plot->resize (500, 450);
  //plot->setCaption (label);
  plot->setName (label);

  plot->update ();
  initPlot3D (plot);

  emit modified ();
  QApplication::restoreOverrideCursor ();
  return plot;
}

Graph3D *
ApplicationWindow::dataPlot3D (const QString & caption,
			       const QString & formula, double xl, double xr,
			       double yl, double yr, double zl, double zr,
			       int scaletype)
{
  //qDebug("formula = %s",formula.ascii());

  QStringList lst = QStringList::split( ',', formula, false );
  QString tableName = lst[0].section( '_', 0, 0 );
  Table *w = table (tableName);
  if (!w)
    return 0;

  int pos = lst[0].findRev('(');
  QString xCol = lst[0].left(pos);
  //qDebug("xCol = %s",xCol.ascii());

  pos = lst[1].findRev('(');
  QString yCol = lst[1].left(pos);
  //qDebug("yCol = %s",yCol.ascii());

  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  plot->addData (w, xCol, yCol, xl, xr, yl, yr, zl, zr, scaletype);
  plot->update ();

  QString label = caption;
  if (alreadyUsedName (label))
    label = generateUniqueName (tr ("Graph"));

  //plot->setCaption (label);
  plot->setName (label);
  initPlot3D (plot);

  return plot;
}

//plot ribbon from the plot wizard
Graph3D *
ApplicationWindow::dataPlot3D (const QString & formula)
{
  //qDebug("formula = %s",formula.ascii());

  QApplication::setOverrideCursor (waitCursor);

  QStringList lst = QStringList::split( ':', formula, false );
  QString caption = lst[0] + "_";
  Table *w = (Table *) table (caption);
  if (!w)
    {
      QApplication::restoreOverrideCursor ();
      return 0;
    }

  QStringList cols = QStringList::split( ',', lst[1], false );

  QString xColName = caption + cols[0].section('<',0,0);
  //qDebug("xColName = %s",xColName.ascii());

  QString yColName = caption + cols[1].section('<',0,0);
  //qDebug("yColName = %s",yColName.ascii());

  QString label = generateUniqueName (tr ("Graph"));

  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  customPlot3D (plot);

  plot->addData (w, xColName, yColName);
  plot->resize (500, 450);
  //plot->setCaption (label);
  plot->setName (label);

  plot->update ();
  initPlot3D (plot);

  emit modified ();
  QApplication::restoreOverrideCursor ();
  return plot;
}

Graph3D *
ApplicationWindow::dataPlotXYZ (Table * table, const QString & zColName,
				int type)
{
  QApplication::setOverrideCursor (waitCursor);

  QString label = generateUniqueName (tr ("Graph"));
  int zCol = table->colIndex (zColName);
  int yCol = table->colY (zCol);
  int xCol = table->colX (zCol);

  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  customPlot3D (plot);

  plot->addData (table, xCol, yCol, zCol, type);
  plot->resize (500, 450);
  //plot->setCaption (label);
  plot->setName (label);

  plot->update ();
  initPlot3D (plot);

  emit modified ();
  QApplication::restoreOverrideCursor ();
  return plot;
}

Graph3D *
ApplicationWindow::dataPlotXYZ (const QString & caption,
				const QString & formula, double xl, double xr,
				double yl, double yr, double zl, double zr,
				int scaletype)
{
  //qDebug("formula = %s",formula.ascii());

  QStringList lst = QStringList::split( ',', formula, false );

  QString wCaption = lst[0].section('_',0,0);

  Table *w = table (wCaption);
  if (!w)
    return 0;

  int pos = lst[0].findRev('(');
  QString xColName = lst[0].left(pos);
  //qDebug("xColName = %s",xColName.ascii());

  pos = lst[1].findRev('(');
  QString yColName = lst[1].left(pos);
  //qDebug("yColName = %s",yColName.ascii());

  pos = lst[2].findRev('(');
  QString zColName = lst[2].left(pos);
  //qDebug("zColName = %s",zColName.ascii());

  int xCol = w->colIndex (xColName);
  int yCol = w->colIndex (yColName);
  int zCol = w->colIndex (zColName);

  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  customPlot3D(plot);

  plot->addData (w, xCol, yCol, zCol, xl, xr, yl, yr, zl, zr, scaletype);
  plot->update ();

  QString label = caption;
  if (alreadyUsedName (label))
    label = generateUniqueName (tr ("Graph"));

  //plot->setCaption (label);
  plot->setName (label);
  initPlot3D (plot);
  return plot;
}

//plot 3D data from plot wizard string
Graph3D *
ApplicationWindow::dataPlotXYZ (const QString & formula)
{
  QApplication::setOverrideCursor (waitCursor);

  QStringList lst = QStringList::split( ':', formula, false );
  QString caption = lst[0] + "_";
  Table *w = (Table *) table (caption);
  if (!w)
    {
      QApplication::restoreOverrideCursor ();
      return 0;
    }

  QStringList cols = QStringList::split( ',', lst[1], false );

  QString xColName = caption + cols[0].section('<',0,0);
  //qDebug("xColName = %s",xColName.ascii());

  QString yColName = caption + cols[1].section('<',0,0);
  //qDebug("yColName = %s",yColName.ascii());

  QString zColName = caption + cols[2].section('<',0,0);
  //qDebug("zColName = %s",zColName.ascii());

  int xCol = w->colIndex (xColName);
  int yCol = w->colIndex (yColName);
  int zCol = w->colIndex (zColName);

  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  customPlot3D (plot);

  plot->addData (w, xCol, yCol, zCol, 1);
  plot->resize (500, 450);

  QString label = generateUniqueName (tr ("Graph"));
  //plot->setCaption (label);
  plot->setName (label);
  plot->update ();
  initPlot3D (plot);

  QApplication::restoreOverrideCursor ();
  return plot;
}

void
ApplicationWindow::customPlot3D (Graph3D * plot)
{
  plot->setDataColors (QColor (plot3DColors[4]), QColor (plot3DColors[0]));
  plot->updateColors (QColor (plot3DColors[2]), QColor (plot3DColors[6]),
		      QColor (plot3DColors[5]), QColor (plot3DColors[1]),
		      QColor (plot3DColors[7]), QColor (plot3DColors[3]));

  plot->setResolution (plot3DResolution);
  plot->showColorLegend (showPlot3DLegend);
  plot->setSmoothMesh (smooth3DMesh);
  plot->setOrtho (orthogonal3DPlots);
  if (showPlot3DProjection)
    plot->setFloorData ();

  plot->setNumbersFont (plot3DNumbersFont);
  plot->setAxisLabelFont (0,plot3DAxesFont);
  plot->setAxisLabelFont (1,plot3DAxesFont);
  plot->setAxisLabelFont (2,plot3DAxesFont);
  plot->setTitleFont (plot3DTitleFont);
}

void
ApplicationWindow::initPlot3D (Graph3D * plot)
{
  connectSurfacePlot (plot);

  plot->setIcon (QPixmap (trajectory_xpm));
  plot->show ();
  plot->setFocus ();
  plot->setIgnoreFonts(false);

  addListViewItem (plot);
  current_folder->addWindow (plot);
  plot->setFolder (current_folder);

  //  if (!plot3DTools->isVisible ())
  //  plot3DTools->show ();

  //if (!plot3DTools->isEnabled ())
  //  plot3DTools->setEnabled (TRUE);

  //customMenu ((QWidget *) plot);
  //customToolBars ((QWidget *) plot);
}

Matrix *
ApplicationWindow::importImage ()
{
  QStringList list = QImage::inputFormatList ();
  QString filter = "Images (*.jpg *.JPG ", aux;
  int i;
  for (i = 0; i < (int) list.count (); i++)
    {
      aux = "*." + (list[i]).lower () + " *." + list[i] + " ";
      filter += aux;
    }
  filter += ");;";

  aux = "*.jpg *.JPG;; ";
  filter += aux;
  for (i = 0; i < (int) list.count (); i++)
    {
      aux = "*." + (list[i]).lower () + " *." + list[i] + ";;";
      filter += aux;
    }

  QString fn = QFileDialog::getOpenFileName (imagesDirPath, filter, this, 0,
					     tr
					     ("QtiPlot - Import image from file"),
					     0, TRUE);
  if (!fn.isEmpty ())
    {
      QFileInfo fi (fn);
      imagesDirPath = fi.dirPath (true);
      return importImage (fn);
    }
  else
    return 0;
}

Matrix *
ApplicationWindow::importImage (const QString & fn)
{
  QApplication::setOverrideCursor (waitCursor);
  QPixmap photo;
  if (fn.endsWith (".jpg", false))
    photo.load (fn, "JPEG", QPixmap::Auto);
  else
    {
      QStringList lst = QImage::inputFormatList ();
      for (int i = 0; i < (int) lst.count (); i++)
	{
	  if (fn.endsWith ("." + lst[i], false))
	    {
	      photo.load (fn, lst[i], QPixmap::Auto);
	      break;
	    }
	}
    }

  Matrix *m = createIntensityMatrix (photo);
  m->setWindowLabel (fn);
  m->setCaptionPolicy (myWidget::Both);
  setListViewLabel (m->getName(), fn);

  QApplication::restoreOverrideCursor ();
  return m;
}

void
ApplicationWindow::loadImage ()
{
  QStringList list = QImage::inputFormatList ();
  QString filter = "Images (*.jpg *.JPG ", aux;
  int i;
  for (i = 0; i < (int) list.count (); i++)
    {
      aux = "*." + (list[i]).lower () + " *." + list[i] + " ";
      filter += aux;
    }
  filter += ");;";

  aux = "*.jpg *.JPG;; ";
  filter += aux;
  for (i = 0; i < (int) list.count (); i++)
    {
      aux = "*." + (list[i]).lower () + " *." + list[i] + ";;";
      filter += aux;
    }

  QString fn = QFileDialog::getOpenFileName (imagesDirPath, filter, this, 0,
					     tr
					     ("QtiPlot - Load image from file"),
					     0, TRUE);
  if (!fn.isEmpty ())
    {
      loadImage (fn);
      QFileInfo fi (fn);
      imagesDirPath = fi.dirPath (true);
    }
}

void
ApplicationWindow::loadImage (const QString & fn)
{
  bool fmax=false;
  if(aw&&(aw->status()==myWidget::Maximized)){
    fmax=true;
    aw->showNormal();
  }

  QApplication::setOverrideCursor (waitCursor);
  QPixmap photo;
  if (fn.endsWith (".jpg", false))
    photo.load (fn, "JPEG", QPixmap::Auto);
  else
    {
      QStringList lst = QImage::inputFormatList ();
      for (int i = 0; i < (int) lst.count (); i++)
	{
	  if (fn.endsWith ("." + lst[i], false))
	    {
	      photo.load (fn, lst[i], QPixmap::Auto);
	      break;
	    }
	}
    }

  MultiLayer *plot = multilayerPlot (generateUniqueName (tr ("Graph")));
  plot->setWindowLabel (fn);
  plot->setCaptionPolicy (myWidget::Both);
  setListViewLabel (plot->getName(), fn);

  if (plot->height () - 20 > photo.height ())
    plot->setGeometry (0, 0, plot->width (), photo.height () + 20);

  plot->showNormal ();

  Graph *g = plot->addLayer (0, 0, plot->width (), plot->height () - 20);
  g->setAxisTitleInverted(QwtPlot::yRight,flipRightAxisTitle);

  g->setTitle ("");
  QMemArray < bool > axesOn (4);
  for (int j = 0; j < 4; j++)
    axesOn[j] = FALSE;
  g->enableAxes (axesOn);

  //g->removeLegend ();
  g->insertImageMarker (photo, fn);

  g->setIgnoreResizeEvents (!autoResizeLayers);

  //plot->connectLayer (g);

  if(fmax) plot->showMaximized();

  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::polishGraph (Graph * g, int style)
{
  if (style == Graph::VerticalBars || style == Graph::HorizontalBars
      || style == Graph::Histogram)
    {
      QValueList < int >ticksList;
      int majTicksStyle = ScaleDraw::Out;
      ticksList << majTicksStyle << majTicksStyle << majTicksStyle <<
	majTicksStyle;
      g->setMajorTicksType (ticksList);
      g->setMinorTicksType (ticksList);
    }
  if (style == Graph::HorizontalBars)
    {
      g->setAxisTitle (0, tr ("Y Axis Title"));
      g->setAxisTitle (1, tr ("X Axis Title"));
    }
}

MultiLayer *
ApplicationWindow::multilayerPlot (const QString & caption)
{
  //fprintf(stderr,"multilayerPlot\n");
  MultiLayer *g = new MultiLayer ("", ws, 0, WDestructiveClose);
  QString label = caption;
  initMultilayerPlot (g, label.replace ("_", "-"));
  return g;
}

MultiLayer *
ApplicationWindow::newGraph ()
{
  //fprintf(stderr,"ApplicationWindow::newGraph\n");
  bool fmax=false;
  if(aw&&(aw->status()==myWidget::Maximized)){
    fmax=true;
    aw->showNormal(); // important
  }

  MultiLayer *g = multilayerPlot (generateUniqueName (tr ("Graph")));

  activeGraph = g->addLayer ();
  customGraph (activeGraph);
  activeGraph->setIgnoreResizeEvents (!autoResizeLayers);
  activeGraph->newLegend();
  activeGraph->replot ();

  if(fmax) g->showMaximized();

  return g;
}

MultiLayer *
ApplicationWindow::multilayerPlot (Table * w, const QStringList & colList, int style)			  
{ //used when plotting selected columns
  //qDebug("multilayerPlot with columns");

  QApplication::setOverrideCursor (waitCursor);

  bool fmax=false;
  if(aw&&(aw->status()==myWidget::Maximized))
    {
      fmax=true;
      aw->showNormal();
    } 

  MultiLayer *m = new MultiLayer ("", ws, 0, WDestructiveClose);
  m->askOnCloseEvent (confirmClosePlot2D);

  activeGraph = m->insertFirstLayer ();
  if (!activeGraph)
    return 0;
  
  activeGraph->insertCurvesList (w, colList, style, defaultCurveLineWidth,
				 defaultSymbolSize);
  //qDebug("calling customGraph");
  customGraph (activeGraph);
  polishGraph (activeGraph, style);
  activeGraph->newLegend();
  initMultilayerPlot (m, generateUniqueName (tr ("Graph")));

  if(fmax) m->showMaximized();

//the following function must be called last in order to avoid resizing problems
  activeGraph->setIgnoreResizeEvents (!autoResizeLayers);
  emit modified ();
  QApplication::restoreOverrideCursor ();
  return m;
}

MultiLayer *
ApplicationWindow::multilayerPlot (int c, int r, int style)
{ //used when plotting with the panel menu
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return 0;

  Table *w = (Table *) ws->activeWindow ();
  if (!w->valid2DPlot ())
    return 0;

  QStringList list = w->selectedYColumns ();
  if ((int) list.count () < 1)
    {
      QMessageBox::warning (0, tr ("QtiPlot - Plot error"),
			    tr ("Please select a Y column to plot!"));
      return 0;
    }

  bool fmax=false;
  if(aw&&(aw->status()==myWidget::Maximized))
    {
      fmax=true;
      aw->showNormal();
    } 

  int curves = (int) list.count ();
  if (r < 0)
    r = curves;

  MultiLayer *m = new MultiLayer ("", ws, 0, WDestructiveClose);
  m->askOnCloseEvent (confirmClosePlot2D);
  initMultilayerPlot (m, generateUniqueName (tr ("Graph")));

  if(fmax) m->showMaximized();

  int layers = c * r;
  if (curves < layers)
    {
      for (int i = 0; i < curves; i++)
	{
	  activeGraph = m->addLayer ();
	  if (activeGraph)
	    {
	      QStringList lst;
	      lst << list[i];
	      activeGraph->insertCurvesList (w, lst, style,
					     defaultCurveLineWidth,
					     defaultSymbolSize);
	      customGraph (activeGraph);
	      activeGraph->newLegend();
	      // activeGraph->setAutoscaleFonts (false);	//in order to avoid to small fonts
	      activeGraph->setIgnoreResizeEvents (!autoResizeLayers);
	      polishGraph (activeGraph, style);
	    }
	}
    }
  else
    {
      for (int i = 0; i < layers; i++)
	{
	  activeGraph = m->addLayer ();
	  if (activeGraph)
	    {
	      QStringList lst;
	      lst << list[i];
	      activeGraph->insertCurvesList (w, lst, style,
					     defaultCurveLineWidth,
					     defaultSymbolSize);
	      customGraph (activeGraph);
	      activeGraph->newLegend();
	      activeGraph->setIgnoreResizeEvents (!autoResizeLayers);
	      polishGraph (activeGraph, style);
	    }
	}
    }
  m->setRows (r);
  m->setCols (c);
  m->arrangeLayers (false, false, false);

  emit modified ();
  return m;
}

MultiLayer *
ApplicationWindow::multilayerPlot (const QStringList & colList)
{ //used when plotting from wizard
  QApplication::setOverrideCursor (waitCursor);

  bool fmax=false;
  if(aw&&(aw->status()==myWidget::Maximized))
    {
      fmax=true;
      aw->showNormal();
    } 

  MultiLayer *g = new MultiLayer ("", ws, 0, WDestructiveClose);
  Graph *ag = g->insertFirstLayer ();
  customGraph (ag);
  polishGraph (ag, defaultCurveStyle);
  int curves = (int) colList.count ();

//We rearrange the list so that the error bars are placed at the end
  QStringList lst = QStringList ();
  for (int j = 0; j < curves; j++)
  {
    if (!colList[j].contains ("(yErr)") && !colList[j].contains ("(xErr)"))
      lst << colList[j];
  }
  int errorBars = 0;
  for (int k = 0; k < curves; k++)
  {
    if (colList[k].contains ("(yErr)") || colList[k].contains ("(xErr)"))
    {
      errorBars++;
      lst << colList[k];
    }
  }

  for (int i = 0; i < curves; i++)
  {
    QStringList clst = QStringList::split(':',lst[i],false);
    QString caption = clst[0] + "_";
    Table *w = (Table *) table (caption);

    QStringList cols = QStringList::split(',',clst[1],false);

    int pos = cols[0].findRev('(');
    QString xColName = caption + cols[0].left(pos);

    pos = cols[1].findRev ('(');
    QString yColName = caption + cols[1].left(pos);

    if (cols.count() > 2)
    {
      int errType;
      if ( (pos = cols[2].find("(yErr)")) != -1 )
        errType = QwtErrorPlotCurve::Vertical;
      else if ( (pos = cols[2].find("(xErr)")) != -1 )
        errType = QwtErrorPlotCurve::Horizontal;
      else 
        qDebug("Error in multilayerPlot");
      QString errColName = caption + cols[2].left(pos);
      ag->addErrorBars (w, xColName, yColName, w, errColName, errType);
    }
    else if (ag->insertCurve (w, xColName, yColName, defaultCurveStyle))
    {
      curveLayout cl =
        ag->initCurveLayout (i, curves - errorBars, defaultCurveStyle);
      cl.lWidth = defaultCurveLineWidth;
      cl.sSize = defaultSymbolSize;
      ag->updateCurveLayout (i, &cl);
    }
  }
  ag->newLegend();
  ag->updatePlot();
  initMultilayerPlot (g, generateUniqueName (tr ("Graph")));

  if(fmax) g->showMaximized();

  ag->setIgnoreResizeEvents (!autoResizeLayers);
  emit modified ();
  QApplication::restoreOverrideCursor ();
  return g;
}

void
ApplicationWindow::initMultilayerPlot (MultiLayer * m, const QString & name)
{
  connectMultilayerPlot (m);

  QString label = name;
  if (alreadyUsedName (label))
    label = generateUniqueName (tr ("Graph"));

  //m->setCaption (label);
  m->setName (label);
  m->setIcon (QPixmap (graph_xpm));

  m->blockSignals(true);
  m->showNormal();
  m->blockSignals(false);

  m->setFocus ();

  addListViewItem (m);
  current_folder->addWindow (m);
  m->setFolder (current_folder);
}

void
ApplicationWindow::customizeTables (const QColor & bgColor,
				    const QColor & textColor,
				    const QColor & headerColor,
				    const QFont & textFont,
				    const QFont & headerFont)
{
  if (tableBkgdColor == bgColor && tableTextColor == textColor &&
      tableHeaderColor == headerColor && tableTextFont == textFont &&
      tableHeaderFont == headerFont)
    return;

  tableBkgdColor = bgColor;
  tableTextColor = textColor;
  tableHeaderColor = headerColor;
  tableTextFont = textFont;
  tableHeaderFont = headerFont;

  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    if (w->inherits ("Table"))
      customTable ((Table *) w);
 
  delete windows;
}

void
ApplicationWindow::customTable (Table * w)
{
  w->setBackgroundColor (tableBkgdColor);
  w->setTextColor (tableTextColor);
  w->setHeaderColor (tableHeaderColor);
  w->setTextFont (tableTextFont);
  w->setHeaderFont (tableHeaderFont);
}

void
ApplicationWindow::customGraph (Graph * g)
{
  if (!g->isPiePlot ())
    {
      if (allAxesOn)
	{
	  QMemArray < bool > axesOn (QwtPlot::axisCnt);
	  axesOn.fill (true);
	  g->enableAxes (axesOn);
	  g->updateSecondaryAxis (QwtPlot::xTop);
	  g->updateSecondaryAxis (QwtPlot::yRight);
	}

      QValueList < int >ticksList;
      ticksList << majTicksStyle << majTicksStyle << majTicksStyle <<
	majTicksStyle;
      g->setMajorTicksType (ticksList);
      ticksList.clear ();
      ticksList << minTicksStyle << minTicksStyle << minTicksStyle <<
	minTicksStyle;
      g->setMinorTicksType (ticksList);

      g->setTicksLength (minTicksLength, majTicksLength);
      g->setAxesLinewidth (axesLineWidth);
      g->drawAxesBackbones (drawBackbones);
    }

  g->initFonts (plotAxesFont, plotNumbersFont);
  g->setTextMarkerDefaults (legendFrameStyle, plotLegendFont, legendTextColor,
			    legendBackground);
  g->customLegend ();

  g->setArrowDefaults (defaultArrowLineWidth, defaultArrowColor,
		       defaultArrowLineStyle, defaultArrowHeadLength,
		       defaultArrowHeadAngle, defaultArrowHeadFill);

  g->initTitle (titleOn, plotTitleFont);
  g->drawCanvasFrame (canvasFrameOn, canvasFrameWidth);
  g->plotWidget ()->setMargin (defaultPlotMargin);
  g->changeSpacing(defaultPlotSpacing);
  g->enableAutoscaling (autoscale2DPlots);
  g->setAutoscaleFonts (autoScaleFonts);
  g->setAxisTitleInverted(QwtPlot::yRight,flipRightAxisTitle); 
  g->setAutoScaleFrameWidth(autoScaleFrameWidth);
  g->setAutoScaleGridWidth(autoScaleGridWidth);
  g->setAutoScaleLineWidth(autoScaleLineWidth);
}

void
ApplicationWindow::newWrksheetPlot (const QString & caption, int r, int c,
				    const QString & text)
{
  Table *w = newTable (caption, r, c, text);
  MultiLayer *plot =
    multilayerPlot (w, QStringList (w->getName() + "_intensity"), 0);
  Graph *g = (Graph *) plot->activeGraph ();
  if (g)
    {
      g->setTitle ("");
      g->setXAxisTitle (tr ("pixels"));
      g->setYAxisTitle (tr ("pixel intensity (a.u.)"));
    }
}

/*
*used when importing an ASCII file
*/
Table *
ApplicationWindow::newTable (const QString & fname, const QString & sep,
			     int lines, bool renameCols, bool stripSpaces,
			     bool simplifySpaces)
{
  Table *w = new Table (scriptEnv, fname, sep, lines, renameCols, stripSpaces,
			simplifySpaces, "", ws, 0, WDestructiveClose);
  if (w)
    {
      QFileInfo fi(fname);
      QString basename = fi.baseName(true);
      basename.replace("_", "-");
      //initTable (w, generateUniqueName (tr ("Table")));
      initTable (w, generateUniqueName (basename,false));
      w->showNormal();
    }
  return w;
}

/*
*creates a new empty table
*/
Table *
ApplicationWindow::newTable ()
{
  Table *w = new Table (scriptEnv, 30, 2, "", ws, 0, WDestructiveClose);
  if(w)
    {
      initTable (w, generateUniqueName (tr ("Table")));
      w->showNormal();
    }
  return w;
}

/*
*used when opening a project file
*/
Table *
ApplicationWindow::newTable (const QString & caption, int r, int c)
{
  Table *w = new Table (scriptEnv, r, c, "", ws, 0, WDestructiveClose);
  initTable (w, caption);
  if (w->getName() != caption)	//the table was renamed
    {
      renamedTables << caption << w->getName();

      QMessageBox::warning (this, tr ("QtiPlot - Renamed Window"),
			    tr
			    ("The table '%1' already exists. It has been renamed '%2'.").arg (caption).arg (w->getName()));
    }
  w->blockSignals(true);
  w->showNormal();
  w->blockSignals(false);
  return w;
}


Table *
ApplicationWindow::newTable (const QString & caption, int r, int c,
			     const QString & text)
{
  QStringList lst = QStringList::split ('\t', caption, false);
  Table *w = new Table (scriptEnv, r, c, lst[1], ws, 0, WDestructiveClose);

  QStringList rows = QStringList::split ('\n', text, FALSE);
  QString rlist = rows[0];
  QStringList list = QStringList::split ('\t', rlist, TRUE);
  w->setHeader (list);

  for (int i = 0; i < r; i++)
    {
      rlist = rows[i + 1];
      list = QStringList::split ('\t', rlist, TRUE);
      for (int j = 0; j < c; j++)
	w->setText (i, j, list[j]);
    }

  initTable (w, lst[0]);
  w->setCaptionPolicy (myWidget::Both);
  w->show();
  return w;
}

/*
*used to return the result of an analysis operation
*/
Matrix *
ApplicationWindow::newHiddenMatrix(const QString &title, int r, int c)
{
  Matrix *m = new Matrix (scriptEnv, r, c, "", ws, 0, WDestructiveClose);
  QString caption = generateUniqueName(title);
  initMatrix (m, caption);
  m->setCaptionPolicy (myWidget::Both);
  outWindows->append (m);
  m->setHidden ();
  return m;
}

/*
*used to return the result of an analysis operation
*/
Table *
ApplicationWindow::newHiddenTable (const QString & caption, int r, int c,
				   int prec, const QString & text)
{
  QStringList lst = QStringList::split ('\t', caption, false);
  Table *w = new Table (scriptEnv, r, c, lst[1], 0, 0, WDestructiveClose);

  QStringList rows = QStringList::split ('\n', text, FALSE);
  QString rlist = rows[0];
  QStringList list = QStringList::split ('\t', rlist, TRUE);
  w->setHeader (list);

  for(int col=0; col<c; col++)
    w->setColNumericFormat( 0, prec, col );

  for (int i = 0; i < r; i++)
    {
      rlist = rows[i + 1];
      list = QStringList::split ('\t', rlist, TRUE);
      for (int j = 0; j < c; j++)
	w->setText (i, j, list[j]);
    }

  initTable (w, lst[0]);
  w->setCaptionPolicy (myWidget::Both);
  outWindows->append (w);
  w->setHidden ();
  return w;
}

Table *
ApplicationWindow::newHiddenTable (const QString & name,
				   const QString & legend, int r, int c,
				   int prec)
{
  Table *w = new Table (scriptEnv, r, c, legend, 0, 0, WDestructiveClose);
  initTable (w, name);
  for(int col=0; col<c; col++)
    w->setColNumericFormat( 0, prec, col );
  w->setCaptionPolicy (myWidget::Both);
  outWindows->append (w);
  w->setHidden ();
  return w;
}

void
ApplicationWindow::initTable (Table * w, const QString & caption)
{
  connectTable (w);
  customTable (w);

  QString name = caption;
  name = name.replace ("_", "-");

  if (alreadyUsedName (name))
    name = generateUniqueName (tr ("Table"));

  tableWindows << name;
  //  w->setCaption (name);
  w->setName (name);
  w->setIcon (QPixmap (worksheet_xpm));
  w->setSpecifications (w->saveToString (windowGeometryInfo (w)));

  addListViewItem (w);
  current_folder->addWindow (w);
  w->setFolder (current_folder);

  emit modified ();
}

void
ApplicationWindow::showHistogramTable (const QString & caption, int r, int c,
				       const QString & text)
{
  Table *w = newTable (caption, r, c, text);
  w->showMaximized ();
}

/*
 * !creates a new table with type statistics on target columns/rows of table base
 */
TableStatistics *
ApplicationWindow::newTableStatistics (Table * base, int type,
				       QValueList < int >target,
				       const QString & caption)
{
  TableStatistics *s =
    new TableStatistics (scriptEnv, ws, base, (TableStatistics::Type) type,
			 target);
  if (caption.isEmpty ())
    initTable (s, s->getName());
  else
    initTable (s, caption);

  connect (base, SIGNAL (modifiedData (Table *, const QString &)),
	   s, SLOT (update (Table *, const QString &)));
  connect (base, SIGNAL (changedColHeader (const QString &, const QString &)),
	   s, SLOT (renameCol (const QString &, const QString &)));
  connect (base, SIGNAL (removedCols (const QStringList &)),
	   s, SLOT (removeCols (const QStringList &)));

  // added by S.Yamamoto
  connect (base, SIGNAL (closedWindow(myWidget *)), 
	   s, SLOT (baseClosed(myWidget *)) );
  //
  s->show();
  return s;
}

/*
*creates a new empty note window
*/
Note *
ApplicationWindow::newNote (const QString & caption)
{
  Note *m = new Note (scriptEnv, "", ws, 0, WDestructiveClose);
  if (caption.isEmpty ())
    initNote (m, generateUniqueName (tr ("Notes")));
  else
    initNote (m, caption);
  m->show();
  return m;
}

void
ApplicationWindow::initNote (Note * m, const QString & caption)
{
  QString name = caption;
  //  qDebug("name = %s",name.ascii());
  if (name.isEmpty () || alreadyUsedName (name))
    name = generateUniqueName (tr ("Notes"));

  //  m->setCaption (name);
  m->setName (name);
  m->setIcon (QPixmap (note_xpm));
  m->askOnCloseEvent (confirmCloseNotes);

  addListViewItem (m);
  current_folder->addWindow (m);
  m->setFolder (current_folder);

  connect (m->textWidget (), SIGNAL (undoAvailable (bool)), actionUndo,
	   SLOT (setEnabled (bool)));
  connect (m->textWidget (), SIGNAL (redoAvailable (bool)), actionRedo,
	   SLOT (setEnabled (bool)));
  connect (m, SIGNAL (modifiedWindow (QWidget *)), this,
	   SLOT (modifiedProject (QWidget *)));
  connect (m, SIGNAL (closedWindow (myWidget *)), this,
	   SLOT (closeWindow (myWidget *)));
  connect (m, SIGNAL (hiddenWindow (myWidget *)), this,
	   SLOT (hideWindow (myWidget *)));
  connect (m, SIGNAL (statusChanged (myWidget *)), this,
	   SLOT (updateWindowStatus (myWidget *)));
  connect (m, SIGNAL (showTitleBarMenu ()), this,
	   SLOT (showWindowTitleBarMenu ()));

  emit modified ();
}

/*
*creates a new empty matrix
*/
Matrix *
ApplicationWindow::newMatrix ()
{
  Matrix *m = new Matrix (scriptEnv, 32, 32, "", ws, 0, WDestructiveClose);
  QString caption = generateUniqueName (tr ("Matrix"));
  initMatrix (m, caption);
  m->show();
  return m;
}

/*
*used when opening a project file
*/
Matrix *
ApplicationWindow::newMatrix (const QString & caption, int r, int c)
{
  Matrix *w = new Matrix (scriptEnv, r, c, "", ws, 0, WDestructiveClose);
  initMatrix (w, caption);
  if (w->getName() != caption)	//the matrix was renamed
    renamedTables << caption << w->getName();

  w->show();
  return w;
}

void
ApplicationWindow::transposeMatrix ()
{
  Matrix *m = (Matrix *) ws->activeWindow ();
  if (!m)
    return;

  QApplication::setOverrideCursor (waitCursor);
  m->transpose ();
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::matrixFFT()
{
  Matrix *m = (Matrix *) ws->activeWindow ();
  if (!m)
    return;

  QApplication::setOverrideCursor (waitCursor);

  int rows = m->numRows();
  int cols = m->numCols();

  double **real = (double **)malloc( rows * sizeof(double *) );
  double **imag = (double **)malloc( rows * sizeof(double *) );
  for (int r = 0; r<rows; r++)
    {
      real[r] = (double *)malloc( cols * sizeof(double) );
      imag[r] = (double *)malloc( cols * sizeof(double) );
    }

  m->forwardFFT( real, imag );

  Matrix *w =
    new Matrix (scriptEnv, rows, cols, "", ws, 0, WDestructiveClose);

  for (int i = 0; i < rows; i++)
    for (int j = 0; j < cols; j++)
      w->setValue(i, j, sqrt(real[i][j]*real[i][j]+imag[i][j]*imag[i][j]) );

  for (int r = 0; r<rows; r++)
    {
      free(real[r]);
      free(imag[r]);
    }

  free(real);
  free(imag);

  QString caption = generateUniqueName (tr ("Matrix"));
  initMatrix (w, caption);
  
  double dfr = 1.0 / fabs( m->xEnd() - m->xStart() );
  double dfc = 1.0 / fabs( m->yEnd() - m->yStart() );
  int r2 = rows/2;
  int c2 = cols/2;
  double xmin = - dfr * r2;
  double xmax = dfr * (rows - 1 - r2);
  double ymin = - dfc * c2;
  double ymax = dfc * (cols - 1 - c2);
  w->setCoordinates( xmin, xmax, ymin, ymax );

  w->setWindowLabel ("forward FFT of "+m->getName());
  w->setCaptionPolicy (myWidget::Both);
  w->show();

  QApplication::restoreOverrideCursor ();
}


void
ApplicationWindow::matrixDeterminant ()
{
  Matrix *m = (Matrix *) ws->activeWindow ();
  if (!m)
    return;

  QDateTime dt = QDateTime::currentDateTime ();
  QString info = tr("[") + dt.toString (Qt::LocalDate) + "]\n";
  info += tr ("Determinant of ") + m->getName() + ":\t";
  info += "det = " + QString::number (m->determinant ()) + "\n";
  info += "-------------------------------------------------------------\n";

  logInfo += info;

  showResults (true);
}

void
ApplicationWindow::invertMatrix ()
{
  Matrix *m = (Matrix *) ws->activeWindow ();
  if (!m)
    return;

  m->invert ();
}

void
ApplicationWindow::showMatrixFilterDialog()
{
  Matrix *m = (Matrix *) ws->activeWindow ();
  if (!m || !m->isA("Matrix"))
    return;

  matrixFilterDialog *mfd =
    new matrixFilterDialog( this,"matrixFilterDialog",
			    false, WStyle_Tool | WDestructiveClose );
  connect(m, SIGNAL (closedWindow (myWidget *)), mfd, SLOT (close ()));
 
  QStringList names = matrixNames();
  int index = names.findIndex( m->getName() );
  mfd->setMatrixNames( names, index );
  mfd->show();
  mfd->setActiveWindow();
}

void
ApplicationWindow::showMatrixInterpDialog()
{
  Matrix *m = (Matrix *) ws->activeWindow ();
  if (!m || !m->isA("Matrix"))
    return;

  matrixInterpDialog *mid =
    new matrixInterpDialog( this, "matrixInterpDialog", 
			    false, WStyle_Tool | WDestructiveClose );
  
  connect(m, SIGNAL (closedWindow (myWidget *)), mid, SLOT (close ()));

  mid->setMatrixNames( matrixNames() );
  mid->setMatrix(m);
  mid->show();
  mid->setActiveWindow();
}

Table *
ApplicationWindow::convertMatrixToTable ()
{
  Matrix *m = (Matrix *) ws->activeWindow ();
  if (!m)
    return 0;

  QApplication::setOverrideCursor (waitCursor);

  int rows = m->numRows ();
  int cols = m->numCols ();

  Table *w = new Table (scriptEnv, rows, cols, "", ws, 0, WDestructiveClose);
  for (int i = 0; i < rows; i++)
    {
      for (int j = 0; j < cols; j++)
	w->setText (i, j, m->text (i, j));
    }

  initTable (w, generateUniqueName (tr ("Table")));

  w->setWindowLabel (m->windowLabel ());
  w->setCaptionPolicy (m->captionPolicy ());
  w->resize (m->size ());
  w->show();

  QApplication::restoreOverrideCursor ();

  return w;
}

void
ApplicationWindow::initMatrix (Matrix * m, const QString & caption)
{
  QString name = caption;
  if (alreadyUsedName (name))
    name = generateUniqueName (tr ("Matrix"));

  //  m->setCaption (name);
  m->setName (name);
  m->setIcon (QPixmap (matrix_xpm));
  m->askOnCloseEvent (confirmCloseMatrix);
  m->setSpecifications( m->saveToString("geometry\n") );

  addListViewItem (m);
  current_folder->addWindow (m);
  m->setFolder (current_folder);

  connect (m, SIGNAL (showTitleBarMenu ()), this,
	   SLOT (showWindowTitleBarMenu ()));
  connect (m, SIGNAL (modifiedWindow (QWidget *)), this,
	   SLOT (modifiedProject (QWidget *)));
  connect (m, SIGNAL (modifiedWindow (QWidget *)), this,
	   SLOT (update3DMatrixPlots (QWidget *)));
  connect (m, SIGNAL (closedWindow (myWidget *)), this,
	   SLOT (closeWindow (myWidget *)));
  connect (m, SIGNAL (hiddenWindow (myWidget *)), this,
	   SLOT (hideWindow (myWidget *)));
  connect (m, SIGNAL (statusChanged (myWidget *)), this,
	   SLOT (updateWindowStatus (myWidget *)));
  connect (m, SIGNAL (showContextMenu ()), this,
	   SLOT (showWindowContextMenu ()));

  emit modified ();
}

Matrix *
ApplicationWindow::convertTableToMatrix ()
{
  Table *m = (Table *) ws->activeWindow ();
  if (!m)
    return 0;

  QApplication::setOverrideCursor (waitCursor);

  int rows = m->tableRows ();
  int cols = m->tableCols ();

  Matrix *w =
    new Matrix (scriptEnv, rows, cols, "", ws, 0, WDestructiveClose);
  for (int i = 0; i < rows; i++)
    {
      for (int j = 0; j < cols; j++)
	w->setText (i, j, m->text (i, j));
    }

  QString caption = generateUniqueName (tr ("Matrix"));
  initMatrix (w, caption);

  w->setWindowLabel (m->windowLabel ());
  w->setCaptionPolicy (m->captionPolicy ());
  w->resize (m->size ());
  w->show();

  QApplication::restoreOverrideCursor ();
  return w;
}

QWidget *
ApplicationWindow::window (const QString & name)
{
  QWidget *w = 0;
  QWidgetList *lst = windowsList ();
  for (w = lst->first (); w; w = lst->next ())
    if (((myWidget *)w)->getName () == name)
      {
	delete lst;
	return w;
      }
  delete lst;
  return 0;
}

Table *
ApplicationWindow::table (const QString & name)
{
  QString caption = name.section('_',0,0);

  QWidget *w;
  QWidgetList *lst = windowsList ();
  for (w = lst->first (); w; w = lst->next ())
    if (w->inherits ("Table") && ((myWidget *)w)->getName() == caption)
      {
	delete lst;
	return (Table *) w;
      }
  delete lst;
  return 0;
}

Matrix *
ApplicationWindow::matrix (const QString & name)
{
  QString caption = name;
  if (!renamedTables.isEmpty () && renamedTables.contains (caption))
    {
      int index = renamedTables.findIndex (caption);
      caption = renamedTables[index + 1];
    }

  QWidget *w;
  QWidgetList *lst = windowsList ();
  for (w = lst->first (); w; w = lst->next ())
    if (w->isA ("Matrix") && ((myWidget *)w)->getName() == caption)
      {
	delete lst;
	return (Matrix *) w;
      }
  delete lst;
  return 0;
}

MultiLayer *
ApplicationWindow::plot (const QString & name)
{
  QWidget *w;
  QWidgetList *lst = windowsList ();
  for (w = lst->first (); w; w = lst->next ())
    if (w->isA ("MultiLayer") && ((myWidget *)w)->getName() == name)
      {
	delete lst;
	return (MultiLayer *) w;
      }
  delete lst;
  return 0;
}

Note *
ApplicationWindow::note (const QString & name)
{
  QWidget *w;
  QWidgetList *lst = windowsList ();
  for (w = lst->first (); w; w = lst->next ())
    if (w->isA ("Note") && ((myWidget *)w)->getName() == name)
      {
	delete lst;
	return (Note *) w;
      }
  delete lst;
  return 0;
}

void
ApplicationWindow::windowActivated (QWidget * w)
{
  if(!w) return;

  // for test
  if((myWidget *)w!=aw)
    btnPointer->setOn(true);

  aw=(myWidget *)w;
  //
  customToolBars (w);
  customMenu (w);

#ifdef Q_OS_WIN
  if(aw->status()==myWidget::Maximized)
	  aw->showMaximized();
#endif

  current_folder->moveToTop(aw);

  // emit modified ();
}

void
ApplicationWindow::addErrorBars ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (g)
    {
      if (g->isPiePlot ())
	QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			      tr
			      ("This functionality is not available for pie plots!"));
      else
	{
	  activeGraph = g;
	  errDialog *ed =
	    new errDialog (this, "errDialog", TRUE,
			   WStyle_Tool | WDestructiveClose);
	  connect (ed,
		   SIGNAL (options
			   (const QString &, int, const QString &, int)),
		   this,
		   SLOT (defineErrorBars
			 (const QString &, int, const QString &, int)));
	  connect (ed,
		   SIGNAL (options (const QString &, const QString &, int)),
		   this,
		   SLOT (defineErrorBars
			 (const QString &, const QString &, int)));

	  QStringList curvesOnPlot = activeGraph->analyzableCurvesList ();

	  ed->setCurveNames (curvesOnPlot);
	  ed->setSrcTables (tableList ());
	  ed->showNormal();
	  ed->setActiveWindow ();
	}
    }
}

void
ApplicationWindow::defineErrorBars (const QString & name, int type,
				    const QString & percent, int direction)
{
  Table *w = table (name);
  if (!w)
    {				//user defined function
      QMessageBox::critical (this, tr ("QtiPlot - Error bars error"),
			     tr
			     ("This feature is not available for user defined function curves!"));
      return;
    }

  QString xColName = activeGraph->curveXColName (name);
  if (xColName.isEmpty ())
    return;

  if (direction == QwtErrorPlotCurve::Horizontal)
    w->addCol (Table::xErr);
  else
    w->addCol (Table::yErr);

  int r = w->tableRows ();
  int c = w->tableCols () - 1;
  int ycol = w->colIndex (name);
  if (!direction)
    ycol = w->colIndex (xColName);

  QMemArray < double >Y (r);
  Y = w->col (ycol);
  QString errColName = w->colName (c);

  double prc = percent.toDouble ();
  double moyenne = 0.0;
  if (type == 0)
    {
      for (int i = 0; i < r; i++)
	{
	  if (!w->table ()->text (i, ycol).isEmpty ())
	    w->setText (i, c, QString::number (Y[i] * prc / 100.0, 'g', 15));
	}
    }
  else if (type == 1)
    {
      int i;
      double dev = 0.0;
      for (i = 0; i < r; i++)
	moyenne += Y[i];
      moyenne /= r;
      for (i = 0; i < r; i++)
	dev += (Y[i] - moyenne) * (Y[i] - moyenne);
      dev = sqrt (dev / (r - 1));
      for (i = 0; i < r; i++)
	{
	  if (!w->table ()->text (i, ycol).isEmpty ())
	    w->setText (i, c, QString::number (dev, 'g', 15));
	}
    }
  activeGraph->addErrorBars (w, xColName, name, w, errColName, direction);
}

void
ApplicationWindow::defineErrorBars (const QString & curveName,
				    const QString & errColumnName,
				    int direction)
{
  Table *w = table (curveName);
  if (!w)
    {				//user defined function --> no worksheet available
      QMessageBox::critical (this, tr ("QtiPlot - Error"),
			     tr
			     ("This feature is not available for user defined function curves!"));
      return;
    }

  Table *errTable = table (errColumnName);
  if (w->tableRows () != errTable->tableRows ())
    {
      QMessageBox::critical (this, tr ("QtiPlot - Error"),
			     tr
			     ("The selected columns have different numbers of rows!"));

      addErrorBars ();
      return;
    }

  int errCol = errTable->colIndex (errColumnName);
  if (errTable->isEmptyColumn (errCol))
    {
      QMessageBox::critical (this, tr ("QtiPlot - Error"),
			     tr ("The selected error column is empty!"));
      addErrorBars ();
      return;
    }
  activeGraph->addErrorBars (w, curveName, errTable, errColumnName,
			     direction);
  emit modified ();
}

void
ApplicationWindow::removeCurves (const QStringList & names)
{
  QApplication::setOverrideCursor (waitCursor);

  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      if (w->isA ("MultiLayer"))
	{
	  QWidgetList *lst = ((MultiLayer *) w)->graphPtrs ();
	  for (Graph *g = (Graph *)lst->first(); g; g=(Graph *)lst->next())
	    {
	      QStringList associations = g->plotAssociations ();
	      QStringList rmcurves;
	      for (int i = 0; i < (int)associations.count(); i++)
		{
		  QStringList ass = 
		    QStringList::split (',', associations[i], false);
		  for (int j = 0; j< (int)names.count(); j++ )
		    if (ass.contains(names[j]))
		      {
			rmcurves << associations[i];
			break;
		      }
		}
	      if(!rmcurves.isEmpty())
		g->removeCurves( rmcurves );
	    }
	}
      else if (w->isA ("Graph3D"))
	{
	  QString ass = ((Graph3D *) w)->formula();
	  QStringList lst = QStringList::split(',',ass,false);
	  lst.gres(QRegExp("\\(.*\\)"),"");
	  for (int j = 0; j < (int)names.count(); j++ )
	    if(lst.contains(names[j]))
	      {
		((Graph3D *) w)->clearData ();
		break;
	      }
	}
    }
  delete windows;
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::removeCurves (Table *t)
{
  QApplication::setOverrideCursor (waitCursor);
  QWidgetList *lst = windowsList ();
  for (QWidget * w = lst->first (); w; w = lst->next ())
    {
      if (w->isA ("MultiLayer"))
	{
	  QWidgetList *g_lst = ((MultiLayer *) w)->graphPtrs ();
	  for(Graph *g = (Graph *)g_lst->first(); g; g =(Graph *) g_lst->next())
	    {
	      bool modified = false;
	      QStringList ass = g->plotAssociations ();
	      QStringList rmcurves;
	      for (int j = 0; j < (int)ass.count(); j++)
		if (ass[j].startsWith(t->getName()+"_"))
		  rmcurves << ass[j];
	      if(!rmcurves.isEmpty())
		g->removeCurves( rmcurves );
	    }
	}
      else if (w->isA ("Graph3D"))
	{
	  QString ass = ((Graph3D *) w)->formula();
	  if(ass.startsWith(t->getName()+"_"))
	    ((Graph3D *) w)->clearData ();
	}
    }
  delete lst;
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::updateCurves (Table * t, const QString & name)
{
  // qDebug("updateCurves is called");

  QApplication::setOverrideCursor (waitCursor);

  QWidgetList *lst = windowsList ();
  for (QWidget * w = lst->first (); w; w = lst->next ())
    {
      if (w->isA ("MultiLayer"))
	{
	  QWidgetList *g_lst = ((MultiLayer *) w)->graphPtrs ();
	  for(Graph *g = (Graph *)g_lst->first(); g; g = (Graph *)g_lst->next())
	    {
	      bool modified = false;
	      QStringList ass = g->plotAssociations();
	      for (int j = 0; j < (int)ass.count(); j++)
		{
		  QStringList lst = QStringList::split(',', ass[j], false);
		  if (lst.contains(name))
		    {
		      modified = true;
		      g->updateCurveData (t, name, j);
		    }
		}
	      if (modified)
		g->updatePlot ();
	    }
	}
      else if (w->isA ("Graph3D"))
	{
	  QString ass = ((Graph3D *) w)->formula();
	  QStringList lst = QStringList::split(',',ass,false);
	  lst.gres(QRegExp("\\(.*\\)"),"");
	  if (lst.contains (name))
	    ((Graph3D *) w)->updateData (t);
	}
    }
  delete lst;
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::updateCurves(Table * t)
{

  QApplication::setOverrideCursor (waitCursor);

  QString tname = t->getName(); 
  QWidgetList *lst = windowsList ();
  for (QWidget * w = lst->first (); w; w = lst->next ())
    {
      if (w->isA ("MultiLayer"))
	{
	  QWidgetList *g_lst = ((MultiLayer *) w)->graphPtrs ();
	  for(Graph *g = (Graph *)g_lst->first(); g; g =(Graph *) g_lst->next())
	    {
	      bool modified = false;
	      QStringList ass = g->plotAssociations ();
	      for (int j = 0; j < (int)ass.count(); j++)
		if (ass[j].startsWith(tname+"_"))
		  {
		    g->updateData( t, j );
		    modified = true;
		  }
	      if (modified)
		g->updatePlot ();
	    }
	}
      else if (w->isA ("Graph3D"))
	{
	  QString ass = ((Graph3D *) w)->formula();
	  if(ass.startsWith(tname+"_"))
	    ((Graph3D *) w)->updateData (t);
	}
    }
  delete lst;
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::showPreferencesDialog ()
{
  configDialog *cd =
    new configDialog (this, "configDialog", TRUE,
		      WStyle_Tool | WDestructiveClose);
  //cd->setColumnSeparator (columnSeparator);
  cd->showNormal ();
  cd->setActiveWindow ();
}

void
ApplicationWindow::setSaveSettings (bool autoSaving, int min)
{
  if (autoSave == autoSaving && autoSaveTime == min)
    return;

  autoSave = autoSaving;
  autoSaveTime = min;

  killTimer (savingTimerId);

  if (autoSave)
    savingTimerId = startTimer (autoSaveTime * 60000);
  else
    savingTimerId = 0;
}

void
ApplicationWindow::changeAppStyle (const QString & s)
{
  if (appStyle == s)
    return;

  if (QStyle * style = QStyleFactory::create (s))
    {
      qApp->setStyle (style);
      appStyle = s;

      QPalette pal = qApp->palette ();
      pal.setColor (QPalette::Active, QColorGroup::Base,
		    QColor (panelsColor));
      qApp->setPalette (pal, true, 0);
    }
}

void
ApplicationWindow::changeAppFont (const QFont & f)
{
  if (appFont == f)
    return;

  appFont = f;
  updateAppFonts ();
}

void
ApplicationWindow::updateAppFonts ()
{
  qApp->setFont (appFont);
  this->setFont (appFont);
  scriptingMenu->setFont (appFont);
  windowsMenu->setFont (appFont);
  viewMenu->setFont (appFont);
  graphMenu->setFont (appFont);
  fileMenu->setFont (appFont);
  formatMenu->setFont (appFont);
  graphAnalysisMenu->setFont (appFont);
  editMenu->setFont (appFont);
  tableAnalysisMenu->setFont (appFont);
  matrixAnalysisMenu->setFont (appFont);
  recentMenu->setFont (appFont);
  helpMenu->setFont (appFont);
  typeMenu->setFont (appFont);
  importMenu->setFont (appFont);
  plot2DMenu->setFont (appFont);
  plot3DMenu->setFont (appFont);
  matrixPlotMenu->setFont (appFont);
  matrixMenu->setFont (appFont);
  specialPlot->setFont (appFont);
  panelsMenu->setFont (appFont);
  statMenu2->setFont (appFont);
  smoothMenu->setFont (appFont);
  filterMenu->setFont (appFont);
  deconvMenu->setFont (appFont);
  decayMenu->setFont (appFont);
  plotDataMenu->setFont (appFont);
  tablesDepend->setFont (appFont);
  tableMenu->setFont (appFont);
  exportPlot->setFont (appFont);
  normMenu->setFont (appFont);
  translateMenu->setFont (appFont);
  fillMenu->setFont (appFont);
  setAsMenu->setFont (appFont);
  multiPeakMenu->setFont (appFont);
  info->
    setFont (QFont
	     //(appFont.family (), 2 + appFont.pointSize (), QFont::Bold,
	     (appFont.family (), appFont.pointSize (), QFont::Bold,
	      FALSE));
}

void
ApplicationWindow::updateConfirmOptions (bool askTables, bool askMatrixes,
					 bool askPlots2D, bool askPlots3D,
					 bool askNotes)
{
  QWidgetList *windows = windowsList ();
  if (confirmCloseTable != askTables)
    {
      confirmCloseTable = askTables;
      //for (int i = 0; i < int (windows->count ()); i++)
      for (QWidget *w = windows->first (); w; w = windows->next ())
	  if (w->isA ("Table"))
	    ((myWidget *)w)->askOnCloseEvent (confirmCloseTable);
    }

  if (confirmCloseMatrix != askMatrixes)
    {
      confirmCloseMatrix = askMatrixes;
      //for (int i = 0; i < int (windows->count ()); i++)
      for (QWidget * w = windows->first (); w; w = windows->next ())
	  if (w->isA ("Matrix"))
	    ((myWidget *)w)->askOnCloseEvent (confirmCloseMatrix);
    }

  if (confirmClosePlot2D != askPlots2D)
    {
      confirmClosePlot2D = askPlots2D;
      //for (int i = 0; i < int (windows->count ()); i++)
      for (QWidget * w = windows->first (); w; w = windows->next ())
	  if (w->isA ("MultiLayer"))
	    ((myWidget *)w)->askOnCloseEvent (confirmClosePlot2D);
    }

  if (confirmClosePlot3D != askPlots3D)
    {
      confirmClosePlot3D = askPlots3D;
      //for (int i = 0; i < int (windows->count ()); i++)
      for (QWidget * w = windows->first (); w; w = windows->next ())
	  if (w->isA ("Graph3D"))
	    ((myWidget *)w)->askOnCloseEvent (confirmClosePlot3D);
    }

  if (confirmCloseNotes != askNotes)
    {
      confirmCloseNotes = askNotes;
      //for (int i = 0; i < int (windows->count ()); i++)
      for (QWidget * w = windows->first (); w; w = windows->next ())
	  if (w->isA ("Note"))
	    ((myWidget *)w)->askOnCloseEvent (confirmCloseNotes);
    }

  delete windows;
}

void
ApplicationWindow::setGraphDefaultSettings (bool autoscale,
					    bool resizeLayers,
					    bool scaleFrame,
					    bool scaleGrid,
					    bool scaleLineWidth,
					    bool scaleFonts)
{

  autoscale2DPlots = autoscale;
  bool resetGeometry = false;
  if(!autoResizeLayers && !resizeLayers)
    resetGeometry = true;
  autoResizeLayers = !resizeLayers;

  autoScaleFrameWidth = scaleFrame;
  autoScaleGridWidth = scaleGrid;
  autoScaleLineWidth = scaleLineWidth;
  autoScaleFonts = scaleFonts;

  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    if (w->isA ("MultiLayer"))
      {
	QWidgetList *g_lst = ((MultiLayer *) w)->graphPtrs ();
	for (Graph *g = (Graph *)g_lst->first(); g; g = (Graph *)g_lst->next())
	  {
	    g->enableAutoscaling (autoscale2DPlots);
	    g->updateScale ();
	    g->setIgnoreResizeEvents (!autoResizeLayers);
	    g->setAutoScaleFrameWidth( autoScaleFrameWidth );
	    g->setAutoScaleGridWidth( autoScaleGridWidth );
	    g->setAutoScaleLineWidth( autoScaleLineWidth );
	    g->setAutoscaleFonts (autoScaleFonts);
	  }
	if(resetGeometry)
	  ((MultiLayer *)w)->resetRelativeGeometries();
      }

  delete windows;
}

void
ApplicationWindow::setLegendDefaultSettings (int frame, const QFont & font,
					     const QColor & textCol,
					     const QColor & backgroundCol)
{
  if (legendFrameStyle == frame &&
      legendTextColor == textCol &&
      legendBackground == backgroundCol && plotLegendFont == font)
    return;

  legendFrameStyle = frame;
  legendTextColor = textCol;
  legendBackground = backgroundCol;
  plotLegendFont = font;

  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    if (w->isA ("MultiLayer"))
      {
	QWidgetList *g_lst = ((MultiLayer *) w)->graphPtrs ();
	for (Graph *g = (Graph *)g_lst->first(); g; g = (Graph *)g_lst->next())
	  g->setTextMarkerDefaults (frame, font, textCol, backgroundCol);
      }

  delete windows;
  saveSettings ();
}

void
ApplicationWindow::setArrowDefaultSettings (int lineWidth, const QColor & c,
					    Qt::PenStyle style,
					    int headLength, int headAngle,
					    bool fillHead)
{
  if (defaultArrowLineWidth == lineWidth &&
      defaultArrowColor == c &&
      defaultArrowLineStyle == style &&
      defaultArrowHeadLength == headLength &&
      defaultArrowHeadAngle == headAngle && defaultArrowHeadFill == fillHead)
    return;

  defaultArrowLineWidth = lineWidth;
  defaultArrowColor = c;
  defaultArrowLineStyle = style;
  defaultArrowHeadLength = headLength;
  defaultArrowHeadAngle = headAngle;
  defaultArrowHeadFill = fillHead;

  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    if (w->isA ("MultiLayer"))
      {
	QWidgetList *g_lst = ((MultiLayer *) w)->graphPtrs ();
	for (Graph *g = (Graph *)g_lst->first(); g; g = (Graph *)g_lst->next ())
	  g->setArrowDefaults (defaultArrowLineWidth, defaultArrowColor,
			       defaultArrowLineStyle,
			       defaultArrowHeadLength,
			       defaultArrowHeadAngle, defaultArrowHeadFill);
      }
  
  delete windows;
  saveSettings ();
}

ApplicationWindow *
ApplicationWindow::plotFile (const QString & fn)
{
  QApplication::setOverrideCursor (waitCursor);
  ApplicationWindow *app = new ApplicationWindow ();
  app->applyUserSettings ();
  app->showMaximized ();

  Table *t = app->newTable (fn, app->columnSeparator, 0, true,
			    app->strip_spaces, app->simplify_spaces);
  t->setCaptionPolicy (myWidget::Both);
  app->multilayerPlot (t, t->YColumns (), Graph::LineSymbols);
  QApplication::restoreOverrideCursor ();
  return 0;
}

void
ApplicationWindow::showImportDialog ()
{
  importDialog *id =
    new importDialog (this, "importDialog", true,
		      WStyle_ContextHelp | WDestructiveClose);
  connect (id, SIGNAL (options (const QString &, int, bool, bool, bool)),
	   this,
	   SLOT (setImportOptions (const QString &, int, bool, bool, bool)));

  id->setSeparator (columnSeparator);
  id->setLines (ignoredLines);
  id->renameCols (renameColumns);
  id->setWhiteSpaceOptions (strip_spaces, simplify_spaces);
  id->showNormal ();
  id->disableApplyButton ();
  id->setActiveWindow ();
}

void
ApplicationWindow::setImportOptions (const QString & sep, int lines,
				     bool rename, bool strip, bool simplify)
{
  columnSeparator = sep;
  ignoredLines = lines;
  renameColumns = rename;
  strip_spaces = strip;
  simplify_spaces = simplify;
}

void
ApplicationWindow::loadASCII ()
{
  QString filter =
    tr ("All files") + " *;;" + tr ("Text") + " (*.TXT *.txt);;" +
    tr ("Data") + " (*DAT *.dat);;" + tr ("Comma Separated Values") +
    " (*.CSV *.csv);;";
  QString fn = QFileDialog::getOpenFileName (asciiDirPath, filter, this, 0,
					     tr
					     ("QtiPlot - Import ASCII File"),
					     0, TRUE);
  if (!fn.isEmpty ())
    {
      Table *t = (Table *) ws->activeWindow ();
      if (t && t->isA ("Table"))
	{
	  t->importASCII (fn, columnSeparator, ignoredLines, renameColumns,
			  strip_spaces, simplify_spaces, 3); //overwrite table
	  t->setWindowLabel (fn);
	  QFileInfo fi(fn);
	  QString name = t->getName();
	  QString basename = fi.baseName(true);
	  if(basename!=name)
	    {
	      QString newName = generateUniqueName(basename,false);
	      if(renameWindow(t,newName))
		renameListViewItem (name, newName);
	    }
	}
      else
	t = newTable (fn, columnSeparator, ignoredLines, renameColumns,
		      strip_spaces, simplify_spaces);

      t->setCaptionPolicy (myWidget::Both);
      setListViewLabel (t->getName(), fn);
      QFileInfo fi (fn);
      asciiDirPath = fi.dirPath (true);
    }
}

void
ApplicationWindow::loadMultiple ()
{
  Table *t = (Table *) ws->activeWindow ();
  if (t && t->isA ("Table"))
    {
      ImportFilesDialog *fd = new ImportFilesDialog (true, this, 0);
      fd->setDir (asciiDirPath);
      if (fd->exec () == QDialog::Accepted)
	{
	  asciiDirPath = fd->dirPath ();
	  loadMultipleASCIIFiles (fd->selectedFiles (), fd->importFileAs ());
	}
    }
  else
    {
      ImportFilesDialog *fd = new ImportFilesDialog (false, this, 0);
      fd->setDir (asciiDirPath);
      if (fd->exec () == QDialog::Accepted)
	{
	  asciiDirPath = fd->dirPath ();
	  loadMultipleASCIIFiles (fd->selectedFiles (), 0);
	}
    }
}

void
ApplicationWindow::loadMultipleASCIIFiles (const QStringList & fileNames,
					   int importFileAs)
{
  //  fprintf(stderr,"loadMultipleASCIIFiles\n");
  int files = fileNames.count ();

  if (!files)
    return;

  if (!importFileAs)
    {
      bool fmax = false;
      if(aw && (aw->status()==myWidget::Maximized))
	{
	  fmax = true;
	  aw->showNormal();
	}

      QString fn;
      int dx,dy;
      Table *w;
      for (int i = 0; i < files; i++)
	{
	  fn = fileNames[i];
	  w = newTable (fn, columnSeparator, ignoredLines, renameColumns,
		      strip_spaces, simplify_spaces);
	  if (w)
	    {
	      w->setCaptionPolicy (myWidget::Both);
	      setListViewLabel (w->getName(), fn);
	      if(i==0){
		dx = w->verticalHeaderWidth ();
		dy =
		  w->parentWidget ()->frameGeometry ().height () -
		  w->height ();
	      }
	      w->parentWidget ()->move (QPoint (i * dx, i * dy));
	    }
	}
      if(w && fmax) w->showMaximized();

      emit modifiedProject ();
    }
  else
    {
      if (ws->activeWindow () && ws->activeWindow ()->isA ("Table"))
	{
	  Table *t = (Table *) ws->activeWindow ();

	  for (int i = 0; i < files; i++)
	    t->importASCII(fileNames[i], columnSeparator,
			   ignoredLines, renameColumns,
			   strip_spaces, simplify_spaces,
			   importFileAs);
	  t->setWindowLabel (fileNames.join ("; "));
	  t->setCaptionPolicy (myWidget::Name);
	  emit modifiedProject (t);
	}
    }
}

void
ApplicationWindow::open ()
{
  QString filter = tr ("QtiPlot project") + " (*.qti);;";
  filter += tr ("Compressed QtiPlot project") + " (*.qti.gz);;";
  filter += tr ("Origin project") + " (*.opj *.OPJ);;";
  filter += tr ("All files") + " (*);;";

  QString fn = QFileDialog::getOpenFileName (workingDir, filter, this, 0,
					     tr ("QtiPlot - Open Project"), 0,
					     TRUE);
  if (!fn.isEmpty ())
    {
      QFileInfo fi (fn);
      workingDir = fi.dirPath (true);

      if (projectname != "untitled")
	{
	  QFileInfo fi (projectname);
	  QString pn = fi.absFilePath ();
	  if (fn == pn)
	    {
	      QMessageBox::warning (this,
				    tr ("QtiPlot - File openning error"),
				    tr
				    ("The file: <b>%1</b> is the current file!").
				    arg (fn));
	      return;
	    }
	}

      if (fn.contains (QRegExp("\\.(qti|qti\\.gz|opj|OPJ)~*$")))
	{
	  QFileInfo f (fn);
	  if (!f.exists ())
	    {
	      QMessageBox::critical (this,
				     tr ("QtiPlot - File openning error"),
				     tr
				     ("The file: <b>%1</b> doesn't exist!").
				     arg (fn));
	      return;
	    }

	  saveSettings ();	//the recent projects must be saved

	  ApplicationWindow *a = open (fn);
	  if (a)
	    {
	      this->close ();
	    }
	}
      else
	{
	  QMessageBox::critical (this, tr ("QtiPlot - File openning error"),
				 tr
				 ("The file: <b>%1</b> is not a QtiPlot or Origin project file!").
				 arg (fn));
	  return;
	}
    }
}

ApplicationWindow *
ApplicationWindow::open (const QString & fn)
{
#ifdef Q_OS_WIN
  LPTSTR lpn = new TCHAR[512];
  int len = GetLongPathName( (LPCTSTR)qt_winTchar(fn, TRUE), lpn, sizeof(TCHAR)*512 );
  QString fname = qt_winQString(lpn);
  delete[] lpn;
#else
  QString fname = fn;
#endif

  if (fname.endsWith("opj",false))
    return importOPJ (fname);
  else if (!fname.contains (QRegExp("\\.(qti|qti\\.gz)~*$")))
    return plotFile (fname);

  /*
  QFile f (fname);
  QTextStream t (&f);
  f.open (IO_ReadOnly);
  QString s = t.readLine ();
  f.close ();
  */
  
  char buf[256];
  gzFile f = gzopen(fname.local8Bit(),"r");
  // gzgets can read both compressed and uncompressed files equally
  gzgets( f, buf, 256 ); 
  QString s = buf;
  gzclose( f );

  //QStringList list = QStringList::split (QRegExp ("\\s"), s, false);
  //QString fileType = list[0];
  QString fileType = s.section(QRegExp("\\s"),0,0);

  if (fileType != "QtiPlot")
    {
      if (QFile::exists (fname + "~"))
	{
	  int choice =
	    QMessageBox::question (this, tr ("QtiPlot - File opening error"),
				   tr
				   ("The file <b>%1</b> is corrupted, but there exists a backup copy.<br>Do you want to open the backup instead?").
				   arg (fname),
				   QMessageBox::Yes | QMessageBox::Default,
				   QMessageBox::No | QMessageBox::Escape);
	  if (choice == QMessageBox::Yes)
	    return open (fname + "~");
	}
      else
	QMessageBox::critical (this, tr ("QtiPlot - File opening error"),
			       tr
			       ("The file: <b> %1 </b> was not created using QtiPlot!").
			       arg (fname));
      return 0;
    }

  ApplicationWindow *app = openProject (fname);

  return app;
}

void
ApplicationWindow::openRecentProject (int index)
{
  QString fn = recentMenu->text (index);
  //int pos = fn.find (" ", 0);
  //fn = fn.right (fn.length () - pos - 1);
  fn = fn.section(' ',1);

  QFile f (fn);
  if (!f.exists ())
    {
      QMessageBox::critical (this, tr ("QtiPlot - File Open Error"),
			     tr
			     ("The file: <b> %1 </b> <p>does not exist anymore!"
			      "<p>It will be removed from the list.").
			     arg (fn));

      recentProjects.remove (fn);
      updateRecentProjectsList ();
      return;
    }

  if (projectname != "untitled")
    {
      QFileInfo fi (projectname);
      QString pn = fi.absFilePath ();
      if (fn == pn)
	{
	  QMessageBox::warning (this, tr ("QtiPlot - File openning error"),
				tr
				("The file: <p><b> %1 </b><p> is the current file!").
				arg (fn));
	  return;
	}
    }

  if (!fn.isEmpty ())
    {
      saveSettings ();		//the recent projects must be saved 

      ApplicationWindow *a = open (fn);
      if (a)
	{
	  this->close ();
	}
    }
}

bool
ApplicationWindow::insertProject(const QString &fn, bool append)
{
  QString fname;
  if (fn.contains (QRegExp("\\.gz~*$")))
    { //decomress using zlib
      gzFile in = gzopen( fn.local8Bit(),"rb" );
#ifdef Q_OS_WIN
      fname = getenv("TEMP");
      fname.append("\\dummy.qti");
#else
      fname = "/tmp/dummy.qti"; 
#endif
      FILE *out = fopen( fname.local8Bit(),"wb" );
      gz_uncompress(in,out);
    }
  else
    fname = fn;

  QFile f (fname);
  QTextStream t (&f);
  t.setEncoding (QTextStream::UnicodeUTF8);
  f.open (IO_ReadOnly);

  QString s = t.readLine();
  QStringList lst = QStringList::split (QRegExp ("\\s"), s, false);
  QString fileType = lst[0], version = lst[1];
  QStringList vl = QStringList::split ('.', version, false);
  fileVersion =
    100 * (vl[0]).toInt () + 10 * (vl[1]).toInt () + (vl[2]).toInt ();

  if (fileType != "QtiPlot")
    {
      QMessageBox::critical (this, tr ("QtiPlot - File opening error"),
			     tr
			     ("The file: <b> %1 </b> was not created using QtiPlot!").
			     arg (fn));
      return false;
    }

  QFileInfo fi (fn);
  QString baseName = fi.baseName (true);

  //t.readLine ();
  if (fileVersion < 73)
    t.readLine ();

  s = t.readLine ();
  lst = QStringList::split ('\t', s, FALSE);
  if(lst[0] == "<scripting-lang>")
    {
      if(!append){
	ScriptingEnv *newEnv = ScriptingLangManager::newEnv (lst[1], this);
	if (newEnv && newEnv->isInitialized ())
	  {
	    ScriptingChangeEvent *sce = new ScriptingChangeEvent (newEnv);
	    QApplication::sendEvent (this, sce);
	    delete sce;
	  }
	else
	  {
	    if (newEnv)
	      delete newEnv;
	    QMessageBox::warning (this, tr ("QtiPlot - File opening error"),
				  tr("The file \"%1\" was created using \"%2\" as scripting language.\n\n"
				     "Initializing support for this language FAILED; I'm using \"%3\" instead.\n"
				     "Various parts of this file may not be displayed as expected.").
				  arg (fname).arg (lst[1]).arg (scriptEnv->name()));
	  }
      }
      s = t.readLine ();
      lst = QStringList::split ('\t', s, FALSE);
    }
  int aux=0, widgets;
  if(lst[0].contains("<windows>"))
    widgets = lst[1].toInt ();
  else {
    fprintf(stderr,"file format error\n");
    fprintf(stderr,"%s\n",s.ascii());
    return false;
  }

  QString titleBase = tr ("Window") + ": ";
  QString title = titleBase + "1/" + QString::number (widgets) + "  ";
  //QProgressDialog progress (this, 0, true, WStyle_StaysOnTop | WStyle_Tool);
  QProgressDialog progress (this, 0, true, WStyle_Tool);
  progress.setMinimumWidth (width () / 2);
  progress.setCaption (tr ("QtiPlot - Opening file") + ": " + fi.fileName());
  progress.setLabelText (title);
  progress.setTotalSteps (widgets);
  progress.setActiveWindow ();
//progress.setMinimumDuration(10000);
//progress.move(0,0);


  folderview->blockSignals (true);
  blockSignals (true);

  if(append)
      hideFolderWindows();

  if(append)
    {
      FolderListItem *item = (FolderListItem *) current_folder->folderListItem ();
      lst = current_folder->subfolders ();
      int n = (int) lst.contains (baseName);
      if (n)
	{                           //avoid identical subfolder names
	  while ((int) lst.contains (baseName + QString::number (n)))
	    n++;
	  baseName += QString::number (n);
	}
      current_folder = new Folder (current_folder, baseName);
      FolderListItem *fli = new FolderListItem (item, current_folder);
      current_folder->setFolderListItem (fli);
    }
  else
    {
      //rename project folder item
      FolderListItem *item = (FolderListItem *) folderview->firstChild ();
      item->setText (0, baseName);
      item->folder ()->setFolderName (baseName);
    }

  Folder *cf=current_folder;

  if(append && aw && (aw->status()==myWidget::Maximized))
    maximized_window = aw;
  else
    maximized_window = 0;

//process tables and matrix information
  while (!t.eof () && !progress.wasCanceled ())
    {
      s = t.readLine ();
      lst.clear ();
      if (s.left (8) == "<folder>")
	{
	  if(maximized_window)
	    {
	      maximized_window->resize(500,450);
	      maximized_window=0;
	    }
	  lst = QStringList::split ('\t', s, TRUE);
	  Folder *f = new Folder (current_folder, lst[1]);
	  f->setBirthDate (lst[2]);
	  f->setModificationDate (lst[3]);
	  if (lst[4] == "current")
	    cf = f;

	  FolderListItem *fli =
	    new FolderListItem (current_folder->folderListItem (), f);
	  fli->setText (0, lst[1]);
	  f->setFolderListItem (fli);
	  current_folder = f;
	}
      else if (s == "<table>")
	{
	  title =
	    titleBase + QString::number (++aux) + "/" +
	    QString::number (widgets);
	  progress.setLabelText (title);
	  while (s != "</table>")
	    {
	      s = t.readLine ();
	      lst << s;
	    }
	  lst.pop_back ();
	  openTable (lst);
	  progress.setProgress (aux);
	}
      else if (s.left (17) == "<TableStatistics>")
	{
	  while (s != "</TableStatistics>")
	    {
	      s = t.readLine ();
	      lst << s;
	    }
	  lst.pop_back ();
	  openTableStatistics (lst);
	}
      else if (s == "<matrix>")
	{
	  title =
	    titleBase + QString::number (++aux) + "/" +
	    QString::number (widgets);
	  progress.setLabelText (title);
	  while (s != "</matrix>")
	    {
	      s = t.readLine ();
	      lst << s;
	    }
	  lst.pop_back ();
	  openMatrix (lst);
	  progress.setProgress (aux);
	}
      else if (s == "<note>")
	{
	  title =
	    titleBase + QString::number (++aux) + "/" +
	    QString::number (widgets);
	  progress.setLabelText (title);
	  for (int i = 0; i < 3; i++)
	    {
	      s = t.readLine ();
	      lst << s;
	    }
	  Note *m = openNote (lst);
	  QStringList cont;
	  while (s != "</note>")
	    {
	      s = t.readLine ();
	      cont << s;
	    }
	  cont.pop_back ();
	  m->restore (cont);
	  progress.setProgress (aux);
	}
      else if (s == "</folder>")
	{
	  if(maximized_window)
	    {
	      maximized_window->resize(500,450);
	      maximized_window=0;
	    }
	  Folder *parent = (Folder *) current_folder->parent ();
	  if (parent)
	    current_folder = parent;
	  else
	    current_folder = projectFolder ();
	}
    }
  f.close ();

  if (progress.wasCanceled ())
    {
      return false;
    }

  //$B$3$N>l=j$K$OI,MW$J$$(B?
  if(maximized_window)
    {
      maximized_window->resize(500,450);
      maximized_window=0;
    }

//process the rest
  f.open (IO_ReadOnly);

  MultiLayer *plot = 0;
  while (!t.eof () && !progress.wasCanceled ())
    {
      s = t.readLine ();
      if (s.left (8) == "<folder>")
	{
	  if(maximized_window)
	    {
	      maximized_window->resize(500,450);
	      maximized_window=0;
	    }
	  lst = QStringList::split ('\t', s, TRUE);
	  current_folder = current_folder->findSubfolder (lst[1]);
	}
      else if (s == "<multiLayer>")
	{ //process multilayers information
	  title =
	    titleBase + QString::number (++aux) + "/" +
	    QString::number (widgets);
	  progress.setLabelText (title);

	  s = t.readLine ();
	  QStringList graph = QStringList::split ('\t', s, TRUE);
	  QString caption = graph[0];

	  //qDebug("reading %s",caption.ascii());

	  plot = multilayerPlot (caption);
	  plot->setCols (graph[1].toInt ());
	  plot->setRows (graph[2].toInt ());
	  QString date = QString::null;
	  date = graph[3];

	  setListViewDate (caption, date);
	  plot->setBirthDate (date);

	  restoreWindowGeometry (plot, t.readLine ());
	  plot->hide();

	  plot->blockSignals (true);

	  if (fileVersion > 71)
	    {
	      lst = QStringList::split ('\t', t.readLine (), true);
	      plot->setWindowLabel (lst[1]);
	      setListViewLabel (plot->getName(), lst[1]);
	      plot->setCaptionPolicy ((myWidget::CaptionPolicy) lst[2].
				      toInt ());
	    }
	  if (fileVersion > 83)
	    {
	      lst = QStringList::split ('\t', t.readLine (), false);
	      plot->setMargins (lst[1].toInt (), lst[2].toInt (),
				lst[3].toInt (), lst[4].toInt ());
	      lst = QStringList::split ('\t', t.readLine (), false);
	      plot->setSpacing (lst[1].toInt (), lst[2].toInt ());
	      lst = QStringList::split ('\t', t.readLine (), false);
	      plot->setLayerCanvasSize (lst[1].toInt (), lst[2].toInt ());
	      lst = QStringList::split ('\t', t.readLine (), false);
	      plot->setAlignement (lst[1].toInt (), lst[2].toInt ());
	    }

	  while (s != "</multiLayer>")
	    {			//open layers
	      s = t.readLine ();
	      if (s.left (7) == "<graph>")
		{
		  lst.clear ();
		  while (s != "</graph>")
		    {
		      s = t.readLine ();
		      lst << s;
		    }
		  openGraph (plot, lst);
		}
	    }
	  plot->blockSignals (false);
	  progress.setProgress (aux);
	}
      else if (s == "<SurfacePlot>")
	{			//process 3D plots information
	  lst.clear ();
	  title =
	    titleBase + QString::number (++aux) + "/" +
	    QString::number (widgets);
	  progress.setLabelText (title);
	  while (s != "</SurfacePlot>")
	    {
	      s = t.readLine ();
	      lst << s;
	    }
	  openSurfacePlot (lst);
	  progress.setProgress (aux);
	}
      else if (s == "</folder>")
	{
	  if(maximized_window)
	    {
	      maximized_window->resize(500,450);
	      maximized_window=0;
	    }
	  Folder *parent = (Folder *)current_folder->parent ();
	  if (parent)
	    current_folder = parent;
	  else
	    current_folder = projectFolder ();
	}
      else if((!append)&&(s.left (5) == "<log>"))
	{			//process analysis information
	  s = t.readLine ();
	  while (s != "</log>")
	    {
	      logInfo += s + "\n";
	      s = t.readLine ();
	    }
	  results->setText (logInfo);
	}
    }
  f.close ();

  if(maximized_window)
    {
      maximized_window->resize(500,450);
      maximized_window=0;
    }  

  if (progress.wasCanceled ())
    {
      return false;
    }

  //change folder to user defined current folder
  //changeFolder also does first showing of windows after the file open
  folderview->setCurrentItem (cf->folderListItem ());
  folderview->blockSignals (false);
  changeFolder (cf, true);

  if (aw)
    {
      aw->setFocus ();
  //    app->customMenu (app->aw);
  //    app->customToolBars (app->aw);
    }

  blockSignals (false);
  renamedTables.clear ();

  updatePlotsTransparency();

  return true;
}

ApplicationWindow *
ApplicationWindow::openProject (const QString & fn)
{
  if (QFile::exists(fn+".lock"))
    {
      QMessageBox::critical(0, tr("QtiPlot - File opening error"),
			    tr("The file <b>%1</b> is already opened by another process. If it is not the case, please delete <b>%2</b>.lock by hand.").arg(fn).arg(fn));
      return 0;
    }

  QFileInfo fi(fn);
  QString fileName = fi.absFilePath ();

  ApplicationWindow *app = new ApplicationWindow();
  app->applyUserSettings();
  app->projectname = fn;
  app->workingDir = fi.dirPath (true);
  app->fileVersion = fileVersion;
  app->setCaption (tr ("QtiPlot") + " - " + fi.fileName());
  app->showMaximized ();

  if(!app->insertProject( fn, false ))
    {
      app->saved = true;
      app->close ();
      return 0;
    }

  app->logInfo = app->logInfo.remove ("</log>\n", FALSE);

  app->recentProjects.remove (fileName);
  app->recentProjects.push_front (fileName);
  app->updateRecentProjectsList ();

  app->executeNotes ();
  app->saved = TRUE;
  app->actionSaveProject->setEnabled (false);

  QFile f(fn+".lock");
  f.open(IO_WriteOnly);
  f.close();

  return app;
}

void
ApplicationWindow::executeNotes ()
{
  QPtrList < myWidget > lst = projectFolder ()->windowsList ();
  for (myWidget * i = lst.first (); i; i = lst.next ())
    if (i->isA ("Note") && ((Note *) i)->autoexec ())
      ((Note *) i)->executeAll ();
}

void
ApplicationWindow::scriptError (const QString & message,
				const QString & scriptName, int lineNumber)
{
  QString st = scriptEnv->stackTraceString ();
  QString errmsg = message;
  if (st.isNull ())
    errmsg += "\nat " + scriptName + ":" + QString::number (lineNumber);
  else
    errmsg += "\n" + st;
  QMessageBox::critical (this, "QtiPlot - Script Error", errmsg);
}

void
ApplicationWindow::scriptPrint (const QString & text)
{
#ifdef SCRIPTING_CONSOLE
  if (!text.stripWhiteSpace ().isEmpty ())
    console->append (text);
#else
  printf (text.ascii ());
#endif
}

bool
ApplicationWindow::setScriptingLang (const QString & lang)
{
  if (lang == scriptEnv->name ())
    return true;
  if (lang.isEmpty ())
    return false;

  ScriptingEnv *newEnv = ScriptingLangManager::newEnv (lang, this);
  if (newEnv && newEnv->isInitialized ())
    {
      ScriptingChangeEvent *sce = new ScriptingChangeEvent (newEnv);
      QApplication::sendEvent (this, sce);
      delete sce;
      QObjectList *receivers = queryList ();
      for (QObjectListIt i (*receivers); !i.atLast (); ++i)
	QApplication::postEvent (i, new ScriptingChangeEvent (newEnv));
      delete receivers;
      if (scriptWindow)
	{
	  receivers = scriptWindow->queryList ();
	  for (QObjectListIt i (*receivers); !i.atLast (); ++i)
	    QApplication::postEvent (i, new ScriptingChangeEvent (newEnv));
	  delete receivers;
	}
      return true;
    }
  return false;
}

void
ApplicationWindow::showScriptingLangDialog ()
{
  ScriptingLangDialog *d =
    new ScriptingLangDialog (scriptEnv, this, "scriptingLangDialog", TRUE,
			     WStyle_Tool | WDestructiveClose);
  d->showNormal ();
  d->setActiveWindow ();
}

void
ApplicationWindow::restartScriptingEnv ()
{
  ScriptingEnv *newEnv =
    ScriptingLangManager::newEnv (scriptEnv->name (), this);
  if (!newEnv || !newEnv->isInitialized ())
    {
      if (newEnv)
	delete newEnv;
      QMessageBox::critical (this, tr ("QtiPlot - Scripting Error"),
			     tr
			     ("Scripting language \"%1\" failed to initialize.").
			     arg (scriptEnv->name ()));
      return;
    }
  ScriptingChangeEvent *sce = new ScriptingChangeEvent (newEnv);
  QApplication::sendEvent (this, sce);
  QObjectList *receivers = queryList ();
  for (QObjectListIt i (*receivers); !i.atLast (); ++i)
    QApplication::sendEvent (i, sce);
  delete receivers;
  delete sce;
  executeNotes ();
}

void
ApplicationWindow::openTemplate ()
{
  QString filter = "QtiPlot 2D Graph Template (*.qpt);;";
  filter += "QtiPlot 3D Surface Template (*.qst);;";
  filter += "QtiPlot Table Template (*.qtt);;";
  filter += "QtiPlot Matrix Template (*.qmt);;";

  QString fn = QFileDialog::getOpenFileName (templatesDir, filter, this, 0,
					     tr
					     ("QtiPlot - Open Template File"),
					     0, TRUE);
  if (!fn.isEmpty ())
    {
      QFileInfo fi (fn);
      templatesDir = fi.dirPath (true);
      if (fn.endsWith (".qmt", true) || fn.endsWith (".qpt", true) ||
	  fn.endsWith (".qtt", true) || fn.endsWith (".qst", true))
	{
	  if (!fi.exists ())
	    {
	      QMessageBox::critical (this,
				     tr ("QtiPlot - File opening error"),
				     tr
				     ("The file: <b>%1</b> doesn't exist!").
				     arg (fn));
	      return;
	    }
	  QFile f (fn);
	  QTextStream t (&f);
	  t.setEncoding (QTextStream::UnicodeUTF8);
	  f.open (IO_ReadOnly);
	  QStringList l =
	    QStringList::split (QRegExp ("\\s"), t.readLine (), false);
	  QString fileType = l[0];
	  if (fileType != "QtiPlot")
	    {
	      QMessageBox::critical (this,
				     tr ("QtiPlot - File openning error"),
				     tr
				     ("The file: <b> %1 </b> was not created using QtiPlot!").
				     arg (fn));
	      return;
	    }
	  QStringList vl = QStringList::split ('.', l[1], false);
	  fileVersion =
	    100 * (vl[0]).toInt () + 10 * (vl[1]).toInt () + (vl[2]).toInt ();

	  QApplication::setOverrideCursor (waitCursor);
	  myWidget *w = 0;
	  QString templateType;
	  t >> templateType;

	  if (templateType == "<SurfacePlot>")
	    {
	      t.skipWhiteSpace ();
	      QStringList lst;
	      while (!t.atEnd ())
		lst << t.readLine ();
	      w = openSurfacePlot (lst);
	      if (w)
		((Graph3D *) w)->clearData ();
	    }
	  else
	    {
	      int rows, cols;
	      t >> rows;
	      t >> cols;
	      t.skipWhiteSpace ();
	      QString geometry = t.readLine ();

	      if (templateType == "<multiLayer>")
		{
		  w = multilayerPlot (generateUniqueName (tr ("Graph")));
		  if (w)
		    {
		      ((MultiLayer *) w)->setCols (cols);
		      ((MultiLayer *) w)->setRows (rows);
		      restoreWindowGeometry (w, geometry);
		      w->hide();
		      if (fileVersion > 83)
			{
			  QStringList lst =
			    QStringList::split ('\t', t.readLine (), false);
			  ((MultiLayer *) w)->setMargins (lst[1].toInt (),
							  lst[2].toInt (),
							  lst[3].toInt (),
							  lst[4].toInt ());
			  lst =
			    QStringList::split ('\t', t.readLine (), false);
			  ((MultiLayer *) w)->setSpacing (lst[1].toInt (),
							  lst[2].toInt ());
			  lst =
			    QStringList::split ('\t', t.readLine (), false);
			  ((MultiLayer *) w)->setLayerCanvasSize (lst[1].
								  toInt (),
								  lst[2].
								  toInt ());
			  lst =
			    QStringList::split ('\t', t.readLine (), false);
			  ((MultiLayer *) w)->setAlignement (lst[1].toInt (),
							     lst[2].toInt ());
			}
		      while (!t.atEnd ())
			{	//open layers
			  QString s = t.readLine ();
			  if (s.left (7) == "<graph>")
			    {
			      QStringList lst;
			      while (s != "</graph>")
				{
				  s = t.readLine ();
				  lst << s;
				}
			      openGraph ((MultiLayer *) w, lst);
			    }
			}
		    }
		}
	      else
		{
		  if (templateType == "<table>")
		    w = newTable (tr ("Table1"), rows, cols);
		  else if (templateType == "<matrix>")
		    w = newMatrix (tr ("Matrix1"), rows, cols);
		  if (w)
		    {
		      QStringList lst;
		      while (!t.atEnd ())
			lst << t.readLine ();
		      w->restore (lst);
		      restoreWindowGeometry ( w, geometry);
		      w->hide();
		    }
		}
	    }

	  f.close ();
	  if (w)
	    {
	      customMenu ((QWidget *) w);
	      customToolBars ((QWidget *) w);
	    }
	  QApplication::restoreOverrideCursor ();
	}
      else
	{
	  QMessageBox::critical (this, tr ("QtiPlot - File openning error"),
				 tr
				 ("The file: <b>%1</b> is not a QtiPlot template file!").
				 arg (fn));
	  return;
	}
    }
}

void
ApplicationWindow::updatePlotsTransparency ()
{
  QApplication::setOverrideCursor (waitCursor);
  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    if (w->isA ("MultiLayer") && ((MultiLayer *) w)->hasOverlapingLayers ())
      ((MultiLayer *) w)->updateTransparency ();

  delete windows;
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::readSettings ()
{
#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
  QSettings settings (QSettings::Ini);
#else
  QSettings settings;
#endif
  
  settings.setPath ("ProIndependent", "QtiPlot", QSettings::User);
  settings.insertSearchPath( QSettings::Windows, "/QtiPlot" );
  settings.beginGroup ("/General");
  //  autoSearchUpdates = settings.readBoolEntry ("/AutoSearchUpdates", false, 0);
  askForSupport = settings.readBoolEntry ("/Support", true, 0);
  appLanguage = settings.readEntry ("/Language", "en");
  show_windows_policy =
    (ShowWindowsPolicy) settings.readNumEntry ("/ShowWindowsPolicy",
					       ActiveFolder);
  recentProjects = settings.readListEntry ("/RecentProjects");
  updateRecentProjectsList ();

  changeAppStyle (settings.readEntry ("/Style", appStyle));
  autoSave = settings.readBoolEntry ("/AutoSave", true);
  autoSaveTime = settings.readNumEntry ("/AutoSaveTime", 15);
  defaultScriptingLang = settings.readEntry ("/ScriptingLang");
  defaultPrinterOrientation = settings.readEntry("/PrinterOrientation","Automatic");

//restore dock windows and tool bars
  QString str = settings.readEntry ("/DockWindows");
  QTextIStream in1 (&str);
  in1 >> *this;

  str = settings.readEntry ("/ExplorerSplitter");
  QTextIStream in2 (&str);
  in2 >> *explorerSplitter;

  QStringList applicationFont = settings.readListEntry ("/Font");
  if (applicationFont.size () == 4)
    appFont =
      QFont (applicationFont[0], applicationFont[1].toInt (),
	     applicationFont[2].toInt (), applicationFont[3].toInt ());

  settings.beginGroup ("/Colors");
  workspaceColor = QColor (settings.readEntry ("/Workspace", "darkGray"));
  panelsColor = QColor (settings.readEntry ("/Panels", "#ffffff"));
  panelsTextColor = QColor (settings.readEntry ("/PanelsText", "#000000"));
  settings.endGroup ();

  settings.beginGroup ("/Paths");
  workingDir =
    settings.readEntry ("/WorkingDir", qApp->applicationDirPath ());
  templatesDir =
    settings.readEntry ("/TemplatesDir", qApp->applicationDirPath ());
  exportDir =
    settings.readEntry ("/ExportDir", qApp->applicationDirPath ());

  helpFilePath = "/usr/share/doc/qtiplot/index.html";
#ifdef Q_OS_WIN			// Windows systems
  helpFilePath = qApp->applicationDirPath () + "/index.html";
#endif
  helpFilePath = settings.readEntry ("/HelpFile", helpFilePath);

  fitPluginsPath = settings.readEntry ("/FitPlugins", "fitPlugins");
  asciiDirPath = settings.readEntry ("/ASCII", qApp->applicationDirPath ());
  imagesDirPath = settings.readEntry ("/Images", qApp->applicationDirPath ());
  settings.endGroup ();
  settings.endGroup ();

  settings.beginGroup ("/UserFunctions");
  //fitFunctions = settings.readListEntry ("/FitFunctions");
  surfaceFunc = settings.readListEntry ("/SurfaceFunctions");
  xFunctions = settings.readListEntry ("/xFunctions");
  yFunctions = settings.readListEntry ("/yFunctions");
  rFunctions = settings.readListEntry ("/rFunctions");
  tetaFunctions = settings.readListEntry ("/tetaFunctions");
  settings.endGroup ();

  settings.beginGroup ("/Confirmations");
  confirmCloseFolder = settings.readBoolEntry ("/Folder", true, 0);
  confirmCloseTable = settings.readBoolEntry ("/Table", true, 0);
  confirmCloseMatrix = settings.readBoolEntry ("/Matrix", true, 0);
  confirmClosePlot2D = settings.readBoolEntry ("/Plot2D", true, 0);
  confirmClosePlot3D = settings.readBoolEntry ("/Plot3D", true, 0);
  confirmCloseNotes = settings.readBoolEntry ("/Note", true, 0);
  settings.endGroup ();

  settings.beginGroup ("/Tables");
  tbl_output_precision = settings.readNumEntry ("/TblOutputPrecision", 15);
  QStringList tableFonts = settings.readListEntry ("/Fonts");
  if (tableFonts.size () == 8)
    {
      tableTextFont =
	QFont (tableFonts[0], tableFonts[1].toInt (), tableFonts[2].toInt (),
	       tableFonts[3].toInt ());
      tableHeaderFont =
	QFont (tableFonts[4], tableFonts[5].toInt (), tableFonts[6].toInt (),
	       tableFonts[7].toInt ());
    }
  /*
  uselessTablesList = settings.readListEntry("/UselessTables");
   */
  settings.beginGroup ("/Colors");
  tableBkgdColor = QColor (settings.readEntry ("/Background", "#ffffff"));
  tableTextColor = QColor (settings.readEntry ("/Text", "#000000"));
  tableHeaderColor = QColor (settings.readEntry ("/Header", "#000000"));
  settings.endGroup ();
  settings.endGroup ();

  settings.beginGroup ("/2DPlots");
  settings.beginGroup ("/General");
  titleOn = settings.readBoolEntry ("/Title", true, 0);
  allAxesOn = settings.readBoolEntry ("/AllAxes", false, 0);
  canvasFrameOn = settings.readBoolEntry ("/CanvasFrame", false, 0);
  canvasFrameWidth = settings.readNumEntry ("/CanvasFrameWidth", 0, 0);
  defaultPlotMargin = settings.readNumEntry ("/Margin", 0, 0);
  defaultPlotSpacing = settings.readNumEntry ("/Spacing", 2, 0);
  drawBackbones = settings.readBoolEntry ("/AxesBackbones", true, 0);
  axesLineWidth = settings.readNumEntry ("/AxesLineWidth", 1, 0);
  autoscale2DPlots = settings.readBoolEntry ("/Autoscale", true, 0);
  autoScaleFonts = settings.readBoolEntry ("/AutoScaleFonts", true, 0);
  autoResizeLayers = settings.readBoolEntry ("/AutoResizeLayers", true, 0);
  flipRightAxisTitle = settings.readBoolEntry("/FlipRightAxisTitle", false, 0);
  autoScaleFrameWidth = settings.readBoolEntry("/AutoScaleFrameWidth", false,0);
  autoScaleGridWidth = settings.readBoolEntry("/AutoScaleGridWidth", false, 0);
  autoScaleLineWidth = settings.readBoolEntry("/AutoScaleLineWidth", false, 0);
  d_show_current_folder = settings.readBoolEntry("/ShowCurrentFolder", false, 0);

  QStringList graphFonts = settings.readListEntry ("/Fonts");
  if (graphFonts.size () == 16)
    {
      plotAxesFont =
	QFont (graphFonts[0], graphFonts[1].toInt (), graphFonts[2].toInt (),
	       graphFonts[3].toInt ());
      plotNumbersFont =
	QFont (graphFonts[4], graphFonts[5].toInt (), graphFonts[6].toInt (),
	       graphFonts[7].toInt ());
      plotLegendFont =
	QFont (graphFonts[8], graphFonts[9].toInt (), graphFonts[10].toInt (),
	       graphFonts[11].toInt ());
      plotTitleFont =
	QFont (graphFonts[12], graphFonts[13].toInt (),
	       graphFonts[14].toInt (), graphFonts[15].toInt ());
    }
  settings.endGroup ();

  settings.beginGroup ("/Curves");
  defaultCurveStyle = settings.readNumEntry ("/Style", Graph::LineSymbols, 0);
  defaultCurveLineWidth = settings.readNumEntry ("/LineWidth", 1, 0);
  defaultSymbolSize = settings.readNumEntry ("/SymbolSize", 7, 0);
  settings.endGroup ();

  settings.beginGroup ("/Ticks");
  majTicksStyle = settings.readNumEntry ("/MajTicksStyle", ScaleDraw::Out, 0);
  minTicksStyle = settings.readNumEntry ("/MinTicksStyle", ScaleDraw::Out, 0);
  minTicksLength = settings.readNumEntry ("/MinTicksLength", 5, 0);
  majTicksLength = settings.readNumEntry ("/MajTicksLength", 9, 0);
  settings.endGroup ();

  settings.beginGroup ("/Legend");
  legendFrameStyle =
    settings.readNumEntry ("/FrameStyle", LegendMarker::Line, 0);
  legendTextColor = QColor (settings.readEntry ("/TextColor", "#000000"));	//default color Qt::black
  legendBackground = QColor (settings.readEntry ("/BackgroundColor", "#ffffff"));	//default color Qt::white
  settings.endGroup ();

  settings.beginGroup ("/Arrows");
  defaultArrowLineWidth = settings.readNumEntry ("/Width", 1);
  defaultArrowColor = QColor (settings.readEntry ("/Color", "#000000"));	//default color Qt::black
  defaultArrowHeadLength = settings.readNumEntry ("/HeadLength", 8);
  defaultArrowHeadAngle = settings.readNumEntry ("/HeadAngle", 25);
  defaultArrowHeadFill = settings.readBoolEntry ("/HeadFill", true);
  defaultArrowLineStyle =
    Graph::getPenStyle (settings.readEntry ("/LineStyle", "SolidLine"));
  settings.endGroup ();
  settings.endGroup ();

  settings.beginGroup ("/3DPlots");
  showPlot3DLegend = settings.readBoolEntry ("/Legend", true);
  showPlot3DProjection = settings.readBoolEntry ("/Projection", false);
  smooth3DMesh = settings.readBoolEntry ("/Antialiasing", true);
  plot3DResolution = settings.readNumEntry ("/Resolution", 1);
  orthogonal3DPlots = settings.readBoolEntry ("/Orthogonal", false);

  QStringList plot3DFonts = settings.readListEntry ("/Fonts");
  if (plot3DFonts.size () == 12)
    {
      plot3DTitleFont =
	QFont (plot3DFonts[0], plot3DFonts[1].toInt (),
	       plot3DFonts[2].toInt (), plot3DFonts[3].toInt ());
      plot3DNumbersFont =
	QFont (plot3DFonts[4], plot3DFonts[5].toInt (),
	       plot3DFonts[6].toInt (), plot3DFonts[7].toInt ());
      plot3DAxesFont =
	QFont (plot3DFonts[8], plot3DFonts[9].toInt (),	 
	       plot3DFonts[10].toInt (), plot3DFonts[11].toInt ());
    }

  settings.beginGroup ("/Colors");
  plot3DColors << QColor (settings.readEntry ("/MaxData", "blue")).name ();
  plot3DColors << QColor (settings.readEntry ("/Labels", "#000000")).name ();
  plot3DColors << QColor (settings.readEntry ("/Mesh", "#000000")).name ();
  plot3DColors << QColor (settings.readEntry ("/Grid", "#000000")).name ();
  plot3DColors << QColor (settings.readEntry ("/MinData", "red")).name ();
  plot3DColors << QColor (settings.readEntry ("/Numbers", "#000000")).name ();
  plot3DColors << QColor (settings.readEntry ("/Axes", "#000000")).name ();
  plot3DColors << QColor (settings.readEntry ("/Background", "#ffffff")).
    name ();
  settings.endGroup ();
  settings.endGroup ();

  settings.beginGroup ("/Fitting");
  fit_output_precision = settings.readNumEntry ("/OutputPrecision", 15);
  pasteFitResultsToPlot =
    settings.readBoolEntry ("/PasteResultsToPlot", false);
  writeFitResultsToLog = settings.readBoolEntry ("/WriteResultsToLog", true);
  generateUniformFitPoints =
    settings.readBoolEntry ("/GenerateUniformFitPoints", true);
  generateFunction =
    settings.readBoolEntry ("/GenerateFunction", true);
  fitPoints = settings.readNumEntry ("/Points", 100);
  generatePeakCurves = settings.readBoolEntry ("/GeneratePeakCurves", true);
  peakCurvesColor = settings.readNumEntry ("/PeaksColor", 2);	//green color
  //fit_scale_errors = settings.readBoolEntry ("/ScaleErrors", false);
  settings.endGroup ();

  settings.beginGroup ("/ImportASCII");
  columnSeparator = settings.readEntry ("/ColumnSeparator", "Auto");
  ignoredLines = settings.readNumEntry ("/IgnoreLines", 0);
  renameColumns = settings.readBoolEntry ("/RenameColumns", false);
  strip_spaces = settings.readBoolEntry ("/StripSpaces", true);
  simplify_spaces = settings.readBoolEntry ("/SimplifySpaces", true);
  settings.endGroup ();

  settings.beginGroup ("/FWHM");
  fwhmPeakFrac = settings.readNumEntry("/peakFrac",20);
  fwhmHalfFrac = settings.readNumEntry("/halfFrac",30);
  settings.endGroup();
}

void
ApplicationWindow::saveSettings ()
{
#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
  QSettings settings (QSettings::Ini);
#else
  QSettings settings;
#endif
  
  settings.setPath ("ProIndependent", "QtiPlot", QSettings::User);
  settings.insertSearchPath( QSettings::Windows, "/QtiPlot" );
  settings.beginGroup ("/General");
  //  settings.writeEntry ("/AutoSearchUpdates", autoSearchUpdates);
  settings.writeEntry ("/Support", askForSupport);
  settings.writeEntry ("/Language", appLanguage);
  settings.writeEntry ("/ShowWindowsPolicy", show_windows_policy);
  settings.writeEntry ("/RecentProjects", recentProjects);
  settings.writeEntry ("/Style", appStyle);
  settings.writeEntry ("/AutoSave", autoSave);
  settings.writeEntry ("/AutoSaveTime", autoSaveTime);
  settings.writeEntry ("/ScriptingLang", defaultScriptingLang);
  settings.writeEntry ("/PrinterOrientation", defaultPrinterOrientation);

  QString str;
  QTextOStream out1 (&str);
  out1 << *this;
  settings.writeEntry ("/DockWindows", str);

  QTextOStream out2 (&str);
  out2 << *explorerSplitter;
  settings.writeEntry ("/ExplorerSplitter", str);

  QStringList applicationFont;
  applicationFont << appFont.family ();
  applicationFont << QString::number (appFont.pointSize ());
  applicationFont << QString::number (appFont.weight ());
  applicationFont << QString::number (appFont.italic ());
  settings.writeEntry ("/Font", applicationFont);

  settings.beginGroup ("/Colors");
  settings.writeEntry ("/Workspace", workspaceColor.name ());
  settings.writeEntry ("/Panels", panelsColor.name ());
  settings.writeEntry ("/PanelsText", panelsTextColor.name ());
  settings.endGroup ();

  settings.beginGroup ("/Paths");
  settings.writeEntry ("/WorkingDir", workingDir);
  settings.writeEntry ("/TemplatesDir", templatesDir);
  settings.writeEntry ("/ExportDir", exportDir);
  settings.writeEntry ("/HelpFile", helpFilePath);
  settings.writeEntry ("/FitPlugins", fitPluginsPath);
  settings.writeEntry ("/ASCII", asciiDirPath);
  settings.writeEntry ("/Images", imagesDirPath);
  settings.endGroup ();
  settings.endGroup ();

  settings.beginGroup ("/UserFunctions");
  //settings.writeEntry ("/FitFunctions", fitFunctions);
  settings.writeEntry ("/SurfaceFunctions", surfaceFunc);
  settings.writeEntry ("/xFunctions", xFunctions);
  settings.writeEntry ("/yFunctions", yFunctions);
  settings.writeEntry ("/rFunctions", rFunctions);
  settings.writeEntry ("/tetaFunctions", tetaFunctions);
  settings.endGroup ();

  settings.beginGroup ("/Confirmations");
  settings.writeEntry ("/Folder", confirmCloseFolder);
  settings.writeEntry ("/Table", confirmCloseTable);
  settings.writeEntry ("/Matrix", confirmCloseMatrix);
  settings.writeEntry ("/Plot2D", confirmClosePlot2D);
  settings.writeEntry ("/Plot3D", confirmClosePlot3D);
  settings.writeEntry ("/Note", confirmCloseNotes);
  settings.endGroup ();

  settings.beginGroup ("/Tables");
  settings.writeEntry ("/TblOutputPrecision", tbl_output_precision);
  QStringList tableFonts;
  tableFonts << tableTextFont.family ();
  tableFonts << QString::number (tableTextFont.pointSize ());
  tableFonts << QString::number (tableTextFont.weight ());
  tableFonts << QString::number (tableTextFont.italic ());
  tableFonts << tableHeaderFont.family ();
  tableFonts << QString::number (tableHeaderFont.pointSize ());
  tableFonts << QString::number (tableHeaderFont.weight ());
  tableFonts << QString::number (tableHeaderFont.italic ());
  settings.writeEntry ("/Fonts", tableFonts);
  /*
  settings.writeEntry ("/UselessTables",uselessTablesList);
  */
  settings.beginGroup ("/Colors");
  settings.writeEntry ("/Background", tableBkgdColor.name ());
  settings.writeEntry ("/Text", tableTextColor.name ());
  settings.writeEntry ("/Header", tableHeaderColor.name ());
  settings.endGroup ();
  settings.endGroup ();

  settings.beginGroup ("/2DPlots");
  settings.beginGroup ("/General");
  settings.writeEntry ("/Title", titleOn);
  settings.writeEntry ("/AllAxes", allAxesOn);
  settings.writeEntry ("/CanvasFrame", canvasFrameOn);
  settings.writeEntry ("/CanvasFrameWidth", canvasFrameWidth);
  settings.writeEntry ("/Margin", defaultPlotMargin);
  settings.writeEntry ("/Spacing", defaultPlotSpacing);
  settings.writeEntry ("/AxesBackbones", drawBackbones);
  settings.writeEntry ("/AxesLineWidth", axesLineWidth);
  settings.writeEntry ("/Autoscale", autoscale2DPlots);
  settings.writeEntry ("/AutoScaleFonts", autoScaleFonts);
  settings.writeEntry ("/AutoResizeLayers", autoResizeLayers);
  settings.writeEntry ("/FlipRightAxisTitle", flipRightAxisTitle);
  settings.writeEntry ("/AutoScaleFrameWidth", autoScaleFrameWidth);
  settings.writeEntry ("/AutoScaleGridWidth", autoScaleGridWidth);
  settings.writeEntry ("/AutoScaleLineWidth", autoScaleLineWidth);
  settings.writeEntry ("/ShowCurrentFolder", d_show_current_folder);

  QStringList graphFonts;
  graphFonts << plotAxesFont.family ();
  graphFonts << QString::number (plotAxesFont.pointSize ());
  graphFonts << QString::number (plotAxesFont.weight ());
  graphFonts << QString::number (plotAxesFont.italic ());
  graphFonts << plotNumbersFont.family ();
  graphFonts << QString::number (plotNumbersFont.pointSize ());
  graphFonts << QString::number (plotNumbersFont.weight ());
  graphFonts << QString::number (plotNumbersFont.italic ());
  graphFonts << plotLegendFont.family ();
  graphFonts << QString::number (plotLegendFont.pointSize ());
  graphFonts << QString::number (plotLegendFont.weight ());
  graphFonts << QString::number (plotLegendFont.italic ());
  graphFonts << plotTitleFont.family ();
  graphFonts << QString::number (plotTitleFont.pointSize ());
  graphFonts << QString::number (plotTitleFont.weight ());
  graphFonts << QString::number (plotTitleFont.italic ());
  settings.writeEntry ("/Fonts", graphFonts);
  settings.endGroup ();

  settings.beginGroup ("/Curves");
  settings.writeEntry ("/Style", defaultCurveStyle);
  settings.writeEntry ("/LineWidth", defaultCurveLineWidth);
  settings.writeEntry ("/SymbolSize", defaultSymbolSize);
  settings.endGroup ();

  settings.beginGroup ("/Ticks");
  settings.writeEntry ("/MajTicksStyle", majTicksStyle);
  settings.writeEntry ("/MinTicksStyle", minTicksStyle);
  settings.writeEntry ("/MinTicksLength", minTicksLength);
  settings.writeEntry ("/MajTicksLength", majTicksLength);
  settings.endGroup ();

  settings.beginGroup ("/Legend");
  settings.writeEntry ("/FrameStyle", legendFrameStyle);
  settings.writeEntry ("/TextColor", legendTextColor.name ());
  settings.writeEntry ("/BackgroundColor", legendBackground.name ());
  settings.endGroup ();

  settings.beginGroup ("/Arrows");
  settings.writeEntry ("/Width", defaultArrowLineWidth);
  settings.writeEntry ("/Color", defaultArrowColor.name ());
  settings.writeEntry ("/HeadLength", defaultArrowHeadLength);
  settings.writeEntry ("/HeadAngle", defaultArrowHeadAngle);
  settings.writeEntry ("/HeadFill", defaultArrowHeadFill);
  settings.writeEntry ("/LineStyle",
		       Graph::penStyleName (defaultArrowLineStyle));
  settings.endGroup ();
  settings.endGroup ();

  settings.beginGroup ("/3DPlots");
  settings.writeEntry ("/Legend", showPlot3DLegend);
  settings.writeEntry ("/Projection", showPlot3DProjection);
  settings.writeEntry ("/Antialiasing", smooth3DMesh);
  settings.writeEntry ("/Resolution", plot3DResolution);
  settings.writeEntry ("/Orthogonal", orthogonal3DPlots);

  QStringList plot3DFonts;
  plot3DFonts << plot3DTitleFont.family ();
  plot3DFonts << QString::number (plot3DTitleFont.pointSize ());
  plot3DFonts << QString::number (plot3DTitleFont.weight ());
  plot3DFonts << QString::number (plot3DTitleFont.italic ());
  plot3DFonts << plot3DNumbersFont.family ();
  plot3DFonts << QString::number (plot3DNumbersFont.pointSize ());
  plot3DFonts << QString::number (plot3DNumbersFont.weight ());
  plot3DFonts << QString::number (plot3DNumbersFont.italic ());
  plot3DFonts << plot3DAxesFont.family ();
  plot3DFonts << QString::number (plot3DAxesFont.pointSize ());
  plot3DFonts << QString::number (plot3DAxesFont.weight ());
  plot3DFonts << QString::number (plot3DAxesFont.italic ());
  settings.writeEntry ("/Fonts", plot3DFonts);

  settings.beginGroup ("/Colors");
  settings.writeEntry ("/MaxData", plot3DColors[0]);
  settings.writeEntry ("/Labels", plot3DColors[1]);
  settings.writeEntry ("/Mesh", plot3DColors[2]);
  settings.writeEntry ("/Grid", plot3DColors[3]);
  settings.writeEntry ("/MinData", plot3DColors[4]);
  settings.writeEntry ("/Numbers", plot3DColors[5]);
  settings.writeEntry ("/Axes", plot3DColors[6]);
  settings.writeEntry ("/Background", plot3DColors[7]);
  settings.endGroup ();
  settings.endGroup ();

  settings.beginGroup ("/Fitting");
  settings.writeEntry ("/OutputPrecision", fit_output_precision);
  settings.writeEntry ("/PasteResultsToPlot", pasteFitResultsToPlot);
  settings.writeEntry ("/WriteResultsToLog", writeFitResultsToLog);
  settings.writeEntry ("/GenerateUniformFitPoints", generateUniformFitPoints);
  settings.writeEntry ("/GenerateFunction", generateFunction);
  settings.writeEntry ("/Points", fitPoints);
  settings.writeEntry ("/GeneratePeakCurves", generatePeakCurves);
  settings.writeEntry ("/PeaksColor", peakCurvesColor);
  //settings.writeEntry ("/ScaleErrors", fit_scale_errors);
  settings.endGroup ();

  settings.beginGroup ("/ImportASCII");
  settings.writeEntry ("/ColumnSeparator", columnSeparator);
  settings.writeEntry ("/IgnoreLines", ignoredLines);
  settings.writeEntry ("/RenameColumns", renameColumns);
  settings.writeEntry ("/StripSpaces", strip_spaces);
  settings.writeEntry ("/SimplifySpaces", simplify_spaces);
  settings.endGroup ();

  settings.beginGroup("/FWHM");
  settings.writeEntry ("/peakFrac",fwhmPeakFrac);
  settings.writeEntry ("/halfFrac",fwhmHalfFrac);
  settings.endGroup();
}

void
ApplicationWindow::exportGraph ()
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return;

  if (w->isA ("MultiLayer"))
    {
      MultiLayer *plot = (MultiLayer *) w;
      if (plot->isEmpty ())
	{
	  QMessageBox::warning (this, tr ("QtiPlot - Warning"),
				tr
				("<h4>There are no plot layers available in this window!</h4>"));
	  return;
	}

      GraphExportDialog *ied = new GraphExportDialog (this, 0);
      ied->setDir (exportDir);
      if (ied->exec () == QDialog::Accepted)
	{
	  exportDir = ied->dirPath ();
	  QString fname = ied->selectedFile ();
	  QString selectedFilter = ied->selectedFilter ();

	  QFileInfo fi (fname);
	  QString baseName = fi.fileName ();

	  if (baseName.contains ('.') == 0)
	    fname.append (selectedFilter.remove ('*'));

	  if (QFile::exists (fname) &&
	      QMessageBox::question (0, tr ("QtiPlot - Overwrite File?"),
				     tr
				     ("A file called: <p><b>%1</b><p>already exists. "
				      "Do you want to overwrite it?").
				     arg (fname), tr ("&Yes"), tr ("&No"),
				     QString::null, 0, 1))
	    return;
	  else
	    {
	      QFile f (fname);
	      if (!f.open (IO_WriteOnly))
		{
		  QMessageBox::critical (0, tr ("QtiPlot - Export Error"),
					 tr
					 ("Could not write to file: <br><h4> %1 </h4><p>Please verify that you have the right to write to this location!").
					 arg (fname));
		  return;
		}

	      if (selectedFilter.contains (".svg"))
		{
		  plot->exportToSVG (fname);
		  return;
		}
	      else if (selectedFilter.contains (".eps"))
		{
		  if (ied->showExportOptions ())
		    {
		      epsExportDialog *ed =
			new epsExportDialog (fname, this, "exportDialog",
					     TRUE,
					     WStyle_Tool | WDestructiveClose);
		      connect (ed,
			       SIGNAL (exportToEPS
				       (const QString &, int,
					QPrinter::Orientation,
					QPrinter::PageSize,
					QPrinter::ColorMode)), plot,
			       SLOT (exportToEPS
				     (const QString &, int,
				      QPrinter::Orientation,
				      QPrinter::PageSize,
				      QPrinter::ColorMode)));

		      ed->showNormal ();
		      ed->setActiveWindow ();
		    }
		  else
		    plot->exportToEPS (fname);
		  return;
		}

	      QStringList list = QImage::outputFormatList ();
	      for (int i = 0; i < (int) list.count (); i++)
		{
		  if (selectedFilter.contains ("." + (list[i]).lower ()))
		    {
		      if (ied->showExportOptions ())
			{
			  imageExportDialog *ed =
			    new imageExportDialog (false, this,
						   "exportDialog", TRUE,
						   WStyle_Tool |
						   WDestructiveClose);
			  connect (ed,
				   SIGNAL (options
					   (const QString &, const QString &,
					    int, bool)), plot,
				   SLOT (exportImage
					 (const QString &, const QString &,
					  int, bool)));

			  ed->setExportPath (fname, list[i]);
			  ed->enableTransparency ();
			  ed->showNormal ();
			  ed->setActiveWindow ();
			}
		      else
			plot->exportImage (fname, list[i], 100, false);
		      return;
		    }
		}
	    }
	}
    }

  else if (w->isA ("Graph3D"))
    ((Graph3D *) w)->saveImage ();
}

void
ApplicationWindow::exportLayer ()
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) w)->activeGraph ();
  if (!g)
    return;

  GraphExportDialog *ied = new GraphExportDialog (this, 0);
  ied->setDir (exportDir);
  if (ied->exec () == QDialog::Accepted)
    {
      exportDir = ied->dirPath ();
      QString fname = ied->selectedFile ();
      QString selectedFilter = ied->selectedFilter ();

      QFileInfo fi (fname);
      QString baseName = fi.fileName ();

      if (baseName.contains ('.') == 0)
	fname.append (selectedFilter.remove ('*'));

      if (QFile::exists (fname) &&
	  QMessageBox::question (0, tr ("QtiPlot - Overwrite File?"),
				 tr
				 ("A file called: <p><b>%1</b><p>already exists. "
				  "Do you want to overwrite it?").arg (fname),
				 tr ("&Yes"), tr ("&No"), QString::null, 0,
				 1))
	return;
      else
	{
	  QFile f (fname);
	  if (!f.open (IO_WriteOnly))
	    {
	      QMessageBox::critical (0, tr ("QtiPlot - Export Error"),
				     tr
				     ("Could not write to file: <br><h4> %1 </h4><p>Please verify that you have the right to write to this location!").
				     arg (fname));
	      return;
	    }

	  if (selectedFilter.contains (".eps"))
	    {
	      if (ied->showExportOptions ())
		{
		  epsExportDialog *ed =
		    new epsExportDialog (fname, this, "exportDialog", TRUE,
					 WStyle_Tool | WDestructiveClose);
		  connect (ed,
			   SIGNAL (exportToEPS
				   (const QString &, int,
				    QPrinter::Orientation, QPrinter::PageSize,
				    QPrinter::ColorMode)), g,
			   SLOT (exportToEPS
				 (const QString &, int, QPrinter::Orientation,
				  QPrinter::PageSize, QPrinter::ColorMode)));

		  ed->showNormal ();
		  ed->setActiveWindow ();
		}
	      else
		g->exportToEPS (fname);

	      if (((MultiLayer *) w)->hasOverlapingLayers ())
		((MultiLayer *) w)->updateTransparency ();
	      return;
	    }
	  else if (selectedFilter.contains (".svg"))
	    {
	      g->exportToSVG (fname);
	      return;
	    }
	  /*else if (selectedFilter.contains(".wmf"))
	     {
	     g->exportToWmf(fname);
	     return;
	     } */
	  QStringList list = QImage::outputFormatList ();
	  for (int i = 0; i < (int) list.count (); i++)
	    {
	      if (selectedFilter.contains ("." + (list[i]).lower ()))
		{
		  if (ied->showExportOptions ())
		    {
		      imageExportDialog *ed =
			new imageExportDialog (false, this, "exportDialog",
					       TRUE,
					       WStyle_Tool |
					       WDestructiveClose);
		      connect (ed,
			       SIGNAL (options
				       (const QString &, const QString &, int,
					bool)), g,
			       SLOT (exportImage
				     (const QString &, const QString &, int,
				      bool)));

		      ed->setExportPath (fname, list[i]);
		      ed->enableTransparency ();
		      ed->showNormal ();
		      ed->setActiveWindow ();
		    }
		  else
		    g->exportImage (fname, list[i], 100, false);

		  if (((MultiLayer *) w)->hasOverlapingLayers ())
		    ((MultiLayer *) w)->updateTransparency ();
		  return;
		}
	    }
	}
    }
}

void
ApplicationWindow::exportAllGraphs ()
{
  QString dir =
    QFileDialog::getExistingDirectory (exportDir, this,
				       tr ("Get existing directory"),
				       tr
				       ("Choose a directory to export the graphs to"),
				       true, true);
  if (!dir.isEmpty ())
    {
      imageExportDialog *ed =
	new imageExportDialog (true, this, "exportDialog", TRUE,
			       WStyle_Tool | WDestructiveClose);
      connect (ed,
	       SIGNAL (exportAll
		       (const QString &, const QString &, int, bool)), this,
	       SLOT (exportAllGraphs
		     (const QString &, const QString &, int, bool)));

      exportDir = dir;
      ed->setExportDirPath (dir);
      ed->enableTransparency ();
      ed->showNormal ();
      ed->setActiveWindow ();
    }
}

void
ApplicationWindow::exportAllGraphs (const QString & dir,
				    const QString & format, int quality,
				    bool transparency)
{
  QApplication::setOverrideCursor (waitCursor);

  QWidgetList *windows = windowsList ();
  QString fileType = format;
  fileType.lower ();
  fileType.prepend (".");

  bool confirmOverwrite = true;

  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      MultiLayer *plot = 0;
      Graph3D *splot = 0;

      if (w->isA ("MultiLayer"))
	plot = (MultiLayer *) w;
      else if (w->isA ("Graph3D"))
	splot = (Graph3D *) w;

      if (plot || splot)
	{
	  QString fileName = dir + "/" + ((myWidget *)w)->getName() + fileType;
	  QFile f (fileName);
	  if (f.exists (fileName) && confirmOverwrite)
	    {
	      QApplication::restoreOverrideCursor ();
	      switch (QMessageBox::
		      question (0, tr ("QtiPlot - Overwrite File?"),
				tr
				("A file called: <p><b>%1</b><p>already exists. "
				 "Do you want to overwrite it?").
				arg (fileName), tr ("&Yes"), tr ("&All"),
				tr ("&Cancel"), 0, 1))
		{
		case 0:
		  if (plot)
		    export2DPlotToFile (plot, fileName, format, quality,
					transparency);
		  else if (splot)
		    export3DPlotToFile (splot, fileName, format);
		  break;

		case 1:
		  confirmOverwrite = false;
		  if (plot)
		    export2DPlotToFile (plot, fileName, format, quality,
					transparency);
		  else if (splot)
		    export3DPlotToFile (splot, fileName, format);
		  break;

		case 2:
		  return;
		  break;
		}
	    }
	  else
	    {
	      if (plot)
		export2DPlotToFile (plot, fileName, format, quality,
				    transparency);
	      else if (splot)
		export3DPlotToFile (splot, fileName, format);
	    }
	}
    }

  delete windows;
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::export2DPlotToFile (MultiLayer * plot,
				       const QString & fileName,
				       const QString & format, int quality,
				       bool transparency)
{
  if (plot->isEmpty ())
    {
      QApplication::restoreOverrideCursor ();
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("There are no plot layers available in window <b>"
			     + plot->getName() +
			     "</b>.<br>Graph window not exported!"));
      return;
    }

  QFile f (fileName);
  if (!f.open (IO_WriteOnly))
    {
      QApplication::restoreOverrideCursor ();
      QMessageBox::critical (0, tr ("QtiPlot - Export Error"),
			     tr
			     ("Could not write to file: <br><h4>%1</h4><p>Please verify that you have the right to write to this location!").
			     arg (fileName));

      return;
    }

  if (format == "EPS")
    plot->exportToEPS (fileName);
  else
    plot->exportImage (fileName, format, quality, transparency);
}

void
ApplicationWindow::export3DPlotToFile (Graph3D * plot,
				       const QString & fileName,
				       const QString & format)
{
  QFile f (fileName);
  if (!f.open (IO_WriteOnly))
    {
      QMessageBox::critical (0, tr ("QtiPlot - Export Error"),
			     tr
			     ("Could not write to file: <br><h4>%1</h4><p>Please verify that you have the right to write to this location!").
			     arg (fileName));

      QApplication::restoreOverrideCursor ();
      return;
    }

  plot->saveImageToFile (fileName, format);
}

QString
ApplicationWindow::windowGeometryInfo (QWidget * w)
{
  QString s = "geometry\t";

  if (((myWidget *) w)->status () == myWidget::Maximized)
    {
      s += "20\t20\t";
      s += QString::number(w->parentWidget()->frameGeometry().width())+"\t";
      s += QString::number(w->parentWidget()->frameGeometry().height())+"\t";
      if (w == ws->activeWindow ())
	return s + "maximized\tactive\n";
      else
	return s + "maximized\n";
    }

  if (!w->parent ())
    s += "0\t0\t500\t450\t";  
  else
    {
      QPoint p = w->parentWidget ()->pos ();	// store position
      s += QString::number(p.x ()) + "\t";
      s += QString::number(p.y ()) + "\t";
      s += QString::number(w->parentWidget()->frameGeometry().width())+"\t";
      s += QString::number(w->parentWidget()->frameGeometry().height())+"\t";
    }
  if (((myWidget *) w)->status () == myWidget::Minimized)
    return s + "minimized\n";
  
  bool hide = hidden (w);
  if (w == ws->activeWindow() && !hide)
    s += "active\n";
  else if (hide)
    s += "hidden\n";
  else
    s += "\n";

  return s;
}

Folder *
ApplicationWindow::projectFolder ()
{
  return ((FolderListItem *) folderview->firstChild ())->folder ();
}

bool
ApplicationWindow::saveProject ()
{
  if (projectname == "untitled" || projectname.endsWith(".opj",false))
    {
      saveProjectAs ();
      return false;
    }

  saveFolder (projectFolder (), projectname);

  QFileInfo fi(projectname);
  setCaption ("QtiPlot - " + fi.fileName());

  saved = TRUE;
  actionSaveProject->setEnabled (false);
  actionUndo->setEnabled (false);
  actionRedo->setEnabled (false);

  if (autoSave)
    {
      if (savingTimerId)
	killTimer (savingTimerId);
      savingTimerId = startTimer (autoSaveTime * 60000);
    }
  else
    savingTimerId = 0;

  QApplication::restoreOverrideCursor ();
  return true;
}

void
ApplicationWindow::saveProjectAs ()
{
  QString filter = tr ("QtiPlot project") + " (*.qti);;";
  filter += tr ("Compressed QtiPlot project") + " (*.qti.gz)";

  QString selectedFilter;
  QString fn =
    QFileDialog::getSaveFileName (workingDir, filter, this, "project",
				  tr ("Save Project As"), &selectedFilter,
				  false);
  if (!fn.isEmpty ())
    {
      QFileInfo fi (fn);
      workingDir = fi.dirPath (true);
      //QString filename = fi.fileName();
      if(!fn.contains(QRegExp("\\.(qti|qti\\.gz)$")))
	fn.append(".qti");
      if(selectedFilter.contains(".gz")&&!fn.endsWith(".gz",true))
	fn.append(".gz");

      if(QFile::exists (fn) &&
	 QMessageBox::question (this, tr ("QtiPlot -- Overwrite File? "),
				tr("A file called: <p><b>%1</b><p>already exists.\n"
				   "Do you want to overwrite it?").arg (fn),
				tr ("&Yes"), tr ("&No"), QString::null, 0, 1))
	return;
      
      //delete old lock file
      QString lockfile = projectname + ".lock";
      if(QFile::exists(lockfile))
	QFile::remove(lockfile);

      projectname = fn;
      if (saveProject ())
	{
	  recentProjects.remove (projectname);
	  recentProjects.push_front (projectname);
	  updateRecentProjectsList ();
	  
	  QFileInfo fi (fn);
	  QString baseName = fi.baseName (true);
	  FolderListItem *item =
	    (FolderListItem *) folderview->firstChild ();
	  item->setText (0, baseName);
	  item->folder ()->setFolderName (baseName);

	  //generate new lock file
	  QFile f(projectname+".lock");
	  f.open(IO_WriteOnly);
	  f.close();
	}
    }
}

void
ApplicationWindow::saveNoteAs ()
{
  Note *w = (Note *) ws->activeWindow ();
  if (!w)
    return;

  QString filter = " *.txt;;";
  if (QString (scriptEnv->name ()) == "Python")
    filter += tr ("Python Source") + " (*.py);;";
  filter += tr ("All Files") + " (*)";

  QString selectedFilter;
  QString fn =
    QFileDialog::getSaveFileName (workingDir + "/" + w->getName(), filter, this,
				  0,
				  tr ("Save Notes As"), &selectedFilter,
				  false);
  if (!fn.isEmpty ())
    {
      QFileInfo fi (fn);
      workingDir = fi.dirPath (true);
      QString baseName = fi.fileName ();
      if (!baseName.contains ('.'))
	{
	  if (selectedFilter.contains (".txt"))
	    fn.append (".txt");
	  else if (selectedFilter.contains (".py"))
	    fn.append (".py");
	}

      if (QFile::exists (fn) &&
	  QMessageBox::question (this, tr ("QtiPlot -- Overwrite File? "),
				 tr
				 ("A file called: <p><b>%1</b><p>already exists.\n"
				  "Do you want to overwrite it?").arg (fn),
				 tr ("&Yes"), tr ("&No"), QString::null, 0,
				 1))
	return;
      else
	{
	  QFile f (fn);
	  if (!f.open (IO_WriteOnly))
	    {
	      QMessageBox::critical (0, tr ("QtiPlot - File Save Error"),
				     tr
				     ("Could not write to file: <br><h4> %1 </h4><p>Please verify that you have the right to write to this location!").
				     arg (fn));
	      return;
	    }
	  QApplication::setOverrideCursor (waitCursor);
	  QTextStream t (&f);
	  t.setEncoding (QTextStream::UnicodeUTF8);
	  t << w->text ();
	  f.close ();
	  QApplication::restoreOverrideCursor ();
	}
    }
}

void
ApplicationWindow::saveAsTemplate ()
{
  myWidget *w = (myWidget *) ws->activeWindow ();
  if (!w)
    return;

  QString filter;
  if (w->isA ("Matrix"))
    filter = tr ("QtiPlot Matrix Template") + " (*.qmt)";
  else if (w->isA ("MultiLayer"))
    filter = tr ("QtiPlot 2D Graph Template") + " (*.qpt)";
  else if (w->isA ("Table"))
    filter = tr ("QtiPlot Table Template") + " (*.qtt)";
  else if (w->isA ("Graph3D"))
    filter = tr ("QtiPlot 3D Surface Template") + " (*.qst)";

  QString selectedFilter;
  QString fn =
    QFileDialog::getSaveFileName (templatesDir + "/" + w->getName(), filter,
				  this, 0,
				  tr ("Save Window As Template"),
				  &selectedFilter, false);
  if (!fn.isEmpty ())
    {
      QFileInfo fi (fn);
      templatesDir = fi.dirPath (true);
      QString baseName = fi.fileName ();
      if (!baseName.contains ('.'))
	{
	  selectedFilter = selectedFilter.right (5).left (4);
	  fn.append (selectedFilter);
	}

      if (QFile::exists (fn) &&
	  QMessageBox::question (this, tr ("QtiPlot -- Overwrite File? "),
				 tr
				 ("A file called: <p><b>%1</b><p>already exists.\n"
				  "Do you want to overwrite it?").arg (fn),
				 tr ("&Yes"), tr ("&No"), QString::null, 0,
				 1))
	return;
      else
	{
	  QFile f (fn);
	  if (!f.open (IO_WriteOnly))
	    {
	      QMessageBox::critical (0, tr ("QtiPlot - Export Error"),
				     tr
				     ("Could not write to file: <br><h4> %1 </h4><p>Please verify that you have the right to write to this location!").
				     arg (fn));
	      return;
	    }
	  QApplication::setOverrideCursor (waitCursor);
	  QString text =
	    "QtiPlot " + QString::number (majVersion) + "." +
	    QString::number (minVersion) + "." +
	    QString::number (patchVersion) + " template file\n";
	  text += w->saveAsTemplate (windowGeometryInfo (w));
	  QTextStream t (&f);
	  t.setEncoding (QTextStream::UnicodeUTF8);
	  t << text;
	  f.close ();
	  QApplication::restoreOverrideCursor ();
	}
    }
}

void
ApplicationWindow::rename ()
{
  myWidget *m = (myWidget *) ws->activeWindow ();
  if (!m)
    return;

  renameWindowDialog *rwd =
    new renameWindowDialog (this, "polyDialog", TRUE,
			    WStyle_Tool | WDestructiveClose);
  rwd->setWidget (m);
  rwd->showNormal ();
  rwd->setActiveWindow ();
}

void
ApplicationWindow::renameWindow ()
{
  WindowListItem *it = (WindowListItem *) listview->currentItem ();
  myWidget *w = it->window ();
  if (!w)
    return;

  renameWindowDialog *rwd =
    new renameWindowDialog (this, "polyDialog", TRUE,
			    WStyle_Tool | WDestructiveClose);
  rwd->setWidget (w);
  rwd->showNormal ();
  rwd->setActiveWindow ();
}

void
ApplicationWindow::renameWindow (QListViewItem * item)
{
  myWidget *w = ((WindowListItem *)item)->window ();
  if (!w)
    return;

  renameWindowDialog *rwd =
    new renameWindowDialog (this, "polyDialog", TRUE,
			    WStyle_Tool | WDestructiveClose);
  rwd->setWidget (w);
  rwd->showNormal ();
  rwd->setActiveWindow ();
}

void
ApplicationWindow::renameWindow (QListViewItem * item, int,
				 const QString & text)
{
  if (!item)
    return;

  myWidget *w = ((WindowListItem *) item)->window ();
  QString name = w->getName();

  if (!w || text == name)
    return;

  while (!renameWindow (w, text))
    {
      //item->setText(0,name);
      item->setRenameEnabled (0, true);
      item->startRename (0);
      return;
    }
}

bool
ApplicationWindow::renameWindow (myWidget * w, const QString & text)
{
  if (!w)
    return false;

  QString name = w->getName();

  QString newName = text;
  newName.replace ("-", "_");
  if (newName.isEmpty ())
    {
      QMessageBox::critical (0, tr ("QtiPlot - Error"),
			     tr ("Please enter a valid name!"));
      return false;
    }
  //else if (newName.contains (QRegExp ("\\W")))
  else if (newName.contains (QRegExp ("[\\/\\?\\\\\\*\\|<>,;:\"]")))
    {
      QMessageBox::critical (0, tr ("QtiPlot - Error"),
			     tr
			     ("You can not use \\/:,;*?<>|\" for window name!")
			     + "<p>" + tr ("Please choose another name!"));
      return false;
    }

  newName.replace ("_", "-");

  if (alreadyUsedName (newName))
    {
      QMessageBox::critical (this, tr ("QtiPlot - Error"),
			     tr ("Name <b>%1</b> already exists!").
			     arg (newName) + "<p>" +
			     tr ("Please choose another name!") + "<p>" +
			     tr
			     ("Warning: for internal consistency reasons the underscore character is replaced with a minus sign."));
      return false;
    }

  if (w->inherits ("Table"))
    {
      QStringList labels = ((Table *) w)->colNames ();
      if (labels.contains (newName) > 0)
	{
	  QMessageBox::critical (0, tr ("QtiPlot - Error"),
				 tr
				 ("The table name must be different from the names of its columns!")
				 + "<p>" +
				 tr ("Please choose another name!"));
	  return false;
	}

      int id = tableWindows.findIndex (name);
      tableWindows[id] = newName;
      updateTableNames (name, newName);    
    }
  else if (w->isA ("Matrix"))
    changeMatrixName (name, newName);

  w->setName (newName);
  w->setCaptionPolicy (w->captionPolicy ());
  return true;
}

QStringList
ApplicationWindow::columnsList (Table::PlotDesignation plotType)
{
  QWidgetList *windows = windowsList ();
  QStringList list;
  //for (int i = 0; i < (int) windows->count (); i++)
  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      if (!w->inherits ("Table"))
	continue;

      Table *t = (Table *)w;
      for (int j = 0; j < t->tableCols (); j++)
	if (t->colPlotDesignation (j) == plotType || plotType == Table::All)
	  list << t->getName() + "_" + t->colLabel (j);
    }
  list.sort();
  delete windows;
  return list;
}

void
ApplicationWindow::showCurvesDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  if (((MultiLayer *) ws->activeWindow ())->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Error"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      return;
    }

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g)
    return;

  if (g->isPiePlot ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Error"),
			    tr
			    ("This functionality is not available for pie plots!"));
    }
  else
    {
      activeGraph = g;

      curvesDialog *crvDialog =
	new curvesDialog (this, "curves", TRUE,
			  WStyle_StaysOnTop | WStyle_Tool |
			  WDestructiveClose);
      connect (crvDialog, SIGNAL (showPlotAssociations (int)), this,
	       SLOT (showPlotAssociations (int)));
      connect (crvDialog, SIGNAL (showFunctionDialog (Graph *, int)), this,
	       SLOT (showFunctionDialog (Graph *, int)));

      crvDialog->insertCurvesToDialog ();
      crvDialog->setCurveDefaultSettings (defaultCurveStyle,
					  defaultCurveLineWidth,
					  defaultSymbolSize);
      crvDialog->setGraph (g);
      crvDialog->initTablesList (tableList ());
      crvDialog->showNormal ();
      crvDialog->setActiveWindow ();
    }
}

QWidgetList *
ApplicationWindow::tableList ()
{
  QWidgetList *lst = new QWidgetList ();
  QWidgetList *windows = windowsList ();
  for(QWidget *w = windows->first(); w; w = windows->next())
    if (w->inherits ("Table"))
      lst->append(w);

  delete windows;
  return lst;
}

void
ApplicationWindow::showPlotAssociations (int curve)
{
  if (!activeGraph)
    return;

  associationsDialog *ad =
    new associationsDialog (this, "curves", true,
			    WStyle_StaysOnTop | WStyle_Tool |
			    WDestructiveClose);
  ad->setGraph (activeGraph);
  ad->initTablesList (tableList (), curve);
  ad->showNormal ();
  ad->setActiveWindow ();
}

void
ApplicationWindow::showTitleDialog ()
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return;

  if (w->isA ("MultiLayer"))
    {
      Graph *g = ((MultiLayer *) w)->activeGraph ();
      if (g)
	{
	  TextDialog *td =
	    new TextDialog (TextDialog::AxisTitle, this, "TextDialog", TRUE,
			    WStyle_Tool | WDestructiveClose);
	  connect (td, SIGNAL (changeFont (const QFont &)), g,
		   SLOT (setTitleFont (const QFont &)));
	  connect (td, SIGNAL (changeText (const QString &)), g,
		   SLOT (setTitle (const QString &)));
	  connect (td, SIGNAL (changeColor (const QColor &)), g,
		   SLOT (setTitleColor (const QColor &)));
	  connect (td, SIGNAL (changeAlignment (int)), g,
		   SLOT (setTitleAlignment (int)));

	  QwtText t = g->plotWidget ()->title ();
	  td->setText (t.text ());
	  td->setFont (t.font ());
	  td->setTextColor (t.color ());
	  td->setAlignment (t.renderFlags ());
	  td->showNormal ();
	  td->setActiveWindow ();
	}
    }
  else if (w->isA ("Graph3D"))
    {
      plot3DDialog *pd = (plot3DDialog *) showPlot3dDialog ();
      if (pd)
	pd->showTitleTab ();
    }
}

void
ApplicationWindow::showXAxisTitleDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    {
      TextDialog *td =
	new TextDialog (TextDialog::AxisTitle, this, "axesDialog", TRUE,
			WStyle_Tool | WDestructiveClose);
      connect (td, SIGNAL (changeFont (const QFont &)), g,
	       SLOT (setXAxisTitleFont (const QFont &)));
      connect (td, SIGNAL (changeText (const QString &)), g,
	       SLOT (setXAxisTitle (const QString &)));
      connect (td, SIGNAL (changeColor (const QColor &)), g,
	       SLOT (setXAxisTitleColor (const QColor &)));
      connect (td, SIGNAL (changeAlignment (int)), g,
	       SLOT (setXAxisTitleAlignment (int)));

      QStringList t = g->scalesTitles ();
      td->setText (t[0]);
      td->setFont (g->axisTitleFont (2));
      td->setTextColor (g->axisTitleColor (2));
      td->setAlignment (g->axisTitleAlignment (2));
      td->setCaption (tr ("QtiPlot - X Axis Title"));
      td->showNormal ();
      td->setActiveWindow ();
    }
}

void
ApplicationWindow::showYAxisTitleDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    {
      TextDialog *td =
	new TextDialog (TextDialog::AxisTitle, this, "axesDialog", TRUE,
			WStyle_Tool | WDestructiveClose);
      connect (td, SIGNAL (changeFont (const QFont &)), g,
	       SLOT (setYAxisTitleFont (const QFont &)));
      connect (td, SIGNAL (changeText (const QString &)), g,
	       SLOT (setYAxisTitle (const QString &)));
      connect (td, SIGNAL (changeColor (const QColor &)), g,
	       SLOT (setYAxisTitleColor (const QColor &)));
      connect (td, SIGNAL (changeAlignment (int)), g,
	       SLOT (setYAxisTitleAlignment (int)));

      QStringList t = g->scalesTitles ();
      td->setText (t[1]);
      td->setFont (g->axisTitleFont (0));
      td->setTextColor (g->axisTitleColor (0));
      td->setAlignment (g->axisTitleAlignment (0));
      td->setCaption (tr ("QtiPlot - Y Axis Title"));
      td->showNormal ();
      td->setActiveWindow ();
    }
}

void
ApplicationWindow::showRightAxisTitleDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    {
      TextDialog *td =
	new TextDialog (TextDialog::AxisTitle, this, "axesDialog", TRUE,
			WStyle_Tool | WDestructiveClose);
      connect (td, SIGNAL (changeFont (const QFont &)), g,
	       SLOT (setRightAxisTitleFont (const QFont &)));
      connect (td, SIGNAL (changeText (const QString &)), g,
	       SLOT (setRightAxisTitle (const QString &)));
      connect (td, SIGNAL (changeColor (const QColor &)), g,
	       SLOT (setRightAxisTitleColor (const QColor &)));
      connect (td, SIGNAL (changeAlignment (int)), g,
	       SLOT (setRightAxisTitleAlignment (int)));

      QStringList t = g->scalesTitles ();
      td->setText (t[3]);
      td->setFont (g->axisTitleFont (1));
      td->setTextColor (g->axisTitleColor (1));
      td->setAlignment (g->axisTitleAlignment (1));
      td->setCaption (tr ("QtiPlot - Right Axis Title"));
      td->showNormal ();
      td->setActiveWindow ();
    }
}

void
ApplicationWindow::showTopAxisTitleDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    {
      TextDialog *td =
	new TextDialog (TextDialog::AxisTitle, this, "axesDialog", TRUE,
			WStyle_Tool | WDestructiveClose);
      connect (td, SIGNAL (changeFont (const QFont &)), g,
	       SLOT (setTopAxisTitleFont (const QFont &)));
      connect (td, SIGNAL (changeText (const QString &)), g,
	       SLOT (setTopAxisTitle (const QString &)));
      connect (td, SIGNAL (changeColor (const QColor &)), g,
	       SLOT (setTopAxisTitleColor (const QColor &)));
      connect (td, SIGNAL (changeAlignment (int)), g,
	       SLOT (setTopAxisTitleAlignment (int)));

      QStringList t = g->scalesTitles ();
      td->setText (t[2]);
      td->setFont (g->axisTitleFont (3));
      td->setTextColor (g->axisTitleColor (3));
      td->setAlignment (g->axisTitleAlignment (3));
      td->setCaption (tr ("QtiPLot - Top Axis Title"));
      td->showNormal ();
      td->setActiveWindow ();
    }
}

void
ApplicationWindow::showExportASCIIDialog ()
{
  if (ws->activeWindow () && ws->activeWindow ()->inherits ("Table"))
    {
      exportDialog *ed =
	new exportDialog (this, "exportDialog", TRUE,
			  WStyle_ContextHelp | WDestructiveClose);
      connect (ed,
	       SIGNAL (exportTable
		       (const QString &, const QString &, bool, bool)), this,
	       SLOT (exportASCII
		     (const QString &, const QString &, bool, bool)));
      connect (ed, SIGNAL (exportAllTables (const QString &, bool, bool)),
	       this, SLOT (exportAllTables (const QString &, bool, bool)));

      ed->setTableNames (tableWindows);
      ed->setActiveTableName ( ((myWidget *)ws->activeWindow ())->getName() );
      ed->setColumnSeparator (columnSeparator);
      ed->showNormal ();
      ed->setActiveWindow ();
    }
}

void
ApplicationWindow::exportAllTables (const QString & sep, bool colNames,
				    bool expSelection)
{
  QString dir =
    QFileDialog::getExistingDirectory (exportDir, this,
				       "get existing directory",
				       tr
				       ("Choose a directory to export the tables to"),
				       true, true);
  if (!dir.isEmpty ())
    {
      QApplication::setOverrideCursor (waitCursor);
      QWidgetList *windows = windowsList ();
      exportDir = dir;

      bool confirmOverwrite = true;
      bool success = true;
      for (QWidget * w = windows->first (); w; w = windows->next ())
	{
	  if (w->inherits ("Table"))
	    {
	      Table *t = (Table *) w;
	      QString fileName = dir + "/" + t->getName() + ".txt";
	      QFile f (fileName);
	      if (f.exists (fileName) && confirmOverwrite)
		{
		  QApplication::restoreOverrideCursor ();
		  switch(QMessageBox::
			 question (0, tr ("QtiPlot - Overwrite File?"),
				    tr("A file called: <p><b>%1</b><p>already exists. "
				       "Do you want to overwrite it?").
				   arg (fileName), tr ("&Yes"), tr ("&All"),
				   tr ("&Cancel"), 0, 1))
		    {
		    case 0:
		      success = t->exportToASCIIFile (fileName, sep, colNames,
						      expSelection);
		      break;

		    case 1:
		      confirmOverwrite = false;
		      success =	t->exportToASCIIFile (fileName, sep, colNames,
						      expSelection);
		      break;

		    case 2:
		      return;
		      break;
		    }
		}
	      else
		success = t->exportToASCIIFile (fileName, sep, colNames,
						expSelection);

	      if (!success)
		break;
	    }
	}
      delete windows;
      QApplication::restoreOverrideCursor ();
    }
}

void
ApplicationWindow::exportASCII (const QString & tableName,
				const QString & sep, bool colNames,
				bool expSelection)
{
  Table *t = table (tableName);
  if (!t)
    return;

  QString selectedFilter;
  QString fname =
    QFileDialog::getSaveFileName (asciiDirPath,
				  "*.txt;;*.dat;;*.DAT;;" + tr ("All files") +
				  " *;;", this, "file dialog",
				  tr ("Choose a filename to save under"),
				  &selectedFilter, TRUE);

  if (!fname.isEmpty ())
    {				// the user gave a file name  
      QFileInfo fi (fname);
      QString baseName = fi.fileName ();
      if (baseName.contains ('.') == 0)
	fname.append (selectedFilter.remove ('*'));

      asciiDirPath = fi.dirPath (true);

      if (QFile::exists (fname) &&
	  QMessageBox::question (0, tr ("QtiPlot - Overwrite File?"),
				 tr
				 ("A file called: <p><b>%1</b><p>already exists. "
				  "Do you want to overwrite it?").arg (fname),
				 tr ("&Yes"), tr ("&No"), QString::null, 0,
				 1))
	return;
      else
	{
	  QApplication::setOverrideCursor (waitCursor);
	  t->exportToASCIIFile (fname, sep, colNames, expSelection);
	  QApplication::restoreOverrideCursor ();
	}
    }
}

void
ApplicationWindow::showRowsDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  bool ok;
  int rows =
    QInputDialog::getInteger (tr ("QtiPlot - Enter rows number"), tr ("Rows"),
			      ((Table *) ws->activeWindow ())->tableRows (),
			      0, 1000000, 1, &ok, this);
  if (ok)
    ((Table *) ws->activeWindow ())->resizeRows (rows);
}

void
ApplicationWindow::showColsDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  bool ok;
  int cols =
    QInputDialog::getInteger (tr ("QtiPlot - Enter columns number"),
			      tr ("Columns"),
			      ((Table *) ws->activeWindow ())->tableCols (),
			      0, 1000000, 1, &ok, this);
  if (ok)
    ((Table *) ws->activeWindow ())->resizeCols (cols);
}

void
ApplicationWindow::showColumnValuesDialog ()
{
  Table *w = (Table *) ws->activeWindow ();
  if (w && w->isA ("Table"))
    {
      if (int (w->selectedColumns ().count ()) > 0
	  || w->table ()->currentSelection () >= 0)
	{
	  setColValuesDialog *vd =
	    new setColValuesDialog (scriptEnv, this, "valuesDialog", TRUE,
				    WDestructiveClose);
	  vd->setTable (w);
	  vd->showNormal ();
	  vd->setActiveWindow ();
	}
      else
	QMessageBox::warning (this, tr ("QtiPlot - Column selection error"),
			      tr ("Please select a column first!"));
    }
}

void
ApplicationWindow::recalculateTable ()
{
  Table *w = (Table *) ws->activeWindow ();
  if (w && w->isA ("Table"))
    w->calculate ();
}

void
ApplicationWindow::invertColumns ()
{
  Table *w = (Table *) ws->activeWindow ();
  if (w && w->isA ("Table"))
    w->invertColumns ();
}

void
ApplicationWindow::sortActiveTable ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  if (int (((Table *) ws->activeWindow ())->selectedColumns ().count ()) > 0)
    ((Table *) ws->activeWindow ())->sortTableDialog ();
  else
    QMessageBox::warning (this, "QtiPlot - Column selection error",
			  "Please select a column first!");
}

void
ApplicationWindow::sortSelection ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  ((Table *) ws->activeWindow ())->sortColumnsDialog ();
}

void
ApplicationWindow::generateFormula()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  Table *t = (Table *)ws->activeWindow();
  if ( t->selectedColsNumber() < 2 )
    {
      QMessageBox::critical(this, "QtiPlot - Column selection error",
			    "You need more than 1 column selected to do this operation.");
      return;
    }

  t->generateFormula();
}

void
ApplicationWindow::normalizeActiveTable ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  if (int (((Table *) ws->activeWindow ())->selectedColumns ().count ()) > 0)
    ((Table *) ws->activeWindow ())->normalizeTable ();
  else
    QMessageBox::warning (this, "QtiPlot - Column selection error",
			  "Please select a column first!");
}

void
ApplicationWindow::normalizeSelection ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  if (int (((Table *) ws->activeWindow ())->selectedColumns ().count ()) > 0)
    ((Table *) ws->activeWindow ())->normalizeSelection ();
  else
    QMessageBox::warning (this, "QtiPlot - Column selection error",
			  "Please select a column first!");
}

void
ApplicationWindow::correlate ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  ((Table *) ws->activeWindow ())->correlate ();
}

void
ApplicationWindow::convolute ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  ((Table *) ws->activeWindow ())->convolute (1);
}

void
ApplicationWindow::deconvolute ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  ((Table *) ws->activeWindow ())->convolute (-1);
}

void
ApplicationWindow::showColStatistics ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;
  Table *t = (Table *) ws->activeWindow ();

  if (t->selectedColsNumber() > 0)
    {
      QValueList < int >targets;
      for (int i = 0; i < t->tableCols (); i++)
	if (t->isColumnSelected (i, true))
	  targets << i;
      newTableStatistics (t, TableStatistics::column, targets)->showNormal ();
    }
  else if (t->selectedRows() > 0)
    {
      QString text = t->partialColStatistics(tbl_output_precision);
      if(text!=QString::null)
	{
	  QString caption = generateUniqueName("Table") + "\t" + 
	    "Partial Statistics of " + t->getName();
	  Table *w = newTable( caption, t->tableCols(), 10, text );
	  //QStringList ct;
	  //ct << tr("1") << tr("0;0/")+QString::number(tbl_output_precision);
	  //w->setColumnTypes( ct );
	}
    }
  else
    QMessageBox::warning (this, tr ("QtiPlot - Column statistics error"),
			  tr ("Please select rows or columns first!"));
}

void
ApplicationWindow::showRowStatistics ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;
  Table *t = (Table *) ws->activeWindow ();

  if (t->selectedRows () > 0)
    {
      QValueList < int >targets;
      for (int i = 0; i < t->tableRows (); i++)
	if (t->isRowSelected (i, true))
	  targets << i;
      newTableStatistics (t, TableStatistics::row, targets)->showNormal ();
    }
  else if(t->selectedColsNumber() > 0)
    {
      QString text = t->partialRowStatistics(tbl_output_precision);
      if(text!=QString::null)
	{
	  QString caption = generateUniqueName("Table") + "\t" + 
	    "Partial Statistics of " + t->getName();
	  Table *w = newTable( caption, t->tableRows(), 10, text );
	  //QStringList ct;
	  //ct << tr("1") << tr("0;0/")+QString::number(tbl_output_precision);
	  //w->setColumnTypes( ct );
	}
    }
  else
    QMessageBox::warning (this, tr ("QtiPlot - Row statistics error"),
			  tr ("Please select rows or columns first!"));
}

void
ApplicationWindow::showXYStatistics()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;
  Table *t = (Table *) ws->activeWindow ();

  QStringList colslist = t->selectedYColumns();
  if(colslist.count()!=1)
    {
      QMessageBox::critical (0, tr ("QtiPlot - Error"),
			     tr("You can select only one Y column for this operation.") );
      return;
    }
  int ycol = t->colIndex (colslist[0]);
  int xcol = t->colX(ycol);
  if(xcol == -1)
    {
      QMessageBox::critical (0, tr ("QtiPlot - Error"),
			     tr("You need X column for the selected Y column.") );
      return;
    }

  QString text = t->xyStatistics(xcol, ycol, tbl_output_precision);
  if(text!=QString::null)
    {
      QString caption = generateUniqueName("Table") + "\t" + 
	"XY Statistics of " + colslist[0];
      Table *t2 = newTable( caption, text.contains('\n')-1, 6, text );
      t2->sortTable( 1, 0, t2->colName(0) );
      QStringList ct;
      ct << t->columnFormat(xcol) << tr("0;0/")+QString::number(tbl_output_precision);
      t2->setColumnTypes( ct );
    }
}



void
ApplicationWindow::showMatrixAreaStatistics()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA("Matrix"))
    return;

  Matrix *m = (Matrix *) ws->activeWindow ();

  QString text = m->areaStatistics(tbl_output_precision);
  if(text!=QString::null)
    {
      //updateLog( text );
      QString caption = generateUniqueName("AreaStat") + "\t" +
	tr("Area Statistics of ")+m->getName();
      Table *w = newTable( caption, 8, 2, text );
      QStringList ct;
      ct << tr("1") << tr("0;0/")+QString::number(tbl_output_precision);
      w->setColumnTypes( ct );
    }
}

void
ApplicationWindow::showTableAreaStatistics()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA("Table"))
    return;

  Table *t = (Table *) ws->activeWindow ();

  QString text = t->areaStatistics(tbl_output_precision);
  if(text!=QString::null)
    {
      //updateLog( text );
      QString caption = generateUniqueName("AreaStat") + "\t" +
	tr("Area Statistics of ")+t->getName();
      Table *w = newTable( caption, 8, 2, text );
      QStringList ct;
      ct << tr("1") << tr("0;0/")+QString::number(tbl_output_precision);
      w->setColumnTypes( ct );
    }
}

void
ApplicationWindow::showMomentAnalysis()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;
  Table *t = (Table *) ws->activeWindow ();

  QString text = t->momentAnalysis(tbl_output_precision);
  if(text!=QString::null)
    {
      int cols = text.contains('\n');
      QString caption = generateUniqueName("Moment") + "\t" +
	tr("Moment Analysis of ")+t->getName();
      Table *w = newTable( caption, cols-1, 7, text );
      QStringList ct;
      ct << "1";
      for (int i=0;i<6; i++)
        ct << tr("0;0/")+QString::number(tbl_output_precision);
      w->setColumnTypes( ct );
    }
}


void
ApplicationWindow::showColMenu (int c)
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  Table *w = (Table *) ws->activeWindow ();

  QPopupMenu Legend (this);
  QPopupMenu plot (this);
  QPopupMenu specialPlot (this);
  QPopupMenu fill (this);
  QPopupMenu sorting (this);
  QPopupMenu colType (this);
  colType.setCheckable (true);
  QPopupMenu panels (this);
  QPopupMenu stat (this);
  QPopupMenu norm (this);
  QPopupMenu statistics (this);

  if ((int) w->selectedColumns ().count () == 1)
    {
      w->setSelectedCol (c);
      plot.insertItem (QPixmap (lPlot_xpm), tr ("&Line"), w, SLOT (plotL ()));
      plot.insertItem (QPixmap (pPlot_xpm), tr ("&Scatter"), w,
		       SLOT (plotP ()));
      plot.insertItem (QPixmap (lpPlot_xpm), tr ("Line + s&ymbol"), w,
		       SLOT (plotLP ()));

      specialPlot.insertItem (QPixmap (dropLines_xpm),
			      tr ("Vertical &drop lines"), w,
			      SLOT (plotVerticalDropLines ()));
      specialPlot.insertItem (QPixmap (spline_xpm), tr ("&Spline"), w,
			      SLOT (plotSpline ()));
      specialPlot.insertItem (QPixmap (vert_steps_xpm),
			      tr ("&Vertical Steps"), w,
			      SLOT (plotVertSteps ()));
      specialPlot.insertItem (QPixmap (hor_steps_xpm),
			      tr ("&Horizontal Steps"), w,
			      SLOT (plotHorSteps ()));
      plot.insertItem (tr ("Special Line/Symb&ol"), &specialPlot);
      plot.insertSeparator ();

      plot.insertItem (QPixmap (vertBars_xpm), tr ("&Columns"), w,
		       SLOT (plotVB ()));
      plot.insertItem (QPixmap (hBars_xpm), tr ("&Rows"), w,
		       SLOT (plotHB ()));
      plot.insertItem (QPixmap (area_xpm), tr ("&Area"), w,
		       SLOT (plotArea ()));

      plot.insertItem (QPixmap (pie_xpm), tr ("&Pie"), w, SLOT (plotPie ()));
      plot.insertSeparator ();

      plot.insertItem (QPixmap (ribbon_xpm), tr ("3D Ribbo&n"), w,
		       SLOT (plot3DRibbon ()));
      plot.insertItem (QPixmap (bars_xpm), tr ("3D &Bars"), w,
		       SLOT (plot3DBars ()));
      plot.insertItem (QPixmap (scatter_xpm), tr ("3&D Scatter"), w,
		       SLOT (plot3DScatter ()));
      plot.insertItem (QPixmap (trajectory_xpm), tr ("3D &Trajectory"), w,
		       SLOT (plot3DTrajectory ()));

      plot.insertSeparator ();

      actionBoxPlot->addTo (&stat);
      stat.insertItem (QPixmap (histogram_xpm), tr ("&Histogram"), w,
		       SLOT (plotHistogram ()));
      plot.insertItem (tr ("Statistical &Graphs"), &stat);

      Legend.insertItem (tr ("&Plot"), &plot);
      Legend.insertSeparator ();

      Legend.insertItem (QPixmap (cut_xpm), tr ("Cu&t"), w,
			 SLOT (cutSelection ()));
      Legend.insertItem (QPixmap (copy_xpm), tr ("&Copy"), w,
			 SLOT (copySelection ()));
      Legend.insertItem (QPixmap (paste_xpm), tr ("Past&e"), w,
			 SLOT (pasteSelection ()));
      Legend.insertItem (QPixmap (paste_xpm), tr ("Paste Transposed"), w,
			 SLOT (pasteTransposed ()));
      Legend.insertSeparator ();

      int xColID = colType.insertItem (tr ("X"), this, SLOT (setXCol ()));
      int yColID = colType.insertItem (tr ("Y"), this, SLOT (setYCol ()));
      int zColID = colType.insertItem (tr ("Z"), this, SLOT (setZCol ()));
      colType.insertSeparator ();
      int xErrColID =
	colType.insertItem (QPixmap (xerrors_xpm), tr ("X Error"), this,
			    SLOT (setXErrCol ()));
      int yErrColID =
	colType.insertItem (QPixmap (errors_xpm), tr ("Y Error"), this,
			    SLOT (setYErrCol ()));
      colType.insertSeparator ();
      int noneID =
	colType.insertItem (tr ("None"), this, SLOT (disregardCol ()));

      if (w->colPlotDesignation (c) == Table::X)
	colType.setItemChecked (xColID, true);
      else if (w->colPlotDesignation (c) == Table::Y)
	colType.setItemChecked (yColID, true);
      else if (w->colPlotDesignation (c) == Table::Z)
	colType.setItemChecked (zColID, true);
      else if (w->colPlotDesignation (c) == Table::xErr)
	colType.setItemChecked (xErrColID, true);
      else if (w->colPlotDesignation (c) == Table::yErr)
	colType.setItemChecked (yErrColID, true);
      else
	colType.setItemChecked (noneID, true);

      Legend.insertItem (tr ("Set as"), &colType);
      if (ws->activeWindow ()->isA ("Table"))
	{
	  Legend.insertSeparator ();

	  actionShowColumnValuesDialog->addTo (&Legend);
	  actionTableRecalculate->addTo (&Legend);

	  
	  actionSetAscValues->addTo (&fill);
	  actionSetRandomValues->addTo (&fill);
	  Legend.insertItem (tr ("&Fill column with"), &fill);

	  norm.insertItem (tr ("&Column"), w, SLOT (normalizeSelection ()));
	  actionNormalizeTable->addTo (&norm);
	  Legend.insertItem (tr ("&Normalize"), &norm);

	  actionColumnInvert->addTo(&Legend);

	  Legend.insertSeparator ();

	  Legend.insertItem (tr("Statistics"), &statistics);
	 
	  actionShowColStatistics->addTo (&statistics);
	  actionShowXYStatistics->addTo (&statistics);
	  statistics.insertItem ( tr ("Area Statistics"),
				 this, SLOT (showTableAreaStatistics()));
	  statistics.insertItem ( tr ("Moment Analysis"),
				 this, SLOT (showMomentAnalysis()));
	  Legend.insertSeparator ();

	  Legend.insertItem (QPixmap (erase_xpm), tr ("Clea&r"), w,
			     SLOT (clearCol ()));
	  Legend.insertItem (QPixmap (close_xpm), tr ("&Delete"), w,
			     SLOT (removeCol ()));
	  Legend.insertItem (tr ("&Insert"), w, SLOT (insertCol ()));
	  Legend.insertItem (tr ("&Add column"), w, SLOT (addCol ()));
	  Legend.insertSeparator ();

	  sorting.insertItem (tr ("&Ascending"), w, SLOT (sortColAsc ()));
	  sorting.insertItem (tr ("&Descending"), w, SLOT (sortColDesc ()));
	  Legend.insertItem (tr ("Sort Colu&mn"), &sorting);

	  actionSortTable->addTo (&Legend);
	}
      Legend.insertSeparator ();
      actionShowColumnOptionsDialog->addTo (&Legend);
    }
  else if ((int) w->selectedColumns ().count () > 1)
    {
      plot.insertItem (QPixmap (lPlot_xpm), tr ("&Line"), w, SLOT (plotL ()));
      plot.insertItem (QPixmap (pPlot_xpm), tr ("&Scatter"), w,
		       SLOT (plotP ()));
      plot.insertItem (QPixmap (lpPlot_xpm), tr ("Line + s&ymbol"), w,
		       SLOT (plotLP ()));

      specialPlot.insertItem (QPixmap (dropLines_xpm),
			      tr ("Vertical &drop lines"), w,
			      SLOT (plotVerticalDropLines ()));
      specialPlot.insertItem (QPixmap (spline_xpm), tr ("&Spline"), w,
			      SLOT (plotSpline ()));
      specialPlot.insertItem (QPixmap (vert_steps_xpm),
			      tr ("&Vertical Steps"), w,
			      SLOT (plotVertSteps ()));
      specialPlot.insertItem (QPixmap (hor_steps_xpm),
			      tr ("&Horizontal Steps"), w,
			      SLOT (plotHorSteps ()));
      plot.insertItem (tr ("Special Line/Symb&ol"), &specialPlot);
      plot.insertSeparator ();

      plot.insertItem (QPixmap (vertBars_xpm), tr ("&Columns"), w,
		       SLOT (plotVB ()));
      plot.insertItem (QPixmap (hBars_xpm), tr ("&Rows"), w,
		       SLOT (plotHB ()));
      plot.insertItem (QPixmap (area_xpm), tr ("&Area"), w,
		       SLOT (plotArea ()));
      plot.insertItem (QPixmap (vectXYXY_xpm), tr ("Vectors &XYXY"), w,
		       SLOT (plotVectXYXY ()));
      plot.insertItem (QPixmap (vectXYAM_xpm), tr ("Vectors &XYAM"), w,
		       SLOT (plotVectXYAM ()));
      plot.insertSeparator ();

      actionBoxPlot->addTo (&stat);
      stat.insertItem (QPixmap (histogram_xpm), tr ("&Histogram"), w,
		       SLOT (plotHistogram ()));
      stat.insertItem (QPixmap (stacked_hist_xpm), tr ("&Stacked Histograms"),
		       this, SLOT (plotStackedHistograms ()));
      plot.insertItem (tr ("Statistical &Graphs"), &stat);

      panels.insertItem (QPixmap (panel_v2_xpm), tr ("&Vertical 2 Layers"),
			 this, SLOT (plot2VerticalLayers ()));
      panels.insertItem (QPixmap (panel_h2_xpm), tr ("&Horizontal 2 Layers"),
			 this, SLOT (plot2HorizontalLayers ()));
      panels.insertItem (QPixmap (panel_4_xpm), tr ("&4 Layers"), this,
			 SLOT (plot4Layers ()));
      panels.insertItem (QPixmap (stacked_xpm), tr ("&Stacked Layers"), this,
			 SLOT (plotStackedLayers ()));
      plot.insertItem (tr ("Pa&nel"), &panels);

      Legend.insertItem (tr ("&Plot"), &plot);
      Legend.insertSeparator ();
      Legend.insertItem (QPixmap (cut_xpm), tr ("Cu&t"), w,
			 SLOT (cutSelection ()));
      Legend.insertItem (QPixmap (copy_xpm), tr ("&Copy"), w,
			 SLOT (copySelection ()));
      Legend.insertItem (QPixmap (paste_xpm), tr ("Past&e"), w,
			 SLOT (pasteSelection ()));
      Legend.insertSeparator ();

      if (ws->activeWindow ()->isA ("Table"))
	{
	  Legend.insertItem (QPixmap (erase_xpm), tr ("Clea&r"), w,
			     SLOT (clearSelection ()));
	  Legend.insertItem (QPixmap (close_xpm), tr ("&Delete"), w,
			     SLOT (removeCol ()));
	  Legend.insertSeparator ();
	  Legend.insertItem (tr ("&Insert"), w, SLOT (insertCol ()));
	  Legend.insertItem (tr ("&Add column"), w, SLOT (addCol ()));
	  Legend.insertSeparator ();
	}

      actionSetXCol->addTo (&colType);
      actionSetYCol->addTo (&colType);
      actionSetZCol->addTo (&colType);
      colType.insertSeparator ();
      actionSetXErrCol->addTo (&colType);
      actionSetYErrCol->addTo (&colType);
      colType.insertSeparator ();
      actionDisregardCol->addTo (&colType);
      Legend.insertItem (tr ("Set as"), &colType);

      if (ws->activeWindow ()->isA ("Table"))
	{
	  Legend.insertSeparator ();

	  actionSetAscValues->addTo (&fill);
	  actionSetRandomValues->addTo (&fill);
	  Legend.insertItem (tr ("&Fill columns with"), &fill);

	  actionNormalizeSelection->addTo (&norm);
	  actionNormalizeTable->addTo (&norm);
	  Legend.insertItem (tr ("&Normalize"), &norm);

	  actionColumnInvert->addTo(&Legend);

	  Legend.insertSeparator ();
	  actionSortSelection->addTo (&Legend);
	  actionSortTable->addTo (&Legend);
	  Legend.insertSeparator ();
	  
	  Legend.insertItem (tr("Statistics"), &statistics);
	  actionShowColStatistics->addTo (&statistics);
	  if((int) w->selectedColumns ().count () == w->tableCols())
	    actionShowRowStatistics->addTo (&statistics);
	  statistics.insertItem ( tr ("Area Statistics"),
				 this,SLOT (showTableAreaStatistics()));
	  statistics.insertItem ( tr ("Moment Analysis"),
			     this, SLOT (showMomentAnalysis()));
	  Legend.insertSeparator ();
	  actionGenerateFormula->addTo(&Legend);
	}
    }

  QPoint posMouse = QCursor::pos ();
  Legend.exec (posMouse);
}

void
ApplicationWindow::plot2VerticalLayers ()
{
  multilayerPlot (1, 2, defaultCurveStyle);
}

void
ApplicationWindow::plot2HorizontalLayers ()
{
  multilayerPlot (2, 1, defaultCurveStyle);
}

void
ApplicationWindow::plot4Layers ()
{
  multilayerPlot (2, 2, defaultCurveStyle);
}

void
ApplicationWindow::plotStackedLayers ()
{
  multilayerPlot (1, -1, defaultCurveStyle);
}

void
ApplicationWindow::plotStackedHistograms ()
{
  multilayerPlot (1, -1, Graph::Histogram);
}

void
ApplicationWindow::showMatrixDialog ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Matrix"))
    {
      Matrix *w = (Matrix *) ws->activeWindow ();

      matrixDialog *md =
	new matrixDialog (this, "matrixDialog", true, WDestructiveClose);
      connect (md, SIGNAL (changeColumnsWidth (int)), w,
	       SLOT (setColumnsWidth (int)));
      connect (md, SIGNAL (changeTextFormat (const QChar &, int)), w,
	       SLOT (setNumericFormat (const QChar &, int)));
      connect (md, SIGNAL (finished ()), w, SLOT (forgetSavedCells ()));

      w->saveCellsToMemory ();
      md->setTextFormat (w->textFormat (), w->precision ());
      md->setColumnsWidth (w->columnsWidth ());
      md->showNormal ();
    }
}

void
ApplicationWindow::showMatrixSizeDialog ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Matrix"))
    {
      Matrix *w = (Matrix *) ws->activeWindow ();

      matrixSizeDialog *md =
	new matrixSizeDialog (this, "matrixDialog", true, WDestructiveClose);
      connect (md, SIGNAL (changeDimensions (int, int)), w,
	       SLOT (setMatrixDimensions (int, int)));
      connect (md,
	       SIGNAL (changeCoordinates (double, double, double, double)), w,
	       SLOT (setCoordinates (double, double, double, double)));

      md->setCoordinates (w->xStart (), w->xEnd (), w->yStart (), w->yEnd ());
      md->setColumns (w->numCols ());
      md->setRows (w->numRows ());
      md->showNormal ();
    }
}

void
ApplicationWindow::showMatrixValuesDialog ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Matrix"))
    {
      matrixValuesDialog *md =
	new matrixValuesDialog (scriptEnv, this, 0, true, WDestructiveClose);
      md->setMatrix ((Matrix *) ws->activeWindow ());
      md->showNormal ();
    }
}

void
ApplicationWindow::showColumnOptionsDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  Table *w = (Table *) ws->activeWindow ();

  if (int (w->selectedColumns ().count ()) > 0)
    {
      tableDialog *td =
	new tableDialog (this, "optionsDialog", true, WDestructiveClose);

      connect( td, SIGNAL( finished() ), w, SLOT( forgetSavedCol() ) );

      td->setWorksheet (w);
      td->showNormal ();
    }
  else
    QMessageBox::warning (this, tr ("QtiPlot"),
			  tr ("Please select a column first!"));
}

void
ApplicationWindow::showAxis (int axis, int type,
			     const QString & labelsColName, bool axisOn,
			     int majTicksType, int minTicksType,
			     bool labelsOn, const QColor & c, int format,
			     int prec, int rotation, int baselineDist,
			     const QString & formula,
			     const QColor & labelsColor)
{
  Table *w = table (labelsColName);
  if ((type == Graph::Txt || type == Graph::ColHeader) && !w)
    return;

  activeGraph->showAxis (axis, type, labelsColName, w, axisOn, majTicksType,
			 minTicksType, labelsOn, c, format, prec, rotation,
			 baselineDist, formula, labelsColor);
}

void
ApplicationWindow::showGeneralPlotDialog ()
{
  QWidget *plot = ws->activeWindow ();
  if (!plot)
    return;

  QDialog *gd = showScaleDialog ();
  if (gd && plot->isA ("MultiLayer") && ((MultiLayer *) plot)->layers ())
    {
      Graph *g = ((MultiLayer *) plot)->activeGraph ();
      if (!g->isPiePlot ())
	((axesDialog *) gd)->showGeneralPage ();
      else
	((pieDialog *) gd)->showGeneralPage ();
    }
  else if (gd && plot->isA ("Graph3D"))
    ((plot3DDialog *) gd)->showGeneralTab ();
}

void
ApplicationWindow::showAxisDialog ()
{
  QWidget *plot = (QWidget *) ws->activeWindow ();
  if (!plot)
    return;

  QDialog *gd = showScaleDialog ();
  if (gd && plot->isA ("MultiLayer") && ((MultiLayer *) plot)->layers ())
    ((axesDialog *) gd)->showAxesPage ();
  else if (gd && plot->isA ("Graph3D"))
    ((plot3DDialog *) gd)->showAxisTab ();
}

void
ApplicationWindow::showGridDialog ()
{
  axesDialog *gd = (axesDialog *) showScaleDialog ();
  if (gd)
    gd->showGridPage ();
}

QDialog *
ApplicationWindow::showScaleDialog ()
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return 0;

  if (w->isA ("MultiLayer"))
    {
      if (((MultiLayer *) w)->isEmpty ())
	return 0;

      Graph *g = ((MultiLayer *) w)->activeGraph ();
      if (!g->isPiePlot ())
	{
	  activeGraph = g;

	  axesDialog *ad =
	    new axesDialog (this, 0, TRUE, WStyle_Tool | WDestructiveClose);
	  connect (ad, SIGNAL (updateAxisTitle (int, const QString &)), g,
		   SLOT (setAxisTitle (int, const QString &)));
	  connect (ad, SIGNAL (changeAxisFont (int, const QFont &)), g,
		   SLOT (setAxisFont (int, const QFont &)));
	  connect (ad,
		   SIGNAL (showAxis
			   (int, int, const QString &, bool, int, int, bool,
			    const QColor &, int, int, int, int,
			    const QString &, const QColor &)), this,
		   SLOT (showAxis
			 (int, int, const QString &, bool, int, int, bool,
			  const QColor &, int, int, int, int, const QString &,
			  const QColor &)));

	  ad->setMultiLayerPlot ((MultiLayer *) w);
	  ad->insertColList (columnsList (Table::All));
	  ad->insertTablesList (tableWindows);
	  ad->setAxesLabelsFormatInfo (g->axesLabelsFormatInfo ());
	  ad->setEnabledAxes (g->enabledAxes ());
	  ad->setAxesType (g->axesType ());
	  ad->setAxesBaseline (g->axesBaseline ());
	  ad->initAxisFonts (g->axisFont (2), g->axisFont (0),
			     g->axisFont (3), g->axisFont (1));
	  ad->setAxisTitles (g->scalesTitles ());
	  ad->updateTitleBox (0);
	  ad->putGridOptions (g->getGridOptions ());
	  ad->setTicksType (g->plotWidget ()->getMajorTicksType (),
			    g->plotWidget ()->getMinorTicksType ());
	  ad->setEnabledTickLabels (g->enabledTickLabels ());
	  ad->initLabelsRotation (g->labelsRotation (QwtPlot::xBottom),
				  g->labelsRotation (QwtPlot::xTop));
	  ad->showNormal ();
	  ad->setActiveWindow ();
	  return ad;
	}
      else if (g->isPiePlot ())
	return showPieDialog ();
    }
  else if (w->isA ("Graph3D"))
    return showPlot3dDialog ();

  return 0;
}

axesDialog *
ApplicationWindow::showScalePageFromAxisDialog (int axisPos)
{
  axesDialog *gd = (axesDialog *) showScaleDialog ();
  if (gd)
    gd->setCurrentScale (axisPos);

  return gd;
}

axesDialog *
ApplicationWindow::showAxisPageFromAxisDialog (int axisPos)
{
  axesDialog *gd = (axesDialog *) showScaleDialog ();
  if (gd)
    {
      gd->showAxesPage ();
      gd->setCurrentScale (axisPos);
    }
  return gd;
}

QDialog *
ApplicationWindow::showPlot3dDialog ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      Graph3D *g = (Graph3D *) ws->activeWindow ();
      if (!g->hasData ())
	{
	  QApplication::restoreOverrideCursor ();
	  QMessageBox::warning (this, tr ("QtiPlot - Warning"),
				tr
				("Not available for empty 3D surface plots!"));
	  return 0;
	}

      plot3DDialog *pd =
	new plot3DDialog (this, "plot3DDialog", TRUE,
			  WStyle_Tool | WDestructiveClose);
      pd->setPlot (g);

      connect (pd,
	       SIGNAL (updateColors
		       (const QColor &, const QColor &, const QColor &,
			const QColor &, const QColor &, const QColor &)), g,
	       SLOT (updateColors
		     (const QColor &, const QColor &, const QColor &,
		      const QColor &, const QColor &, const QColor &)));
      connect (pd, SIGNAL (updateDataColors (const QColor &, const QColor &)),
	       g, SLOT (setDataColors (const QColor &, const QColor &)));
      connect (pd, SIGNAL (setDataColorMap (const QString &)), g,
	       SLOT (setDataColorMap (const QString &)));
      connect (pd,
	       SIGNAL (updateTitle
		       (const QString &, const QColor &, const QFont &)), g,
	       SLOT (updateTitle
		     (const QString &, const QColor &, const QFont &)));
      connect (pd, SIGNAL (updateResolution (int)), g,
	       SLOT (setResolution (int)));
      connect (pd, SIGNAL (showColorLegend (bool)), g,
	       SLOT (showColorLegend (bool)));
      connect (pd, SIGNAL (setOrtho (bool)), g, SLOT (setOrtho (bool)));
      connect (pd, SIGNAL (updateLabel (int, const QString &, const QFont &)),
	       g, SLOT (updateLabel (int, const QString &, const QFont &)));
      connect (pd, SIGNAL (updateScale (int, const QStringList &)),
	       g, SLOT (updateScale (int, const QStringList &)));
      connect (pd, SIGNAL (adjustLabels (int)), g, SLOT (adjustLabels (int)));
      connect (pd, SIGNAL (updateTickLength (int, double, double)),
	       g, SLOT (updateTickLength (int, double, double)));
      connect (pd, SIGNAL (setNumbersFont (const QFont &)),
	       g, SLOT (setNumbersFont (const QFont &)));
      connect (pd, SIGNAL (updateMeshLineWidth (int)),
	       g, SLOT (setMeshLineWidth (int)));
      connect (pd, SIGNAL (updateBars (double)), g,
	       SLOT (updateBars (double)));
      connect (pd, SIGNAL (updatePoints (double, bool)), g,
	       SLOT (updatePoints (double, bool)));
      connect (pd, SIGNAL (updateTransparency (double)), g,
	       SLOT (changeTransparency (double)));
      connect (pd, SIGNAL (showWorksheet ()), g, SLOT (showWorksheet ()));
      connect (pd, SIGNAL (updateZoom (double)), g,
	       SLOT (updateZoom (double)));
      connect (pd, SIGNAL (updateScaling (double, double, double)), g,
	       SLOT (updateScaling (double, double, double)));
      connect (pd, SIGNAL (updateCones (double, int)), g,
	       SLOT (updateCones (double, int)));
      connect (pd, SIGNAL (updateCross (double, double, bool, bool)), g,
	       SLOT (updateCross (double, double, bool, bool)));

      pd->setMeshLineWidth (g->meshLineWidth ());
      pd->setTransparency (g->transparency ());
      pd->setDataColors (g->minDataColor (), g->maxDataColor ());
      pd->setColors (g->titleColor (), g->meshColor (), g->axesColor (),
		     g->numColor (), g->labelColor (), g->bgColor (),
		     g->gridColor ());

      pd->setTitle (g->plotTitle ());
      pd->setTitleFont (g->titleFont ());

      //pd->setZoom (g->zoom ());
      pd->setZoomAndScaling (g->zoom(), g->xScale (), g->yScale (), g->zScale ());
      pd->setResolution (g->resolution ());
      pd->showLegend (g->isLegendOn ());
      pd->setOrthogonal (g->isOrthogonal ());
      pd->setAxesLabels (g->axesLabels ());
      pd->setAxesTickLengths (g->axisTickLengths ());
      pd->setAxesFonts (g->axisLabelFont (0), g->axisLabelFont (1),
			g->axisLabelFont (2));
      pd->setScales (g->scaleLimits ());
      pd->setLabelsDistance (g->labelsDistance ());
      pd->setNumbersFonts (g->numbersFont ());

      if (g->coordStyle () == Qwt3D::NOCOORD)
	pd->disableAxesOptions ();

      Qwt3D::PLOTSTYLE style = g->plotStyle ();
      Graph3D::PointStyle pt = g->pointType ();

      if (style == Qwt3D::USER)
	{
	  switch (pt)
	    {
	    case Graph3D::None:
	      break;

	    case Graph3D::Dots:
	      pd->disableMeshOptions ();
	      pd->initPointsOptionsStack ();
	      pd->showPointsTab (g->pointsSize (), g->smoothPoints ());
	      break;

	    case Graph3D::VerticalBars:
	      pd->showBarsTab (g->barsRadius ());
	      break;

	    case Graph3D::HairCross:
	      pd->disableMeshOptions ();
	      pd->initPointsOptionsStack ();
	      pd->showCrossHairTab (g->crossHairRadius (),
				    g->crossHairLinewidth (),
				    g->smoothCrossHair (),
				    g->boxedCrossHair ());
	      break;

	    case Graph3D::Cones:
	      pd->disableMeshOptions ();
	      pd->initPointsOptionsStack ();
	      pd->showConesTab (g->coneRadius (), g->coneQuality ());
	      break;
	    }
	}
      else if (style == Qwt3D::FILLED)
	pd->disableMeshOptions ();
      else if (style == Qwt3D::HIDDENLINE || style == Qwt3D::WIREFRAME)
	pd->disableLegend ();

      if (g->grids () == 0)
	pd->disableGridOptions ();

      if (g->userFunction ())
	pd->customWorksheetBtn (QString::null);
      else if (g->getTable ())
	pd->customWorksheetBtn (tr ("&Worksheet"));
      else if (g->getMatrix ())
	pd->customWorksheetBtn (tr ("&Matrix"));

      pd->showNormal ();
      pd->setActiveWindow ();
      return pd;
    }
  else
    return 0;
}

QDialog *
ApplicationWindow::showPieDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return 0;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    {
      activeGraph = g;

      pieDialog *pd =
	new pieDialog (this, "plotDialog", TRUE,
		       WStyle_Tool | WDestructiveClose);
      connect (pd, SIGNAL (drawFrame (bool, int, const QColor &)), g,
	       SLOT (drawCanvasFrame (bool, int, const QColor &)));
      connect (pd, SIGNAL (toggleCurve ()), g, SLOT (removePie ()));
      connect (pd,
	       SIGNAL (updatePie
		       (const QPen &, const Qt::BrushStyle &, int, int)), g,
	       SLOT (updatePie
		     (const QPen &, const Qt::BrushStyle &, int, int)));
      connect (pd, SIGNAL (worksheet (const QString &)), this,
	       SLOT (showTable (const QString &)));

      pd->setMultiLayerPlot ((MultiLayer *) ws->activeWindow ());
      pd->showNormal ();
      pd->setActiveWindow ();
      return pd;
    }
  return 0;
}

void
ApplicationWindow::showPlotDialog ()
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return;

  if (w->isA ("MultiLayer"))
    {
      Graph *g = ((MultiLayer *) w)->activeGraph ();
      if (!g)
	return;

      if (g->curves () > 0)
	{
	  if (!g->isPiePlot ())
	    {
	      plotDialog *pd =
		new plotDialog (this, "plotDialog", true,
				WStyle_Tool | WDestructiveClose);
	      pd->insertColumnsList (columnsList (Table::All));
	      pd->setGraph (g);
	      pd->selectCurve (0);
	      pd->showNormal ();
	      pd->setActiveWindow ();

	      activeGraph = g;
	    }
	  else
	    showPieDialog ();
	}
      else if (g->curves () == 0)
	QMessageBox::warning (this, tr ("QtiPlot - Empty plot"),
			      tr
			      ("There are actually no curves on the active layer!"));
    }
  else if (w->isA ("Graph3D"))
    showPlot3dDialog ();
}

void
ApplicationWindow::showPlotDialog (int curveindex)
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return;

  if (w->isA ("MultiLayer"))
    {
      Graph *g = ((MultiLayer *) w)->activeGraph ();
      if (!g || g->curves () <= 0)
	return;

      if (!g->isPiePlot ())
	{
	  plotDialog *pd =
	    new plotDialog (this, "plotDialog", true,
			    WStyle_Tool | WDestructiveClose);
	  pd->insertColumnsList (columnsList (Table::All));
	  pd->setGraph (g);
	  pd->selectCurve (curveindex);
	  pd->showNormal ();
	  pd->setActiveWindow ();

	  activeGraph = g;
	}
      else
	showPieDialog ();
    }
}

void
ApplicationWindow::showCurveContextMenu (int index)
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  activeGraph = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  QwtPlotCurve *c = activeGraph->curve (index);
  if (!c)
    return;

  QPopupMenu curveMenu (this);
  int id =
    curveMenu.insertItem (c->title ().text (), this,
			  SLOT (showPlotDialog (int)));
  curveMenu.setItemParameter (id, index);
  curveMenu.insertSeparator ();

  if (c->rtti () == FunctionCurve::RTTI)
    {
      int id =
	curveMenu.insertItem (tr ("&Edit Function"), this,
			      SLOT (showFunctionDialog (int)));
      curveMenu.setItemParameter (id, index);
    }

  id =
    curveMenu.insertItem (tr ("&Worksheet"), this,
			  SLOT (showCurveWorksheet (int)));
  curveMenu.setItemParameter (id, index);
  id =
    curveMenu.insertItem (tr ("&Plot details"), this,
			  SLOT (showPlotDialog (int)));
  curveMenu.setItemParameter (id, index);
  curveMenu.insertSeparator ();
  id =
    curveMenu.insertItem (QPixmap (close_xpm), tr ("&Delete"), this,
			  SLOT (removeCurve (int)));
  curveMenu.setItemParameter (id, index);

  curveMenu.exec (QCursor::pos ());
}

void
ApplicationWindow::removeCurve (int index)
{
  activeGraph->removeCurve (index, true);
}

void
ApplicationWindow::showCurveWorksheet (int index)
{
  const QwtPlotItem *c =
    activeGraph->curve (index);
  if (!c)
    return;

  QString curveTitle = c->title ().text ();
  if (c->rtti () == FunctionCurve::RTTI)
    activeGraph->createWorksheet (curveTitle);
  else if (c->rtti () == QwtPlotItem::Rtti_PlotSpectrogram)
    {
      Spectrogram *sp = (Spectrogram *) c;
      if (sp->matrix ())
	{
	  if(aw && aw->status()==myWidget::Maximized)
	    sp->matrix ()->showMaximized ();
	  else
	    sp->matrix ()->showNormal();
	}
    }
  else
    showTable (curveTitle);
}

void
ApplicationWindow::zoomIn ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      btnPointer->setOn (true);
      return;
    }

  if ((Graph *) plot->activeGraph ()->isPiePlot ())
    {
      if (btnZoomIn->isOn ())
	QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			      tr
			      ("This functionality is not available for pie plots!"));
      btnPointer->setOn (true);
      return;
    }

  QWidgetList *g_lst = plot->graphPtrs ();
  for (Graph *g = (Graph *)g_lst->first(); g; g = (Graph *)g_lst->next())
    if (!g->isPiePlot ())
      g->zoom (true);
}

void
ApplicationWindow::zoomOut ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty () || (Graph *) plot->activeGraph ()->isPiePlot ())
    return;

  ((Graph *) plot->activeGraph ())->zoomOut ();
  btnPointer->setOn (true);
}

void
ApplicationWindow::setAutoScale ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"));
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (g)
    {
      g->setAutoScale ();
      btnPointer->setOn(true);
      g->zoom(false);
      emit modified ();
    }
}

void
ApplicationWindow::removePoints ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      btnPointer->setOn (true);
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    {
      btnPointer->setOn (true);
      return;
    }

  if (g->isPiePlot ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("This functionality is not available for pie plots!"));
      btnPointer->setOn (true);
      return;
    }
  else
    {
      switch (QMessageBox::warning (this, tr ("QtiPlot"),
				    tr
				    ("This will modify the data in the worksheets!\nAre you sure you want to continue?"),
				    tr ("Continue"), tr ("Cancel"), 0, 1))
	{
	case 0:
	  g->removePoints (true);
	  info->setText ("Select point and double click to remove it!");
	  showDisplayBar();
	  break;

	case 1:
	  btnPointer->setOn (TRUE);
	  break;
	}
    }
}

void
ApplicationWindow::movePoints ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      btnPointer->setOn (true);
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    {
      btnPointer->setOn (true);
      return;
    }

  if (g->isPiePlot ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("This functionality is not available for pie plots!"));

      btnPointer->setOn (true);
      return;
    }
  else
    {
      switch (QMessageBox::warning (this, "QtiPlot",
				    "This will modify the data in the worksheets! \
	\nAre you sure you want to continue?", "Continue", "Cancel", 0,
				    1))
	{
	case 0:
	  if (g)
	    {
	      g->movePoints (true);
	      info->setText ("Please, click on plot and move cursor!");
	      showDisplayBar();
	    }
	  break;

	case 1:
	  btnPointer->setOn (TRUE);
	  break;
	}
    }
}

void
ApplicationWindow::print (QWidget * w)
{
  if (w->isA ("MultiLayer") && ((MultiLayer *) w)->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"));
      return;
    }

  ((myWidget *) w)->print ();
}

//print active window
void
ApplicationWindow::print ()
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return;

  print (w);
}

// print window from project explorer
void
ApplicationWindow::printWindow ()
{
  WindowListItem *it = (WindowListItem *) listview->currentItem ();
  myWidget *w = it->window ();
  if (!w)
    return;

  print (w);
}

void
ApplicationWindow::printAllPlots ()
{
  QPrinter printer;
  printer.setResolution (84);

  if(defaultPrinterOrientation=="Portrait")
    printer.setOrientation (QPrinter::Portrait);
  else
    printer.setOrientation (QPrinter::Landscape);

  printer.setColorMode (QPrinter::Color);
  printer.setFullPage (TRUE);

  if (printer.setup ())
    {
      QPainter *paint = new QPainter (&printer, false);
      QWidgetList *windows = windowsList ();

      int plots = 0;
      QWidget *w = 0;
      for (w = windows->first (); w; w = windows->next ())
	if (w->isA ("MultiLayer"))
	  plots++;

      printer.setMinMax (0, plots);
      printer.setFromTo (0, plots);

      for (w = windows->first (); w; w = windows->next ())
	if (w->isA ("MultiLayer") && printer.newPage ())
	  ((MultiLayer *) w)->printAllLayers (paint);

      paint->end ();
      delete paint;
      delete windows;
    }
}

void
ApplicationWindow::showExpGrowthDialog ()
{
  showExpDecayDialog (-1);
}

void
ApplicationWindow::showExpDecayDialog ()
{
  showExpDecayDialog (1);
}

void
ApplicationWindow::showExpDecayDialog (int type)
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  activeGraph = g;
  aw = (myWidget *) ws->activeWindow ();

  expDecayDialog *edd =
    new expDecayDialog (type, this, "polyDialog", true,
			WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), edd, SLOT (close ()));

  edd->setGraph (g);
  edd->showNormal ();
  edd->setActiveWindow ();
}

void
ApplicationWindow::showTwoExpDecayDialog ()
{
  showExpDecayDialog (2);
}

void
ApplicationWindow::showExpDecay3Dialog ()
{
  showExpDecayDialog (3);
}

void
ApplicationWindow::showFitDialog ()
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return;

  MultiLayer *plot = 0;
  if (w->isA ("MultiLayer"))
    plot = (MultiLayer *) w;
  else if (w->isA ("Table"))
    plot =
      multilayerPlot ((Table *) w, ((Table *) w)->drawableColumnSelection (),
		      Graph::LineSymbols);

  if (!plot)
    return;

  Graph *g = (Graph *) plot->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  activeGraph = g;
  aw = (myWidget *) ws->activeWindow ();

  fitDialog *fd = new fitDialog (this, "fitDialog", false, WDestructiveClose);
  //connect (fd, SIGNAL (clearFunctionsList ()), this,
  //	   SLOT (clearFitFunctionsList ()));
  connect (fd, SIGNAL (saveFunctionsList (const QStringList &)), this,
	   SLOT (saveFitFunctionsList (const QStringList &)));
  connect (plot, SIGNAL (closedWindow (myWidget *)), fd, SLOT (close ()));

#ifdef Q_OS_MAC                 // Mac
  QSettings settings (QSettings::Ini);
#else
  QSettings settings;
#endif
  settings.setPath ("ProIndependent", "QtiPlot", QSettings::User);
  settings.beginGroup ("/UserFunctions");
  fitFunctions = settings.readListEntry ("/FitFunctions");
  settings.endGroup();

  fd->insertFunctionsList (fitFunctions);
  fd->setGraph (g);
  fd->setSrcTables (tableList ());
  fd->showNormal ();
  fd->setActiveWindow ();
}

void
ApplicationWindow::showFilterDialog (int filter)
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g && g->validCurvesDataSize ())
    {
      filterDialog *fd =
	new filterDialog (filter, this, "filterDialog", TRUE,
			  WStyle_Tool | WDestructiveClose);
      fd->setGraph (g);
      fd->show ();
      fd->setActiveWindow ();
    }
}

void
ApplicationWindow::lowPassFilterDialog ()
{
  showFilterDialog (filterDialog::LowPass);
}

void
ApplicationWindow::highPassFilterDialog ()
{
  showFilterDialog (filterDialog::HighPass);
}

void
ApplicationWindow::bandPassFilterDialog ()
{
  showFilterDialog (filterDialog::BandPass);
}

void
ApplicationWindow::bandBlockFilterDialog ()
{
  showFilterDialog (filterDialog::BandBlock);
}

void
ApplicationWindow::bandInterpFilterDialog ()
{
  showFilterDialog (filterDialog::BandInterp);
}

void
ApplicationWindow::showFFTDialog ()
{
  QWidget *w = ws->activeWindow ();
  if (!w)
    return;

  FFTDialog *sd = 0;
  if (w->isA ("MultiLayer"))
    {
      Graph *g = ((MultiLayer *) w)->activeGraph ();
      if (g && g->validCurvesDataSize ())
	{
	  sd =
	    new FFTDialog (FFTDialog::onGraph, this, "smoothDialog", TRUE,
			   WStyle_Tool | WDestructiveClose);
	  sd->setGraph (g);
	}
    }
  else if (w->isA ("Table"))
    {
      sd =
	new FFTDialog (FFTDialog::onTable, this, "smoothDialog", TRUE,
		       WStyle_Tool | WDestructiveClose);
      sd->setTable ((Table *) w);
    }

  if (sd)
    {
      sd->show ();
      sd->setActiveWindow ();
    }
}

void
ApplicationWindow::showConvDialog(int conv)
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) w)->activeGraph ();
  if (g && g->validCurvesDataSize ())
    {
      convolutionDialog *cd =
	new convolutionDialog(conv,
			      this, "convolutionDialog",
			      true, WStyle_Tool | WDestructiveClose);
      cd->setGraph(g);
      cd->show ();
      cd->setActiveWindow ();
    }
}

void
ApplicationWindow::showConvolutionDialog()
{
  showConvDialog(convolutionDialog::ConvFFT);
}

void
ApplicationWindow::showDeconvFFTDialog()
{
  showConvDialog(convolutionDialog::DeconvFFT);
}

void
ApplicationWindow::showDeconvAndreDialog()
{
  showConvDialog(convolutionDialog::DeconvAndre);
}

void
ApplicationWindow::showDeconvGSDialog()
{
  showConvDialog(convolutionDialog::DeconvGS);
}

void
ApplicationWindow::showSmoothSavGolDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  smoothCurveDialog *sd =
    new smoothCurveDialog (smoothCurveDialog::SavitzkyGolay,
			   this, "smoothDialog", TRUE,
			   WStyle_Tool | WDestructiveClose);
  sd->setGraph (g);
  sd->show ();
  sd->setActiveWindow ();
}

void
ApplicationWindow::showGaussConvDialog()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  smoothCurveDialog *sd =
    new smoothCurveDialog (smoothCurveDialog::GaussConv, this,
			   "smoothDialog", TRUE,
			   WStyle_Tool | WDestructiveClose);
  sd->setGraph (g);
  sd->show ();
  sd->setActiveWindow ();
}
void

ApplicationWindow::showLorentzConvDialog()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  smoothCurveDialog *sd =
    new smoothCurveDialog (smoothCurveDialog::LorentzConv, this,
			   "smoothDialog", TRUE,
			   WStyle_Tool | WDestructiveClose);
  sd->setGraph (g);
  sd->show ();
  sd->setActiveWindow ();
}

void
ApplicationWindow::showSmoothAverageDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  smoothCurveDialog *sd =
    new smoothCurveDialog (smoothCurveDialog::Average, this, "smoothDialog",
			   TRUE, WStyle_Tool | WDestructiveClose);
  sd->setGraph (g);
  sd->show ();
  sd->setActiveWindow ();
}

void
ApplicationWindow::showInterpolationDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  interpolationDialog *id =
    new interpolationDialog (this, "interpolationDialog", true,
			     WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), id, SLOT (close ()));
  id->setGraph (g);
  id->show ();
  id->setActiveWindow ();
}

void
ApplicationWindow::showCascadeDialog()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *m = (MultiLayer *)ws->activeWindow();
  Graph *g = m->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  cascadeDialog *cd =
    new cascadeDialog (this, "cascadeDialog", true,
			     WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), cd, SLOT (close ()));
  cd->setGraph (g, m->getName());
  cd->show ();
  cd->setActiveWindow ();
}

void
ApplicationWindow::showNormalizeDialog()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *m = (MultiLayer *)ws->activeWindow();
  Graph *g = m->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  normalizeDialog *nd =
    new normalizeDialog (this, "normalizeDialog", true,
			     WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), nd, SLOT (close ()));
  nd->setGraph (g, m->getName());
  nd->show ();
  nd->setActiveWindow ();
}

void
ApplicationWindow::showAlignCurvesDialog()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *m = (MultiLayer *)ws->activeWindow();
  Graph *g = m->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  alignCurvesDialog *nd =
    new alignCurvesDialog (this, "alignCurvesDialog", true,
			     WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), nd, SLOT (close ()));
  nd->setGraph (g, m->getName());
  nd->show ();
  nd->setActiveWindow ();
}


void
ApplicationWindow::showFitPolynomDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  activeGraph = g;
  aw = (myWidget *) ws->activeWindow ();

  polynomFitDialog *pfd =
    new polynomFitDialog (this, "polyDialog", true,
			  WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), pfd, SLOT (close ()));

  pfd->setGraph (g);
  pfd->showNormal ();
  pfd->setActiveWindow ();
}

void
ApplicationWindow::showSineFitDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  activeGraph = g;
  aw = (myWidget *) ws->activeWindow ();

  sineFitDialog *sfd =
    new sineFitDialog (this, "sineFitDialog", true,
			  WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), sfd, SLOT (close ()));

  sfd->setGraph (g);
  sfd->showNormal ();
  sfd->setActiveWindow ();
}

void
ApplicationWindow::fitLinear ()
{
  analysis ("fitLinear");
}

void
ApplicationWindow::updateLog (const QString & result)
{
  if (!result.isEmpty ())
    {
      logInfo += result;
      showResults (true);
      emit modified ();
    }
}

void
ApplicationWindow::showIntDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  intDialog *id =
    new intDialog (this, "intDialog", true, WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), id, SLOT (close ()));
  id->setGraph (g);
  id->showNormal ();
  id->setActiveWindow ();
  activeGraph = g;
}

void
ApplicationWindow::showFwhmDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  fwhmDialog *id =
    new fwhmDialog (this, "fwhmDialog", true, WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), id, SLOT (close ()));
  id->setGraph (g);
  id->showNormal ();
  id->setActiveWindow ();
  activeGraph = g;
}

void
ApplicationWindow::showMomentDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *m = (MultiLayer *) ws->activeWindow (); 
  Graph *g = m->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  momentDialog *id =
    new momentDialog (this, "momentDialog", true,
		      WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), id, SLOT (close ()));
  id->setGraph (g, m->getName());
  id->showNormal ();
  id->setActiveWindow ();
  activeGraph = g;
}

void
ApplicationWindow::fitSigmoidal ()
{
  analysis ("fitSigmoidal");
}

void
ApplicationWindow::fitGauss ()
{
  analysis ("fitGauss");
}

void
ApplicationWindow::fitLorentz ()
{
  analysis ("fitLorentz");
}

void
ApplicationWindow::differentiate ()
{
  analysis ("differentiate");
}

void
ApplicationWindow::showResults (bool ok)
{
  if (ok)
    {
      if (!logInfo.isEmpty ())
	results->setText (logInfo);
      else
	results->setText (tr ("Sorry, there are no results to display!"));

      logWindow->show ();
      results->scrollToBottom ();
    }
  else
    logWindow->hide ();
}

void
ApplicationWindow::showResults (const QString & s)
{
  logInfo += s;
  showResults (true);
}

void
ApplicationWindow::showScreenReader ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      btnPointer->setOn (true);
      return;
    }

  QWidgetList *g_lst = plot->graphPtrs ();
  for (Graph *g = (Graph *)g_lst->first(); g; g = (Graph *)g_lst->next())
    g->showPlotPicker (true);

  info->setText (tr ("Click on plot or move cursor to display coordinates!"));
  showDisplayBar();
}

void
ApplicationWindow::showRangeSelectors ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("There are no plot layers available in this window!"));
      btnPointer->setOn (true);
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (!g)
    return;

  if (!g->curves ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("There are no curves available on this plot!"));
      btnPointer->setOn (true);
      return;
    }
  else if (g->isPiePlot ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("This functionality is not available for pie plots!"));
      btnPointer->setOn (true);
      return;
    }


  activeGraph = g;
  if (g->enableRangeSelectors (true))
    {
      info->
	setText
	("Click or use Ctrl+arrow key to select range (arrows select active cursor)!");
      showDisplayBar();
    }
}

void
ApplicationWindow::showCursor ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      btnPointer->setOn (true);
      return;
    }

  if ((Graph *) plot->activeGraph ()->isPiePlot ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("This functionality is not available for pie plots!"));

      btnPointer->setOn (true);
      return;
    }

  QWidgetList *g_lst = plot->graphPtrs ();
  for (Graph *g = (Graph *)g_lst->first(); g; g = (Graph *)g_lst->next())
    if (!g->isPiePlot () && g->validCurvesDataSize ())
      g->enableCursor (true);

  info->setText (tr ("Click on data point to display information!"));
  showDisplayBar();
}

void
ApplicationWindow::newLegend ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (g)
    g->newLegend ();
}

void
ApplicationWindow::addTimeStamp ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (g)
    g->addTimeStamp ();
}

void
ApplicationWindow::disableAddText ()
{
  actionAddText->setOn (false);
  showTextDialog ();
}

void
ApplicationWindow::addText ()
{
  if (!btnPointer->isOn ())
    btnPointer->setOn (TRUE);

  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();

  switch (QMessageBox::information (this,
				    tr ("QtiPlot - Add new layer?"),
				    tr
				    ("Do you want to add the text on a new layer or on the active layer?"),
				    tr ("On &New Layer"),
				    tr ("On &Active Layer"), tr ("&Cancel"),
				    1, 2))
    {
    case 0:
      plot->addTextLayer (legendFrameStyle, plotLegendFont, legendTextColor,
			  legendBackground);
      break;

    case 1:
      {
	if (plot->isEmpty ())
	  {
	    QMessageBox::warning (this, tr ("QtiPlot - Warning"),
				  tr
				  ("<h4>There are no plot layers available in this window.</h4>"
				   "<p><h4>Please add a layer and try again!</h4>"));

	    actionAddText->setOn (false);
	    return;
	  }

	Graph *g = (Graph *) plot->activeGraph ();
	if (g)
	  g->drawText (true);
      }
      break;

    case 2:
      actionAddText->setOn (false);
      return;
      break;
    }
}

void
ApplicationWindow::addImage ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (g)
    {
      QStringList list = QImage::inputFormatList ();
      QString filter = tr ("Images") + " (*.jpg *JPG ", aux;
      int i;
      for (i = 0; i < (int) list.count (); i++)
	{
	  aux = "*." + (list[i]).lower () + " *. " + list[i] + " ";
	  filter += aux;
	}
      filter += ");;";
      for (i = 0; i < (int) list.count (); i++)
	{
	  aux = "*." + (list[i]).lower () + " *. " + list[i] + ";;";
	  filter += aux;
	}

      QString fn =
	QFileDialog::getOpenFileName (imagesDirPath, filter, this, 0,
				      tr ("QtiPlot - Insert image from file"),
				      0, TRUE);
      if (!fn.isEmpty ())
	{
	  QApplication::setOverrideCursor (waitCursor);
	  QPixmap photo;
	  for (i = 0; i < (int) list.count (); i++)
	    {
	      if (fn.endsWith ("." + list[i], false))
		{
		  photo.load (fn, list[i], QPixmap::Color);
		  break;
		}
	    }

	  if (fn.endsWith (".jpg", false))
	    photo.load (fn, "JPEG", QPixmap::Color);

	  g->insertImageMarker (photo, fn);

	  QFileInfo fi (fn);
	  imagesDirPath = fi.dirPath (true);
	  QApplication::restoreOverrideCursor ();
	}
    }
}

void
ApplicationWindow::drawLine ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));

      btnPointer->setOn (true);
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (g)
    {
      g->drawLine (true);
      //showDisplayBar();
      emit modified ();
    }
}

void
ApplicationWindow::drawArrow ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));

      btnPointer->setOn (true);
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (g)
    {
      g->drawLine (true, 1);
      //showDisplayBar();
      emit modified ();
    }
}

void
ApplicationWindow::subtractLine()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));

      //btnPointer->setOn (true);
      return;
    }

  btnPointer->setOn (true);

  Graph *g = (Graph *) plot->activeGraph ();
  if (g) g->setSubtractLine(true);
  //showDisplayBar();
}

void
ApplicationWindow::showSubtractDialog()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  activeGraph = g;
  aw = (myWidget *) ws->activeWindow ();

  subtBackDialog *sbd =
    new subtBackDialog (this, "subtBackDialog", true,
			  WStyle_Tool | WDestructiveClose);
  connect((myWidget *) ws->activeWindow (),
	  SIGNAL (closedWindow (myWidget *)), sbd, SLOT (close ()));
  connect( sbd, SIGNAL(updateTableColumn(const QString &,
					 QMemArray <double> &)),
	   this, SLOT(updateTableColumn(const QString &,
					QMemArray <double> &)));

  sbd->setGraph (g);
  sbd->showNormal ();
  sbd->setActiveWindow ();
}

void
ApplicationWindow::showRemoveSpikeDialog()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  activeGraph = g;
  aw = (myWidget *) ws->activeWindow ();

  removeSpikeDialog *sbd =
    new removeSpikeDialog (this, "removeSpikeDialog", true,
			  WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), sbd, SLOT (close ()));
  connect ( sbd, SIGNAL(updateTableColumn(const QString &, 
					  QMemArray <double> &)),
	    this, SLOT(updateTableColumn(const QString &,
					 QMemArray <double> &)) );

  sbd->setGraph (g);
  sbd->showNormal ();
  sbd->setActiveWindow ();
}


void
ApplicationWindow::showCurveCalcDialog()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *m = (MultiLayer *) ws->activeWindow (); 
  Graph *g = m->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  activeGraph = g;
  aw = (myWidget *) ws->activeWindow ();

  curveCalcDialog *ccd =
    new curveCalcDialog (this, "curveCalcDialog", true,
			  WStyle_Tool | WDestructiveClose);
  connect ((myWidget *) ws->activeWindow (),
	   SIGNAL (closedWindow (myWidget *)), ccd, SLOT (close ()));

  ccd->setGraph (g, m->getName());
  ccd->showNormal ();
  ccd->setActiveWindow ();
}

void
ApplicationWindow::showImageDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    {
      ImageMarker *im = (ImageMarker *) g->selectedMarkerPtr ();
      if (!im)
	return;

      imageDialog *id =
	new imageDialog (0, "imageDialog", TRUE,
			 WStyle_Tool | WDestructiveClose);
      connect (id, SIGNAL (options (int, int, int, int)), g,
	       SLOT (updateImageMarker (int, int, int, int)));
      id->setIcon (QPixmap (logo_xpm));
      id->setOrigin (im->getOrigin ());
      id->setSize (im->size ());
      id->showNormal ();
      id->setFocus ();
    }
}

void
ApplicationWindow::showLayerDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("There are no plot layers available in this window."));
      return;
    }

  layerDialog *id =
    new layerDialog (this, "layerDialog", TRUE,
		     WStyle_Tool | WDestructiveClose);
  id->setMultiLayer (plot);
  id->initFonts (plotTitleFont, plotAxesFont, plotNumbersFont,
		 plotLegendFont);
  id->showNormal ();
  id->setActiveWindow ();
}

void
ApplicationWindow::showLayerOrderDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("There are no plot layers available in this window."));
      return;
    }

  layerOrderDialog *id =
    new layerOrderDialog (this, "layerOrderDialog", TRUE,
			  WStyle_Tool | WDestructiveClose);
  id->setMultiLayer (plot);
  id->showNormal ();
  id->setActiveWindow ();
}

void
ApplicationWindow::showPlotGeometryDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  Graph *g = plot->activeGraph ();
  if (g)
    {
      imageDialog *id =
	new imageDialog (0, "imageDialog", TRUE,
			 WStyle_Tool | WDestructiveClose);
      connect (id, SIGNAL (options (int, int, int, int)), plot,
	       SLOT (setGraphGeometry (int, int, int, int)));
      id->setIcon (QPixmap (logo_xpm));
      id->setCaption (tr ("QtiPlot - Layer Geometry"));
      id->setOrigin (g->pos ());
      id->setSize (g->plotWidget ()->size ());
      id->showNormal ();
      id->setActiveWindow ();
    }
}

void
ApplicationWindow::showTextDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    {
      LegendMarker *m = (LegendMarker *) g->selectedMarkerPtr ();
      if (!m)
	return;

      TextDialog *td =
	new TextDialog (TextDialog::TextMarker, this, 0, TRUE,
			WStyle_Tool | WDestructiveClose);
      connect (td,
	       SIGNAL (values
		       (const QString &, int, int, const QFont &,
			const QColor &, const QColor &)), g,
	       SLOT (updateTextMarker
		     (const QString &, int, int, const QFont &,
		      const QColor &, const QColor &)));

      td->setIcon (QPixmap (logo_xpm));
      td->setText (m->getText ());
      td->setFont (m->getFont ());
      td->setTextColor (m->getTextColor ());
      td->setBackgroundColor (m->backgroundColor ());
      td->setBackgroundType (m->getBkgType ());
      td->setAngle (m->getAngle ());
      td->showNormal ();
      td->setActiveWindow ();
    }
}

void
ApplicationWindow::showLineDialog ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    {
      LineMarker *lm = (LineMarker *) g->selectedMarkerPtr ();
      if (!lm)
	return;

      lineDialog *ld =
	new lineDialog (lm, this, "lineDialog", TRUE,
			WStyle_Tool | WDestructiveClose);
      ld->showNormal ();
      ld->setFocus ();
      g->deselectMarker();
    }
}

void
ApplicationWindow::addColToTable ()
{
  Table *m = (Table *) ws->activeWindow ();
  if (m)
    m->addCol ();
}

void
ApplicationWindow::addRowToTable ()
{
  Table *m = (Table *) ws->activeWindow ();
  if (m)
    m->addRow ();
}

void
ApplicationWindow::clearSelection ()
{
  //fprintf(stderr,"clearSelection\n");

  if (listview->hasFocus ())
    {
      deleteSelectedItems ();
      return;
    }

  QWidget *m = (QWidget *) ws->activeWindow ();
  if (!m)
    return;

  if (m->inherits ("Table"))
    ((Table *) m)->clearSelection ();
  else if (m->isA ("Matrix"))
    ((Matrix *) m)->clearSelection ();
  else if (m->isA ("MultiLayer"))
    {
      Graph *g = ((MultiLayer *) m)->activeGraph ();
      if (!g)
	return;

      if (g->titleSelected ())
	g->removeTitle ();
      else if (g->markerSelected ())
	g->removeMarker ();
    }
  else if (m->isA ("Note"))
    ((Note *) m)->textWidget ()->clear ();
  emit modified ();
}

void
ApplicationWindow::copySelection ()
{
  if (results->hasFocus ())
    {
      results->copy ();
      return;
    }

  QWidget *m = (QWidget *) ws->activeWindow ();
  if (!m)
    return;

  if (m->inherits ("Table"))
    ((Table *) m)->copySelection ();
  else if (m->isA ("Matrix"))
    ((Matrix *) m)->copySelection ();
  else if (m->isA ("MultiLayer"))
    {
      MultiLayer *plot = (MultiLayer *) m;
      if (!plot || plot->layers () == 0)
	return;

      plot->copyAllLayers ();
      Graph *g = (Graph *) plot->activeGraph ();
      if (g && g->markerSelected ())
	copyMarker ();
      else
	copyActiveLayer ();
    }
  else if (m->isA ("Note"))
    ((Note *) m)->textWidget ()->copy ();
}

void
ApplicationWindow::cutSelection ()
{
  QWidget *m = (QWidget *) ws->activeWindow ();
  if (!m)
    return;

  if (m->inherits ("Table"))
    ((Table *) m)->cutSelection ();
  else if (m->isA ("Matrix"))
    ((Matrix *) m)->cutSelection ();
  else if (m->isA ("MultiLayer"))
    {
      MultiLayer *plot = (MultiLayer *) m;
      if (!plot || plot->layers () == 0)
	return;

      Graph *g = (Graph *) plot->activeGraph ();
      copyMarker ();
      g->removeMarker ();
    }
  else if (m->isA ("Note"))
    ((Note *) m)->textWidget ()->cut ();

  emit modified ();
}

void
ApplicationWindow::copyMarker ()
{
  QWidget *m = (QWidget *) ws->activeWindow ();
  MultiLayer *plot = (MultiLayer *) m;
  Graph *g = (Graph *) plot->activeGraph ();
  if (g && g->markerSelected ())
    {
      g->copyMarker ();
      copiedMarkerType = g->copiedMarkerType ();
      QRect rect = g->copiedMarkerRect ();
      auxMrkStart = rect.topLeft ();
      auxMrkEnd = rect.bottomRight ();

      if (copiedMarkerType == Graph::Text)
	{
	  LegendMarker *m = (LegendMarker *) g->selectedMarkerPtr ();
	  auxMrkText = m->getText ();
	  auxMrkColor = m->getTextColor ();
	  auxMrkFont = m->getFont ();
	  auxMrkBkg = m->getBkgType ();
	  auxMrkBkgColor = m->backgroundColor ();
	}
      else if (copiedMarkerType == Graph::Arrow)
	{
	  LineMarker *m = (LineMarker *) g->selectedMarkerPtr ();
	  auxMrkWidth = m->width ();
	  auxMrkColor = m->color ();
	  auxMrkStyle = m->style ();
	  startArrowOn = m->hasStartArrow ();
	  endArrowOn = m->hasEndArrow ();
	  arrowHeadLength = m->headLength ();
	  arrowHeadAngle = m->headAngle ();
	  fillArrowHead = m->filledArrowHead ();
	}
      else if (copiedMarkerType == Graph::Image)
	{
	  ImageMarker *im = (ImageMarker *) g->selectedMarkerPtr ();
	  auxMrkFileName = im->getFileName ();
	}
    }
  copiedLayer = FALSE;
}

void
ApplicationWindow::pasteSelection ()
{
  QWidget *m = (QWidget *) ws->activeWindow ();
  if (!m)
    return;

  if (m->isA ("Table"))
    ((Table *) m)->pasteSelection ();
  else if (m->isA ("Matrix"))
    ((Matrix *) m)->pasteSelection ();
  else if (m->isA ("Note"))
    ((Note *) m)->textWidget ()->paste ();
  else if (m->isA ("MultiLayer"))
    {
      MultiLayer *plot = (MultiLayer *) m;
      if (!plot)
	return;
      if (copiedLayer)
	{
	  QApplication::setOverrideCursor (waitCursor);

	  /*
	  Graph *g = plot->addLayer ();
	  g->setIgnoreResizeEvents (!autoResizeLayers);

	  g->copy (lastCopiedLayer);
	  QPoint pos = plot->mapFromGlobal (QCursor::pos ());
	  plot->setGraphGeometry (pos.x (), pos.y () - 20,
				  lastCopiedLayer->width (),
				  lastCopiedLayer->height ());
	  */
	  QPoint pos = plot->mapFromGlobal (QCursor::pos ());
	  Graph *g = plot->addLayer(pos.x (), pos.y () - 20,
				    lastCopiedLayer->width (),
				    lastCopiedLayer->height ());
	  g->setAxisTitleInverted(QwtPlot::yRight,flipRightAxisTitle);
	  g->setIgnoreResizeEvents (!autoResizeLayers);
	  g->copy (lastCopiedLayer);
	  
	  //plot->connectLayer (g);
	  plot->updateTransparency();

	  QApplication::restoreOverrideCursor ();
	}
      else
	{
	  if (plot->layers () == 0)
	    return;

	  Graph *g = (Graph *) plot->activeGraph ();
	  if (!g)
	    return;

	  g->setCopiedMarkerType (copiedMarkerType);
	  g->setCopiedMarkerEnds (auxMrkStart, auxMrkEnd);

	  if (copiedMarkerType == Graph::Text)
	    g->setCopiedTextOptions (auxMrkBkg, auxMrkText, auxMrkFont,
				     auxMrkColor, auxMrkBkgColor);
	  if (copiedMarkerType == Graph::Arrow)
	    g->setCopiedArrowOptions (auxMrkWidth, auxMrkStyle, auxMrkColor,
				      startArrowOn, endArrowOn,
				      arrowHeadLength, arrowHeadAngle,
				      fillArrowHead);
	  if (copiedMarkerType == Graph::Image)
	    g->setCopiedImageName (auxMrkFileName);
	  g->pasteMarker ();
	}
    }
  emit modified ();
}

Table *
ApplicationWindow::copyTable ()
{
  Table *w = 0, *m = (Table *) ws->activeWindow ();
  if (m)
    {
      QString caption = generateUniqueName (tr ("Table"));
      w = newTable (caption, m->tableRows (), m->tableCols ());
      w->copy (m);

      // QString spec = m->saveToString ("geometry\n");
      //w->setSpecifications (spec.replace (m->getName(), caption));
      w->setSpecifications (w->saveToString("geometry\n"));

      w->showNormal ();
      
      setListViewSize (caption, m->sizeToString ());
      emit modified ();
    }
  return w;
}

Matrix *
ApplicationWindow::cloneMatrix ()
{
  Matrix *w = 0, *m = (Matrix *) ws->activeWindow ();
  if (m)
    {
      QString caption = generateUniqueName (tr ("Matrix"));
      int c = m->numCols ();
      int r = m->numRows ();
      w = newMatrix (caption, r, c);
      w->setCoordinates (m->xStart (), m->xEnd (), m->yStart (), m->yEnd ());

      for (int i = 0; i < r; i++)
	for (int j = 0; j < c; j++)
	  {
	    w->setText (i, j, m->text (i, j));
	  }

      w->setColumnsWidth (m->columnsWidth ());
      w->setFormula (m->formula ());
      w->setTextFormat (m->textFormat (), m->precision ());
      w->setSpecifications(w->saveToString("geometry\n"));
      w->showNormal ();
      
      setListViewSize (caption, m->sizeToString ());
      emit modified ();
    }
  return w;
}

Matrix*
ApplicationWindow::shrinkMatrix(int w, int h)
{
  Matrix *m2 = 0, *m = (Matrix *) ws->activeWindow ();
  if (m)
    {
      QString caption = generateUniqueName (tr ("Matrix"));
      int c = m->numCols () / w;
      int r = m->numRows () / h;
      m2 = newMatrix (caption, r, c);
      m2->setCoordinates (m->xStart (), m->xEnd (), m->yStart (), m->yEnd ());
      m2->setColumnsWidth (m->columnsWidth ());
      m2->setFormula (m->formula ());
      m2->setTextFormat (m->textFormat (), m->precision ());

      for (int i = 0; i < r; i++)    //row
	for (int j = 0; j < c; j++)  //column
	  {
	    double sum = 0;
	    for (int k=0; k<h; k++) //row
	      for (int l=0; l<w; l++)  //column
		sum += m->text(h*i+k,w*j+l).toDouble();
	    m2->setValue (i, j, sum/(w*h) );
	  }

      m2->setSpecifications(m2->saveToString("geometry\n"));
      m2->showNormal ();
      
      setListViewSize (caption, m->sizeToString ());
      emit modified ();
    }
  return m2;
}

void
ApplicationWindow::showShrinkMatrixDialog()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Matrix"))
    {
      // Matrix *w = (Matrix *) ws->activeWindow ();

      shrinkMatrixDialog *md =
        new shrinkMatrixDialog (this, "shrinkMatrixDialog", true, 
				WDestructiveClose);
      md->showNormal ();
    }
}

Graph3D *
ApplicationWindow::copySurfacePlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      Graph3D *g = (Graph3D *) ws->activeWindow ();
      if (!g->hasData ())
	{
	  QApplication::restoreOverrideCursor ();
	  QMessageBox::warning (this, tr ("QtiPlot - Duplicate error"),
				tr
				("Empty 3D surface plots cannot be duplicated!"));
	  return 0;
	}

      QString caption = generateUniqueName (tr ("Graph"));
      Graph3D *g2 = 0;
      QString s = g->formula ();
      if (g->userFunction ())
	{
	  g2 = newPlot3D (caption, g->formula (), g->xStart (), g->xStop (),
			  g->yStart (), g->yStop (), g->zStart (),
			  g->zStop (),g->getScaleType());
	}
      else if (s.endsWith ("(Z)", TRUE))
	g2 = dataPlotXYZ (caption, s, g->xStart (), g->xStop (),
			  g->yStart (), g->yStop (), g->zStart (),
			  g->zStop (), g->getScaleType());
      else if (s.endsWith ("(Y)", TRUE))
	g2 = dataPlot3D (caption, s, g->xStart (), g->xStop (), g->yStart (), g->yStop (), g->zStart (), g->zStop (), g->getScaleType());	//Ribbon plot
      else
	g2 = openMatrixPlot3D (caption, s, g->xStart (), g->xStop (),
			       g->yStart (), g->yStop (), g->zStart (),
			       g->zStop (),g->getScaleType());

      if (!g2)
	return 0;

      Graph3D::PointStyle pt = g->pointType ();
      if (g->plotStyle () == Qwt3D::USER)
	{
	  switch (pt)
	    {
	    case Graph3D::None:
	      break;

	    case Graph3D::Dots:
	      g2->setPointOptions (g->pointsSize (), g->smoothPoints ());
	      break;

	    case Graph3D::VerticalBars:
	      g2->setBarsRadius (g->barsRadius ());
	      break;

	    case Graph3D::HairCross:
	      g2->setCrossOptions (g->crossHairRadius (),
				   g->crossHairLinewidth (),
				   g->smoothCrossHair (),
				   g->boxedCrossHair ());
	      break;

	    case Graph3D::Cones:
	      g2->setConesOptions (g->coneRadius (), g->coneQuality ());
	      break;
	    }
	}
      g2->setStyle (g->coordStyle (), g->floorStyle (), g->plotStyle (), pt);
      g2->setGrid (g->grids ());
      g2->setTitle (g->plotTitle (), g->titleColor (), g->titleFont ());
      if (!g->colorMap ().isEmpty ())
	g2->setDataColorMap (g->colorMap ());
      else
	g2->setDataColors (g->minDataColor (), g->maxDataColor ());
      g2->setColors (g->meshColor (), g->axesColor (), g->numColor (),
		     g->labelColor (), g->bgColor (), g->gridColor ());
      g2->setTransparency (g->transparency ());
      g2->setAxesLabels (g->axesLabels ());
      g2->setTicks (g->scaleTicks ());
      g2->setTickLengths (g->axisTickLengths ());
      g2->setOptions (g->isLegendOn (), g->resolution (),
		      g->labelsDistance ());
      g2->setNumbersFont (g->numbersFont ());
      g2->setAxisLabelFont (0,g->axisLabelFont (0));
      g2->setAxisLabelFont (1,g->axisLabelFont (1));
      g2->setAxisLabelFont (2,g->axisLabelFont (2));
      g2->setRotation (g->xRotation (), g->yRotation (), g->zRotation ());
      g2->setZoom (g->zoom ());
      g2->setScale (g->xScale (), g->yScale (), g->zScale ());
      g2->setShift (g->xShift (), g->yShift (), g->zShift ());
      g2->setMeshLineWidth ((int) g->meshLineWidth ());
      g2->setOrtho (g->isOrthogonal ());
      g2->update ();
      g2->animate (g->isAnimated ());
      customToolBars ((QWidget *) g2);

      setListViewSize (caption, g->sizeToString ());
      return g2;
    }
  else
    return 0;
}

MultiLayer *
ApplicationWindow::copyGraph ()
{
  MultiLayer *plot2 = 0;
  if (ws->activeWindow () && ws->activeWindow ()->isA ("MultiLayer"))
    {
      MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
      QString caption = generateUniqueName (tr ("Graph"));

      plot2 = multilayerPlot (caption);
      plot2->showNormal ();
      
      plot2->resize (plot->size ());
      plot2->setSpacing (plot->rowsSpacing (), plot->colsSpacing ());
      plot2->setAlignement (plot->horizontalAlignement (),
			    plot->verticalAlignement ());
      plot2->setMargins (plot->leftMargin (), plot->rightMargin (),
			 plot->topMargin (), plot->bottomMargin ());

      QWidgetList *g_lst = plot->graphPtrs ();
      for(Graph *g = (Graph *)g_lst->first(); g; g = (Graph *)g_lst->next())
	{
	  Graph *g2 =
	    plot2->addLayer (g->pos ().x (), g->pos ().y (), g->width (),
			     g->height ());
	  g->setAxisTitleInverted(QwtPlot::yRight,flipRightAxisTitle);
	  //g2->setIgnoreResizeEvents (true);
	  g2->setAutoscaleFonts (false);
	  g2->copy (g);
	  g2->updateScale ();
	  // plot2->connectLayer (g2);
	  g2->setIgnoreResizeEvents (!autoResizeLayers);
	  g2->setAutoscaleFonts (autoScaleFonts);
	}
      setListViewSize (caption, plot->sizeToString ());
    }
  return plot2;
}

myWidget *
ApplicationWindow::copyWindow ()
{
  myWidget *w = 0;
  myWidget *g = (myWidget *) ws->activeWindow ();
  if (!g)
    {
      QMessageBox::critical (this, tr ("QtiPlot - Duplicate window error"),
			     tr
			     ("There are no windows available in this project!"));
      return w;
    }

  QApplication::setOverrideCursor (waitCursor);

  if (g->isA ("MultiLayer"))
    w = copyGraph ();
  else if (g->inherits ("Table"))
    w = copyTable ();
  else if (g->isA ("Graph3D"))
    w = copySurfacePlot ();
  else if (g->isA ("Matrix"))
    w = cloneMatrix ();
  else if (g->isA ("Note"))
    {
      w = newNote ();
      if (w)
	((Note *) w)->setText (((Note *) g)->text ());
    }

  if (w)
    {
      if (g->isA ("MultiLayer"))
	{
	  ((MultiLayer *) w)->updateTransparency ();
	  if (g->status () == myWidget::Maximized)
	    w->showMaximized ();
	}
      else if (g->isA ("Graph3D"))
	{
	  ((Graph3D *) w)->setIgnoreFonts (true);
	  if (g->status () == myWidget::Maximized)
	    {
	      g->showNormal ();
	      g->resize (500, 450);
	      w->resize (g->size ());
	      w->showMaximized ();
	    }
	  else
	    w->resize (g->size ());
	  ((Graph3D *) w)->setIgnoreFonts (false);
	}
      else
	w->resize (g->size ());

      w->setWindowLabel (g->windowLabel ());
      w->setCaptionPolicy (g->captionPolicy ());
      setListViewLabel (w->getName(), g->windowLabel ());
    }

  QApplication::restoreOverrideCursor ();
  return w;
}


QString extractName( QString & spec )
{
  QTextStream ts (&spec, IO_ReadOnly);
  ts.readLine ();
  QString s = ts.readLine ();
  return s.section('\t',0,0);
}

void
ApplicationWindow::undo ()
{
  if (!lastModified)
    return;

  QApplication::setOverrideCursor (waitCursor);

  if (lastModified->isA ("Table"))
    {
      Table *t = (Table *) lastModified;
      redoBuffer = t->getSpecifications(); // save current spec
      QString newCaption = extractName( undoBuffer );
      QString name = t->getName();
      if (newCaption != name)
	{
	  int id = tableWindows.findIndex (name);
	  tableWindows[id] = newCaption;
	  updateTableNames (name, newCaption);
	  renameListViewItem (name, newCaption);
	}
      t->restore (undoBuffer);
      actionUndo->setEnabled (FALSE);
      actionRedo->setEnabled (TRUE);
    }
  else if (lastModified->isA ("Matrix"))
    {
      Matrix *m = (Matrix *) lastModified;
      redoBuffer = m->getSpecifications(); // save current spec
      QString newCaption = extractName( undoBuffer );
      QString name = m->getName();
      if (newCaption != name)
	{
	  changeMatrixName( name, newCaption );
	  renameListViewItem (name, newCaption );
	}
      m->restore (undoBuffer);
      actionUndo->setEnabled (FALSE);
      actionRedo->setEnabled (TRUE);
    }
  else if (lastModified->isA ("Note"))
    {
      ((Note *) lastModified)->textWidget ()->undo ();
      actionUndo->setEnabled (FALSE);
      actionRedo->setEnabled (TRUE);
    }

  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::redo ()
{
  if (!lastModified)
    return;

  QApplication::setOverrideCursor (waitCursor);
  if (lastModified->isA ("Table"))
    {
      Table *t = (Table *) lastModified;
      QString newCaption = extractName( redoBuffer );
      QString name = t->getName();
      if (newCaption != name)
	{
	  int id = tableWindows.findIndex (name);
	  tableWindows[id] = newCaption;
	  updateTableNames (name, newCaption);
	  renameListViewItem (name, newCaption);
	}
      t->restore (redoBuffer);
      actionUndo->setEnabled (TRUE);
      actionRedo->setEnabled (FALSE);
    }
  else if (lastModified->isA ("Matrix"))
    {
      Matrix *m = (Matrix *) lastModified;
      QString newCaption = extractName( redoBuffer );
      QString name = m->getName();
      if (newCaption != name)
	{
	  changeMatrixName(name, newCaption);
	  renameListViewItem (name, newCaption);
	}
      m->restore (redoBuffer);
      actionUndo->setEnabled (TRUE);
      actionRedo->setEnabled (FALSE);
    }
  else if (lastModified->isA ("Note"))
    {
      ((Note *) lastModified)->textWidget ()->redo ();
      actionUndo->setEnabled (TRUE);
      actionRedo->setEnabled (FALSE);
    }
  QApplication::restoreOverrideCursor ();
}

bool
ApplicationWindow::hidden (QWidget * window)
{
  if (hiddenWindows->containsRef (window) || outWindows->containsRef (window))
    return TRUE;

  return FALSE;
}

void
ApplicationWindow::updateWindowStatus (myWidget * w)
{
  //  qDebug("updateWindowStatus");

  setListView (w->getName(), w->aspect ());

  if (w->status () == myWidget::Maximized)
    {//set any other window having status = Maximized to status = Normal
      QPtrList < myWidget > lst = current_folder->windowsList ();
      if (!lst.containsRef (w))
	return;

      myWidget *aw;
      for (aw = lst.first (); aw; aw = lst.next ())
	{
	  if (aw != w && aw->status () == myWidget::Maximized)
	    {
	      aw->setNormal ();
	      return;
	    }
	}
    }
}

void
ApplicationWindow::resizeActiveWindow ()
{
  QWidget *w = (QWidget *) ws->activeWindow ();
  if (!w)
    return;

  imageDialog *id =
    new imageDialog (this, "imageDialog", TRUE,
		     WStyle_Tool | WDestructiveClose);
  connect (id, SIGNAL (options (int, int, int, int)), w->parentWidget (),
	   SLOT (setGeometry (int, int, int, int)));

  id->setCaption (tr ("QtiPlot - Window Geometry"));
  id->setOrigin (w->parentWidget ()->pos ());
  id->setSize (w->parentWidget ()->size ());
  id->showNormal ();
  id->setActiveWindow ();
}

void
ApplicationWindow::hideActiveWindow ()
{
  myWidget *w = (myWidget *) ws->activeWindow ();
  if (!w)
    return;

  hideWindow (w);
}

void
ApplicationWindow::hideWindow (myWidget * w)
{
  hiddenWindows->append (w);
  w->setHidden ();
  // if(aw==w) aw=0; //no need of this 
  emit modified ();
}

void
ApplicationWindow::hideWindow ()
{
  WindowListItem *it = (WindowListItem *) listview->currentItem ();
  myWidget *w = it->window ();
  if (!w)
    return;

  hideWindow (w);
}

void
ApplicationWindow::showWindow ()
{
  WindowListItem *it = (WindowListItem *) listview->currentItem ();
  myWidget *w = it->window ();
  if (!w)
    return;

  activateWindow(w);
}

void
ApplicationWindow::resizeWindow ()
{
  WindowListItem *it = (WindowListItem *) listview->currentItem ();
  myWidget *w = it->window ();
  if (!w)
    return;

  imageDialog *id =
    new imageDialog (this, "imageDialog", TRUE,
		     WStyle_Tool | WDestructiveClose);
  connect (id, SIGNAL (options (int, int, int, int)), w->parentWidget (),
	   SLOT (setGeometry (int, int, int, int)));

  id->setCaption (tr ("QtiPlot - Window Geometry"));
  id->setOrigin (w->parentWidget ()->pos ());
  id->setSize (w->parentWidget ()->size ());
  id->showNormal ();
  id->setActiveWindow ();
}

void
ApplicationWindow::activateWindow ()
{
  WindowListItem *it = (WindowListItem *) listview->currentItem ();
  activateWindow (it->window ());
}

void
ApplicationWindow::activateWindow (QWidget * w)
{
  if (!w)
    return;

  // add by S.Yamamoto
  if(aw==(myWidget *)w && ((myWidget *)w)->status() != myWidget::Minimized)
    return;
  //

  updateWindowLists (w);

  if(aw->status()==myWidget::Maximized) 
    w->showMaximized();
  else 
    w->showNormal ();

  w->setActiveWindow ();
  //emit modified ();
}

void
ApplicationWindow::activateWindow (QListViewItem * lbi)
{
  if (!lbi || lbi->rtti () == FolderListItem::RTTI)
    return;

  QWidget *w = ((WindowListItem *) lbi)->window ();
  if (!w)
    return;

  activateWindow(w);
}

void
ApplicationWindow::maximizeWindow (QListViewItem * lbi)
{
  if (!lbi || lbi->rtti () == FolderListItem::RTTI)
    return;

  QWidget *w = ((WindowListItem *) lbi)->window ();
  if (!w)
    return;

  updateWindowLists (w);
  w->showMaximized ();
  emit modified ();
}

void
ApplicationWindow::maximizeWindow ()
{
  maximizeWindow (listview->currentItem ());
}

void
ApplicationWindow::minimizeWindow ()
{
  WindowListItem *it = (WindowListItem *) listview->currentItem ();
  myWidget *w = it->window ();
  if (!w)
    return;

  updateWindowLists (w);
  w->showMinimized ();
  emit modified ();
}

void
ApplicationWindow::updateWindowLists (QWidget * w)
{
  if (!w)
    return;

  if (hiddenWindows->containsRef (w))
    hiddenWindows->take (hiddenWindows->find (w));
  else if (outWindows->containsRef (w))
    {
      outWindows->take (outWindows->find (w));
      w->reparent (ws, WDestructiveClose, w->pos (), false);
    }
}

void
ApplicationWindow::closeActiveWindow ()
{
  QWidget *w = (QWidget *) ws->activeWindow ();
  if (w)
    w->close ();
}

void
ApplicationWindow::removeWindowFromLists (QWidget * w)
{
  QString caption = ((myWidget *)w)->getName();
  if (w->isA ("Table"))
    {
      Table *m = (Table *) w;
      removeCurves( m );

      tableWindows.remove (caption);
      if (w == lastModified)
	{
	  actionUndo->setEnabled (FALSE);
	  actionRedo->setEnabled (FALSE);
	}
      if ((int) tableWindows.count () <= 0)
	tableTools->hide ();
    }
  else if (w->isA ("TableStatistics"))
    {
      Table *m = (Table *) w;
      removeCurves( m );
      tableWindows.remove (caption);
      if ((int) tableWindows.count () <= 0)
	tableTools->hide ();
    }
  else if (w->isA ("MultiLayer"))
    {
      MultiLayer *ml = (MultiLayer *) w;
      Graph *g = ml->activeGraph ();

      if (g
	  && (g->selectorsEnabled () || g->zoomOn ()
	      || g->removePointActivated () || g->movePointsActivated ()
	      || g->enabledCursor () || g->pickerActivated ()))
	{
	  btnPointer->setOn (true);
	  activeGraph = 0;
	}
      if (!projectHas2DPlots ())
	plotTools->hide ();
    }
  else if (w->isA ("Matrix"))
    {
      remove3DMatrixPlots ((Matrix *) w);
      if (w == lastModified)
	{
	  actionUndo->setEnabled (FALSE);
	  actionRedo->setEnabled (FALSE);
	}
      if (!projectHasMatrices ())
	plotMatrixBar->hide ();
    }
  else if (w->isA ("Graph3D"))
    {
      if (!projectHas3DPlots ())
	plot3DTools->hide ();
    }

  if (hiddenWindows->containsRef (w))
    hiddenWindows->take (hiddenWindows->find (w));
  else if (outWindows->containsRef (w))
    outWindows->take (outWindows->find (w));
}

void
ApplicationWindow::closeWindow (myWidget * window)
{
  if (!window)
    return;

  window_to_close = window;

  removeWindowFromLists (window);
  window->folder ()->removeWindow ((myWidget *) window);

//update list view in project explorer
  QListViewItem *it =
    listview->findItem (window->getName(), 0, Qt::ExactMatch | Qt::CaseSensitive);
  if (it)
    listview->takeItem (it);

  //delete window;
  emit modified ();
}

void
ApplicationWindow::about ()
{
  QString version = "QtiPlot " + QString::number (majVersion) + "." +
    QString::number (minVersion) + "." + QString::number (patchVersion) +
    d_extra_version;

  QMessageBox::about (this, tr ("About QtiPlot"),
		      tr ("<h2>" + version + "</h2>"
			  "<p><h3>Copyright(C): Ion Vasilief</h3>"
			  "<p><h3>modified by Sekika Yamamoto</h3>"));
}

void
ApplicationWindow::cascade ()
{
  QPtrList < myWidget > lst = current_folder->windowsList ();
  myWidget *w;
  for(w=lst.first(); w; w=lst.next())
    w->setMinimumSize(500,450);
  ws->cascade ();
  for(w=lst.first(); w; w=lst.next())
    w->setMinimumSize(0,0);
}

void
ApplicationWindow::windowsMenuAboutToShow ()
{
  QWidgetList windows = ws->windowList ();
  int n = int (windows.count ());
  if (!n)
    return;

  windowsMenu->clear ();
  windowsMenu->insertItem (tr ("&Cascade"), this, SLOT (cascade ()));
  windowsMenu->insertItem (tr ("&Tile"), ws, SLOT (tile ()));
  windowsMenu->insertItem (tr ("&Minimize All"), this, 
			   SLOT (minimizeAllFolderWindows ()));
  windowsMenu->insertItem (tr ("&Hide All"), this,
			   SLOT (hideAllFolderWindows())); 
  windowsMenu->insertItem (tr ("&Show All"), this,
			   SLOT (showAllFolderWindows())); 
  windowsMenu->insertSeparator ();
  windowsMenu->insertItem (QPixmap (next_xpm), tr ("&Next"),
			   ws, SLOT (activateNextWindow ()), Key_F5);
  windowsMenu->insertItem (QPixmap (prev_xpm), tr ("&Previous"),
			   ws, SLOT (activatePrevWindow ()), Key_F6);
  windowsMenu->insertSeparator ();
  actionRename->addTo (windowsMenu);
  actionCopyWindow->addTo (windowsMenu);
  windowsMenu->insertSeparator ();
#ifdef SCRIPTING_PYTHON
  actionShowScriptWindow->addTo (windowsMenu);
  windowsMenu->insertSeparator ();
#endif
  actionResizeActiveWindow->addTo (windowsMenu);
  windowsMenu->insertItem (tr ("&Hide Window"),
			   this, SLOT (hideActiveWindow ()));
  windowsMenu->insertItem (QPixmap (close_xpm), tr ("Close &Window"),
			   this, SLOT (closeActiveWindow ()), CTRL + Key_W);

  if (n > 0 && n < 10)
    {
      windowsMenu->insertSeparator ();
      for (int i = 0; i < n; ++i)
	{
	  int id = windowsMenu->insertItem(((myWidget *)windows.at(i))->getName(),
					   this,
					   SLOT (windowsMenuActivated(int)));
	  windowsMenu->setItemParameter (id, i);
	  windowsMenu->setItemChecked (id,
				       ws->activeWindow () == windows.at (i));
	}
    }
  else if (n >= 10)
    {
      windowsMenu->insertSeparator ();
      for (int i = 0; i < 9; ++i)
	{
	  int id = windowsMenu->insertItem (((myWidget *)windows.at(i))->getName(),
					    this,
					    SLOT (windowsMenuActivated(int)));
	  windowsMenu->setItemParameter (id, i);
	  windowsMenu->setItemChecked (id,
				       ws->activeWindow () == windows.at (i));
	}
      windowsMenu->insertSeparator ();
      windowsMenu->insertItem (tr ("More windows"), this,
			       SLOT (showMoreWindows ()));
    }
}

void
ApplicationWindow::showMarkerPopupMenu ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  QPopupMenu markerMenu (this);

  if (g->imageMarkerSelected ())
    {
      markerMenu.insertItem (QPixmap (pixelProfile_xpm),
			     tr ("&View pixel line profile"), this,
			     SLOT (pixelLineProfile ()));
      markerMenu.insertItem (tr ("&Intensity Matrix"), this,
			     SLOT (intensityTable ()));
      markerMenu.insertSeparator ();
    }

  markerMenu.insertItem (QPixmap (cut_xpm), tr ("&Cut"), this,
			 SLOT (cutSelection ()));
  markerMenu.insertItem (QPixmap (copy_xpm), tr ("&Copy"), this,
			 SLOT (copySelection ()));
  markerMenu.insertItem (QPixmap (erase_xpm), tr ("&Delete"), this,
			 SLOT (clearSelection ()));
  markerMenu.insertSeparator ();
  if (g->arrowMarkerSelected ())
    markerMenu.insertItem (tr ("&Properties"), this,
			   SLOT (showLineDialog ()));
  else if (g->imageMarkerSelected ())
    markerMenu.insertItem (tr ("&Properties"), this,
			   SLOT (showImageDialog ()));
  else
    markerMenu.insertItem (tr ("&Properties"), this,
			   SLOT (showTextDialog ()));

  markerMenu.exec (QCursor::pos ());
}

void
ApplicationWindow::showMoreWindows ()
{
  if (explorerWindow->isVisible ())
    QMessageBox::information (this, "QtiPlot",
			      tr
			      ("Please use the project explorer to select a window!"));
  else
    showExplorer ();
}

void
ApplicationWindow::windowsMenuActivated (int id)
{
  QWidgetList windows = ws->windowList ();
  myWidget *w = (myWidget *)windows.at (id);
  if (w)
    {
      w->showNormal ();
      w->setFocus ();
      if (hidden (w))
	{
	  hiddenWindows->take (hiddenWindows->find (w));
	  setListView (w->getName(), tr ("Normal"));
	}
    }
}

void
ApplicationWindow::newProject ()
{
  saveSettings ();		//the recent projects must be saved 

  ApplicationWindow *ed = new ApplicationWindow ();
  ed->applyUserSettings ();
  ed->newTable ();

  if (this->isMaximized ())
    ed->showMaximized ();
  else
    ed->show ();
  
  ed->saved = TRUE;
  ed->actionSaveProject->setEnabled (false);

  this->close ();
}

void
ApplicationWindow::modifiedProject ()
{
  //fprintf(stderr,"modifiedProject1\n");

  actionSaveProject->setEnabled (TRUE);
  saved = FALSE;
}

void
ApplicationWindow::modifiedProject (QWidget * w)
{
  //qDebug("modifiedProject : %s\n",((myWidget *)w)->getName().ascii());

  actionSaveProject->setEnabled (TRUE);
  saved = FALSE;

  actionUndo->setEnabled (TRUE);
  actionRedo->setEnabled (FALSE);
  lastModified = w;

  if (lastModified->isA ("Table"))
    {
      Table *t = (Table *) lastModified;
      undoBuffer = t->getSpecifications();
      t->setSpecifications( t->saveToString("geometry\n") );
    }
  else if (lastModified->isA ("Matrix"))
    {
      Matrix *m = (Matrix *) lastModified;
      undoBuffer = m->getSpecifications();
      m->setSpecifications( m->saveToString("geometry\n") );
    }
}

void
ApplicationWindow::timerEvent (QTimerEvent * e)
{
  if (e->timerId () == savingTimerId)
    saveProject ();
  else
    QWidget::timerEvent (e);
}

void
ApplicationWindow::dropEvent (QDropEvent * e)
{
  //fprintf(stderr,"dropEvent\n");
/*  for test */
/*
  QStrList dummy;
  if (QUriDrag::decode(e, dummy))
    {
      int c = dummy.count();
      for (int i = 0 ; i < c; i++)
        {
          showResults( QString(dummy.at(i))+"\n" );
          showResults( tr("URI : ") );
          showResults( QUriDrag::uriToUnicodeUri(dummy.at(i))+"\n" );
          showResults( tr("Filename : ")); 
          showResults( QUriDrag::uriToLocalFile(dummy.at(i))+"\n" );
        }
    }
*/
  QStringList fileNames;
  if (QUriDrag::decodeLocalFiles (e, fileNames))
    {
      QStringList lst = QImage::inputFormatList () << "JPG";
      QStringList asciiFiles;
      int files = (int) fileNames.count();
      for (int i = 0; i < files; i++)
	{
	  QString fn = fileNames[i];
	  // showResults( tr("Importing : ") + fn + "\n" );
          // I don't know why, but QUriDrag::decodeLocalFiles puts "/" at the top of the path for the network file. 
          // This must be trimmed.
#ifdef Q_OS_WIN
          fn.replace("/\\\\","\\\\");
#endif
          //
          QFileInfo fi(fn);
	  if(fi.exists()){
	    QString ext = fi.extension().lower();
	    QStringList l = lst.grep (ext, false);
	    if (!ext.isEmpty() && l.count()>0)
	      loadImage (fn);
	    else if (ext == "opj" || ext == "qti")
	      open (fn);
	    else
	      asciiFiles << fn;
	  }
	}
      loadMultipleASCIIFiles (asciiFiles, 0);
    }
}

void
ApplicationWindow::dragEnterEvent (QDragEnterEvent * e)
{
  if (e->source ())
    {
      e->ignore ();
      return;
    }

  e->accept (QUriDrag::canDecode (e));
}

void
ApplicationWindow::closeEvent (QCloseEvent * ce)
{
  if (!saved)
    {
      QString s =
	tr ("Save changes to project: <p><b> %1 </b> ?").arg (projectname);
      switch (QMessageBox::
	      information (this, tr ("QtiPlot"), s, tr ("Yes"), tr ("No"),
			   tr ("Cancel"), 0, 2))
	{
	case 0:
	  if (!saveProject ())
	    {
	      ce->ignore ();
	      return;
	    }
	  saveSettings ();	//the recent projects must be saved 
	  ce->accept ();
	  break;

	case 1:
	default:
	  saveSettings ();	//the recent projects must be saved 
	  ce->accept ();
	  break;

	case 2:
	  ce->ignore ();
	  return;
	}
    }
  else
    {
      saveSettings ();		//the recent projects must be saved 
      ce->accept ();
    }

  QString lockname = projectname+".lock";
  if(QFile::exists(lockname))
    QFile::remove(lockname);
}

void
ApplicationWindow::customEvent (QCustomEvent * e)
{
  if (e->type () == SCRIPTING_CHANGE_EVENT)
    {
      scriptingChangeEvent ((ScriptingChangeEvent *) e);
      connect (scriptEnv,
	       SIGNAL (error (const QString &, const QString &, int)), this,
	       SLOT (scriptError (const QString &, const QString &, int)));
    }
}

void
ApplicationWindow::deleteSelectedItems ()
{
  if (folderview->hasFocus ()
      && folderview->currentItem () != folderview->firstChild ())
    { //we never allow the user to delete the project folder item
      deleteFolder ();
      return;
    }

  QListViewItem *item;
  QPtrList < QListViewItem > lst;
  for (item = listview->firstChild (); item; item = item->nextSibling ())
    {
      if (item->isSelected ())
	lst.append (item);
    }

  int choice = QMessageBox::question(0, tr ("QtiPlot"),
				     tr ("Do you really want to delete these items?"),
				     QMessageBox::Yes | QMessageBox::Default,
				     QMessageBox::No | QMessageBox::Escape);
 if (choice == QMessageBox::Yes)
   {
     folderview->blockSignals (true);
     for (item = lst.first (); item; item = lst.next ())
       {
	 if (item->rtti () == FolderListItem::RTTI)
	   {
	     Folder *f = ((FolderListItem *) item)->folder ();
	     if (deleteFolder (f))
	       delete item;
	   }
	 else
	   {
	     ((WindowListItem *) item)->window ()->askOnCloseEvent(false);
	     ((WindowListItem *) item)->window ()->close ();
	   }
       }
     folderview->blockSignals (false);
   }
}

void
ApplicationWindow::minimizeSelectedItems()
{
  QListViewItem *item;
  QPtrList < QListViewItem > lst;
  for (item = listview->firstChild (); item; item = item->nextSibling ())
    {
      if (item->isSelected ())
	lst.append (item);
    }

  for (item = lst.first (); item; item = lst.next ())
    {
      if (item->rtti () != FolderListItem::RTTI)
	{
	  ((WindowListItem *) item)->window ()->showMinimized();
	}
    }
}

void
ApplicationWindow::hideSelectedItems()
{
  QListViewItem *item;
  QPtrList < QListViewItem > lst;
  for (item = listview->firstChild (); item; item = item->nextSibling ())
    {
      if (item->isSelected ())
	lst.append (item);
    }

  for (item = lst.first (); item; item = lst.next ())
    {
      if (item->rtti () != FolderListItem::RTTI)
	{
	  hideWindow( ((WindowListItem *) item)->window ());
	}
    }
}

void
ApplicationWindow::showSelectedItems()
{
  QListViewItem *item;
  QPtrList < QListViewItem > lst;
  for (item = listview->firstChild (); item; item = item->nextSibling ())
    {
      if (item->isSelected ())
	lst.append (item);
    }

  for (item = lst.first (); item; item = lst.next ())
    {
      if (item->rtti () != FolderListItem::RTTI)
	{
	  ((WindowListItem *) item)->window ()->showNormal();
	}
    }
}

void
ApplicationWindow::showListViewSelectionMenu (const QPoint & p)
{
  QPopupMenu cm (this);
  cm.insertItem (tr ("&Delete Selection"), this,
		 SLOT (deleteSelectedItems ()), Key_F8);
  cm.insertItem (tr ("&Minimize Selected Windows"), this,
		 SLOT (minimizeSelectedItems ()));
  cm.insertItem (tr ("&Hide Selected Windows"), this,
		 SLOT (hideSelectedItems ()));
  cm.insertItem (tr ("&Show Selected Windows"), this,
		 SLOT (showSelectedItems ()));
  cm.exec (p);
}

void
ApplicationWindow::showListViewPopupMenu (const QPoint & p)
{
  QPopupMenu cm (this);
  QPopupMenu window (this);

  actionNewTable->addTo (&window);
  actionNewMatrix->addTo (&window);
  actionNewNote->addTo (&window);
  actionNewGraph->addTo (&window);
  actionNewFunctionPlot->addTo (&window);
  actionNewSurfacePlot->addTo (&window);
  cm.insertItem (tr ("New &Window"), &window);

  cm.insertItem (QPixmap (newfolder_xpm), tr ("New F&older"), this,
		 SLOT (addSubFolder ()), Key_F7);
  cm.insertSeparator ();
  cm.insertItem (tr ("Auto &Column Width"), listview, SLOT (adjustColumns ()));
  cm.exec (p);
}

void
ApplicationWindow::showWindowPopupMenu (QListViewItem * it, const QPoint & p,
					int)
{
  if (folderview->isRenaming ())
    return;

  if (!it)
    {
      showListViewPopupMenu (p);
      return;
    }

  QListViewItem *item;
  int selected = 0;
  for (item = listview->firstChild (); item; item = item->nextSibling ())
    {
      if (item->isSelected ())
	selected++;

      if (selected > 1)
	{
	  showListViewSelectionMenu (p);
	  return;
	}
    }

  if (it->rtti () == FolderListItem::RTTI)
    {
      current_folder = ((FolderListItem *) it)->folder ();
      showFolderPopupMenu (it, p, false);
      return;
    }

  myWidget *w = ((WindowListItem *) it)->window ();
  if (w)
    {
      QPopupMenu cm (this);
      QPopupMenu plots (this);

      actionActivateWindow->addTo (&cm);
      actionMinimizeWindow->addTo (&cm);
      actionMaximizeWindow->addTo (&cm);
      cm.insertSeparator ();
      if (!hidden (w))
	actionHideWindow->addTo (&cm);
      else
	actionShowWindow->addTo (&cm);
      cm.insertItem (QPixmap (close_xpm), tr ("&Delete Window"), w,
		     SLOT (close ()), Key_F8);
      cm.insertSeparator ();
      cm.insertItem (tr ("&Rename Window"), this, SLOT (renameWindow ()),
		     Key_F2);
      actionResizeWindow->addTo (&cm);
      cm.insertSeparator ();
      actionPrintWindow->addTo (&cm);
      cm.insertSeparator ();
      cm.insertItem (tr ("&Properties"), this, SLOT (windowProperties ()));

      if (w->isA ("Table"))
	{
	  QStringList graphs = dependingPlots (w->getName());
	  if (int (graphs.count ()) > 0)
	    {
	      cm.insertSeparator ();
	      for (int i = 0; i < int (graphs.count ()); i++)
		plots.insertItem (graphs[i], (myWidget *)window (graphs[i]),
				  SLOT (activate()));
	      cm.insertItem (tr ("D&epending Graphs"), &plots);
	    }
	}
      else if (w->isA ("Matrix"))
	{
	  QStringList graphs = depending3DPlots ((Matrix *) w);//How about Spectrograms?
	  if (int (graphs.count ()) > 0)
	    {
	      cm.insertSeparator ();
	      for (int i = 0; i < int (graphs.count ()); i++)
		plots.insertItem (graphs[i], (myWidget *)window (graphs[i]),
				  SLOT (activate()));
	      cm.insertItem (tr ("D&epending 3D Graphs"), &plots);
	    }
	}
      else if (w->isA ("MultiLayer"))
	{
	  tablesDepend->clear ();
	  QStringList tbls = multilayerDependencies (w);
	  int n = int (tbls.count ());
	  if (n > 0)
	    {
	      cm.insertSeparator ();
	      for (int i = 0; i < n; i++)
		tablesDepend->insertItem (tbls[i], (myWidget *)window(tbls[i]),
					  SLOT(activate()));
	      cm.insertItem (tr ("D&epends on"), tablesDepend);
	    }
	}
      else if (w->isA ("Graph3D"))
	{
	  Graph3D *sp = (Graph3D *) w;
	  Matrix *m = sp->getMatrix ();
	  QString formula = sp->formula ();
	  if (!formula.isEmpty ())
	    {
	      if (formula.contains ('_'))
		{
		  cm.insertSeparator ();
		  QString tl = formula.section('_',0,0);
                  tablesDepend->clear ();
		  tablesDepend->insertItem (tl, (myWidget *)window(tl),
					    SLOT(activate()));
		  cm.insertItem (tr ("D&epends on"), tablesDepend);
		}
	      else if (m)
		{
		  cm.insertSeparator ();
		  plots.insertItem (m->getName(), (myWidget *)m,
				    SLOT (activate()));
		  cm.insertItem (tr ("D&epends on"), &plots);
		}
	    }
	}
      cm.exec (p);
    }
}

void
ApplicationWindow::showTable (int i)
{
  Table *t = table (tablesDepend->text (i));
  if (!t)
    return;
  updateWindowLists (t);

  if(aw && aw->status()==myWidget::Maximized)
    t->showMaximized ();
  else
    t->showNormal();
}

void
ApplicationWindow::showTable (const QString & curve)
{
  Table *w = table (curve);
  if (!w)
    return;

  updateWindowLists (w);
  int colIndex = w->colIndex (curve);
  w->setSelectedCol (colIndex);
  w->table ()->clearSelection ();
  w->table ()->selectColumn (colIndex);

  if(aw && aw->status()==myWidget::Maximized)
    w->showMaximized ();
  else
    w->showNormal();
}

QStringList
ApplicationWindow::depending3DPlots (Matrix * m)
{
  QWidgetList *windows = windowsList ();
  QStringList plots;
  //for (int i = 0; i < (int) windows->count (); i++)
  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      if (w->isA ("Graph3D") && ((Graph3D *) w)->getMatrix () == m)
	plots << ((myWidget *)w)->getName();
      else if (w->isA ("MultiLayer"))
	{
	  QWidgetList *lst = ((MultiLayer *) w)->graphPtrs ();
	  for (Graph *g = (Graph *)lst->first(); g; g = (Graph *)lst->next())
            for (int i = 0; i < g->curves(); i++)
              {
                Spectrogram *sp = (Spectrogram *) g->curve (i);
                if (sp && sp->rtti () == QwtPlotItem::Rtti_PlotSpectrogram
		    && sp->matrix() == m)
		  plots << ((myWidget *)w)->getName();
              }
	}
    }
  delete windows;
  return plots;
}

QStringList
ApplicationWindow::dependingPlots (const QString & name)
{
  QWidgetList *windows = windowsList ();
  QStringList onPlot, plots;

  QRegExp rex("^"+name+"_");

  //for (int i = 0; i < (int) windows->count (); i++)
  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      QString wname = ((myWidget *)w)->getName();
      
      if (w->isA ("MultiLayer"))
	{
	  QWidgetList *lst = ((MultiLayer *) w)->graphPtrs ();
	  for (Graph *g = (Graph *)lst->first(); g; g = (Graph *)lst->next())
	    {
	      onPlot = g->curvesList ();
	      onPlot = onPlot.grep (rex);
	      if (int (onPlot.count ()) && plots.contains (wname) <= 0)
		plots << wname;
	    }
	}
      else if (w->isA ("Graph3D"))
	{
	  if ((((Graph3D *) w)->formula ()).contains (rex)
	      && plots.contains (wname) <= 0)
	    plots << wname;
	}
    }
  delete windows;
  return plots;
}

QStringList
ApplicationWindow::multilayerDependencies (QWidget * w)
{
  QStringList windows;
  MultiLayer *m = (MultiLayer *) w;
  QWidgetList *lst = m->graphPtrs ();
  for (Graph *g = (Graph *)lst->first(); g; g = (Graph *)lst->next())
    for (int i = 0; i < g->curves (); i++)
      {
	QwtPlotCurve *c = g->curve (i);
	if (c->rtti () == QwtPlotItem::Rtti_PlotSpectrogram)
	  {
	    QString title = ((Spectrogram *)c)->matrix()->getName();
	    if(windows.contains(title) <= 0)
	      windows << title;
	  }
	else if(c->rtti () == QwtPlotItem::Rtti_PlotCurve)
	  {
	    QString title = c->title().text().section('_',0,0);
	    if(windows.contains(title) <= 0)
	      windows << title;
	  }
      }
  return windows;
}

void
ApplicationWindow::showGraphContextMenu ()
{
  QWidget *w = (QWidget *) ws->activeWindow ();
  if (!w)
    return;

  if (w->isA ("MultiLayer"))
    {
      MultiLayer *plot = (MultiLayer *) w;
      QPopupMenu cm (this);
      QPopupMenu exports (this);
      QPopupMenu copy (this);
      QPopupMenu prints (this);
      QPopupMenu graphAnalysisMenu (this);
      QPopupMenu smooth (this);
      QPopupMenu filter (this);
      QPopupMenu decay (this);
      QPopupMenu translate (this);
      QPopupMenu multiPeakMenu (this);
      QPopupMenu deconvolute (this);

      Graph *ag = (Graph *) plot->activeGraph ();

      if (ag->isPiePlot ())
	cm.insertItem (tr ("Re&move pie curve"), ag, SLOT (removePie ()));
      else
	{
	  actionShowCurvesDialog->addTo (&cm);
	  actionAddErrorBars->addTo (&cm);
	  actionAddFunctionCurve->addTo (&cm);
	  actionNewLegend->addTo (&cm);

	  cm.insertSeparator ();

	  actionResetColors->addTo (&cm);
	  actionBlackenColors->addTo (&cm);
	  actionResetSymbols->addTo (&cm);
	  actionResetFonts->addTo (&cm);

	  cm.insertSeparator ();

	  actionTranslateVert->addTo (&translate);
	  actionTranslateHor->addTo (&translate);
	  graphAnalysisMenu.insertItem (tr ("&Translate"), &translate);

	  actionSubtractLine->addTo (&graphAnalysisMenu);
	  actionSubtractBack->addTo (&graphAnalysisMenu);
	  actionRemoveSpike->addTo (&graphAnalysisMenu);

	  graphAnalysisMenu.insertSeparator ();

	  actionCurveCalc->addTo(&graphAnalysisMenu);

	  graphAnalysisMenu.insertSeparator ();

	  actionDifferentiate->addTo (&graphAnalysisMenu);
	  actionShowIntDialog->addTo (&graphAnalysisMenu);
	  actionShowFwhmDialog->addTo (&graphAnalysisMenu);
	  actionShowMomentDialog->addTo (&graphAnalysisMenu);
	  graphAnalysisMenu.insertSeparator ();
	  actionSmoothSavGol->addTo (&smooth);
	  actionSmoothAverage->addTo (&smooth);
	  actionGaussConv->addTo (&smooth);
	  actionLorentzConv->addTo (&smooth);
	  graphAnalysisMenu.insertItem (tr ("&Smooth"), &smooth);

	  actionLowPassFilter->addTo (&filter);
	  actionHighPassFilter->addTo (&filter);
	  actionBandPassFilter->addTo (&filter);
	  actionBandBlockFilter->addTo (&filter);
	  actionBandInterpFilter->addTo (&filter);
	  graphAnalysisMenu.insertItem (tr ("&FFT filter"), &filter);
	  graphAnalysisMenu.insertSeparator ();
	  actionInterpolate->addTo (&graphAnalysisMenu);
	  actionFFT->addTo (&graphAnalysisMenu);
	  
	  actionConvoluteCurves->addTo (&graphAnalysisMenu);
	  actionDeconvFFT->addTo(&deconvolute);
          actionDeconvAndre->addTo(&deconvolute);
	  actionDeconvGS->addTo(&deconvolute);
	  graphAnalysisMenu.insertItem(tr("Deconvolute"),&deconvolute);

	  graphAnalysisMenu.insertSeparator ();
	  actionFitLinear->addTo (&graphAnalysisMenu);
	  actionShowFitPolynomDialog->addTo (&graphAnalysisMenu);
	  actionShowSineFitDialog->addTo (&graphAnalysisMenu);
	  graphAnalysisMenu.insertSeparator ();
	  actionShowExpDecayDialog->addTo (&decay);
	  actionShowTwoExpDecayDialog->addTo (&decay);
	  actionShowExpDecay3Dialog->addTo (&decay);
	  graphAnalysisMenu.insertItem (tr ("Fit E&xponential Decay"), &decay);
	  actionFitExpGrowth->addTo (&graphAnalysisMenu);
	  actionFitSigmoidal->addTo (&graphAnalysisMenu);
	  actionFitGauss->addTo (&graphAnalysisMenu);
	  actionFitLorentz->addTo (&graphAnalysisMenu);

	  actionMultiPeakGauss->addTo (&multiPeakMenu);
	  actionMultiPeakLorentz->addTo (&multiPeakMenu);
	  graphAnalysisMenu.insertItem (tr ("Fit &Multi-peak"), &multiPeakMenu);
	  graphAnalysisMenu.insertSeparator ();
	  actionShowFitDialog->addTo (&graphAnalysisMenu);
	  cm.insertItem (tr ("Anal&yse"), &graphAnalysisMenu);
	}

      if (copiedLayer)
	{
	  cm.insertSeparator ();
	  cm.insertItem (QPixmap (paste_xpm), tr ("&Paste layer"), this,
			 SLOT (pasteSelection ()));
	}
      else if (copiedMarkerType >= 0)
	{
	  cm.insertSeparator ();
	  if (copiedMarkerType == Graph::Text)
	    cm.insertItem (QPixmap (paste_xpm), tr ("&Paste text"), plot,
			   SIGNAL (pasteMarker ()));
	  else if (copiedMarkerType == Graph::Arrow)
	    cm.insertItem (QPixmap (paste_xpm), tr ("&Paste line/arrow"),
			   plot, SIGNAL (pasteMarker ()));
	  else if (copiedMarkerType == Graph::Image)
	    cm.insertItem (QPixmap (paste_xpm), tr ("&Paste image"), plot,
			   SIGNAL (pasteMarker ()));
	}
      cm.insertSeparator ();
      copy.insertItem (tr ("&Layer"), this, SLOT (copyActiveLayer ()));
      copy.insertItem (tr ("&Window"), plot, SLOT (copyAllLayers ()));
      cm.insertItem (QPixmap (copy_xpm), tr ("&Copy"), &copy);

      exports.insertItem (tr ("&Layer"), this, SLOT (exportLayer ()));
      exports.insertItem (tr ("&Window"), this, SLOT (exportGraph ()));
      cm.insertItem (tr ("E&xport"), &exports);

      prints.insertItem (tr ("&Layer"), plot, SLOT (printActiveLayer ()));
      prints.insertItem (tr ("&Window"), plot, SLOT (print ()));
      cm.insertItem (QPixmap (fileprint_xpm), tr ("&Print"), &prints);
      cm.insertSeparator ();
      cm.insertItem (QPixmap (resize_xpm), tr ("&Geometry"), plot,
		     SIGNAL (showGeometryDialog ()));
      cm.insertItem (tr ("P&roperties"), this,
		     SLOT (showGeneralPlotDialog ()));
      cm.insertSeparator ();
      cm.insertItem (tr ("Align Curves"), this,
		     SLOT (showAlignCurvesDialog ()));
      cm.insertItem (tr ("Cascade Curves"), this,
		     SLOT (showCascadeDialog ()));
      cm.insertItem (tr ("Normalize Curves"), this,
		     SLOT (showNormalizeDialog ()));
      cm.insertSeparator ();
      cm.insertItem( tr("Resize Layer"), plot, SLOT(highlightActiveLayer()));
      cm.insertItem (QPixmap (close_xpm), tr ("&Delete Layer"), plot,
		     SLOT (confirmRemoveLayer ()));
      cm.exec (QCursor::pos ());
    }
}

void
ApplicationWindow::showWindowContextMenu ()
{
  QWidget *w = (QWidget *) ws->activeWindow ();
  if (!w)
    return;

  QPopupMenu cm (this);
  QPopupMenu plot3D (this);
  if (w->isA ("MultiLayer"))
    {
      MultiLayer *g = (MultiLayer *) w;
      if (copiedLayer)
	{
	  cm.insertItem (QPixmap (paste_xpm), tr ("&Paste Layer"), this,
			 SLOT (pasteSelection ()));
	  cm.insertSeparator ();
	}

      actionAddLayer->addTo (&cm);
      cm.insertSeparator ();
      if (g->layers () != 0)
	{
          actionResizeLayer->addTo (&cm);
	  actionDeleteLayer->addTo (&cm);
	  cm.insertSeparator ();
	  actionShowPlotGeometryDialog->addTo (&cm);
	  actionShowLayerDialog->addTo (&cm);
	  actionShowLayerOrderDialog->addTo (&cm);
	  cm.insertSeparator ();
	}
      actionRename->addTo (&cm);
      actionCopyWindow->addTo (&cm);
      cm.insertSeparator ();
      cm.insertItem (QPixmap (copy_xpm), tr ("&Copy Page"), g,
		     SLOT (copyAllLayers ()));
      cm.insertItem (tr ("E&xport Page"), this, SLOT (exportGraph ()));
      actionPrint->addTo (&cm);
      cm.insertSeparator ();
      actionCloseWindow->addTo (&cm);
    }
  else if (w->isA ("Graph3D"))
    {
      Graph3D *g = (Graph3D *) w;
      if (!g->hasData ())
	{
	  cm.insertItem (tr ("&Graph 3D"), &plot3D);
	  actionAdd3DData->addTo (&plot3D);
	  plot3D.insertItem (tr ("&Matrix"), this,
			     SLOT (add3DMatrixPlot ()));
	  actionEditSurfacePlot->addTo (&plot3D);
	}
      else
	{
	  if (g->getTable ())
	    cm.insertItem (tr ("Choose &Data Set"), this,
			   SLOT (change3DData ()));
	  else if (g->getMatrix ())
	    cm.insertItem (tr ("Choose &Matrix"), this,
			   SLOT (change3DMatrix ()));
	  else if (g->userFunction ())
	    actionEditSurfacePlot->addTo (&cm);
	  cm.insertItem (QPixmap (erase_xpm), tr ("C&lear"), g,
			 SLOT (clearData ()));
	}

      cm.insertSeparator ();
      actionRename->addTo (&cm);
      actionCopyWindow->addTo (&cm);
      cm.insertSeparator ();
      cm.insertItem (tr ("&Copy Graph"), g, SLOT (copyImage ()));
      cm.insertItem (tr ("&Export"), g, SLOT (saveImage ()));
      actionPrint->addTo (&cm);
      cm.insertSeparator ();
      actionCloseWindow->addTo (&cm);
    }
  else if (w->isA ("Matrix"))
    {
      Matrix *t = (Matrix *) w;
      cm.insertItem (QPixmap (cut_xpm), tr ("Cu&t"), t,
		     SLOT (cutSelection ()));
      cm.insertItem (QPixmap (copy_xpm), tr ("&Copy"), t,
		     SLOT (copySelection ()));
      cm.insertItem (QPixmap (paste_xpm), tr ("&Paste"), t,
		     SLOT (pasteSelection ()));
      cm.insertItem (QPixmap (paste_xpm), tr ("Paste Transposed"), t,
		     SLOT (pasteTransposed ()));
      cm.insertSeparator ();
      if (t->rowsSelected ())
	{
	  cm.insertItem (tr ("&Insert Row"), t, SLOT (insertRow ()));
	  cm.insertItem (tr ("&Add Row"), t, SLOT (appendRow ()));
	  cm.insertItem (QPixmap (close_xpm), tr ("&Delete Rows"), t,
			 SLOT (deleteSelectedRows ()));
	}
      else if (t->columnsSelected ())
	{
	  cm.insertItem (tr ("&Insert Column"), t, SLOT (insertColumn ()));
	  cm.insertItem (QPixmap (close_xpm), tr ("&Delete Columns"), t,
			 SLOT (deleteSelectedColumns ()));
	}
      cm.insertItem (QPixmap (erase_xpm), tr ("Clea&r"), t,
		     SLOT (clearSelection ()));
      cm.insertSeparator ();
      cm.insertItem( tr("Area Statistics"), this, 
		     SLOT (showMatrixAreaStatistics()));
    }
  cm.exec (QCursor::pos ());
}

void
ApplicationWindow::showWindowTitleBarMenu ()
{
  if (!ws->activeWindow ())
    return;

  QPopupMenu cm (this);

  if (ws->activeWindow ()->inherits ("Table"))
    {
      actionShowExportASCIIDialog->addTo (&cm);
      cm.insertSeparator ();
    }

  if (ws->activeWindow ()->isA ("Note"))
    actionSaveNote->addTo (&cm);
  else
    actionSaveTemplate->addTo (&cm);

  actionPrint->addTo (&cm);
  cm.insertSeparator ();
  actionRename->addTo (&cm);
  actionCopyWindow->addTo (&cm);
  cm.insertSeparator ();
  cm.insertItem (tr ("&Hide Window"), this, SLOT (hideActiveWindow ()));
  actionCloseWindow->addTo (&cm);
  cm.exec (QCursor::pos ());
}

void
ApplicationWindow::showTableContextMenu (bool selection)
{
  Table *t = (Table *) ws->activeWindow ();
  if (!t)
    return;

  QPopupMenu cm (this);
  if (selection)
    {
      if ((int) t->selectedColumns ().count () > 0)
	{
	  showColMenu (t->firstSelectedColumn ());
	  return;
	}
      else if (t->selectedRows () == 1)
	{
	  cm.insertItem (QPixmap (cut_xpm), tr ("Cu&t"), t,
			 SLOT (cutSelection ()));
	  cm.insertItem (QPixmap (copy_xpm), tr ("&Copy"), t,
			 SLOT (copySelection ()));
	  cm.insertItem (QPixmap (paste_xpm), tr ("&Paste"), t,
			 SLOT (pasteSelection ()));
	  cm.insertItem (QPixmap (paste_xpm), tr ("Paste Transposed"), t,
			 SLOT (pasteTransposed ()));
	  cm.insertSeparator ();
	  actionTableRecalculate->addTo (&cm);
	  cm.insertItem (tr ("&Insert Row"), t, SLOT (insertRow ()));
	  cm.insertItem (tr ("&Add Row"), t, SLOT (addRow ()));
	  cm.insertItem (QPixmap (close_xpm), tr ("&Delete Row"), t,
			 SLOT (deleteSelectedRows ()));
	  cm.insertItem (QPixmap (erase_xpm), tr ("Clea&r Row"), t,
			 SLOT (clearSelection ()));
	  cm.insertSeparator ();
	  actionShowRowStatistics->addTo (&cm);
	  cm.insertItem ( tr ("Area Statistics"), this,
	  		 SLOT (showTableAreaStatistics()));
	}
      else if (t->selectedRows () > 1)
	{
	  cm.insertItem (QPixmap (cut_xpm), tr ("Cu&t"), t,
			 SLOT (cutSelection ()));
	  cm.insertItem (QPixmap (copy_xpm), tr ("&Copy"), t,
			 SLOT (copySelection ()));
	  cm.insertItem (QPixmap (paste_xpm), tr ("&Paste"), t,
			 SLOT (pasteSelection ()));
	  cm.insertItem (QPixmap (paste_xpm), tr ("Paste Transposed"), t,
			 SLOT (pasteTransposed ()));
	  cm.insertSeparator ();
	  actionTableRecalculate->addTo (&cm);
	  cm.insertItem (QPixmap (close_xpm), tr ("&Delete Rows"), t,
			 SLOT (deleteSelectedRows ()));
	  cm.insertItem (QPixmap (erase_xpm), tr ("Clea&r Rows"), t,
			 SLOT (clearSelection ()));
	  cm.insertSeparator ();
	  actionShowRowStatistics->addTo (&cm);
	  cm.insertItem ( tr ("Area Statistics"), this,
	  		 SLOT (showTableAreaStatistics()));
	}
      else
	{
	  cm.insertItem (QPixmap (cut_xpm), tr ("Cu&t"), t,
			 SLOT (cutSelection ()));
	  cm.insertItem (QPixmap (copy_xpm), tr ("&Copy"), t,
			 SLOT (copySelection ()));
	  cm.insertItem (QPixmap (paste_xpm), tr ("&Paste"), t,
			 SLOT (pasteSelection ()));
	  cm.insertItem (QPixmap (paste_xpm), tr ("Paste Transposed"), t,
			 SLOT (pasteTransposed ()));
	  cm.insertSeparator ();
	  actionTableRecalculate->addTo (&cm);
	  cm.insertItem (QPixmap (erase_xpm), tr ("Clea&r"), t,
			 SLOT (clearSelection ()));
	  cm.insertSeparator ();
	  cm.insertItem ( tr ("Area Statistics"), this,
	  		 SLOT (showTableAreaStatistics()));
	}
    }
  else
    {
      actionShowExportASCIIDialog->addTo (&cm);
      cm.insertSeparator ();
      actionAddColToTable->addTo (&cm);
      actionClearTable->addTo (&cm);
      cm.insertSeparator ();
      actionAddRowToTable->addTo (&cm);
      actionGoToRow->addTo (&cm);
    }
  cm.exec (QCursor::pos ());
}

void
ApplicationWindow::chooseHelpFolder ()
{
  QString dir =
    QFileDialog::getExistingDirectory (qApp->applicationDirPath (), this,
				       QString::null,
				       tr
				       ("Choose the location of the QtiPlot help folder!"),
				       true, true);

  if (!dir.isEmpty ())
    {
      helpFilePath = dir + "/index.html";

      QFile helpFile (helpFilePath);
      if (!helpFile.exists ())
	{
	  QMessageBox::critical (this,
				 tr ("QtiPlot - index.html File Not Found!"),
				 tr
				 ("There is no file called <b>index.html</b> in this folder.<br>Please choose another folder!"));
	}
    }
}

void
ApplicationWindow::showStandAloneHelp ()
{
// qassistantclient crashes on Windows7
#ifndef Q_OS_WIN
  delete assistant;
  assistant = new QAssistantClient ("");
#endif
  showHelp ();
}

void
ApplicationWindow::showHelp ()
{
  QFile helpFile (helpFilePath);
  if (!helpFile.exists ())
    {
      QMessageBox::critical (this, tr ("QtiPlot - Help Files Not Found!"),
			     tr("Please indicate the location of the help file!<br><br>"
				"<p>The manual can be downloaded from the following internet address:</p>"
				"<p><font color=blue>'http://soft.proindependent.com/manuals.html'</font></p>"));
      QString fn =
	QFileDialog::getOpenFileName (QDir::currentDirPath (), "*.html",
				      this);
      if (!fn.isEmpty ())
	{
	  QFileInfo fi (fn);
	  helpFilePath = fi.absFilePath ();
	  saveSettings ();
	}
    }

#ifdef Q_OS_WIN
  QProcess proc;
  proc.addArgument(qApp->applicationDirPath()+"/assistant.exe");
  proc.addArgument(helpFilePath);
  if(!proc.start())
  {
    QMessageBox::critical (this, tr ("QtiPlot - assistant can not be started!"),
			   tr("The assistant could not start because of unknown reason!"));
  }
#else
  QFileInfo fi (helpFilePath);
  QString profilePath = QString (fi.dirPath (true) + "/qtiplot.adp");
  if (!QFile (profilePath).exists ())
    {
      QMessageBox::critical (this, tr ("QtiPlot - Help Profile Not Found!"),
			     tr("The assistant could not start because the file <b>%1</b> was not found in the help file directory!"
				"<p>This file is provided with the QtiPlot manual which can be downloaded from the following internet address:</p>"
				"<p><font color=blue>'http://soft.proindependent.com/manuals.html'</font></p>").
			     arg ("qtiplot.adp"));
      return;
    }

  QStringList cmdLst = QStringList () << "-profile" << profilePath;
  assistant->setArguments (cmdLst);
  assistant->showPage (helpFilePath);
#endif
}

void
ApplicationWindow::showPlotWizard ()
{
  if (tableWindows.count () > 0)
    {
      plotWizard *pw =
	new plotWizard (this, "plotWizard", TRUE,
			WStyle_Tool | WDestructiveClose);
      connect (pw, SIGNAL (plot (const QStringList &)), this,
	       SLOT (multilayerPlot (const QStringList &)));
      connect (pw, SIGNAL (plot3D (const QString &)), this,
	       SLOT (dataPlotXYZ (const QString &)));
      connect (pw, SIGNAL (plot3DRibbon (const QString &)), this,
	       SLOT (dataPlot3D (const QString &)));

      pw->insertTablesList (tableWindows);
      pw->setColumnsList (columnsList ());
      pw->changeColumnsList (tableWindows[0]);
      pw->show ();
      pw->setActiveWindow ();
    }
  else
    QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			  tr
			  ("<h4>There are no tables available in this project.</h4>"
			   "<p><h4>Please create a table and try again!</h4>"));
}

void
ApplicationWindow::showFunctionDialog (int index)
{
  if (!activeGraph)
    return;
  showFunctionDialog (activeGraph, index);
}

void
ApplicationWindow::showFunctionDialog (Graph * g, int index)
{
  if (!g)
    return;

  FunctionDialog *fd = functionDialog ();
  fd->setCaption (tr ("QtiPlot - Edit function"));
  fd->setCurveToModify (g, index);
}

FunctionDialog *
ApplicationWindow::functionDialog ()
{
  FunctionDialog *fd =
    new FunctionDialog (this, "FunctionDialog", TRUE,
			WStyle_Tool | WDestructiveClose);
  connect (fd, SIGNAL (clearParamFunctionsList ()), this,
	   SLOT (clearParamFunctionsList ()));
  connect (fd, SIGNAL (clearPolarFunctionsList ()), this,
	   SLOT (clearPolarFunctionsList ()));

  fd->insertParamFunctionsList (xFunctions, yFunctions);
  fd->insertPolarFunctionsList (rFunctions, tetaFunctions);
  fd->show ();
  fd->setActiveWindow ();

  return fd;
}

void
ApplicationWindow::addFunctionCurve ()
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA ("MultiLayer"))
    return;

  if (((MultiLayer *) w)->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      return;
    }

  Graph *g = ((MultiLayer *) w)->activeGraph ();
  if (g)
    {
      activeGraph = g;
      FunctionDialog *fd = functionDialog ();
      if (fd)
	fd->setGraph (g);
    }
}

void
ApplicationWindow::updateFunctionLists (int type, QStringList & formulas)
{
  int maxListSize = 10;
  if (type == 2)
    {
      rFunctions.remove (formulas[0]);
      rFunctions.push_front (formulas[0]);

      tetaFunctions.remove (formulas[1]);
      tetaFunctions.push_front (formulas[1]);

      while ((int) rFunctions.size () > maxListSize)
	rFunctions.pop_back ();
      while ((int) tetaFunctions.size () > maxListSize)
	tetaFunctions.pop_back ();
    }
  else if (type == 1)
    {
      xFunctions.remove (formulas[0]);
      xFunctions.push_front (formulas[0]);

      yFunctions.remove (formulas[1]);
      yFunctions.push_front (formulas[1]);

      while ((int) xFunctions.size () > maxListSize)
	xFunctions.pop_back ();
      while ((int) yFunctions.size () > maxListSize)
	yFunctions.pop_back ();
    }
}

void
ApplicationWindow::newFunctionPlot ()
{
  FunctionDialog *fd = functionDialog ();
  if (fd)
    connect (fd,
	     SIGNAL (newFunctionPlot
		     (int, QStringList &, const QString &,
		      QValueList < double >&, int)), this,
	     SLOT (newFunctionPlot
		   (int, QStringList &, const QString &,
		    QValueList < double >&, int)));

}

void
ApplicationWindow::newFunctionPlot (int type, QStringList & formulas,
				    const QString & var,
				    QValueList < double >&ranges, int points)
{
  bool fmax=false;
  if(aw&&(aw->status()==myWidget::Maximized))
    {
      fmax=true;
      aw->showNormal();
    }
  
  MultiLayer *plot = multilayerPlot (generateUniqueName (tr ("Graph")));
  Graph *g = plot->addLayer ();
  customGraph (g);
  g->addFunctionCurve (type, formulas, var, ranges, points);

  plot->showNormal ();

  g->setIgnoreResizeEvents (!autoResizeLayers);

  if(fmax) plot->showMaximized();

  setListViewSize (plot->getName(), plot->sizeToString ());
  updateFunctionLists (type, formulas);
}

void
ApplicationWindow::clearLogInfo ()
{
  if (!logInfo.isEmpty ())
    {
      logInfo = "";
      results->setText (logInfo);
      emit modified ();
    }
}

void
ApplicationWindow::clearParamFunctionsList ()
{
  xFunctions.clear ();
  yFunctions.clear ();
}

void
ApplicationWindow::clearPolarFunctionsList ()
{
  rFunctions.clear ();
  tetaFunctions.clear ();
}

/*
void
ApplicationWindow::clearFitFunctionsList ()
{
  fitFunctions.clear ();
}
*/

void
ApplicationWindow::saveFitFunctionsList (const QStringList & l)
{
  fitFunctions = l;

#ifdef Q_OS_MAC                 // Mac
  QSettings settings (QSettings::Ini);
#else
  QSettings settings;
#endif
  settings.setPath ("ProIndependent", "QtiPlot", QSettings::User);
  settings.beginGroup ("/UserFunctions");
  settings.writeEntry ("/FitFunctions", fitFunctions);
  settings.endGroup ();
}

void
ApplicationWindow::clearSurfaceFunctionsList ()
{
  surfaceFunc.clear ();
}

void
ApplicationWindow::setFramed3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      ((Graph3D *) ws->activeWindow ())->setFramed ();
      actionShowAxisDialog->setEnabled (TRUE);
    }
}

void
ApplicationWindow::setBoxed3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      ((Graph3D *) ws->activeWindow ())->setBoxed ();
      actionShowAxisDialog->setEnabled (TRUE);
    }
}

void
ApplicationWindow::removeAxes3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      ((Graph3D *) ws->activeWindow ())->setNoAxes ();
      actionShowAxisDialog->setEnabled (FALSE);
    }
}

void
ApplicationWindow::removeGrid3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setNoGrid ();
}

void
ApplicationWindow::setHiddenLineGrid3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setHiddenLineGrid ();
}

void
ApplicationWindow::setPoints3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setPointsMesh ();
}

void
ApplicationWindow::setCones3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setConesMesh ();
}

void
ApplicationWindow::setCrosses3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setCrossMesh ();
}

void
ApplicationWindow::setBars3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setBarsPlot ();
}

void
ApplicationWindow::setLineGrid3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setLineGrid ();
}

void
ApplicationWindow::setFilledMesh3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setFilledMesh ();
}

void
ApplicationWindow::setFloorData3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setFloorData ();
}

void
ApplicationWindow::setFloorIso3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setFloorIsolines ();
}

void
ApplicationWindow::setEmptyFloor3DPlot ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setEmptyFloor ();
}

void
ApplicationWindow::setFrontGrid3DPlot (bool on)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setFrontGrid (on);
}

void
ApplicationWindow::setBackGrid3DPlot (bool on)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setBackGrid (on);
}

void
ApplicationWindow::setFloorGrid3DPlot (bool on)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setFloorGrid (on);
}

void
ApplicationWindow::setCeilGrid3DPlot (bool on)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setCeilGrid (on);
}

void
ApplicationWindow::setRightGrid3DPlot (bool on)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setRightGrid (on);
}

void
ApplicationWindow::setLeftGrid3DPlot (bool on)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->setLeftGrid (on);
}

void
ApplicationWindow::pickPlotStyle (QAction * action)
{
  if (!action)
    return;

  if (action == polygon)
    {
      removeGrid3DPlot ();
    }
  else if (action == filledmesh)
    {
      setFilledMesh3DPlot ();
    }
  else if (action == wireframe)
    {
      setLineGrid3DPlot ();
    }
  else if (action == hiddenline)
    {
      setHiddenLineGrid3DPlot ();
    }
  else if (action == pointstyle)
    {
      setPoints3DPlot ();
    }
  else if (action == conestyle)
    {
      setCones3DPlot ();
    }
  else if (action == crossHairStyle)
    {
      setCrosses3DPlot ();
    }
  else if (action == barstyle)
    {
      setBars3DPlot ();
    }
  emit modified ();
}


void
ApplicationWindow::pickCoordSystem (QAction * action)
{
  if (!action)
    return;

  if (action == Box || action == Frame)
    {
      if (action == Box)
	setBoxed3DPlot ();
      if (action == Frame)
	setFramed3DPlot ();
      grids->setEnabled (true);
    }
  else if (action == None)
    {
      removeAxes3DPlot ();
      grids->setEnabled (false);
    }

  emit modified ();
}

void
ApplicationWindow::pickFloorStyle (QAction * action)
{
  if (!action)
    return;

  if (action == floordata)
    {
      setFloorData3DPlot ();
    }
  else if (action == flooriso)
    {
      setFloorIso3DPlot ();
    }
  else
    {
      setEmptyFloor3DPlot ();
    }

  emit modified ();
}

void
ApplicationWindow::custom3DActions (QWidget * w)
{
  if (w && w->isA ("Graph3D"))
    {
      Graph3D *plot = (Graph3D *) w;

      actionAnimate->setOn (plot->isAnimated ());
      actionPerspective->setOn (!plot->isOrthogonal ());

      switch (plot->plotStyle ())
	{
	case FILLEDMESH:
	  wireframe->setOn (FALSE);
	  hiddenline->setOn (FALSE);
	  polygon->setOn (FALSE);
	  filledmesh->setOn (TRUE);
	  pointstyle->setOn (FALSE);
	  barstyle->setOn (FALSE);
	  conestyle->setOn (FALSE);
	  crossHairStyle->setOn (FALSE);
	  break;

	case FILLED:
	  wireframe->setOn (FALSE);
	  hiddenline->setOn (FALSE);
	  polygon->setOn (TRUE);
	  filledmesh->setOn (FALSE);
	  pointstyle->setOn (FALSE);
	  barstyle->setOn (FALSE);
	  conestyle->setOn (FALSE);
	  crossHairStyle->setOn (FALSE);
	  break;

	case Qwt3D::USER:
	  wireframe->setOn (FALSE);
	  hiddenline->setOn (FALSE);
	  polygon->setOn (FALSE);
	  filledmesh->setOn (FALSE);

	  if (plot->pointType () == Graph3D::VerticalBars)
	    {
	      pointstyle->setOn (FALSE);
	      conestyle->setOn (FALSE);
	      crossHairStyle->setOn (FALSE);
	      barstyle->setOn (TRUE);
	    }
	  else if (plot->pointType () == Graph3D::Dots)
	    {
	      pointstyle->setOn (TRUE);
	      barstyle->setOn (FALSE);
	      conestyle->setOn (FALSE);
	      crossHairStyle->setOn (FALSE);
	    }
	  else if (plot->pointType () == Graph3D::HairCross)
	    {
	      pointstyle->setOn (FALSE);
	      barstyle->setOn (FALSE);
	      conestyle->setOn (FALSE);
	      crossHairStyle->setOn (true);
	    }
	  else if (plot->pointType () == Graph3D::Cones)
	    {
	      pointstyle->setOn (FALSE);
	      barstyle->setOn (FALSE);
	      conestyle->setOn (true);
	      crossHairStyle->setOn (FALSE);
	    }
	  break;

	case WIREFRAME:
	  wireframe->setOn (TRUE);
	  hiddenline->setOn (FALSE);
	  polygon->setOn (FALSE);
	  filledmesh->setOn (FALSE);
	  pointstyle->setOn (FALSE);
	  barstyle->setOn (FALSE);
	  conestyle->setOn (FALSE);
	  crossHairStyle->setOn (FALSE);
	  break;

	case HIDDENLINE:
	  wireframe->setOn (FALSE);
	  hiddenline->setOn (TRUE);
	  polygon->setOn (FALSE);
	  filledmesh->setOn (FALSE);
	  pointstyle->setOn (FALSE);
	  barstyle->setOn (FALSE);
	  conestyle->setOn (FALSE);
	  crossHairStyle->setOn (FALSE);
	  break;

	default:
	  break;
	}

      switch (plot->coordStyle ())
	{
	case Qwt3D::NOCOORD:
	  None->setOn (TRUE);
	  Box->setOn (FALSE);
	  Frame->setOn (FALSE);
	  break;

	case Qwt3D::BOX:
	  None->setOn (FALSE);
	  Box->setOn (TRUE);
	  Frame->setOn (FALSE);
	  break;

	case Qwt3D::FRAME:
	  None->setOn (FALSE);
	  Box->setOn (FALSE);
	  Frame->setOn (TRUE);
	  break;
	}

      switch (plot->floorStyle ())
	{
	case NOFLOOR:
	  floornone->setOn (TRUE);
	  flooriso->setOn (FALSE);
	  floordata->setOn (FALSE);
	  break;

	case FLOORISO:
	  floornone->setOn (FALSE);
	  flooriso->setOn (TRUE);
	  floordata->setOn (FALSE);
	  break;

	case FLOORDATA:
	  floornone->setOn (FALSE);
	  flooriso->setOn (FALSE);
	  floordata->setOn (TRUE);
	  break;
	}
      custom3DGrids (plot->grids ());
    }
}

void
ApplicationWindow::custom3DGrids (int grids)
{
  if (Qwt3D::BACK & grids)
    back->setOn (TRUE);
  else
    back->setOn (FALSE);

  if (Qwt3D::FRONT & grids)
    front->setOn (TRUE);
  else
    front->setOn (FALSE);

  if (Qwt3D::CEIL & grids)
    ceil->setOn (TRUE);
  else
    ceil->setOn (FALSE);

  if (Qwt3D::FLOOR & grids)
    floor->setOn (TRUE);
  else
    floor->setOn (FALSE);

  if (Qwt3D::RIGHT & grids)
    right->setOn (TRUE);
  else
    right->setOn (FALSE);

  if (Qwt3D::LEFT & grids)
    left->setOn (TRUE);
  else
    left->setOn (FALSE);
}

void
ApplicationWindow::initPlot3DToolBar ()
{
  plot3DTools = new QToolBar (this, "plot3d operations");
  plot3DTools->setCloseMode (QDockWindow::Undocked);
  addToolBar (plot3DTools, tr ("Surface 3D"), Top, FALSE);

  coord = new QActionGroup (this, "coord");
  coord->setUsesDropDown (FALSE);
  Box = new QAction (coord, "Box");
  Box->setToggleAction (TRUE);
  Box->setIconSet (QPixmap (box_xpm));
  Box->setOn (TRUE);

  Frame = new QAction (coord, "Frame");
  Frame->setToggleAction (TRUE);
  Frame->setIconSet (QPixmap (free_axes_xpm));
  None = new QAction (coord, "None");
  None->setToggleAction (TRUE);
  None->setIconSet (QPixmap (no_axes_xpm));

  Frame->addTo (plot3DTools);
  Box->addTo (plot3DTools);
  None->addTo (plot3DTools);

  plot3DTools->addSeparator ();

  // grid actions
  grids = new QActionGroup (this, "grids");
  grids->setEnabled (TRUE);
  grids->setExclusive (FALSE);
  grids->setUsesDropDown (FALSE);
  front = new QAction (grids, "front");
  front->setToggleAction (TRUE);
  front->setIconSet (QPixmap (frontGrid_xpm));
  back = new QAction (grids, "back");
  back->setToggleAction (TRUE);
  back->setIconSet (QPixmap (backGrid_xpm));
  right = new QAction (grids, "right");
  right->setToggleAction (TRUE);
  right->setIconSet (QPixmap (leftGrid_xpm));
  left = new QAction (grids, "left");
  left->setToggleAction (TRUE);
  left->setIconSet (QPixmap (rightGrid_xpm));
  ceil = new QAction (grids, "ceil");
  ceil->setToggleAction (TRUE);
  ceil->setIconSet (QPixmap (ceilGrid_xpm));
  floor = new QAction (grids, "floor");
  floor->setToggleAction (TRUE);
  floor->setIconSet (QPixmap (floorGrid_xpm));

  front->addTo (plot3DTools);
  back->addTo (plot3DTools);
  right->addTo (plot3DTools);
  left->addTo (plot3DTools);
  ceil->addTo (plot3DTools);
  floor->addTo (plot3DTools);

  plot3DTools->addSeparator ();

  actionPerspective = new QAction (this);
  actionPerspective->setToggleAction (TRUE);
  actionPerspective->setIconSet (QPixmap (perspective_xpm));
  actionPerspective->addTo (plot3DTools);
  orthogonal3DPlots=false; // for dummy
  actionPerspective->setOn (!orthogonal3DPlots);
  connect (actionPerspective, SIGNAL (toggled (bool)), this,
	   SLOT (togglePerspective (bool)));

  actionResetRotation = new QAction (this);
  actionResetRotation->setToggleAction (false);
  actionResetRotation->setIconSet (QPixmap (reset_rotation_xpm));
  actionResetRotation->addTo (plot3DTools);
  connect (actionResetRotation, SIGNAL (activated ()), this,
	   SLOT (resetRotation ()));

  actionFitFrame = new QAction (this);
  actionFitFrame->setToggleAction (false);
  actionFitFrame->setIconSet (QPixmap (fit_frame_xpm));
  actionFitFrame->addTo (plot3DTools);
  connect (actionFitFrame, SIGNAL (activated ()), this,
	   SLOT (fitFrameToLayer ()));

  plot3DTools->addSeparator ();

  //plot style actions
  plotstyle = new QActionGroup (this, "plotstyle");
  plotstyle->setUsesDropDown (FALSE);
  wireframe = new QAction (plotstyle, "wireframe");
  wireframe->setToggleAction (TRUE);
  wireframe->setEnabled (TRUE);
  wireframe->setIconSet (QPixmap (lineMesh_xpm));
  hiddenline = new QAction (plotstyle, "hiddenline");
  hiddenline->setToggleAction (TRUE);
  hiddenline->setEnabled (TRUE);
  hiddenline->setIconSet (QPixmap (grid_only_xpm));
  polygon = new QAction (plotstyle, "polygon");
  polygon->setToggleAction (TRUE);
  polygon->setEnabled (TRUE);
  polygon->setIconSet (QPixmap (no_grid_xpm));
  filledmesh = new QAction (plotstyle, "filledmesh");
  filledmesh->setToggleAction (TRUE);
  filledmesh->setOn (TRUE);
  filledmesh->setIconSet (QPixmap (grid_poly_xpm));
  pointstyle = new QAction (plotstyle, "pointstyle");
  pointstyle->setToggleAction (TRUE);
  pointstyle->setIconSet (QPixmap (pointsMesh_xpm));

  conestyle = new QAction (plotstyle, "conestyle");
  conestyle->setToggleAction (TRUE);
  conestyle->setIconSet (QPixmap (cones_xpm));

  crossHairStyle = new QAction (plotstyle, "crossHairStyle");
  crossHairStyle->setToggleAction (TRUE);
  crossHairStyle->setIconSet (QPixmap (crosses_xpm));

  barstyle = new QAction (plotstyle, "barstyle");
  barstyle->setToggleAction (TRUE);
  barstyle->setIconSet (QPixmap (plot_bars_xpm));

  barstyle->addTo (plot3DTools);
  pointstyle->addTo (plot3DTools);

  conestyle->addTo (plot3DTools);
  crossHairStyle->addTo (plot3DTools);
  plot3DTools->addSeparator ();

  wireframe->addTo (plot3DTools);
  hiddenline->addTo (plot3DTools);
  polygon->addTo (plot3DTools);
  filledmesh->addTo (plot3DTools);

  plot3DTools->addSeparator ();

  //floor actions
  floorstyle = new QActionGroup (this, "floorstyle");
  floorstyle->setUsesDropDown (FALSE);
  floordata = new QAction (floorstyle, "floordata");
  floordata->setToggleAction (TRUE);
  floordata->setIconSet (QPixmap (floor_xpm));
  flooriso = new QAction (floorstyle, "flooriso");
  flooriso->setToggleAction (TRUE);
  flooriso->setIconSet (QPixmap (isolines_xpm));
  floornone = new QAction (floorstyle, "floornone");
  floornone->setToggleAction (TRUE);
  floornone->setOn (TRUE);
  floornone->setIconSet (QPixmap (no_floor_xpm));

  floordata->addTo (plot3DTools);
  flooriso->addTo (plot3DTools);
  floornone->addTo (plot3DTools);

  plot3DTools->addSeparator ();

  actionAnimate = new QAction (this);
  actionAnimate->setToggleAction (true);
  actionAnimate->setIconSet (QPixmap (movie_xpm));
  connect (actionAnimate, SIGNAL (toggled (bool)), this,
	   SLOT (toggle3DAnimation (bool)));
  actionAnimate->addTo (plot3DTools);

  plot3DTools->hide ();

  connect (coord, SIGNAL (selected (QAction *)), this,
	   SLOT (pickCoordSystem (QAction *)));
  connect (floorstyle, SIGNAL (selected (QAction *)), this,
	   SLOT (pickFloorStyle (QAction *)));
  connect (plotstyle, SIGNAL (selected (QAction *)), this,
	   SLOT (pickPlotStyle (QAction *)));

  connect (left, SIGNAL (toggled (bool)), this,
	   SLOT (setLeftGrid3DPlot (bool)));
  connect (right, SIGNAL (toggled (bool)), this,
	   SLOT (setRightGrid3DPlot (bool)));
  connect (ceil, SIGNAL (toggled (bool)), this,
	   SLOT (setCeilGrid3DPlot (bool)));
  connect (floor, SIGNAL (toggled (bool)), this,
	   SLOT (setFloorGrid3DPlot (bool)));
  connect (back, SIGNAL (toggled (bool)), this,
	   SLOT (setBackGrid3DPlot (bool)));
  connect (front, SIGNAL (toggled (bool)), this,
	   SLOT (setFrontGrid3DPlot (bool)));
}

void
ApplicationWindow::pixelLineProfile ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    {
      bool ok;
      int res =
	QInputDialog::
	getInteger (tr ("QtiPlot - Set the number of pixels to average"),
		    tr ("Number of averaged pixels"), 1, 1, 2000, 2,
		    &ok, this);
      if (ok)
	{
	  QApplication::setOverrideCursor (waitCursor);
	  g->calculateProfile (res, TRUE);
	  QApplication::restoreOverrideCursor ();
	}
      else
	return;
    }
}

void
ApplicationWindow::intensityTable ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (g)
    g->showIntensityTable ();
}

Matrix *
ApplicationWindow::createIntensityMatrix (const QPixmap & pic)
{
  QImage image = pic.convertToImage ();
  QSize size = pic.size ();
  int cols = size.width ();
  int rows = size.height ();

  QApplication::setOverrideCursor (waitCursor);
  Matrix *w = newMatrix ("Matrix1", rows, cols);
  for (int i = 0; i < rows; i++)
    {
      int l = rows - i - 1;
      for (int j = 0; j < cols; j++)
	{
	  QRgb pixel = image.pixel (j, l);
	  w->setText (i, j, QString::number (qGray (pixel)));
	}
    }

  w->setSpecifications(w->saveToString("geometry\n"));
  w->show ();
  QApplication::restoreOverrideCursor ();
  return w;
}

void
ApplicationWindow::autoArrangeLayers ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  plot->setMargins (0, 0, 0, 0);
  plot->setSpacing (0, 0);
  plot->arrangeLayers (true, false, false);
}

void
ApplicationWindow::addLayer ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) ws->activeWindow ();
  switch (QMessageBox::information (this,
				    tr
				    ("QtiPlot - Guess best origin for the new layer?"),
				    tr
				    ("Do you want QtiPlot to guess the best position for the new layer?\n Warning: this will rearrange existing layers!"),
				    tr ("&Guess"), tr ("&Top-left corner"),
				    tr ("&Cancel"), 0, 2))
    {
    case 0:
      {
	//customGraph (plot->addLayer ());
	Graph *g = plot->addLayer();
	g->setIgnoreResizeEvents (!autoResizeLayers);
	customGraph( g );
 
	plot->arrangeLayers (true, false, false);
      }
      break;

    case 1:
      {
      //customGraph (plot->addLayerToOrigin ());
      Graph *g = plot->addLayer();
      g->setIgnoreResizeEvents (!autoResizeLayers);
      customGraph( g );

      plot->updateTransparency ();
      }
      break;      
    case 2:
      return;
      break;
    }
}

void
ApplicationWindow::deleteLayer ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  ((MultiLayer *) ws->activeWindow ())->confirmRemoveLayer ();
}

void
ApplicationWindow::resizeLayer ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  ((MultiLayer *) ws->activeWindow ())->highlightActiveLayer ();
}

void
ApplicationWindow::restoreWindowGeometry(myWidget * w, const QString s)
{
  w->blockSignals (true);
  QString caption = w->getName();
  QStringList lst = QStringList::split ('\t', s, TRUE);
  if (lst.count() > 4)
    w->parentWidget()->setGeometry(lst[1].toInt(),lst[2].toInt(),
				   lst[3].toInt(),lst[4].toInt());
  else
    w->parentWidget ()->setGeometry (0, 0, 500, 450);
  
  if (s.contains ("minimized"))
    {
      //w->showMinimized ();
      w->setStatus (myWidget::Minimized);
      setListView (caption, tr ("Minimized"));
    }
  else if (s.contains ("maximized"))
    {
      w->setStatus (myWidget::Maximized);
      maximized_window = w;
      setListView (caption, tr ("Maximized"));
    }
  else
    {
      w->setStatus (myWidget::Normal);
      if((lst.count()>5)&&(lst[5] == "hidden"))
	{
	  //fprintf(stderr,"hiding window\n");
	  hideWindow (w);
	}
    }

  if (s.contains ("active"))
    aw = w;

  w->blockSignals (false);
}

Note *
ApplicationWindow::openNote (const QStringList & flist)
{
  QStringList lst = QStringList::split ('\t', flist[0], false);
  QString caption = lst[0];
  Note *w = newNote (caption);
  w->blockSignals(true);
  setListViewDate (caption, lst[1]);
  w->setBirthDate (lst[1]);
  restoreWindowGeometry ( w, flist[1]);
  w->hide();

  lst = QStringList::split ('\t', flist[2], true);
  w->setWindowLabel (lst[1]);
  w->setCaptionPolicy ((myWidget::CaptionPolicy) lst[2].toInt ());
  setListViewLabel (w->getName(), lst[1]);
  w->blockSignals(false);
  return w;
}

Matrix *
ApplicationWindow::openMatrix (const QStringList & flist)
{
  QStringList::const_iterator line = flist.begin ();

  QStringList list = QStringList::split ('\t', *line, TRUE);
  QString caption = list[0];
  int rows = list[1].toInt ();
  int cols = list[2].toInt ();

  Matrix *w = newMatrix (caption, rows, cols);
  w->blockSignals(true);
  setListViewDate (caption, list[3]);
  w->setBirthDate (list[3]);

  for (line++; line != flist.end (); line++)
    {
      QStringList fields = QStringList::split ('\t', *line, true);
      if (fields[0] == "geometry")
	{
	  restoreWindowGeometry ( w, *line);
	  w->hide();
	}
      else if (fields[0] == "ColWidth")
	{
	  w->setColumnsWidth (fields[1].toInt ());
	}
      else if (fields[0] == "Formula")
	{
	  w->setFormula (fields[1]);
	}
      else if (fields[0] == "<formula>")
	{
	  QString formula;
	  for (line++; line != flist.end () && *line != "</formula>"; line++)
	    formula += *line + "\n";
	  formula.truncate (formula.length () - 1);
	  w->setFormula (formula);
	}
      else if (fields[0] == "TextFormat")
	{
	  if (fields[1] == "f")
	    w->setTextFormat ('f', fields[2].toInt ());
	  else
	    w->setTextFormat ('e', fields[2].toInt ());
	}
      else if (fields[0] == "WindowLabel")
	{			// fileVersion > 71
	  w->setWindowLabel (fields[1]);
	  w->setCaptionPolicy ((myWidget::CaptionPolicy) fields[2].toInt ());
	  setListViewLabel (w->getName(), fields[1]);
	}
      else if (fields[0] == "Coordinates")
	{			// fileVersion > 81
	  w->setCoordinates (fields[1].toDouble (), fields[2].toDouble (),
			     fields[3].toDouble (), fields[4].toDouble ());
	}
      else			// <data> or values
	break;
    }
  if (*line == "<data>")
    line++;

//read and set table values
  for (; line != flist.end () && *line != "</data>"; line++)
    {
      QStringList fields = QStringList::split ('\t', *line, true);
      int row = fields[0].toInt ();
      for (int col = 0; col < cols; col++)
	w->setText (row, col, fields[col + 1]);
      qApp->eventLoop ()->processEvents (QEventLoop::ExcludeUserInput);
    }

  w->setSpecifications(w->saveToString("geometry\n"));
  w->blockSignals(false);

  return w;
}

Table *
ApplicationWindow::openTable(const QStringList & flist)
{
  QStringList::const_iterator line = flist.begin ();

  QStringList list = QStringList::split ('\t', *line, TRUE);
  QString caption = list[0];

  int rows = list[1].toInt ();
  int cols = list[2].toInt ();

  Table *w = newTable (caption, rows, cols);
  w->blockSignals(true);
  setListViewDate (caption, list[3]);
  w->setBirthDate (list[3]);


  for (line++; line != flist.end (); line++)
    {
      QStringList fields = QStringList::split ('\t', *line, true);
      if (fields[0] == "geometry")
	{
	  restoreWindowGeometry (w, *line);
	  w->hide();
	}
      else if (fields[0] == "header")
	{
	  fields.pop_front ();
	  if (fileVersion >= 78)
	    w->loadHeader (fields);
	  else
	    {
	      w->setColPlotDesignation (list[4].toInt (), Table::X);
	      w->setColPlotDesignation (list[6].toInt (), Table::Y);
	      w->setHeader (fields);
	    }
	}
      else if (fields[0] == "ColWidth")
	{
	  fields.pop_front ();
	  w->setColWidths (fields);
	}
      else if (fields[0] == "com")
	{			// legacy code
	  w->setCommandes (*line);
	}
      else if (fields[0] == "<com>")
	{
	  for (line++; line != flist.end () && *line != "</com>"; line++)
	    {
	      int col = (*line).mid (9, (*line).length () - 11).toInt ();
	      QString formula;
	      for (line++; line != flist.end () && *line != "</col>"; line++)
		formula += *line + "\n";
	      formula.truncate (formula.length () - 1);
	      w->setCommand (col, formula);
	    }
	}
      else if (fields[0] == "ColType")
	{			// fileVersion > 65
	  fields.pop_front ();
	  w->setColumnTypes (fields);
	}
      else if (fields[0] == "Comments")
	{			// fileVersion > 71
	  fields.pop_front ();
	  w->setColComments (fields);
	}
      else if (fields[0] == "WindowLabel")
	{			// fileVersion > 71
	  w->setWindowLabel (fields[1]);
	  w->setCaptionPolicy ((myWidget::CaptionPolicy) fields[2].toInt ());
	  setListViewLabel (w->getName(), fields[1]);
	}
      else			// <data> or values
	break;
    }
  if (*line == "<data>")
    line++;

//read and set table values
  for (; line != flist.end () && *line != "</data>"; line++)
    {
      QStringList fields = QStringList::split ('\t', *line, true);
      int row = fields[0].toInt ();
      for (int col = 0; col < cols; col++)
	w->setText (row, col, fields[col + 1]);
      qApp->eventLoop ()->processEvents (QEventLoop::ExcludeUserInput);
    }

  w->setSpecifications (w->saveToString ("geometry\n"));
  w->blockSignals(false);
  return w;
}

TableStatistics *
ApplicationWindow::openTableStatistics (const QStringList & flist)
{
  QStringList::const_iterator line = flist.begin ();

  QStringList list = QStringList::split ('\t', *line++, TRUE);
  QString caption = list[0];

  QValueList < int >targets;
  for (int i = 1; i <= (*line).contains ('\t'); i++)
    targets << (*line).section ('\t', i, i).toInt ();

  Table *tbl=table(list[1]);

  if (!renamedTables.isEmpty ())
    {
      if (renamedTables.contains (list[1]))
	{ //modify the name of the curve according to the new table name
	  int index = renamedTables.findIndex (list[1]);
	  QString newname = renamedTables[index+1];
	  tbl=table(newname);
	  caption="";
	}
    }

  if(!tbl) return 0;

  TableStatistics *w =
    newTableStatistics (tbl,
			list[2] ==
			"row" ? TableStatistics::row : TableStatistics::
			column, targets, caption);
  w->blockSignals(true);
  setListViewDate (caption, list[3]);
  w->setBirthDate (list[3]);

  for (line++; line != flist.end (); line++)
    {
      QStringList fields = QStringList::split ('\t', *line, true);
      if (fields[0] == "geometry")
	{
	  restoreWindowGeometry (w, *line);
	  w->hide();
	}
      else if (fields[0] == "header")
	{
	  fields.pop_front ();
	  if (fileVersion >= 78)
	    w->loadHeader (fields);
	  else
	    {
	      w->setColPlotDesignation (list[4].toInt (), Table::X);
	      w->setColPlotDesignation (list[6].toInt (), Table::Y);
	      w->setHeader (fields);
	    }
	}
      else if (fields[0] == "ColWidth")
	{
	  fields.pop_front ();
	  w->setColWidths (fields);
	}
      else if (fields[0] == "com")
	{			// legacy code
	  w->setCommandes (*line);
	}
      else if (fields[0] == "<com>")
	{
	  for (line++; line != flist.end () && *line != "</com>"; line++)
	    {
	      int col = (*line).mid (9, (*line).length () - 11).toInt ();
	      QString formula;
	      for (line++; line != flist.end () && *line != "</col>"; line++)
		formula += *line + "\n";
	      formula.truncate (formula.length () - 1);
	      w->setCommand (col, formula);
	    }
	}
      else if (fields[0] == "ColType")
	{			// fileVersion > 65
	  fields.pop_front ();
	  w->setColumnTypes (fields);
	}
      else if (fields[0] == "Comments")
	{			// fileVersion > 71
	  fields.pop_front ();
	  w->setColComments (fields);
	}
      /*     else if (fields[0] == "WindowLabel")
	{			// fileVersion > 71
	  w->setWindowLabel (fields[1]);
	  w->setCaptionPolicy ((myWidget::CaptionPolicy) fields[2].toInt ());
	  setListViewLabel (w->getName(), fields[1]);
	}
      */
    }
  w->blockSignals(false);
  return w;
}

void
ApplicationWindow::openGraph (MultiLayer * plot, const QStringList & list)
{
  Graph *ag = 0;
  int curveID = 0, i;
  Table *w = 0;
  QStringList fList, curve;
  QString caption;
  curveLayout cl;

  //  qDebug("openGraph");
   
  for (int j = 0; j < (int) list.count () - 1; j++)
    {
      QString s = list[j];
      if (s.startsWith ("ggeometry"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag = (Graph *) plot->addLayer (fList[1].toInt (), fList[2].toInt (),
					 fList[3].toInt (), fList[4].toInt ());
	  ag->blockSignals(true);
	  ag->enableAutoscaling (autoscale2DPlots);
	}
      else if (s.startsWith("Background"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  if (QColor (fList[1]) != QColor (255, 255, 255))
	    ag->setBackgroundColor (QColor (fList[1]));
	}
      else if (s.startsWith ("Margin"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->changeMargin (fList[1].toInt ());
	}
      else if (s.startsWith ("Spacing"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->changeSpacing (fList[1].toInt ());
	}
      else if (s.startsWith ("Border"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->setBorder (fList[1].toInt (), QColor (fList[2]));
	}
      else if (s.startsWith ("EnabledAxes"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->enableAxes (fList);
	}
      else if (s.startsWith ("AxesBaseline"))
	{
	  fList = QStringList::split ('\t', s, false);
	  ag->setAxesBaseline (fList);
	}
      else if (s.startsWith ("EnabledTicks"))
	{			//version < 0.8.6
	  fList = QStringList::split ('\t', s, TRUE);
	  fList.pop_front ();
	  fList.gres ("-1", "3");
	  ag->setMajorTicksType (fList);
	  ag->setMinorTicksType (fList);
	}
      else if (s.startsWith ("MajorTicks"))
	{			//version >= 0.8.6
	  fList = QStringList::split ('\t', s, false);
	  fList.pop_front ();
	  ag->setMajorTicksType (fList);
	}
      else if (s.startsWith ("MinorTicks"))
	{			//version >= 0.8.6
	  fList = QStringList::split ('\t', s, false);
	  fList.pop_front ();
	  ag->setMinorTicksType (fList);
	}
      else if (s.startsWith ("TicksLength"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->setTicksLength (fList[1].toInt (), fList[2].toInt ());
	}
      else if (s.startsWith ("EnabledTickLabels"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  fList.pop_front ();
	  ag->setEnabledTickLabels (fList);
	}
      else if (s.startsWith ("AxesColors"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  fList.pop_front ();
	  ag->setAxesColors (fList);
	}
      else if (s.startsWith ("AxesNumberColors"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  fList.pop_front ();
	  ag->setAxesNumColors (fList);
	}
      else if (s.startsWith("grid\t"))
	{
	  QStringList grid = QStringList::split ('\t', s, TRUE);
	  gridOptions gr;
	  gr.majorOnX = grid[1].toInt ();
	  gr.minorOnX = grid[2].toInt ();
	  gr.majorOnY = grid[3].toInt ();
	  gr.minorOnY = grid[4].toInt ();
	  gr.majorCol = grid[5].toInt ();
	  gr.majorStyle = grid[6].toInt ();
	  gr.majorWidth = grid[7].toInt ();
	  gr.minorCol = grid[8].toInt ();
	  gr.minorStyle = grid[9].toInt ();
	  gr.minorWidth = grid[10].toInt ();
	  gr.xZeroOn = grid[11].toInt ();
	  gr.yZeroOn = grid[12].toInt ();
	  if (grid.count () == 15)
	    {
	      gr.xAxis = grid[13].toInt ();
	      gr.yAxis = grid[14].toInt ();
	    }
	  ag->setGridOptions (gr);
	}
      else if (s.startsWith ("PieCurve"))
	{
	  curve = QStringList::split ('\t', s, TRUE);
	  QPen pen =
	    QPen (QColor (curve[3]), curve[2].toInt (),
		  Graph::getPenStyle (curve[4]));
	  ag->plotPie (table (curve[1]), curve[1], pen,
		       curve[5].toInt (), curve[6].toInt (),
		       curve[7].toInt ());
	}
      else if (s.startsWith("curve\t"))
	{
	  curve = QStringList::split ('\t', s, false);
	  if (!renamedTables.isEmpty ())
	    {
	      QString caption = curve[2].section('_', 0, 0);
	      if (renamedTables.contains (caption))
		{//modify the name of the curve according to the new table name
		  int index = renamedTables.findIndex (caption);
		  QString newCaption = renamedTables[++index];
		  curve.gres (caption + "_", newCaption + "_", true);
		}
	    }

	  cl.connectType = curve[4].toInt ();
	  cl.lCol = curve[5].toInt ();
	  cl.lStyle = curve[6].toInt ();
	  cl.lWidth = curve[7].toInt ();
	  cl.sSize = curve[8].toInt ();
	  if (fileVersion <= 78)
	    cl.sType = Graph::obsoleteSymbolStyle (curve[9].toInt ());
	  else
	    cl.sType = curve[9].toInt ();

	  cl.symCol = curve[10].toInt ();
	  cl.fillCol = curve[11].toInt ();
	  cl.filledArea = curve[12].toInt ();
	  cl.aCol = curve[13].toInt ();
	  cl.aStyle = curve[14].toInt ();
	  if (fileVersion <= 77)
	    cl.penWidth = cl.lWidth;
	  else
	    cl.penWidth = curve[15].toInt ();

	  w = table (curve[2]);
	  if (w)
	    {
	      int plotType = curve[3].toInt ();
	      if (plotType == Graph::VectXYXY || plotType == Graph::VectXYAM)
		{
		  QStringList colsList;
		  colsList << curve[2];
		  colsList << curve[20];
		  colsList << curve[21];
		  if (fileVersion < 72)
		    colsList.prepend (w->colName (curve[1].toInt ()));
		  else
		    colsList.prepend (curve[1]);
		  ag->plotVectorCurve (w, colsList, plotType);
		  if (fileVersion <= 77)
		    ag->updateVectorsLayout (w, curveID, curve[15].toInt (),
					     curve[16].toInt (),
					     curve[17].toInt (),
					     curve[18].toInt (),
					     curve[19].toInt (), 0, curve[20],
					     curve[21]);
		  else
		    {
		      if (plotType == Graph::VectXYXY)
			ag->setVectorsLook (curveID, QColor (curve[15]),
					    curve[16].toInt (),
					    curve[17].toInt (),
					    curve[18].toInt (),
					    curve[19].toInt (), 0);
		      else
			ag->setVectorsLook (curveID, QColor (curve[15]),
					    curve[16].toInt (),
					    curve[17].toInt (),
					    curve[18].toInt (),
					    curve[19].toInt (),
					    curve[22].toInt ());
		    }
		}
	      else if (plotType == Graph::Box)
		ag->openBoxDiagram (w, curve);
	      else
		{
		  if (fileVersion < 72)
		    ag->insertCurve (w, curve[1].toInt (), curve[2],
				     plotType);
		  else
		    ag->insertCurve (w, curve[1], curve[2], plotType);
		}

	      if (plotType == Graph::Histogram)
		{
		  if (fileVersion <= 76)
		    ag->updateHistogram (w, curve[2], curveID,
					 curve[16].toInt (),
					 curve[17].toDouble (),
					 curve[18].toDouble (),
					 curve[19].toDouble ());
		  else
		    ag->updateHistogram (w, curve[2], curveID,
					 curve[17].toInt (),
					 curve[18].toDouble (),
					 curve[19].toDouble (),
					 curve[20].toDouble ());
		}

	      if (plotType == Graph::VerticalBars
		  || plotType == Graph::HorizontalBars
		  || plotType == Graph::Histogram)
		{
		  if (fileVersion <= 76)
		    ag->setBarsGap (curveID, curve[15].toInt (), 0);
		  else
		    ag->setBarsGap (curveID, curve[15].toInt (),
				    curve[16].toInt ());
		}
	      ag->updateCurveLayout (curveID, &cl);
	      if (fileVersion >= 88)
		{
		  QwtPlotCurve *c = ag->curve (curveID);
		  if (c && c->rtti () == QwtPlotItem::Rtti_PlotCurve)
		    c->setAxis (curve[curve.count () - 2].toInt (),
				curve[curve.count () - 1].toInt ());
		}
	    }
	  curveID++;
	}
      else if (s.startsWith ("FunctionCurve"))
	{
	  //qDebug("FunctionCurve");
	  curve = QStringList::split ('\t', s, true);

	  cl.connectType = curve[6].toInt ();
	  cl.lCol = curve[7].toInt ();
	  cl.lStyle = curve[8].toInt ();
	  cl.lWidth = curve[9].toInt ();
	  cl.sSize = curve[10].toInt ();
	  cl.sType = curve[11].toInt ();
	  cl.symCol = curve[12].toInt ();
	  cl.fillCol = curve[13].toInt ();
	  cl.filledArea = curve[14].toInt ();
	  cl.aCol = curve[15].toInt ();
	  cl.aStyle = curve[16].toInt ();
	  if (fileVersion <= 77)
	    cl.penWidth = cl.lWidth;
	  else
	    cl.penWidth = curve[17].toInt ();

	  //qDebug("inserting function curve");
	  ag->insertFunctionCurve (curve[1], curve[2].toInt (), fileVersion);
	  //qDebug("CurveID = %d",curveID);

	  ag->setCurveType (curveID, curve[5].toInt ());
	  ag->updateCurveLayout (curveID, &cl);
	  if (fileVersion >= 88)
	    {
	      QwtPlotCurve *c = ag->curve (curveID);
	      if (c)
		c->setAxis (curve[18].toInt (), curve[19].toInt ());
	    }
	  curveID++;
	}
      else if (s.startsWith ("ErrorBars"))
	{
	  curve = QStringList::split ('\t', s, FALSE);
	  w = table (curve[3]);
	  Table *errTable = table (curve[4]);
	  if (w && errTable)
	    {
	      double xOffset = 0, yOffset = 0;
	      if (fileVersion > 85)
		{
		  xOffset = curve[11].toDouble ();
		  yOffset = curve[12].toDouble ();
		}

	      ag->addErrorBars (w, curve[2], curve[3], errTable, curve[4],
				curve[1].toInt (), curve[5].toInt (),
				curve[6].toInt (), QColor (curve[7]),
				curve[8].toInt (), curve[10].toInt (),
				curve[9].toInt (), xOffset, yOffset);
	    }
	  curveID++;
	}
      else if (s == "<spectrogram>")
	{
	  curveID++;
	  QStringList lst;
	  while (s != "</spectrogram>")
	    {
	      s = list[++j];
	      lst << s;
	    }
	  lst.pop_back ();
	  ag->restoreSpectrogram (this, lst);
	}
      else if (s.startsWith("scale\t"))
	{
	  QStringList scl = QStringList::split ('\t', s, true);
	  scl.pop_front ();
	  if (fileVersion < 88)
	    {
	      double step = scl[2].toDouble ();
	      if (scl[5] == "0")
		step = 0.0;
	      ag->setScale (QwtPlot::xBottom, scl[0].toDouble (),
			    scl[1].toDouble (), step, scl[3].toInt (),
			    scl[4].toInt (), scl[6].toInt (),
			    bool (scl[7].toInt ()));
	      ag->setScale (QwtPlot::xTop, scl[0].toDouble (),
			    scl[1].toDouble (), step, scl[3].toInt (),
			    scl[4].toInt (), scl[6].toInt (),
			    bool (scl[7].toInt ()));

	      step = scl[10].toDouble ();
	      if (scl[13] == "0")
		step = 0.0;
	      ag->setScale (QwtPlot::yLeft, scl[8].toDouble (),
			    scl[9].toDouble (), step, scl[11].toInt (),
			    scl[12].toInt (), scl[14].toInt (),
			    bool (scl[15].toInt ()));
	      ag->setScale (QwtPlot::yRight, scl[8].toDouble (),
			    scl[9].toDouble (), step, scl[11].toInt (),
			    scl[12].toInt (), scl[14].toInt (),
			    bool (scl[15].toInt ()));
	    }
	  else
	    ag->setScale (scl[0].toInt (), scl[1].toDouble (),
			  scl[2].toDouble (), scl[3].toDouble (),
			  scl[4].toInt (), scl[5].toInt (), scl[6].toInt (),
			  bool (scl[7].toInt ()));
	}
      else if (s.startsWith ("PlotTitle"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->setTitle (fList[1]);
	  ag->setTitleColor (QColor (fList[2]));
	  ag->setTitleAlignment (fList[3].toInt ());
	}
      else if (s.startsWith ("TitleFont"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  QFont fnt =
	    QFont (fList[1], fList[2].toInt (), fList[3].toInt (),
		   fList[4].toInt ());
	  fnt.setUnderline (fList[5].toInt ());
	  fnt.setStrikeOut (fList[6].toInt ());
	  ag->setTitleFont (fnt);
	}
      else if (s.startsWith ("AxesTitles"))
	{
	  QStringList legend = QStringList::split ('\t', s, TRUE);
	  for (i = 0; i < 4; i++)
	    ag->setAxisTitle (i, legend[i + 1]);
	}
      else if (s.startsWith ("AxesTitleColors"))
	{
	  QStringList colors = QStringList::split ('\t', s, FALSE);
	  ag->setAxesTitleColor (colors);
	}
      else if (s.startsWith ("AxesTitleAlignment"))
	{
	  QStringList align = QStringList::split ('\t', s, FALSE);
	  ag->setAxesTitlesAlignment (align);
	}
      else if (s.startsWith ("ScaleFont"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  QFont fnt =
	    QFont (fList[1], fList[2].toInt (), fList[3].toInt (),
		   fList[4].toInt ());
	  fnt.setUnderline (fList[5].toInt ());
	  fnt.setStrikeOut (fList[6].toInt ());

	  int axis = (fList[0].right (1)).toInt ();
	  ag->setAxisTitleFont (axis, fnt);
	}
      else if (s.startsWith ("AxisFont"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  QFont fnt =
	    QFont (fList[1], fList[2].toInt (), fList[3].toInt (),
		   fList[4].toInt ());
	  fnt.setUnderline (fList[5].toInt ());
	  fnt.setStrikeOut (fList[6].toInt ());

	  int axis = (fList[0].right (1)).toInt ();
	  ag->setAxisFont (axis, fnt);
	}
      else if (s.startsWith ("AxesFormulas"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  fList.remove (fList.first ());
	  ag->setAxesFormulas (fList);
	}
      else if (s.startsWith ("<AxisFormula "))
	{
	  int pos = s.mid (18, s.length () - 20).toInt ();
	  QString formula;
	  for (j++; j < (int) list.count () && list[j] != "</AxisFormula>";
	       j++)
	    formula += list[j] + "\n";
	  formula.truncate (formula.length () - 1);
	  ag->setAxisFormula (pos, formula);
	}
      else if (s.startsWith ("LabelsFormat"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  fList.pop_front ();
	  ag->setLabelsNumericFormat (fList);
	}
      else if (s.startsWith ("LabelsRotation"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->setAxisLabelRotation (QwtPlot::xBottom, fList[1].toInt ());
	  ag->setAxisLabelRotation (QwtPlot::xTop, fList[2].toInt ());
	}
      else if (s.startsWith ("DrawAxesBackbone"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->loadAxesOptions (fList[1]);
	}
      else if (s.startsWith ("AxesLineWidth"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->loadAxesLinewidth (fList[1].toInt ());
	}
      else if (s.startsWith ("CanvasFrame"))
	{
	  QStringList list = QStringList::split ('\t', s, TRUE);
	  ag->drawCanvasFrame (list);
	}
      else if (s.startsWith ("CanvasBackground"))
	{
	  QStringList list = QStringList::split ('\t', s, TRUE);
	  ag->setCanvasBackground (QColor (list[1]));
	}
      else if (s.startsWith ("Legend"))
	{			// version <= 0.8.9
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->insertLegend (fList, fileVersion);
	}
      else if (s.startsWith ("<legend>") && s.endsWith ("</legend>"))
	{
	  fList = QStringList::split ('\t', s.remove("</legend>"), TRUE);
	  ag->insertLegend (fList, fileVersion);
	}
      else if (s.startsWith ("textMarker"))
	{			// version <= 0.8.9
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->insertTextMarker (fList, fileVersion);
	}
      else if (s.startsWith ("<text>") && s.endsWith ("</text>"))
	{
	  fList = QStringList::split ('\t', s.remove ("</text>"), TRUE);
	  ag->insertTextMarker (fList, fileVersion);
	}
      else if (s.startsWith ("lineMarker"))
	{			// version <= 0.8.9
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->insertLineMarker (fList, fileVersion);
	}
      else if (s.startsWith ("<line>") && s.endsWith ("</line>"))
	{
	  fList = QStringList::split ('\t', s.remove ("</line>"), TRUE);
	  ag->insertLineMarker (fList, fileVersion);
	}
      else if (s.startsWith ("ImageMarker")
	       || (s.startsWith ("<image>") && s.endsWith ("</image>")))
	{
	  fList = QStringList::split ('\t', s.remove("</image>"), TRUE);
	  ag->insertImageMarker (fList, fileVersion, workingDir);
	}
      else if (s.startsWith ("FitID"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  ag->setFitID (fList[1].toInt ());
	}
      else if (s.startsWith ("AxisType"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  for (i = 0; i < 4; i++)
	    {
	      QStringList lst = QStringList::split (';', fList[i + 1], false);
	      int format = lst[0].toInt ();
	      if (format == Graph::Day)
		ag->setLabelsDayFormat (i, lst[1].toInt ());
	      else if (format == Graph::Month)
		ag->setLabelsMonthFormat (i, lst[1].toInt ());
	      else if (format == Graph::Time || format == Graph::Date)
		ag->setLabelsDateTimeFormat (i, format,
					     lst[1] + ";" + lst[2]);
	      else
		{
		  Table *nw = table (lst[1]);
		  ag->setLabelsTextFormat (i, format, lst[1], nw);
		}
	    }
	}
      else if (fileVersion < 69 && s.startsWith ("AxesTickLabelsCol"))
	{
	  fList = QStringList::split ('\t', s, TRUE);
	  QValueList < int >axesTypes = ag->axesType ();
	  for (i = 0; i < 4; i++)
	    {
	      QString colName = fList[i + 1];
	      Table *nw = table (colName);
	      ag->setLabelsTextFormat (i, axesTypes[i], colName, nw);
	    }
	}
    }
  ag->replot ();
  ag->setIgnoreResizeEvents (!autoResizeLayers);
  ag->setAutoscaleFonts (autoScaleFonts);
  ag->setAxisTitleInverted(QwtPlot::yRight,flipRightAxisTitle);
  ag->setAutoScaleFrameWidth(autoScaleFrameWidth);
  ag->setAutoScaleGridWidth(autoScaleGridWidth);
  ag->setAutoScaleLineWidth(autoScaleLineWidth);
  ag->setTextMarkerDefaults (legendFrameStyle, plotLegendFont,
			     legendTextColor, legendBackground);
  ag->setArrowDefaults (defaultArrowLineWidth, defaultArrowColor,
			defaultArrowLineStyle,
			defaultArrowHeadLength,
			defaultArrowHeadAngle,
			defaultArrowHeadFill);
  ag->blockSignals(false);
  //plot->connectLayer (ag);
  //fprintf(stderr,"end of openGraph\n");
}

Graph3D *
ApplicationWindow::openSurfacePlot (const QStringList & lst)
{
  QStringList fList = QStringList::split ('\t', lst[0], TRUE);
  QString caption = fList[0];
  QString date = fList[1];
  if (date.isEmpty ())
    date = QDateTime::currentDateTime ().toString (Qt::LocalDate);

  fList = QStringList::split ('\t', lst[2], false);
  if (fList.count()<9) // for old format
    for(int i=0;i<3;i++)
      fList.append("0");
  Graph3D *plot = 0;

  if (fList[1].endsWith ("(Y)", TRUE))	//Ribbon plot
    plot = dataPlot3D (caption, fList[1], fList[2].toDouble (),
		       fList[3].toDouble (), fList[4].toDouble (),
		       fList[5].toDouble (), fList[6].toDouble (),
		       fList[7].toDouble (), fList[10].toInt());
  else if (fList[1].contains ("(Z)", TRUE) > 0)
    plot = dataPlotXYZ (caption, fList[1], fList[2].toDouble (),
			fList[3].toDouble (), fList[4].toDouble (),
			fList[5].toDouble (), fList[6].toDouble (),
			fList[7].toDouble (), fList[10].toInt());
  else if (fList[1].startsWith ("matrix<", TRUE)
	   && fList[1].endsWith (">", false))
    plot = openMatrixPlot3D (caption, fList[1], fList[2].toDouble (),
			     fList[3].toDouble (), fList[4].toDouble (),
			     fList[5].toDouble (), fList[6].toDouble (),
			     fList[7].toDouble (), fList[10].toInt() );
  else
    plot = newPlot3D (caption, fList[1], fList[2].toDouble (),
		      fList[3].toDouble (), fList[4].toDouble (),
		      fList[5].toDouble (), fList[6].toDouble (),
		      fList[7].toDouble (), fList[10].toInt());

  if (!plot)
    return 0;

  plot->blockSignals(true);

  setListViewDate (caption, date);
  plot->setBirthDate (date);

  restoreWindowGeometry (plot, lst[1]);
  plot->hide();

  fList = QStringList::split ('\t', lst[3], false);
  plot->setStyle (fList);

  fList = QStringList::split ('\t', lst[4], false);
  plot->setGrid (fList[1].toInt ());

  fList = QStringList::split ('\t', lst[5], true);
  plot->setTitle (fList);

  fList = QStringList::split ('\t', lst[6], false);
  plot->setColors (fList);

  fList = QStringList::split ('\t', lst[7], false);
  fList.pop_front ();
  plot->setAxesLabels (fList);

  fList = QStringList::split ('\t', lst[8], false);
  plot->setTicks (fList);

  fList = QStringList::split ('\t', lst[9], false);
  plot->setTickLengths (fList);

  fList = QStringList::split ('\t', lst[10], false);
  plot->setOptions (fList);

  fList = QStringList::split ('\t', lst[11], false);
  plot->setNumbersFont (fList);

  fList = QStringList::split ('\t', lst[12], false);
  plot->setAxisLabelFont (0,fList);

  fList = QStringList::split ('\t', lst[13], false);
  plot->setAxisLabelFont (1,fList);

  fList = QStringList::split ('\t', lst[14], false);
  plot->setAxisLabelFont (2,fList);

  fList = QStringList::split ('\t', lst[15], false);
  plot->setRotation (fList[1].toDouble (), fList[2].toDouble (),
		     fList[3].toDouble ());

  fList = QStringList::split ('\t', lst[16], false);
  plot->setZoom (fList[1].toDouble ());

  fList = QStringList::split ('\t', lst[17], false);
  plot->setScale (fList[1].toDouble (), fList[2].toDouble (),
		  fList[3].toDouble ());

  fList = QStringList::split ('\t', lst[18], false);
  plot->setShift (fList[1].toDouble (), fList[2].toDouble (),
		  fList[3].toDouble ());

  fList = QStringList::split ('\t', lst[19], false);
  plot->setMeshLineWidth (fList[1].toInt ());

  if (fileVersion > 71)
    {
      fList = QStringList::split ('\t', lst[20], false);
      plot->setWindowLabel (fList[1]);
      plot->setCaptionPolicy ((myWidget::CaptionPolicy) fList[2].toInt ());
      setListViewLabel (plot->getName(), fList[1]);
    }

  if (fileVersion >= 88)
    {
      fList = QStringList::split ('\t', lst[21], false);
      plot->setOrtho (fList[1].toInt ());
    }

  plot->update ();
  plot->setIgnoreFonts (false);
  plot->blockSignals(false);
  return plot;
}

void
ApplicationWindow::copyActiveLayer ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  copiedLayer = TRUE;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  delete lastCopiedLayer;
  lastCopiedLayer = new Graph (0, 0, WDestructiveClose);
  lastCopiedLayer->setGeometry (0, 0, g->width (), g->height ());
  lastCopiedLayer->copy (g);
  g->copyImage ();
}

void
ApplicationWindow::showAnalysisDialog (const QString & whichFit)
{
  int index = -1;
  QString curvetitle = activeGraph->selectedCurveTitle ();
  QStringList curvelist = activeGraph->analyzableCurvesList ();
  for(int i=0;i<(int)curvelist.count();i++)
    if(curvelist[i]==curvetitle)
      index = i;

  analysisDialog *ad =
    new analysisDialog (0, tr ("Curve:"), "analysisDialog", TRUE,
			WDestructiveClose);
  ad->setCurveNames (curvelist,index);
  ad->setOperationType (whichFit);
  ad->showNormal ();
  ad->setActiveWindow ();

  connect (ad, SIGNAL (analyse (const QString &, const QString &)), this,
	   SLOT (analyzeCurve (const QString &, const QString &)));
}

void
ApplicationWindow::analyzeCurve (const QString & whichFit,
				 const QString & curveTitle)
{
  //fprintf(stderr,"curve = %s\n",curveTitle.ascii());

  if (whichFit == "fitLinear" || whichFit == "fitSigmoidal"
      || whichFit == "fitGauss" || whichFit == "fitLorentz")
    {
      Fit *fitter = 0;
      if (whichFit == "fitLinear")
	fitter = new LinearFit (this, activeGraph);
      else if (whichFit == "fitSigmoidal")
	fitter = new SigmoidalFit (this, activeGraph);
      else if (whichFit == "fitGauss")
	fitter = new GaussFit (this, activeGraph);
      else if (whichFit == "fitLorentz")
	fitter = new LorentzFit (this, activeGraph);

      if (fitter->setDataFromCurve (curveTitle))
	{
	  if (whichFit != "fitLinear")
	    fitter->guessInitialValues ();
	  fitter->setGenerateFunction (generateFunction);
	  QString name = QString(fitter->name()) + tr("Fit");
	  if(generateFunction)
	    name = activeGraph->generateUniqueName( name );
	  else
	    name = generateUniqueName( name );
	  fitter->setCurveName(name);
	  fitter->setGenerateUniformFitPoints (generateUniformFitPoints);
	  fitter->setFitPoints( fitPoints );
	  fitter->fit ();
	}
      delete fitter;	  
    }
  else if (whichFit == "differentiate" && activeGraph->diffCurve (this, curveTitle))
    {
      Table *w = table (tableWindows.last ());
      QStringList list;
      list << w->getName() + "_derivative";
      MultiLayer *d = multilayerPlot (w, list, 0);
      d->setFocus ();
    }
}

void
ApplicationWindow::analysis (const QString & whichFit)
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("MultiLayer"))
    return;

  Graph *g = ((MultiLayer *) ws->activeWindow ())->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  activeGraph = g;
  aw = (myWidget *) ws->activeWindow ();

  if (g->selectorsEnabled ())	// a curve is selected
    analyzeCurve (whichFit, g->selectedCurveTitle ());
  else
    {
      QStringList clist = g->analyzableCurvesList();
      if (clist.count() == 1)
	{
	  const QwtPlotCurve *c = g->curve (clist[0]);
	  if (c)
	    analyzeCurve (whichFit, clist[0]);
	}
      else
	showAnalysisDialog (whichFit);
    }
}

void
ApplicationWindow::pickPointerCursor ()
{
  btnPointer->setOn (true);
  activeGraph = 0;
}

void
ApplicationWindow::disableTools ()
{
  //if(displayBar->isVisible())
  //  displayBar->hide();
  info->setText("");
  
  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    if (w->isA ("MultiLayer"))
      {
	QWidgetList *lst = ((MultiLayer *) w)->graphPtrs ();
	for (Graph *g = (Graph *)lst->first(); g; g = (Graph *)lst->next())
	  g->disableTools ();
      }

  delete windows;
}

void
ApplicationWindow::pickDataTool (QAction * action)
{
  if (!action)
    return;

  disableTools ();

  if (action == btnCursor)
    showCursor ();
  else if (action == btnSelect)
    showRangeSelectors ();
  else if (action == btnPicker)
    showScreenReader ();
  //else if (action == btnSubback)
  //  subtractLine();
  else if (action == btnMovePoints)
    movePoints ();
  else if (action == btnRemovePoints)
    removePoints ();
  else if (action == btnZoomIn)
    zoomIn ();
  else if (action == btnZoomOut)
    zoomOut ();
  else if (action == btnArrow)
    drawArrow ();
  else if (action == btnLine)
    drawLine ();
}

void
ApplicationWindow::connectSurfacePlot (Graph3D * plot)
{
  connect (plot, SIGNAL (showTitleBarMenu ()), this,
	   SLOT (showWindowTitleBarMenu ()));
  connect (plot, SIGNAL (showContextMenu ()), this,
	   SLOT (showWindowContextMenu ()));
  connect (plot, SIGNAL (showOptionsDialog ()), this,
	   SLOT (showPlot3dDialog ()));
  connect (plot, SIGNAL (closedWindow (myWidget *)), this,
	   SLOT (closeWindow (myWidget *)));
  connect (plot, SIGNAL (hiddenWindow (myWidget *)), this,
	   SLOT (hideWindow (myWidget *)));
  connect (plot, SIGNAL (statusChanged (myWidget *)), this,
	   SLOT (updateWindowStatus (myWidget *)));
  connect (plot, SIGNAL (modified ()), this, SIGNAL (modified ()));
  connect (plot, SIGNAL (custom3DActions (QWidget *)), this,
	   SLOT (custom3DActions (QWidget *)));

  plot->askOnCloseEvent (confirmClosePlot3D);
}

void
ApplicationWindow::connectMultilayerPlot (MultiLayer * m)
{
// FIXME: the signal changeActiveLayer does not exist
//connect (g,SIGNAL(changeActiveLayer(Graph *)),this,SLOT(changeActiveGraph(Graph *)));
  connect (m, SIGNAL (showTitleBarMenu ()), this,
	   SLOT (showWindowTitleBarMenu ()));
  connect (m, SIGNAL (showTextDialog ()), this, SLOT (showTextDialog ()));
  connect (m, SIGNAL (showPlotDialog (int)), this,
	   SLOT (showPlotDialog (int)));
  connect (m, SIGNAL (showScaleDialog (int)), this,
	   SLOT (showScalePageFromAxisDialog (int)));
  connect (m, SIGNAL (showAxisDialog (int)), this,
	   SLOT (showAxisPageFromAxisDialog (int)));
  connect (m, SIGNAL (showCurveContextMenu (int)), this,
	   SLOT (showCurveContextMenu (int)));

  connect (m, SIGNAL (showWindowContextMenu ()), this,
	   SLOT (showWindowContextMenu ()));
  connect (m, SIGNAL (showCurvesDialog ()), this, SLOT (showCurvesDialog ()));
  connect (m, SIGNAL (drawLineEnded (bool)), btnPointer, SLOT (setOn (bool)));
  connect (m, SIGNAL (drawTextOff ()), this, SLOT (disableAddText ()));
  connect (m, SIGNAL (showXAxisTitleDialog ()), this,
	   SLOT (showXAxisTitleDialog ()));
  connect (m, SIGNAL (showYAxisTitleDialog ()), this,
	   SLOT (showYAxisTitleDialog ()));
  connect (m, SIGNAL (showRightAxisTitleDialog ()), this,
	   SLOT (showRightAxisTitleDialog ()));
  connect (m, SIGNAL (showTopAxisTitleDialog ()), this,
	   SLOT (showTopAxisTitleDialog ()));
  connect (m, SIGNAL (showMarkerPopupMenu ()), this,
	   SLOT (showMarkerPopupMenu ()));
  connect (m, SIGNAL (closedWindow (myWidget *)), this,
	   SLOT (closeWindow (myWidget *)));
  connect (m, SIGNAL (hiddenWindow (myWidget *)), this,
	   SLOT (hideWindow (myWidget *)));
  connect (m, SIGNAL (statusChanged (myWidget *)), this,
	   SLOT (updateWindowStatus (myWidget *)));
  connect (m, SIGNAL (cursorInfo (const QString &,const QString &,int)),
	   this, SLOT (showCursorInfo(const QString &,const QString &,int)));
  connect (m, SIGNAL (showInfo (const QString &)), info,
	   SLOT (setText (const QString &)));
  connect (m, SIGNAL (showDisplayBar()), this, SLOT(showDisplayBar()));
  connect (m, SIGNAL (showImageDialog ()), this, SLOT (showImageDialog ()));
  connect (m,
	   SIGNAL (createTablePlot
		   (const QString &, int, int, const QString &)), this,
	   SLOT (newWrksheetPlot
		 (const QString &, int, int, const QString &)));
  connect (m,
	   SIGNAL (createHiddenTable
		   (const QString &, int, int, int, const QString &)), this,
	   SLOT (newHiddenTable
		 (const QString &, int, int, int, const QString &)));
  connect (m,
	   SIGNAL (createTable (const QString &, int, int, const QString &)),
	   this,
	   SLOT (newTable (const QString &, int, int, const QString &)));
  connect (m, SIGNAL (showPieDialog ()), this, SLOT (showPieDialog ()));
  connect (m, SIGNAL (viewTitleDialog ()), this, SLOT (showTitleDialog ()));
  connect (m, SIGNAL (modifiedPlot ()), this, SLOT (modifiedProject ()));
  connect (m, SIGNAL (showLineDialog ()), this, SLOT (showLineDialog ()));
  connect (m, SIGNAL (updateTable (const QString &, int, const QString &)),
	   this, SLOT (updateTable (const QString &, int, const QString &)));
  connect (m, SIGNAL (updateTableColumn(const QString&, QMemArray<double>&)),
	   this, SLOT (updateTableColumn(const QString&, QMemArray<double>&)));

  connect (m, SIGNAL (clearCell (const QString &, int)), this,
	   SLOT (clearCellFromTable (const QString &, int)));
  connect (m, SIGNAL (showGeometryDialog ()), this,
	   SLOT (showPlotGeometryDialog ()));
  connect (m, SIGNAL (pasteMarker ()), this, SLOT (pasteSelection ()));
  connect (m, SIGNAL (showGraphContextMenu ()), this,
	   SLOT (showGraphContextMenu ()));

  connect (m, SIGNAL (createIntensityTable (const QPixmap &)),
	   this, SLOT (createIntensityMatrix (const QPixmap &)));
  connect (m,
	   SIGNAL (createHistogramTable
		   (const QString &, int, int, const QString &)), this,
	   SLOT (showHistogramTable
		 (const QString &, int, int, const QString &)));
  connect (m, SIGNAL (setPointerCursor ()), this,
	   SLOT (pickPointerCursor ()));

  m->askOnCloseEvent (confirmClosePlot2D);
}

void
ApplicationWindow::connectTable (Table * w)
{
  connect (w, SIGNAL (showTitleBarMenu ()), this,
	   SLOT (showWindowTitleBarMenu ()));
  connect (w, SIGNAL (statusChanged (myWidget *)), this,
	   SLOT (updateWindowStatus (myWidget *)));
  connect (w, SIGNAL (hiddenWindow (myWidget *)), this,
	   SLOT (hideWindow (myWidget *)));
  connect (w, SIGNAL (closedWindow (myWidget *)), this,
	   SLOT (closeWindow (myWidget *)));
  connect (w, SIGNAL (removedCols (const QStringList &)), this,
	   SLOT (removeCurves (const QStringList &)));
  connect (w, SIGNAL (modifiedData (Table *, const QString &)), this,
	   SLOT (updateCurves (Table *, const QString &)));
  connect (w, SIGNAL (modifiedTable (Table *)), 
	   this, SLOT (updateCurves (Table *)));
  connect (w, SIGNAL (plotCol (Table *, const QStringList &, int)), this,
	   SLOT (multilayerPlot (Table *, const QStringList &, int)));
  connect (w, SIGNAL (modifiedWindow (QWidget *)), this,
	   SLOT (modifiedProject (QWidget *)));
  connect (w, SIGNAL (optionsDialog ()), this,
	   SLOT (showColumnOptionsDialog ()));
  connect (w, SIGNAL (colValuesDialog ()), this,
	   SLOT (showColumnValuesDialog ()));
  connect (w, SIGNAL (showContextMenu (bool)), this,
	   SLOT (showTableContextMenu (bool)));
  connect (w, SIGNAL (changedColHeader (const QString &, const QString &)),
	   this, SLOT (updateColNames (const QString &, const QString &)));
  connect (w,
	   SIGNAL (createTable (const QString &, int, int, const QString &)),
	   this,
	   SLOT (newTable (const QString &, int, int, const QString &)));

//3d plots
  connect (w, SIGNAL (plot3DRibbon (Table *, const QString &)), this,
	   SLOT (dataPlot3D (Table *, const QString &)));
  connect (w, SIGNAL (plotXYZ (Table *, const QString &, int)), this,
	   SLOT (dataPlotXYZ (Table *, const QString &, int)));

  w->askOnCloseEvent (confirmCloseTable);
}

void
ApplicationWindow::setAppColors (const QColor & wc, const QColor & pc,
				 const QColor & tpc)
{
  if (workspaceColor != wc)
    {
      workspaceColor = wc;
      ws->setPaletteBackgroundColor (wc);
    }

  if (panelsColor != pc)
    {
      panelsColor = pc;
      listview->setPaletteBackgroundColor (pc);
      results->setPaper (QBrush (pc, SolidPattern));

      QPalette pal = qApp->palette ();
      pal.setColor (QPalette::Active, QColorGroup::Base,
		    QColor (panelsColor));
      qApp->setPalette (pal, true, 0);
    }

  if (panelsTextColor != tpc)
    {
      panelsTextColor = tpc;
      listview->setPaletteForegroundColor (tpc);

      results->setPaletteForegroundColor (tpc);
      if (results->isVisible ())
	{
	  results->hide ();
	  results->show ();
	}
    }
}

void
ApplicationWindow::setPlot3DOptions ()
{
  QWidgetList *windows = windowsList ();
  for (QWidget * w = windows->first (); w; w = windows->next ())
    if (w->isA ("Graph3D"))
      {
	((Graph3D *)w)->setSmoothMesh (smooth3DMesh);
	((Graph3D *)w)->setOrtho (orthogonal3DPlots);
      }

  delete windows;
}

void
ApplicationWindow::createActions ()
{
  actionNewProject =
    new QAction (QPixmap (new_xpm), tr ("New &Project"), tr ("Ctrl+N"), this);
  connect (actionNewProject, SIGNAL (activated ()), this,
	   SLOT (newProject ()));

  actionNewGraph =
    new QAction (QPixmap (new_graph_xpm), tr ("New &Graph"), tr ("Ctrl+G"),
		 this);
  connect (actionNewGraph, SIGNAL (activated ()), this, SLOT (newGraph ()));

  actionNewNote =
    new QAction (QPixmap (new_note_xpm), tr ("New &Note"), QString::null,
		 this);
  connect (actionNewNote, SIGNAL (activated ()), this, SLOT (newNote ()));

  actionNewTable =
    new QAction (QPixmap (table_xpm), tr ("New &Table"), tr ("Ctrl+T"), this);
  connect (actionNewTable, SIGNAL (activated ()), this, SLOT (newTable ()));

  actionNewMatrix =
    new QAction (QPixmap (new_matrix_xpm), tr ("New &Matrix"), tr ("Ctrl+M"),
		 this);
  connect (actionNewMatrix, SIGNAL (activated ()), this, SLOT (newMatrix ()));

  actionNewFunctionPlot =
    new QAction (QPixmap (newF_xpm), tr ("New &Function Plot"), tr ("Ctrl+F"),
		 this);
  connect (actionNewFunctionPlot, SIGNAL (activated ()), this,
	   SLOT (newFunctionPlot ()));

  actionNewSurfacePlot =
    new QAction (QPixmap (newFxy_xpm), tr ("New &Surface 3D Plot"),
		 tr ("Ctrl+ALT+Z"), this);
  connect (actionNewSurfacePlot, SIGNAL (activated ()), this,
	   SLOT (newSurfacePlot ()));

  actionOpen =
    new QAction (QPixmap (fileopen_xpm), tr ("&Open"), tr ("Ctrl+O"), this);
  connect (actionOpen, SIGNAL (activated ()), this, SLOT (open ()));

  actionLoadImage =
    new QAction (tr ("Open image &file"), tr ("Ctrl+I"), this);
  connect (actionLoadImage, SIGNAL (activated ()), this, SLOT (loadImage ()));

  actionImportImage =
    new QAction (tr ("Import i&mage"), QString::null, this);
  connect (actionImportImage, SIGNAL (activated ()), this,
	   SLOT (importImage ()));

  actionSaveProject =
    new QAction (QPixmap (filesave_xpm), tr ("&Save Project"), tr ("Ctrl+S"),
		 this);
  connect (actionSaveProject, SIGNAL (activated ()), this,
	   SLOT (saveProject ()));
  actionSaveProject->setEnabled (FALSE);
  saved = TRUE;

  actionSaveProjectAs =
    new QAction (tr ("Save Project &as"), QString::null, this);
  connect (actionSaveProjectAs, SIGNAL (activated ()), this,
	   SLOT (saveProjectAs ()));

  actionOpenTemplate =
    new QAction (QPixmap (open_template_xpm), tr ("Open Temp&late"),
		 QString::null, this);
  connect (actionOpenTemplate, SIGNAL (activated ()), this,
	   SLOT (openTemplate ()));

  actionSaveTemplate =
    new QAction (QPixmap (save_template_xpm), tr ("Save as &Template"),
		 QString::null, this);
  connect (actionSaveTemplate, SIGNAL (activated ()), this,
	   SLOT (saveAsTemplate ()));

  actionSaveNote = new QAction (tr ("Save Notes As"), QString::null, this);
  connect (actionSaveNote, SIGNAL (activated ()), this, SLOT (saveNoteAs ()));

  actionLoad =
    new QAction (QPixmap (import_xpm), tr ("&Single file"), tr ("Ctrl+K"),
		 this);
  connect (actionLoad, SIGNAL (activated ()), this, SLOT (loadASCII ()));

  actionLoadMultiple =
    new QAction (QPixmap (multiload_xpm), tr ("&Multiple files"),
		 tr ("Ctrl+Alt+K"), this);
  connect (actionLoadMultiple, SIGNAL (activated ()), this,
	   SLOT (loadMultiple ()));

  actionUndo =
    new QAction (QPixmap (undo_xpm), tr ("&Undo"), tr ("Ctrl+Z"), this);
  connect (actionUndo, SIGNAL (activated ()), this, SLOT (undo ()));
  actionUndo->setEnabled (FALSE);

  actionRedo =
    new QAction (QPixmap (redo_xpm), tr ("&Redo"), tr ("Ctrl+R"), this);
  connect (actionRedo, SIGNAL (activated ()), this, SLOT (redo ()));
  actionRedo->setEnabled (FALSE);

  actionCopyWindow =
    new QAction (QPixmap (duplicate_xpm), tr ("&Duplicate"), QString::null,
		 this);
  connect (actionCopyWindow, SIGNAL (activated ()), this,
	   SLOT (copyWindow ()));

  actionCutSelection =
    new QAction (QPixmap (cut_xpm), tr ("Cu&t selection"), tr ("Ctrl+X"),
		 this);
  connect (actionCutSelection, SIGNAL (activated ()), this,
	   SLOT (cutSelection ()));

  actionCopySelection =
    new QAction (QPixmap (copy_xpm), tr ("&Copy selection"), tr ("Ctrl+C"),
		 this);
  connect (actionCopySelection, SIGNAL (activated ()), this,
	   SLOT (copySelection ()));

  actionPasteSelection =
    new QAction (QPixmap (paste_xpm), tr ("&Paste selection"), tr ("Ctrl+V"),
		 this);
  connect (actionPasteSelection, SIGNAL (activated ()), this,
	   SLOT (pasteSelection ()));

  actionClearSelection =
    new QAction (QPixmap (erase_xpm), tr ("&Delete selection"),
		 tr ("Key_Delete"), this);
  connect (actionClearSelection, SIGNAL (activated ()), this,
	   SLOT (clearSelection ()));

  actionShowExplorer =
    new QAction (QPixmap (folder_xpm), tr ("Project &explorer"),
		 tr ("Ctrl+E"), this);
  actionShowExplorer->setToggleAction (TRUE);
  actionShowExplorer->setOn (FALSE);
  connect (actionShowExplorer, SIGNAL (activated ()), this,
	   SLOT (showExplorer ()));

  actionShowLog =
    new QAction (QPixmap (log_xpm), tr ("Results &Log"), QString::null, this);
  actionShowLog->setToggleAction (TRUE);
  actionShowLog->setOn (FALSE);

#ifdef SCRIPTING_CONSOLE
  actionShowConsole =
    new QAction (0, tr ("Scripting &Console"), QString::null, this);
  actionShowConsole->setToggleAction (true);
  actionShowConsole->setOn (false);
#endif

  actionAddLayer =
    new QAction (QPixmap (newLayer_xpm), tr ("Add La&yer"), tr ("ALT+L"),
		 this);
  connect (actionAddLayer, SIGNAL (activated ()), this, SLOT (addLayer ()));

  actionShowLayerDialog =
    new QAction (QPixmap (arrangeLayers_xpm), tr ("Arran&ge Layers"),
		 tr ("ALT+A"), this);
  connect (actionShowLayerDialog, SIGNAL (activated ()), this,
	   SLOT (showLayerDialog ()));

  actionShowLayerOrderDialog = new QAction (0, tr ("Reorder Layers"),
					    QString::null, this);
  connect (actionShowLayerOrderDialog, SIGNAL (activated ()), this,
	   SLOT (showLayerOrderDialog ()));

  actionAutomaticLayout =
    new QAction (QPixmap (auto_layout_xpm), tr ("Automatic Layout"),
		 QString::null, this);
  connect (actionAutomaticLayout, SIGNAL (activated ()), this,
	   SLOT (autoArrangeLayers ()));

  actionExportGraph = new QAction (tr ("&Current"), tr ("Alt+G"), this);
  connect (actionExportGraph, SIGNAL (activated ()), this,
	   SLOT (exportGraph ()));

  actionExportAllGraphs = new QAction (tr ("&All"), tr ("Alt+X"), this);
  connect (actionExportAllGraphs, SIGNAL (activated ()), this,
	   SLOT (exportAllGraphs ()));

  actionPrint =
    new QAction (QPixmap (fileprint_xpm), tr ("&Print"), tr ("Ctrl+P"), this);
  connect (actionPrint, SIGNAL (activated ()), this, SLOT (print ()));

  actionPrintAllPlots =
    new QAction (tr ("Print All Plo&ts"), QString::null, this);
  connect (actionPrintAllPlots, SIGNAL (activated ()), this,
	   SLOT (printAllPlots ()));

  actionShowExportASCIIDialog =
    new QAction (tr ("E&xport ASCII"), QString::null, this);
  connect (actionShowExportASCIIDialog, SIGNAL (activated ()), this,
	   SLOT (showExportASCIIDialog ()));

  actionShowImportDialog =
    new QAction (tr ("Set import &options"), tr ("Ctrl+Alt+O"), this);
  connect (actionShowImportDialog, SIGNAL (activated ()), this,
	   SLOT (showImportDialog ()));

  actionCloseAllWindows =
    new QAction (QPixmap (quit_xpm), tr ("&Quit"), tr ("Ctrl+Q"), this);
  connect (actionCloseAllWindows, SIGNAL (activated ()), qApp,
	   SLOT (closeAllWindows ()));

  actionClearLogInfo =
    new QAction (tr ("Clear &log information"), QString::null, this);
  connect (actionClearLogInfo, SIGNAL (activated ()), this,
	   SLOT (clearLogInfo ()));

  actionDeleteUselessTables =
    new QAction (QPixmap (close_xpm), tr ("Delete useless tables"),
		 QString::null, this);
  connect (actionDeleteUselessTables, SIGNAL (activated ()), this,
	   SLOT (deleteUselessTables ()));

  actionShowPlotWizard =
    new QAction (QPixmap (wizard_xpm), tr ("Plot &wizard"), tr ("Ctrl+Alt+W"),
		 this);
  connect (actionShowPlotWizard, SIGNAL (activated ()), this,
	   SLOT (showPlotWizard ()));

  actionShowConfigureDialog =
    new QAction (tr ("&Preferences"), QString::null, this);
  connect (actionShowConfigureDialog, SIGNAL (activated ()), this,
	   SLOT (showPreferencesDialog ()));

  actionShowCurvesDialog =
    new QAction (QPixmap (curves_xpm), tr ("Add/Remove &Curve"),
		 tr ("ALT+C"), this);
  connect (actionShowCurvesDialog, SIGNAL (activated ()), this,
	   SLOT (showCurvesDialog ()));

  actionAddErrorBars =
    new QAction (QPixmap (errors_xpm), tr ("Add &Error Bars"),
		 tr ("Ctrl+B"), this);
  connect (actionAddErrorBars, SIGNAL (activated ()), this,
	   SLOT (addErrorBars ()));

  actionAddFunctionCurve =
    new QAction (QPixmap (fx_xpm), tr ("Add &Function"), tr ("Ctrl+Alt+F"),
		 this);
  connect (actionAddFunctionCurve, SIGNAL (activated ()), this,
	   SLOT (addFunctionCurve ()));

  actionUnzoom =
    new QAction (QPixmap (unzoom_xpm), tr ("&Rescale to show all"),
		 tr ("Ctrl+Shift+R"), this);
  connect (actionUnzoom, SIGNAL (activated ()), this, SLOT (setAutoScale ()));

  actionNewLegend =
    new QAction (QPixmap (legend_xpm), tr ("New &Legend"), tr ("Ctrl+L"),
		 this);
  connect (actionNewLegend, SIGNAL (activated ()), this, SLOT (newLegend ()));

  actionResetColors =
    new QAction( tr("Reset Colors"), QString::null, this );
  connect (actionResetColors, SIGNAL(activated()), 
	   this, SLOT( resetGraphColors()));

  actionBlackenColors =
    new QAction( tr("Blacken Colors"), QString::null, this );
  connect (actionBlackenColors, SIGNAL(activated()), 
	   this, SLOT( blackenGraphColors()));

  actionResetSymbols =
    new QAction( tr("Reset Symbols"), QString::null, this );
  connect (actionResetSymbols, SIGNAL(activated()), 
	   this, SLOT( resetGraphSymbols()));

  actionResetFonts =
    new QAction( tr("Reset Fonts"), QString::null, this );
  connect( actionResetFonts, SIGNAL(activated()),
	   this, SLOT( resetGraphFonts() ) );

  actionTimeStamp =
    new QAction (QPixmap (clock_xpm), tr ("Add time stamp"),
		 tr ("Ctrl+ALT+T"), this);
  connect (actionTimeStamp, SIGNAL (activated ()), this,
	   SLOT (addTimeStamp ()));

  actionAddImage =
    new QAction (QPixmap (monalisa_xpm), tr ("Add &Image"), tr ("ALT+I"),
		 this);
  connect (actionAddImage, SIGNAL (activated ()), this, SLOT (addImage ()));

  actionPlotL =
    new QAction (QPixmap (lPlot_xpm), tr ("&Line"), QString::null, this);
  connect (actionPlotL, SIGNAL (activated ()), this, SLOT (plotL ()));

  actionPlotP =
    new QAction (QPixmap (pPlot_xpm), tr ("&Scatter"), QString::null, this);
  connect (actionPlotP, SIGNAL (activated ()), this, SLOT (plotP ()));

  actionPlotLP =
    new QAction (QPixmap (lpPlot_xpm), tr ("Line + S&ymbol"), QString::null,
		 this);
  connect (actionPlotLP, SIGNAL (activated ()), this, SLOT (plotLP ()));

  actionPlotVerticalDropLines =
    new QAction (QPixmap (dropLines_xpm), tr ("Vertical &drop lines"),
		 QString::null, this);
  connect (actionPlotVerticalDropLines, SIGNAL (activated ()), this,
	   SLOT (plotVerticalDropLines ()));

  actionPlotSpline =
    new QAction (QPixmap (spline_xpm), tr ("&Spline"), QString::null, this);
  connect (actionPlotSpline, SIGNAL (activated ()), this,
	   SLOT (plotSpline ()));

  actionPlotHorSteps =
    new QAction (QPixmap (hor_steps_xpm), tr ("&Horizontal Steps"),
		 QString::null, this);
  connect (actionPlotHorSteps, SIGNAL (activated ()), this,
	   SLOT (plotHorSteps ()));

  actionPlotVertSteps =
    new QAction (QPixmap (vert_steps_xpm), tr ("&Vertical Steps"),
		 QString::null, this);
  connect (actionPlotVertSteps, SIGNAL (activated ()), this,
	   SLOT (plotVertSteps ()));

  actionPlotVerticalBars =
    new QAction (QPixmap (vertBars_xpm), tr ("&Columns"), QString::null,
		 this);
  connect (actionPlotVerticalBars, SIGNAL (activated ()), this,
	   SLOT (plotVerticalBars ()));

  actionPlotHorizontalBars =
    new QAction (QPixmap (hBars_xpm), tr ("&Rows"), QString::null, this);
  connect (actionPlotHorizontalBars, SIGNAL (activated ()), this,
	   SLOT (plotHorizontalBars ()));

  actionPlotArea =
    new QAction (QPixmap (area_xpm), tr ("&Area"), QString::null, this);
  connect (actionPlotArea, SIGNAL (activated ()), this, SLOT (plotArea ()));

  actionPlotPie =
    new QAction (QPixmap (pie_xpm), tr ("&Pie"), QString::null, this);
  connect (actionPlotPie, SIGNAL (activated ()), this, SLOT (plotPie ()));

  actionPlotVectXYAM =
    new QAction (QPixmap (vectXYAM_xpm), tr ("Vectors XY&AM"), QString::null,
		 this);
  connect (actionPlotVectXYAM, SIGNAL (activated ()), this,
	   SLOT (plotVectXYAM ()));

  actionPlotVectXYXY =
    new QAction (QPixmap (vectXYXY_xpm), tr ("&Vectors &XYXY"), QString::null,
		 this);
  connect (actionPlotVectXYXY, SIGNAL (activated ()), this,
	   SLOT (plotVectXYXY ()));

  actionPlotHistogram =
    new QAction (QPixmap (histogram_xpm), tr ("&Histogram"), QString::null,
		 this);
  connect (actionPlotHistogram, SIGNAL (activated ()), this,
	   SLOT (plotHistogram ()));

  actionPlotStackedHistograms =
    new QAction (QPixmap (stacked_hist_xpm), tr ("&Stacked Histogram"),
		 QString::null, this);
  connect (actionPlotStackedHistograms, SIGNAL (activated ()), this,
	   SLOT (plotStackedHistograms ()));

  actionPlot2VerticalLayers =
    new QAction (QPixmap (panel_v2_xpm), tr ("&Vertical 2 Layers"),
		 QString::null, this);
  connect (actionPlot2VerticalLayers, SIGNAL (activated ()), this,
	   SLOT (plot2VerticalLayers ()));

  actionPlot2HorizontalLayers =
    new QAction (QPixmap (panel_h2_xpm), tr ("&Horizontal 2 Layers"),
		 QString::null, this);
  connect (actionPlot2HorizontalLayers, SIGNAL (activated ()), this,
	   SLOT (plot2HorizontalLayers ()));

  actionPlot4Layers =
    new QAction (QPixmap (panel_4_xpm), tr ("&4 Layers"), QString::null,
		 this);
  connect (actionPlot4Layers, SIGNAL (activated ()), this,
	   SLOT (plot4Layers ()));

  actionPlotStackedLayers =
    new QAction (QPixmap (stacked_xpm), tr ("&Stacked Layers"), QString::null,
		 this);
  connect (actionPlotStackedLayers, SIGNAL (activated ()), this,
	   SLOT (plotStackedLayers ()));

  actionPlot3DRibbon =
    new QAction (QPixmap (ribbon_xpm), tr ("&Ribbon"), QString::null, this);
  connect (actionPlot3DRibbon, SIGNAL (activated ()), this,
	   SLOT (plot3DRibbon ()));

  actionPlot3DBars =
    new QAction (QPixmap (bars_xpm), tr ("&Bars"), QString::null, this);
  connect (actionPlot3DBars, SIGNAL (activated ()), this,
	   SLOT (plot3DBars ()));

  actionPlot3DScatter =
    new QAction (QPixmap (scatter_xpm), tr ("&Scatter"), QString::null, this);
  connect (actionPlot3DScatter, SIGNAL (activated ()), this,
	   SLOT (plot3DScatter ()));

  actionPlot3DTrajectory =
    new QAction (QPixmap (trajectory_xpm), tr ("&Trajectory"), QString::null,
		 this);
  connect (actionPlot3DTrajectory, SIGNAL (activated ()), this,
	   SLOT (plot3DTrajectory ()));

  actionShowColStatistics =
    new QAction (QPixmap (col_stat_xpm), tr ("Statistics on &Columns"),
		 QString::null, this);
  connect (actionShowColStatistics, SIGNAL (activated ()), this,
	   SLOT (showColStatistics ()));

  actionShowXYStatistics =
    new QAction ( tr ("Statistics on &Columns"), QString::null, this);
  connect (actionShowXYStatistics, SIGNAL (activated ()), this,
	   SLOT (showXYStatistics ()));

  actionShowRowStatistics =
    new QAction (QPixmap (stat_rows_xpm), tr ("Statistics on &Rows"),
		 QString::null, this);
  connect (actionShowRowStatistics, SIGNAL (activated ()), this,
	   SLOT (showRowStatistics ()));

  actionShowIntDialog =
    new QAction (tr ("&Integrate"), QString::null, this);
  connect (actionShowIntDialog, SIGNAL (activated ()), this,
	   SLOT (showIntDialog ()));

  actionShowFwhmDialog =
    new QAction (tr ("&FWHM"), QString::null, this);
  connect (actionShowFwhmDialog, SIGNAL (activated ()), this,
	   SLOT (showFwhmDialog ()));

  actionShowMomentDialog =
    new QAction (tr ("&Moment Analysis"), QString::null, this);
  connect (actionShowMomentDialog, SIGNAL (activated ()), this,
	   SLOT (showMomentDialog ()));

  actionInterpolate =
    new QAction (tr ("Inte&rpolate"), QString::null, this);
  connect (actionInterpolate, SIGNAL (activated ()), this,
	   SLOT (showInterpolationDialog ()));

  actionAlign =
    new QAction (tr ("Align Curves"), QString::null, this);
  connect (actionAlign, SIGNAL (activated ()), this,
	   SLOT (showAlignCurvesDialog ()));

  actionCascade =
    new QAction (tr ("Cascade Curves"), QString::null, this);
  connect (actionCascade, SIGNAL (activated ()), this,
	   SLOT (showCascadeDialog ()));

  actionNormalizeGraph =
    new QAction (tr ("Normalize Curves"), QString::null, this);
  connect (actionNormalizeGraph, SIGNAL (activated ()), this,
	   SLOT (showNormalizeDialog ()));

  actionLowPassFilter =
    new QAction (tr ("&Low Pass"), QString::null, this);
  connect (actionLowPassFilter, SIGNAL (activated ()), this,
	   SLOT (lowPassFilterDialog ()));

  actionHighPassFilter =
    new QAction (tr ("&High Pass"), QString::null, this);
  connect (actionHighPassFilter, SIGNAL (activated ()), this,
	   SLOT (highPassFilterDialog ()));

  actionBandPassFilter =
    new QAction (tr ("&Band Pass"), QString::null, this);
  connect (actionBandPassFilter, SIGNAL (activated ()), this,
	   SLOT (bandPassFilterDialog ()));

  actionBandBlockFilter =
    new QAction (tr ("&Band Block"), QString::null, this);
  connect (actionBandBlockFilter, SIGNAL (activated ()), this,
	   SLOT (bandBlockFilterDialog ()));

  actionBandInterpFilter =
    new QAction (tr ("&Band Block (interpolate)"), QString::null, this);
  connect (actionBandInterpFilter, SIGNAL (activated ()), this,
	   SLOT (bandInterpFilterDialog ()));

  actionFFT = new QAction (tr ("&FFT"), QString::null, this);
  connect (actionFFT, SIGNAL (activated ()), this, SLOT (showFFTDialog ()));

  actionConvoluteCurves = new QAction (tr ("&Convolute"), QString::null, this);
  connect (actionConvoluteCurves, SIGNAL (activated ()), this, 
	   SLOT (showConvolutionDialog ()));

  actionDeconvFFT = new QAction (tr ("FFT"), QString::null, this);
  connect (actionDeconvFFT, SIGNAL (activated ()), this, 
	   SLOT (showDeconvFFTDialog ()));

  actionDeconvAndre = new QAction (tr("FFT(decay)"), QString::null, this);
  connect (actionDeconvAndre, SIGNAL (activated ()), this, 
	   SLOT (showDeconvAndreDialog ()));

  actionDeconvGS = new QAction (tr ("Gauss-Sidel"), QString::null, this);
  connect (actionDeconvGS, SIGNAL (activated ()), this, 
	   SLOT (showDeconvGSDialog ()));

  actionSmoothSavGol = new QAction (tr ("&Savitzky-Golay"), 
				    QString::null, this);
  connect (actionSmoothSavGol, SIGNAL (activated ()), this,
	   SLOT (showSmoothSavGolDialog ()));

  actionSmoothAverage =
    new QAction (tr ("Moving Window &Average"), QString::null, this);
  connect (actionSmoothAverage, SIGNAL (activated ()), this,
	   SLOT (showSmoothAverageDialog ()));

  actionGaussConv =
    new QAction (tr ("&Gaussian Convolugion"), QString::null, this);
  connect (actionGaussConv, SIGNAL (activated ()), this,
	   SLOT (showGaussConvDialog ()));

  actionLorentzConv =
    new QAction (tr ("&Lorentzian Convolugion"), QString::null, this);
  connect (actionLorentzConv, SIGNAL (activated ()), this,
	   SLOT (showLorentzConvDialog ()));

  actionDifferentiate =
    new QAction (tr ("&Differentiate"), QString::null, this);
  connect (actionDifferentiate, SIGNAL (activated ()), this,
	   SLOT (differentiate ()));

  actionFitLinear = new QAction (tr ("Fit &Linear"), QString::null, this);
  connect (actionFitLinear, SIGNAL (activated ()), this, SLOT (fitLinear ()));

  actionShowFitPolynomDialog =
    new QAction (tr ("Fit &Polynomial"), QString::null, this);
  connect (actionShowFitPolynomDialog, SIGNAL (activated ()), this,
	   SLOT (showFitPolynomDialog ()));

  actionShowSineFitDialog =
    new QAction (tr ("Fit Sinusoidal"), QString::null, this);
  connect (actionShowSineFitDialog, SIGNAL (activated ()), this,
	   SLOT (showSineFitDialog ()));

  actionShowExpDecayDialog =
    new QAction (tr ("&First Order"), QString::null, this);
  connect (actionShowExpDecayDialog, SIGNAL (activated ()), this,
	   SLOT (showExpDecayDialog ()));

  actionShowTwoExpDecayDialog =
    new QAction (tr ("&Second Order"), QString::null, this);
  connect (actionShowTwoExpDecayDialog, SIGNAL (activated ()), this,
	   SLOT (showTwoExpDecayDialog ()));

  actionShowExpDecay3Dialog =
    new QAction (tr ("&Third Order"), QString::null, this);
  connect (actionShowExpDecay3Dialog, SIGNAL (activated ()), this,
	   SLOT (showExpDecay3Dialog ()));

  actionFitExpGrowth =
    new QAction (tr ("Fit Exponential Gro&wth"), QString::null, this);
  connect (actionFitExpGrowth, SIGNAL (activated ()), this,
	   SLOT (showExpGrowthDialog ()));

  actionFitSigmoidal =
    new QAction (tr ("Fit &Boltzmann (Sigmoidal)"), QString::null, this);
  connect (actionFitSigmoidal, SIGNAL (activated ()), this,
	   SLOT (fitSigmoidal ()));

  actionFitGauss = new QAction (tr ("Fit &Gaussian"), QString::null, this);
  connect (actionFitGauss, SIGNAL (activated ()), this, SLOT (fitGauss ()));

  actionFitLorentz =
    new QAction (tr ("Fit Lorent&zian"), QString::null, this);
  connect (actionFitLorentz, SIGNAL (activated ()), this,
	   SLOT (fitLorentz ()));

  actionShowFitDialog =
    new QAction (tr ("&Non-linear Curve Fit"), tr ("Ctrl+Y"), this);
  connect (actionShowFitDialog, SIGNAL (activated ()), this,
	   SLOT (showFitDialog ()));

  actionShowPlotDialog = new QAction (tr ("&Plot"), QString::null, this);
  connect (actionShowPlotDialog, SIGNAL (activated ()), this,
	   SLOT (showGeneralPlotDialog ()));

  actionShowCurveFormatDialog =
    new QAction (tr ("&Curves"), QString::null, this);
  connect (actionShowCurveFormatDialog, SIGNAL (activated ()), this,
	   SLOT (showPlotDialog ()));

  actionShowScaleDialog =
    new QAction (tr ("&Scales"), QString::null, this);
  connect (actionShowScaleDialog, SIGNAL (activated ()), this,
	   SLOT (showScaleDialog ()));

  actionShowAxisDialog = new QAction (tr ("&Axes"), QString::null, this);
  connect (actionShowAxisDialog, SIGNAL (activated ()), this,
	   SLOT (showAxisDialog ()));

  actionShowGridDialog = new QAction (tr ("&Grid"), QString::null, this);
  connect (actionShowGridDialog, SIGNAL (activated ()), this,
	   SLOT (showGridDialog ()));

  actionShowTitleDialog =
    new QAction (tr ("&Title"), QString::null, this);
  connect (actionShowTitleDialog, SIGNAL (activated ()), this,
	   SLOT (showTitleDialog ()));

  actionShowColumnOptionsDialog =
    new QAction (tr ("Column &Options"), tr ("Ctrl+Alt+O"), this);
  connect (actionShowColumnOptionsDialog, SIGNAL (activated ()), this,
	   SLOT (showColumnOptionsDialog ()));

  actionShowColumnValuesDialog =
    new QAction (tr ("Set Column &Values"), tr ("Alt+Q"), this);
  connect (actionShowColumnValuesDialog, SIGNAL (activated ()), this,
	   SLOT (showColumnValuesDialog ()));

  actionTableRecalculate =
    new QAction (tr ("Recalculate"), tr ("Ctrl+Return"), this);
  connect (actionTableRecalculate, SIGNAL (activated ()), this,
	   SLOT (recalculateTable ()));

  actionColumnInvert =
    new QAction (tr ("Invert"), tr ("Ctrl+Alt+I"), this);
  connect (actionColumnInvert, SIGNAL (activated ()), this,
	   SLOT (invertColumns ()));

  actionShowColsDialog =
    new QAction (tr ("&Columns"), QString::null, this);
  connect (actionShowColsDialog, SIGNAL (activated ()), this,
	   SLOT (showColsDialog ()));

  actionShowRowsDialog = new QAction (tr ("&Rows"), QString::null, this);
  connect (actionShowRowsDialog, SIGNAL (activated ()), this,
	   SLOT (showRowsDialog ()));

  actionAbout = new QAction (tr ("&About QtiPlot"), tr ("F1"), this);
  connect (actionAbout, SIGNAL (activated ()), this, SLOT (about ()));

  actionShowHelp = new QAction (tr ("&Help"), tr ("Ctrl+H"), this);
  connect (actionShowHelp, SIGNAL (activated ()), this, SLOT (showHelp ()));

  actionChooseHelpFolder =
    new QAction (tr ("&Choose help folder"), QString::null, this);
  connect (actionChooseHelpFolder, SIGNAL (activated ()), this,
	   SLOT (chooseHelpFolder ()));

  actionRename = new QAction (tr ("&Rename Window"), QString::null, this);
  connect (actionRename, SIGNAL (activated ()), this, SLOT (rename ()));

  actionCloseWindow =
    new QAction (QPixmap (close_xpm), tr ("Close &Window"), tr ("Ctrl+W"),
		 this);
  connect (actionCloseWindow, SIGNAL (activated ()), this,
	   SLOT (closeActiveWindow ()));

  actionAddColToTable =
    new QAction (QPixmap (addCol_xpm), tr ("Add column"), QString::null,
		 this);
  connect (actionAddColToTable, SIGNAL (activated ()), this,
	   SLOT (addColToTable ()));

  actionAddRowToTable =
    new QAction (QPixmap (addRow_xpm), tr ("Add row"), QString::null,
		 this);
  connect (actionAddRowToTable, SIGNAL (activated ()), this,
	   SLOT (addRowToTable ()));

  actionGoToRow = new QAction (tr ("&Go to Row"), tr ("Ctrl+Alt+G"), this);
  connect (actionGoToRow, SIGNAL (activated ()), this, SLOT (goToRow ()));

  actionClearTable =
    new QAction (QPixmap (erase_xpm), tr ("Clear"), QString::null, this);
  connect (actionClearTable, SIGNAL (activated ()), this,
	   SLOT (clearTable ()));

  actionDeleteLayer =
    new QAction (QPixmap (erase_xpm), tr ("&Remove Layer"), tr ("Alt+R"),
		 this);
  connect (actionDeleteLayer, SIGNAL (activated ()), this,
	   SLOT (deleteLayer ()));

  actionResizeLayer =
    new QAction (tr ("Resize Layer"), QString::null, this);
  connect (actionResizeLayer, SIGNAL (activated ()), this,
	   SLOT (resizeLayer ()));

  actionResizeActiveWindow =
    new QAction (QPixmap (resize_xpm), tr ("Window &Geometry"),
		 QString::null, this);
  connect (actionResizeActiveWindow, SIGNAL (activated ()), this,
	   SLOT (resizeActiveWindow ()));

  actionHideActiveWindow =
    new QAction (tr ("&Hide Window"), QString::null, this);
  connect (actionHideActiveWindow, SIGNAL (activated ()), this,
	   SLOT (hideActiveWindow ()));

  actionShowMoreWindows =
    new QAction (tr ("More windows"), QString::null, this);
  connect (actionShowMoreWindows, SIGNAL (activated ()), this,
	   SLOT (showMoreWindows ()));

  actionPixelLineProfile =
    new QAction (QPixmap (pixelProfile_xpm), tr ("&View pixel line profile"),
		 QString::null, this);
  connect (actionPixelLineProfile, SIGNAL (activated ()), this,
	   SLOT (pixelLineProfile ()));

  actionIntensityTable =
    new QAction (tr ("&Intensity table"), QString::null, this);
  connect (actionIntensityTable, SIGNAL (activated ()), this,
	   SLOT (intensityTable ()));

  actionShowLineDialog =
    new QAction (tr ("&Properties"), QString::null, this);
  connect (actionShowLineDialog, SIGNAL (activated ()), this,
	   SLOT (showLineDialog ()));

  actionShowImageDialog =
    new QAction (tr ("&Properties"), QString::null, this);
  connect (actionShowImageDialog, SIGNAL (activated ()), this,
	   SLOT (showImageDialog ()));

  actionShowTextDialog =
    new QAction (tr ("&Properties"), QString::null, this);
  connect (actionShowTextDialog, SIGNAL (activated ()), this,
	   SLOT (showTextDialog ()));

  actionActivateWindow =
    new QAction (tr ("&Activate Window"), QString::null, this);
  connect (actionActivateWindow, SIGNAL (activated ()), this,
	   SLOT (activateWindow ()));

  actionMinimizeWindow =
    new QAction (tr ("Mi&nimize Window"), QString::null, this);
  connect (actionMinimizeWindow, SIGNAL (activated ()), this,
	   SLOT (minimizeWindow ()));

  actionMaximizeWindow =
    new QAction (tr ("Ma&ximize Window"), QString::null, this);
  connect (actionMaximizeWindow, SIGNAL (activated ()), this,
	   SLOT (maximizeWindow ()));

  actionHideWindow = new QAction (tr ("&Hide Window"), QString::null, this);
  connect (actionHideWindow, SIGNAL (activated ()), this,
	   SLOT (hideWindow ()));

  actionShowWindow = new QAction (tr ("&Show Window"), QString::null, this);
  connect (actionShowWindow, SIGNAL (activated ()), this,
	   SLOT (showWindow ()));

  actionResizeWindow =
    new QAction (QPixmap (resize_xpm), tr ("Re&size Window"),
		 QString::null, this);
  connect (actionResizeWindow, SIGNAL (activated ()), this,
	   SLOT (resizeWindow ()));

  actionPrintWindow =
    new QAction (QPixmap (fileprint_xpm), tr ("&Print Window"), QString::null,
		 this);
  connect (actionPrintWindow, SIGNAL (activated ()), this,
	   SLOT (printWindow ()));

  actionShowPlotGeometryDialog =
    new QAction (QPixmap (resize_xpm), tr ("&Layer geometry"), QString::null,
		 this);
  connect (actionShowPlotGeometryDialog, SIGNAL (activated ()), this,
	   SLOT (showPlotGeometryDialog ()));

  actionEditSurfacePlot =
    new QAction (tr ("&Surface"), QString::null, this);
  connect (actionEditSurfacePlot, SIGNAL (activated ()), this,
	   SLOT (editSurfacePlot ()));

  actionAdd3DData = new QAction (tr ("&Data Set"), QString::null, this);
  connect (actionAdd3DData, SIGNAL (activated ()), this, SLOT (add3DData ()));

  actionSetMatrixProperties =
    new QAction (tr ("Set &Properties"), QString::null, this);
  connect (actionSetMatrixProperties, SIGNAL (activated ()), this,
	   SLOT (showMatrixDialog ()));

  actionSetMatrixDimensions =
    new QAction (tr ("Set &Dimensions"), QString::null, this);
  connect (actionSetMatrixDimensions, SIGNAL (activated ()), this,
	   SLOT (showMatrixSizeDialog ()));

  actionSetMatrixValues =
    new QAction (tr ("Set &Values"), QString::null, this);
  connect (actionSetMatrixValues, SIGNAL (activated ()), this,
	   SLOT (showMatrixValuesDialog ()));

  actionTransposeMatrix =
    new QAction (tr ("&Transpose"), QString::null, this);
  connect (actionTransposeMatrix, SIGNAL (activated ()), this,
	   SLOT (transposeMatrix ()));

  actionInvertMatrix = new QAction (tr ("&Invert"), QString::null, this);
  connect (actionInvertMatrix, SIGNAL (activated ()), this,
	   SLOT (invertMatrix ()));

  actionMatrixInterpolate =
    new QAction (tr("Interpolate"), QString::null, this);
  connect (actionMatrixInterpolate, SIGNAL (activated()), this,
            SLOT(showMatrixInterpDialog()));

  actionMatrixFFT =
    new QAction (tr("FFT"), QString::null, this);
  connect (actionMatrixFFT, SIGNAL( activated()), this,
	   SLOT(matrixFFT()));

  actionMatrixFilter =
    new QAction (tr("FFT Filter"), QString::null, this);
  connect (actionMatrixFilter, SIGNAL( activated()), this,
	   SLOT(showMatrixFilterDialog()));

  actionMatrixDeterminant =
    new QAction (tr ("&Determinant"), QString::null, this);
  connect (actionMatrixDeterminant, SIGNAL (activated ()), this,
	   SLOT (matrixDeterminant ()));

  actionShrinkMatrix =
    new QAction (tr ("&Shrink"), QString::null, this);
  actionShrinkMatrix->setToolTip("Downsize the matrix by avaraging cells");
  connect (actionShrinkMatrix, SIGNAL (activated ()), this,
	   SLOT (showShrinkMatrixDialog()));

  actionConvertMatrix =
    new QAction (tr ("&Convert to spreadsheet"), QString::null, this);
  connect (actionConvertMatrix, SIGNAL (activated ()), this,
	   SLOT (convertMatrixToTable ()));

  actionConvertTable =
    new QAction (tr ("Convert to &matrix"), QString::null, this);
  connect (actionConvertTable, SIGNAL (activated ()), this,
	   SLOT (convertTableToMatrix ()));

  actionPlot3DWireFrame =
    new QAction (QPixmap (lineMesh_xpm), tr ("3D &Wire Frame"), QString::null,
		 this);
  connect (actionPlot3DWireFrame, SIGNAL (activated ()), this,
	   SLOT (plot3DWireframe ()));

  actionPlot3DHiddenLine =
    new QAction (QPixmap (grid_only_xpm), tr ("3D &Hidden Line"),
		 QString::null, this);
  connect (actionPlot3DHiddenLine, SIGNAL (activated ()), this,
	   SLOT (plot3DHiddenLine ()));

  actionPlot3DPolygons =
    new QAction (QPixmap (no_grid_xpm), tr ("3D &Polygons"), QString::null,
		 this);
  connect (actionPlot3DPolygons, SIGNAL (activated ()), this,
	   SLOT (plot3DPolygons ()));

  actionPlot3DWireSurface =
    new QAction (QPixmap (grid_poly_xpm), tr ("3D Wire &Surface"),
		 QString::null, this);
  connect (actionPlot3DWireSurface, SIGNAL (activated ()), this,
	   SLOT (plot3DWireSurface ()));

  actionColorMap =
    new QAction (QPixmap (color_map_xpm), tr ("Contour - &Color Fill"),
		 QString::null, this);
  connect (actionColorMap, SIGNAL (activated ()), this,
	   SLOT (plotColorMap ()));

  actionContourMap =
    new QAction (QPixmap (contour_map_xpm), tr ("Contour &Lines"),
		 QString::null, this);
  connect (actionContourMap, SIGNAL (activated ()), this,
	   SLOT (plotContour ()));

  actionGrayMap =
    new QAction (QPixmap (gray_map_xpm), tr ("&Gray Scale Map"),
		 QString::null, this);
  connect (actionGrayMap, SIGNAL (activated ()), this,
	   SLOT (plotGrayScale ()));

  actionSortTable = new QAction (tr ("Sort Ta&ble"), QString::null, this);
  connect (actionSortTable, SIGNAL (activated ()), this,
	   SLOT (sortActiveTable ()));

  actionSortSelection =
    new QAction (tr ("Sort Columns"), QString::null, this);
  connect (actionSortSelection, SIGNAL (activated ()), this,
	   SLOT (sortSelection ()));

  actionGenerateFormula =
    new QAction (tr ("Generate Formula"), QString::null, this);
  connect (actionGenerateFormula, SIGNAL (activated ()), this,
	   SLOT (generateFormula()));

  actionNormalizeTable = new QAction (tr ("&Table"), QString::null, this);
  connect (actionNormalizeTable, SIGNAL (activated ()), this,
	   SLOT (normalizeActiveTable ()));

  actionNormalizeSelection =
    new QAction (tr ("&Columns"), QString::null, this);
  connect (actionNormalizeSelection, SIGNAL (activated ()), this,
	   SLOT (normalizeSelection ()));

  actionCorrelate = new QAction (tr ("Co&rrelate"), QString::null, this);
  connect (actionCorrelate, SIGNAL (activated ()), this, SLOT (correlate ()));

  actionConvolute = new QAction (tr ("&Convolute"), QString::null, this);
  connect (actionConvolute, SIGNAL (activated ()), this, SLOT (convolute ()));

  actionDeconvolute = new QAction (tr ("&Deconvolute"), QString::null, this);
  connect (actionDeconvolute, SIGNAL (activated ()), this,
	   SLOT (deconvolute ()));

  actionTranslateHor = new QAction (tr ("&Horizontal"), QString::null, this);
  connect (actionTranslateHor, SIGNAL (activated ()), this,
	   SLOT (translateCurveHor ()));

  actionTranslateVert = new QAction (tr ("&Vertical"), QString::null, this);
  connect (actionTranslateVert, SIGNAL (activated ()), this,
	   SLOT (translateCurveVert ()));

  actionSubtractLine = new QAction(tr("&Subtract Line"), QString::null, this);
  connect (actionSubtractLine, SIGNAL (activated ()), this,
	   SLOT (subtractLine()));

  actionSubtractBack = new QAction(tr("&Subtract Background"),
				   QString::null, this);
  connect (actionSubtractBack, SIGNAL (activated ()),
	   this, SLOT (showSubtractDialog()));

  actionRemoveSpike = new QAction(tr("&Remove Spike Noise"),
				   QString::null, this);
  connect (actionRemoveSpike, SIGNAL (activated ()),
	   this, SLOT (showRemoveSpikeDialog()));

  actionCurveCalc = new QAction(tr("&Two Curve Operation"),
				   QString::null, this);
  connect (actionCurveCalc, SIGNAL (activated ()),
	   this, SLOT (showCurveCalcDialog()));

  actionSetAscValues =
    new QAction (QPixmap (rowNumbers_xpm), tr ("Ro&w Numbers"), QString::null,
		 this);
  connect (actionSetAscValues, SIGNAL (activated ()), this,
	   SLOT (setAscValues ()));

  actionSetRandomValues =
    new QAction (QPixmap (randomNumbers_xpm), tr ("&Random values"),
		 QString::null, this);
  connect (actionSetRandomValues, SIGNAL (activated ()), this,
	   SLOT (setRandomValues ()));

  actionSetXCol = new QAction (tr ("&X"), QString::null, this);
  connect (actionSetXCol, SIGNAL (activated ()), this, SLOT (setXCol ()));

  actionSetYCol = new QAction (tr ("&Y"), QString::null, this);
  connect (actionSetYCol, SIGNAL (activated ()), this, SLOT (setYCol ()));

  actionSetZCol = new QAction (tr ("&Z"), QString::null, this);
  connect (actionSetZCol, SIGNAL (activated ()), this, SLOT (setZCol ()));

  actionSetXErrCol =
    new QAction (QPixmap(xerrors_xpm), tr ("X E&rror"), QString::null, this);
  connect (actionSetXErrCol, SIGNAL (activated ()), this,
	   SLOT (setXErrCol ()));

  actionSetYErrCol =
    new QAction (QPixmap (errors_xpm), tr ("Y &Error"), QString::null, this);
  connect (actionSetYErrCol, SIGNAL (activated ()), this,
	   SLOT (setYErrCol ()));

  actionDisregardCol = new QAction (tr ("&None"), QString::null, this);
  connect (actionDisregardCol, SIGNAL (activated ()), this,
	   SLOT (disregardCol ()));

  actionBoxPlot =
    new QAction (QPixmap (boxPlot_xpm), tr ("&Box Plot"), QString::null,
		 this);
  connect (actionBoxPlot, SIGNAL (activated ()), this,
	   SLOT (plotBoxDiagram ()));

  actionMultiPeakGauss =
    new QAction (tr ("&Gaussian"), QString::null, this);
  connect (actionMultiPeakGauss, SIGNAL (activated ()), this,
	   SLOT (fitMultiPeakGauss ()));

  actionMultiPeakLorentz =
    new QAction (tr ("&Lorentzian"), QString::null, this);
  connect (actionMultiPeakLorentz, SIGNAL (activated ()), this,
	   SLOT (fitMultiPeakLorentz ()));

  /*
    actionCheckUpdates =
    new QAction (tr ("Search for &Updates"), QString::null, this);
  connect (actionCheckUpdates, SIGNAL (activated ()), this,
  	   SLOT (searchForUpdates ()));
  */

  actionHomePage =
    new QAction (tr ("&QtiPlot Homepage"), QString::null, this);
  connect (actionHomePage, SIGNAL (activated ()), this,
	   SLOT (showHomePage ()));

  /*
  actionDownloadManual =
    new QAction (tr ("Download &manual"), QString::null, this);
  connect (actionDownloadManual, SIGNAL (activated ()), this,
	   SLOT (downloadManual ()));

  actionTranslations =
    new QAction (tr ("&Translations"), QString::null, this);
  connect (actionTranslations, SIGNAL (activated ()), this,
	   SLOT (downloadTranslation ()));
  */

  actionDonate = new QAction (tr ("Make a &donation"), QString::null, this);
  connect (actionDonate, SIGNAL (activated ()), this,
	   SLOT (showDonationsPage ()));

  /*
  actionTechnicalSupport =
    new QAction (tr ("Technical &support"), QString::null, this);
  connect (actionTechnicalSupport, SIGNAL (activated ()), this,
	   SLOT (showSupportPage ()));
  */

#ifdef SCRIPTING_DIALOG
  actionScriptingLang =
    new QAction (tr ("Scripting &language"), QString::null, this);
  connect (actionScriptingLang, SIGNAL (activated ()), this,
	   SLOT (showScriptingLangDialog ()));
#endif

  actionRestartScripting =
    new QAction (tr ("&Restart scripting"), QString::null, this);
  connect (actionRestartScripting, SIGNAL (activated ()), this,
	   SLOT (restartScriptingEnv ()));

  actionNoteExecute = new QAction (tr ("E&xecute"), tr ("Ctrl+J"), this);

  actionNoteExecuteAll =
    new QAction (tr ("Execute &All"), tr ("Ctrl+Shift+J"), this);

  actionNoteEvaluate =
    new QAction (tr ("&Evaluate Expression"), tr ("Ctrl+Return"), this);

#ifdef SCRIPTING_PYTHON
  actionShowScriptWindow =
    new QAction (QPixmap (python_xpm), tr ("Script Window"), tr ("F3"), this);
  actionShowScriptWindow->setToggleAction (true);
  actionShowScriptWindow->setOn (false);
  connect (actionShowScriptWindow, SIGNAL (activated ()), this,
	   SLOT (showScriptWindow ()));
#endif
}

void
ApplicationWindow::translateActionsStrings ()
{
  actionNewProject->setMenuText (tr ("New &Project"));
  actionNewProject->setToolTip (tr ("Open a new project"));
  actionNewProject->setAccel (tr ("Ctrl+N"));

  actionNewGraph->setMenuText (tr ("New &Graph"));
  actionNewGraph->setToolTip (tr ("Create an empty 2D plot"));
  actionNewGraph->setAccel (tr ("Ctrl+G"));

  actionNewNote->setMenuText (tr ("New &Note"));
  actionNewNote->setToolTip (tr ("Create an empty note window"));

  actionNewTable->setMenuText (tr ("New &Table"));
  actionNewTable->setAccel (tr ("Ctrl+T"));
  actionNewTable->setToolTip (tr ("New Table"));

  actionNewMatrix->setMenuText (tr ("New &Matrix"));
  actionNewMatrix->setAccel (tr ("Ctrl+M"));
  actionNewMatrix->setToolTip (tr ("New matrix"));

  actionNewFunctionPlot->setMenuText (tr ("New &Function Plot"));
  actionNewFunctionPlot->setToolTip (tr ("Create a new 2D function plot"));
  actionNewFunctionPlot->setAccel (tr ("Ctrl+F"));

  actionNewSurfacePlot->setMenuText (tr ("New &Surface 3D Plot"));
  actionNewSurfacePlot->setToolTip (tr ("Create a new 3D surface plot"));
  actionNewSurfacePlot->setAccel (tr ("Ctrl+ALT+Z"));

  actionOpen->setMenuText (tr ("&Open"));
  actionOpen->setAccel (tr ("Ctrl+O"));
  actionOpen->setToolTip (tr ("Open project"));

  actionLoadImage->setMenuText (tr ("Open image &file"));
  actionLoadImage->setAccel (tr ("Ctrl+I"));

  actionImportImage->setMenuText (tr ("Import i&mage"));

  actionSaveProject->setMenuText (tr ("&Save Project"));
  actionSaveProject->setToolTip (tr ("Save Project"));
  actionSaveProject->setAccel (tr ("Ctrl+S"));

  actionSaveProjectAs->setMenuText (tr ("Save Project &as"));

  actionOpenTemplate->setMenuText (tr ("Open Te&mplate"));
  actionOpenTemplate->setToolTip (tr ("Open Template"));

  actionSaveTemplate->setMenuText (tr ("Save as &Template"));
  actionSaveTemplate->setToolTip (tr ("Save Window as Template"));

  actionLoad->setMenuText (tr ("&Single file"));
  actionLoad->setToolTip (tr ("Import data file"));
  actionLoad->setAccel (tr ("Ctrl+K"));

  actionLoadMultiple->setMenuText (tr ("&Multiple files"));
  actionLoadMultiple->setToolTip (tr ("Import multiple data files"));
  actionLoadMultiple->setAccel (tr ("Ctrl+Alt+K"));

  actionUndo->setMenuText (tr ("&Undo"));
  actionUndo->setToolTip (tr ("Undo changes"));
  actionUndo->setAccel (tr ("Ctrl+Z"));

  actionRedo->setMenuText (tr ("&Redo"));
  actionRedo->setToolTip (tr ("Redo changes"));
  actionRedo->setAccel (tr ("Ctrl+R"));

  actionCopyWindow->setMenuText (tr ("&Duplicate"));
  actionCopyWindow->setToolTip (tr ("Duplicate window"));

  actionCutSelection->setMenuText (tr ("Cu&t selection"));
  actionCutSelection->setToolTip (tr ("Cut selection"));
  actionCutSelection->setAccel (tr ("Ctrl+X"));

  actionCopySelection->setMenuText (tr ("&Copy selection"));
  actionCopySelection->setToolTip (tr ("Copy selection"));
  actionCopySelection->setAccel (tr ("Ctrl+C"));

  actionPasteSelection->setMenuText (tr ("&Paste selection"));
  actionPasteSelection->setToolTip (tr ("Paste selection"));
  actionPasteSelection->setAccel (tr ("Ctrl+V"));

  actionClearSelection->setMenuText (tr ("&Delete selection"));
  actionClearSelection->setToolTip (tr ("Delete selection"));
  actionClearSelection->setAccel (tr ("Key_Delete"));

  actionShowExplorer->setMenuText (tr ("Project &explorer"));
  actionShowExplorer->setAccel (tr ("Ctrl+E"));
  actionShowExplorer->setToolTip (tr ("Show project explorer"));

  actionShowLog->setMenuText (tr ("Results &Log"));
  actionShowLog->setToolTip (tr ("Show calculus results"));

#ifdef SCRIPTING_CONSOLE
  actionShowConsole->setMenuText (tr ("&Console"));
  actionShowConsole->setToolTip (tr ("Show Scripting console"));
#endif

#ifdef SCRIPTING_PYTHON
  actionShowScriptWindow->setMenuText (tr ("Script Window"));
  actionShowScriptWindow->setToolTip (tr ("Script Window"));
  actionShowScriptWindow->setAccel (tr ("F3"));
#endif

  actionAddLayer->setMenuText (tr ("Add La&yer"));
  actionAddLayer->setToolTip (tr ("Add Layer"));
  actionAddLayer->setAccel (tr ("ALT+L"));

  actionShowLayerDialog->setMenuText (tr ("Arran&ge Layers"));
  actionShowLayerDialog->setToolTip (tr ("Arrange Layers"));
  actionShowLayerDialog->setAccel (tr ("ALT+A"));

  actionShowLayerOrderDialog->setMenuText (tr ("Reorder Layers"));
  actionShowLayerOrderDialog->setToolTip (tr ("Reorder Layers"));

  actionAutomaticLayout->setMenuText (tr ("Automatic Layout"));
  actionAutomaticLayout->setToolTip (tr ("Automatic Layout"));

  actionExportGraph->setMenuText (tr ("&Current"));
  actionExportGraph->setAccel (tr ("Alt+G"));
  actionExportGraph->setToolTip (tr ("Export current graph"));

  actionExportAllGraphs->setMenuText (tr ("&All"));
  actionExportAllGraphs->setAccel (tr ("Alt+X"));
  actionExportAllGraphs->setToolTip (tr ("Export all graphs"));

  actionPrint->setMenuText (tr ("&Print"));
  actionPrint->setAccel (tr ("Ctrl+P"));
  actionPrint->setToolTip (tr ("Print window"));

  actionPrintAllPlots->setMenuText (tr ("Print All Plo&ts"));
  actionShowExportASCIIDialog->setMenuText (tr ("E&xport ASCII"));
  actionShowImportDialog->setMenuText (tr ("Set import &options"));

  actionCloseAllWindows->setMenuText (tr ("&Quit"));
  actionCloseAllWindows->setAccel (tr ("Ctrl+Q"));

  actionClearLogInfo->setMenuText (tr ("Clear &log information"));
  actionDeleteUselessTables->setMenuText (tr ("Delete useless tables"));

  actionShowPlotWizard->setMenuText (tr ("Plot &wizard"));
  actionShowPlotWizard->setAccel (tr ("Ctrl+Alt+W"));

  actionShowConfigureDialog->setMenuText (tr ("&Preferences"));

  actionShowCurvesDialog->setMenuText (tr ("Add/Remove &Curve"));
  actionShowCurvesDialog->setAccel (tr ("ALT+C"));
  actionShowCurvesDialog->setToolTip (tr ("Add curve to graph"));

  actionAddErrorBars->setMenuText (tr ("Add &Error Bars"));
  actionAddErrorBars->setToolTip (tr ("Add Error Bars"));
  actionAddErrorBars->setAccel (tr ("Ctrl+B"));

  actionAddFunctionCurve->setMenuText (tr ("Add &Function"));
  actionAddFunctionCurve->setToolTip (tr ("Add Function"));
  actionAddFunctionCurve->setAccel (tr ("Ctrl+Alt+F"));

  actionUnzoom->setMenuText (tr ("&Rescale to show all"));
  actionUnzoom->setAccel (tr ("Ctrl+Shift+R"));
  actionUnzoom->setToolTip (tr ("Best fit"));

  actionNewLegend->setMenuText (tr ("New &Legend"));
  actionNewLegend->setAccel (tr ("Ctrl+L"));
  actionNewLegend->setToolTip (tr ("Add new legend"));

  actionTimeStamp->setMenuText (tr ("Add time stamp"));
  actionTimeStamp->setAccel (tr ("Ctrl+ALT+T"));
  actionTimeStamp->setToolTip (tr ("Date & Time "));

  actionAddImage->setMenuText (tr ("Add &Image"));
  actionAddImage->setToolTip (tr ("Add Image"));
  actionAddImage->setAccel (tr ("ALT+I"));

  actionPlotL->setMenuText (tr ("&Line"));
  actionPlotL->setToolTip (tr ("Plot with line"));

  actionPlotP->setMenuText (tr ("&Scatter"));
  actionPlotP->setToolTip (tr ("Plot with symbols"));

  actionPlotLP->setMenuText (tr ("Line + S&ymbol"));
  actionPlotLP->setToolTip (tr ("Plot with line + symbols"));

  actionPlotVerticalDropLines->setMenuText (tr ("Vertical &drop lines"));

  actionPlotSpline->setMenuText (tr ("&Spline"));
  actionPlotVertSteps->setMenuText (tr ("&Vertical Steps"));
  actionPlotHorSteps->setMenuText (tr ("&Horizontal Steps"));

  actionPlotVerticalBars->setMenuText (tr ("&Columns"));
  actionPlotVerticalBars->setToolTip (tr ("Plot with vertical bars"));

  actionPlotHorizontalBars->setMenuText (tr ("&Rows"));
  actionPlotHorizontalBars->setToolTip (tr ("Plot with horizontal bars"));

  actionPlotArea->setMenuText (tr ("&Area"));
  actionPlotArea->setToolTip (tr ("Plot area"));

  actionPlotPie->setMenuText (tr ("&Pie"));
  actionPlotPie->setToolTip (tr ("Plot pie"));

  actionPlotVectXYXY->setMenuText (tr ("&Vectors XYXY"));
  actionPlotVectXYXY->setToolTip (tr ("Vectors XYXY"));

  actionPlotVectXYAM->setMenuText (tr ("Vectors XY&AM"));
  actionPlotVectXYAM->setToolTip (tr ("Vectors XYAM"));

  actionPlotHistogram->setMenuText (tr ("&Histogram"));
  actionPlotStackedHistograms->setMenuText (tr ("&Stacked Histogram"));
  actionPlot2VerticalLayers->setMenuText (tr ("&Vertical 2 Layers"));
  actionPlot2HorizontalLayers->setMenuText (tr ("&Horizontal 2 Layers"));
  actionPlot4Layers->setMenuText (tr ("&4 Layers"));
  actionPlotStackedLayers->setMenuText (tr ("&Stacked Layers"));

  actionPlot3DRibbon->setMenuText (tr ("&Ribbon"));
  actionPlot3DRibbon->setToolTip (tr ("Plot 3D Ribbon"));

  actionPlot3DBars->setMenuText (tr ("&Bars"));
  actionPlot3DBars->setToolTip (tr ("Plot 3D Bars"));

  actionPlot3DScatter->setMenuText (tr ("&Scatter"));
  actionPlot3DScatter->setToolTip (tr ("Plot 3D Scatter"));

  actionPlot3DTrajectory->setMenuText (tr ("&Trajectory"));
  actionPlot3DTrajectory->setToolTip (tr ("Plot 3D Trajectory"));

  actionColorMap->setMenuText (tr ("Contour + &Color Fill"));
  actionColorMap->setToolTip (tr ("Contour Lines + Color Fill"));

  actionContourMap->setMenuText (tr ("Contour &Lines"));
  actionContourMap->setToolTip (tr ("Contour Lines"));

  actionGrayMap->setMenuText (tr ("&Gray Scale Map"));
  actionGrayMap->setToolTip (tr ("Gray Scale Map"));

  actionShowColStatistics->setMenuText (tr ("Statistics on &Columns"));
  actionShowColStatistics->setToolTip (tr ("Selected columns statistics"));

  actionShowXYStatistics->setMenuText (tr ("XY Statistics on &Column"));
  actionShowXYStatistics->setToolTip (tr ("Selected Y column statistics on X column"));

  actionShowRowStatistics->setMenuText (tr ("Statistics on &Rows"));
  actionShowRowStatistics->setToolTip (tr ("Selected rows statistics"));
  actionShowIntDialog->setMenuText (tr ("&Integrate"));
  actionShowFwhmDialog->setMenuText (tr ("&FWHM"));
  actionShowMomentDialog->setMenuText (tr ("&Moment Analysis"));
  actionInterpolate->setMenuText (tr ("Inte&rpolate"));
  actionLowPassFilter->setMenuText (tr ("&Low Pass"));
  actionHighPassFilter->setMenuText (tr ("&High Pass"));
  actionBandPassFilter->setMenuText (tr ("&Band Pass"));
  actionBandBlockFilter->setMenuText (tr ("&Band Block"));
  actionBandInterpFilter->setMenuText (tr ("&Band Block (interpolate)"));
  actionFFT->setMenuText (tr ("&FFT"));
  actionConvoluteCurves->setMenuText (tr ("&Convolute"));
  actionSmoothSavGol->setMenuText (tr ("&Savitzky-Golay"));
  actionSmoothAverage->setMenuText (tr ("Moving Window &Average"));
  actionGaussConv->setMenuText (tr ("&Gaussian Convolution"));
  actionLorentzConv->setMenuText (tr ("&Lorentzian Convolution"));
  actionDifferentiate->setMenuText (tr ("&Differentiate"));
  actionFitLinear->setMenuText (tr ("Fit &Linear"));
  actionShowFitPolynomDialog->setMenuText (tr ("Fit &Polynomial"));
  actionShowSineFitDialog->setMenuText (tr ("Fit Sinusoidal"));
  actionShowExpDecayDialog->setMenuText (tr ("&First Order"));
  actionShowTwoExpDecayDialog->setMenuText (tr ("&Second Order"));
  actionShowExpDecay3Dialog->setMenuText (tr ("&Third Order"));
  actionFitExpGrowth->setMenuText (tr ("Fit Exponential Gro&wth"));
  actionFitSigmoidal->setMenuText (tr ("Fit &Boltzmann (Sigmoidal)"));
  actionFitGauss->setMenuText (tr ("Fit &Gaussian"));
  actionFitLorentz->setMenuText (tr ("Fit Lorent&zian"));

  actionShowFitDialog->setMenuText (tr ("&Non-linear Curve Fit"));
  actionShowFitDialog->setAccel (tr ("Ctrl+Y"));

  actionShowPlotDialog->setMenuText (tr ("&Plot"));
  actionShowScaleDialog->setMenuText (tr ("&Scales"));
  actionShowCurveFormatDialog->setMenuText (tr ("&Curves"));
  actionShowAxisDialog->setMenuText (tr ("&Axes"));
  actionShowGridDialog->setMenuText (tr ("&Grid"));
  actionShowTitleDialog->setMenuText (tr ("&Title"));
  actionShowColumnOptionsDialog->setMenuText (tr ("Column &Options"));
  actionShowColumnOptionsDialog->setAccel (tr ("Ctrl+Alt+O"));

  actionShowColumnValuesDialog->setMenuText (tr ("Set Column &Values"));
  actionShowColumnValuesDialog->setAccel (tr ("Alt+Q"));
  actionTableRecalculate->setMenuText (tr ("Recalculate"));

  actionShowColsDialog->setMenuText (tr ("&Columns"));
  actionShowRowsDialog->setMenuText (tr ("&Rows"));

  actionAbout->setMenuText (tr ("&About QtiPlot"));
  actionAbout->setAccel (tr ("F1"));

  actionShowHelp->setMenuText (tr ("&Help"));
  actionShowHelp->setAccel (tr ("Ctrl+H"));

  actionChooseHelpFolder->setMenuText (tr ("&Choose help folder"));
  actionRename->setMenuText (tr ("&Rename Window"));

  actionCloseWindow->setMenuText (tr ("Close &Window"));
  actionCloseWindow->setAccel (tr ("Ctrl+W"));

  actionAddColToTable->setMenuText (tr ("Add column"));
  actionAddColToTable->setToolTip (tr ("Add column"));

  actionAddRowToTable->setMenuText (tr ("Add row"));
  actionAddRowToTable->setToolTip (tr ("Add row"));

  actionClearTable->setMenuText (tr ("Clear"));
  actionGoToRow->setMenuText (tr ("&Go to Row"));
  actionGoToRow->setAccel (tr ("Ctrl+Alt+G"));

  actionDeleteLayer->setMenuText (tr ("&Remove Layer"));
  actionDeleteLayer->setAccel (tr ("Alt+R"));

  actionResizeLayer->setMenuText (tr ("Resize Layer"));

  actionResizeActiveWindow->setMenuText (tr ("Window &Geometry"));
  actionHideActiveWindow->setMenuText (tr ("&Hide Window"));
  actionShowMoreWindows->setMenuText (tr ("More windows"));
  actionPixelLineProfile->setMenuText (tr ("&View pixel line profile"));
  actionIntensityTable->setMenuText (tr ("&Intensity table"));
  actionShowLineDialog->setMenuText (tr ("&Properties"));
  actionShowImageDialog->setMenuText (tr ("&Properties"));
  actionShowTextDialog->setMenuText (tr ("&Properties"));
  actionActivateWindow->setMenuText (tr ("&Activate Window"));
  actionMinimizeWindow->setMenuText (tr ("Mi&nimize Window"));
  actionMaximizeWindow->setMenuText (tr ("Ma&ximize Window"));
  actionHideWindow->setMenuText (tr ("&Hide Window"));
  actionShowWindow->setMenuText (tr ("&Show Window"));
  actionResizeWindow->setMenuText (tr ("Re&size Window"));
  actionPrintWindow->setMenuText (tr ("&Print Window"));
  actionShowPlotGeometryDialog->setMenuText (tr ("&Layer geometry"));
  actionEditSurfacePlot->setMenuText (tr ("&Surface"));
  actionAdd3DData->setMenuText (tr ("&Data Set"));
  actionSetMatrixProperties->setMenuText (tr ("Set &Properties"));
  actionSetMatrixDimensions->setMenuText (tr ("Set &Dimensions"));
  actionSetMatrixValues->setMenuText (tr ("Set &Values"));
  actionTransposeMatrix->setMenuText (tr ("&Transpose"));
  actionInvertMatrix->setMenuText (tr ("&Invert"));
  actionMatrixDeterminant->setMenuText (tr ("&Determinant"));
  actionMatrixInterpolate->setMenuText (tr ("Interpolate"));
  actionMatrixFFT->setMenuText(tr("FFT"));
  actionMatrixFilter->setMenuText(tr("FFT Filter"));
  actionConvertMatrix->setMenuText (tr ("&Convert to spreadsheet"));
  actionConvertTable->setMenuText (tr ("Convert to &matrix"));
  actionPlot3DWireFrame->setMenuText (tr ("3D &Wire Frame"));
  actionPlot3DHiddenLine->setMenuText (tr ("3D &Hidden Line"));
  actionPlot3DPolygons->setMenuText (tr ("3D &Polygons"));
  actionPlot3DWireSurface->setMenuText (tr ("3D Wire &Surface"));
  actionSortTable->setMenuText (tr ("Sort Ta&ble"));
  actionSortSelection->setMenuText (tr ("Sort Columns"));
  actionNormalizeTable->setMenuText (tr ("&Table"));
  actionNormalizeSelection->setMenuText (tr ("&Columns"));
  actionCorrelate->setMenuText (tr ("Co&rrelate"));
  actionConvolute->setMenuText (tr ("&Convolute"));
  actionDeconvolute->setMenuText (tr ("&Deconvolute"));
  actionTranslateHor->setMenuText (tr ("&Horizontal"));
  actionTranslateVert->setMenuText (tr ("&Vertical"));
  actionSetAscValues->setMenuText (tr ("Ro&w Numbers"));
  actionSetRandomValues->setMenuText (tr ("&Random values"));
  actionSetXCol->setMenuText (tr ("&X"));
  actionSetYCol->setMenuText (tr ("&Y"));
  actionSetZCol->setMenuText (tr ("&Z"));
  actionSetXErrCol->setMenuText (tr ("X E&rror"));
  actionSetYErrCol->setMenuText (tr ("Y &Error"));
  actionDisregardCol->setMenuText (tr ("&None"));

  actionBoxPlot->setMenuText (tr ("&Box Plot"));
  actionBoxPlot->setToolTip (tr ("Box and Whiskers Plot"));

  actionMultiPeakGauss->setMenuText (tr ("&Gaussian"));
  actionMultiPeakLorentz->setMenuText (tr ("&Lorentzian"));
  actionHomePage->setMenuText (tr ("&QtiPlot Homepage"));
  //actionCheckUpdates->setMenuText (tr ("Search for &Updates"));
  //actionDownloadManual->setMenuText (tr ("Download &manual"));
  //actionTranslations->setMenuText (tr ("&Translations"));
  actionDonate->setMenuText (tr ("Make a &donation"));
  //actionTechnicalSupport->setMenuText (tr ("Technical &support"));
#ifdef SCRIPTING_DIALOG
  actionScriptingLang->setMenuText (tr ("Scripting &language"));
#endif
  actionRestartScripting->setMenuText (tr ("&Restart scripting"));

  actionNoteExecute->setMenuText (tr ("E&xecute"));
  actionNoteExecute->setAccel (tr ("Ctrl+J"));

  actionNoteExecuteAll->setMenuText (tr ("Execute &All"));
  actionNoteExecuteAll->setAccel (tr ("Ctrl+Shift+J"));

  actionNoteEvaluate->setMenuText (tr ("&Evaluate Expression"));
  actionNoteEvaluate->setAccel (tr ("Ctrl+Return"));

  btnPointer->setMenuText (tr ("Disable &tools"));
  btnPointer->setToolTip (tr ("Pointer"));

  btnZoomIn->setMenuText (tr ("&Zoom In"));
  btnZoomIn->setAccel (tr ("Ctrl++"));
  btnZoomIn->setToolTip (tr ("Zoom In"));

  btnZoomOut->setMenuText (tr ("Zoom &Out"));
  btnZoomOut->setAccel (tr ("Ctrl+-"));
  btnZoomOut->setToolTip (tr ("Zoom Out"));

  btnCursor->setMenuText (tr ("&Data reader"));
  btnCursor->setAccel (tr ("CTRL+D"));
  btnCursor->setToolTip (tr ("Data reader"));

  btnSelect->setMenuText (tr ("&Select data range"));
  btnSelect->setAccel (tr ("ALT+S"));
  btnSelect->setToolTip (tr ("Select data range"));

  btnPicker->setMenuText (tr ("S&creen reader"));
  btnPicker->setToolTip (tr ("Screen reader"));

  //btnSubback->setMenuText(tr("S&ubtract line"));
  //btnSubback->setToolTip(tr("Subtract line"));

  btnMovePoints->setMenuText (tr ("&Move Data Points"));
  btnMovePoints->setAccel (tr ("Ctrl+ALT+M"));
  btnMovePoints->setToolTip (tr ("Move data points"));

  btnRemovePoints->setMenuText (tr ("Remove &Bad Data Points"));
  btnRemovePoints->setAccel (tr ("Alt+B"));
  btnRemovePoints->setToolTip (tr ("Remove data points"));

  actionAddText->setMenuText (tr ("Add &Text"));
  actionAddText->setToolTip (tr ("Add Text"));
  actionAddText->setAccel (tr ("ALT+T"));

  btnArrow->setMenuText (tr ("Draw &Arrow"));
  btnArrow->setAccel (tr ("CTRL+ALT+A"));
  btnArrow->setToolTip (tr ("Draw arrow"));

  btnLine->setMenuText (tr ("Draw &Line"));
  btnLine->setAccel (tr ("CTRL+ALT+L"));
  btnLine->setToolTip (tr ("Draw line"));

  coord->setText (tr ("Coordinates"));
  coord->setMenuText (tr ("&Coord"));
  coord->setStatusTip (tr ("Coordinates"));
  Box->setText (tr ("Box"));
  Box->setMenuText (tr ("Box"));
  Box->setToolTip (tr ("Box"));
  Box->setStatusTip (tr ("Box"));
  Frame->setText (tr ("Frame"));
  Frame->setMenuText (tr ("&Frame"));
  Frame->setToolTip (tr ("Frame"));
  Frame->setStatusTip (tr ("Frame"));
  None->setText (tr ("No Axes"));
  None->setMenuText (tr ("No Axes"));
  None->setToolTip (tr ("No Axes"));
  None->setStatusTip (tr ("No Axes"));

  grids->setText (tr ("grid"));
  grids->setMenuText (tr ("grid"));
  grids->setStatusTip (tr ("grid"));
  front->setToolTip (tr ("Front Grid"));
  back->setToolTip (tr ("Back Grid"));
  right->setToolTip (tr ("Right Grid"));
  left->setToolTip (tr ("Left Grid"));
  ceil->setToolTip (tr ("Ceiling Grid"));
  floor->setToolTip (tr ("Floor Grid"));

  plotstyle->setText (tr ("Plot Style"));
  plotstyle->setMenuText (tr ("Plot Style"));
  plotstyle->setStatusTip (tr ("Plot Style"));
  wireframe->setText (tr ("Wireframe"));
  wireframe->setMenuText (tr ("Wireframe"));
  wireframe->setToolTip (tr ("Wireframe"));
  wireframe->setStatusTip (tr ("Wireframe"));
  hiddenline->setText (tr ("Hidden Line"));
  hiddenline->setMenuText (tr ("Hidden Line"));
  hiddenline->setToolTip (tr ("Hidden Line"));
  hiddenline->setStatusTip (tr ("Hidden Line"));
  polygon->setText (tr ("Polygon only"));
  polygon->setMenuText (tr ("Polygon only"));
  polygon->setToolTip (tr ("Polygon only"));
  polygon->setStatusTip (tr ("Polygon only"));
  filledmesh->setText (tr ("Mesh & filled Polygons"));
  filledmesh->setMenuText (tr ("Mesh & filled Polygons"));
  filledmesh->setToolTip (tr ("Mesh & filled Polygons"));
  filledmesh->setStatusTip (tr ("Mesh & filled Polygons"));
  pointstyle->setText (tr ("Dots"));
  pointstyle->setMenuText (tr ("Dots"));
  pointstyle->setToolTip (tr ("Dots"));
  pointstyle->setStatusTip (tr ("Dots"));
  barstyle->setText (tr ("Bars"));
  barstyle->setMenuText (tr ("Bars"));
  barstyle->setToolTip (tr ("Bars"));
  barstyle->setStatusTip (tr ("Bars"));
  conestyle->setText (tr ("Cones"));
  conestyle->setMenuText (tr ("Cones"));
  conestyle->setToolTip (tr ("Cones"));
  conestyle->setStatusTip (tr ("Cones"));
  crossHairStyle->setText (tr ("Cross Hairs"));
  crossHairStyle->setMenuText (tr ("Cross Hairs"));
  crossHairStyle->setToolTip (tr ("Cross Hairs"));
  crossHairStyle->setStatusTip (tr ("Cross Hairs"));

  floorstyle->setText (tr ("Floor Style"));
  floorstyle->setMenuText (tr ("Floor Style"));
  floorstyle->setStatusTip (tr ("Floor Style"));
  floordata->setText (tr ("Floor Data Projection"));
  floordata->setMenuText (tr ("Floor Data Projection"));
  floordata->setToolTip (tr ("Floor Data Projection"));
  floordata->setStatusTip (tr ("Floor Data Projection"));
  flooriso->setText (tr ("Floor Isolines"));
  flooriso->setMenuText (tr ("Floor Isolines"));
  flooriso->setToolTip (tr ("Floor Isolines"));
  flooriso->setStatusTip (tr ("Floor Isolines"));
  floornone->setText (tr ("Empty Floor"));
  floornone->setMenuText (tr ("Empty Floor"));
  floornone->setToolTip (tr ("Empty Floor"));
  floornone->setStatusTip (tr ("Empty Floor"));

  actionAnimate->setText (tr ("Animation"));
  actionAnimate->setMenuText (tr ("Animation"));
  actionAnimate->setToolTip (tr ("Animation"));
  actionAnimate->setStatusTip (tr ("Animation"));

  actionPerspective->setText (tr ("Enable perspective"));
  actionPerspective->setMenuText (tr ("Enable perspective"));
  actionPerspective->setToolTip (tr ("Enable perspective"));
  actionPerspective->setStatusTip (tr ("Enable perspective"));

  actionResetRotation->setText (tr ("Reset rotation"));
  actionResetRotation->setMenuText (tr ("Reset rotation"));
  actionResetRotation->setToolTip (tr ("Reset rotation"));
  actionResetRotation->setStatusTip (tr ("Reset rotation"));

  actionFitFrame->setText (tr ("Fit frame to window"));
  actionFitFrame->setMenuText (tr ("Fit frame to window"));
  actionFitFrame->setToolTip (tr ("Fit frame to window"));
  actionFitFrame->setStatusTip (tr ("Fit frame to window"));
}

Graph3D *
ApplicationWindow::openMatrixPlot3D (const QString & caption,
				     const QString & matrix_name, double xl,
				     double xr, double yl, double yr,
				     double zl, double zr, int scaletype)
{
  QString name = matrix_name;
  name.remove ("matrix<", true);
  name.remove (">");
  Matrix *m = matrix (name);
  if (!m)
    return 0;

  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  plot->blockSignals(true);
  //plot->setCaption (caption);
  plot->setName (caption);
  plot->addMatrixData (m, xl, xr, yl, yr, zl, zr, scaletype);
  plot->update ();
  //
  plot->resize(500,450);
  //
  initPlot3D (plot);
  plot->blockSignals(false);
  return plot;
}

void
ApplicationWindow::plot3DMatrix (int style)
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Matrix"))
    return;

  QApplication::setOverrideCursor (waitCursor);
  QString label = generateUniqueName (tr ("Graph"));

  Graph3D *plot = new Graph3D ("", ws, 0, WDestructiveClose);
  customPlot3D (plot);

  plot->addMatrixData ((Matrix *) ws->activeWindow ());
  plot->customPlotStyle (style);
  plot->update ();

  plot->resize (500, 450);
  //plot->setCaption (label);
  plot->setName (label);
  initPlot3D (plot);

  emit modified ();
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::plotGrayScale ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Matrix"))
    return;

  plotSpectrogram ((Matrix *) ws->activeWindow (), Graph::GrayMap);
}

MultiLayer *
ApplicationWindow::plotGrayScale (Matrix * m)
{
  if (!m)
    return 0;

  return plotSpectrogram (m, Graph::GrayMap);
}

void
ApplicationWindow::plotContour ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Matrix"))
    return;

  plotSpectrogram ((Matrix *) ws->activeWindow (), Graph::ContourMap);
}

MultiLayer *
ApplicationWindow::plotContour (Matrix * m)
{
  if (!m)
    return 0;

  return plotSpectrogram (m, Graph::ContourMap);
}

void
ApplicationWindow::plotColorMap ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Matrix"))
    return;

  plotSpectrogram ((Matrix *) ws->activeWindow (), Graph::ColorMap);
}

MultiLayer *
ApplicationWindow::plotColorMap (Matrix * m)
{
  if (!m)
    return 0;

  return plotSpectrogram (m, Graph::ColorMap);
}

MultiLayer *
ApplicationWindow::plotSpectrogram (Matrix * m, Graph::CurveType type)
{
  bool fmax=false;
  if(aw&&(aw->status()==myWidget::Maximized)){
    fmax=true;
    aw->showNormal();
  }

  QApplication::setOverrideCursor (waitCursor);

  MultiLayer *g = multilayerPlot (generateUniqueName (tr ("Graph")));
  Graph *plot = g->addLayer ();
  customGraph (plot);

  plot->plotSpectrogram (m, type);

  g->showNormal ();

  plot->setIgnoreResizeEvents (!autoResizeLayers);

  if(fmax) g->showMaximized();

  emit modified ();
  QApplication::restoreOverrideCursor ();
  return g;
}

ApplicationWindow *
ApplicationWindow::importOPJ (const QString & filename)
{
  QApplication::setOverrideCursor (waitCursor);

  ApplicationWindow *app = new ApplicationWindow ();
  app->applyUserSettings ();
  app->setCaption ("QtiPlot - " + filename);
  app->showMaximized ();
  app->projectname = filename;
  app->recentProjects.remove (filename);
  app->recentProjects.push_front (filename);
  app->updateRecentProjectsList ();

  ImportOPJ (app, filename);

  QApplication::restoreOverrideCursor ();
  return app;
}

void
ApplicationWindow::deleteTableIfUnused(myWidget *w)
{
  if(!w->isA("Table")) return;

  QApplication::setOverrideCursor (waitCursor);

  QString name = w->getName();
  QWidgetList *windows = windowsList ();
  for (QWidget * w1 = windows->first (); w1; w1 = windows->next ())
    {
      if (w1->isA ("MultiLayer"))
	{
	  QWidgetList *lst = ((MultiLayer *) w1)->graphPtrs ();
	  for (Graph *g = (Graph *)lst->first(); g; g = (Graph *)lst->next())
	    {
	      QStringList associations = g->plotAssociations ();
	      for (int i = 0; i < int (associations.count ()); i++)
		{
		  QString ass = associations[i];
		  if (ass.startsWith(name+"_"))
		    {
		      delete windows;
		      QApplication::restoreOverrideCursor ();
		      return;
		    }
		}
	    }
	}
      else if (w1->isA ("Graph3D"))
	{
	  if ((((Graph3D *) w1)->formula()).startsWith(name+"_"))
	    {
	      delete windows;
	      QApplication::restoreOverrideCursor ();
	      return;
	    }
	}
    }
  delete windows;
  w->askOnCloseEvent (false);
  w->close ();
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::deleteUselessTables()
{
  QWidgetList *windows = windowsList ();
  for (QWidget *w = windows->first (); w; w = windows->next ())
    {
      QString caption = ((myWidget *)w)->getName();
      if (w->isA ("Table") && (caption.startsWith(tr("Normalized"))
			       || caption.startsWith(tr("Aligned"))
			       || caption.startsWith(tr("Cascade"))
			       || caption.startsWith(tr("Smoothed"))
			       || caption.startsWith(tr("Derivative"))
			       || caption.startsWith(tr("Calculated"))
			       || caption.startsWith(tr("Convoluted"))
			       || caption.startsWith(tr("Deconvoluted"))
			       || caption.startsWith(tr("FilteredFFT"))
			       ))
	{
	  deleteTableIfUnused((myWidget *) w );
	}
      /*
        if(w->isA("Table"))
        {
          QString caption = ((myWidget *)w)->getName();
  	  for(int i=0;i<(int)uselessTableList.count();i++)
          {
            if(caption.startsWith( uselessTableList[i] ))
              deleteTableIfUnused((myWidget *) w);
          }
        }
       */
    }
  delete windows;
}


QWidgetList *
ApplicationWindow::windowsList ()
{
  QWidgetList *lst = new QWidgetList;

  QWidgetList windows = ws->windowList (QWorkspace::StackingOrder);
  QWidget *w;
  for(w = windows.first(); w; w = windows.next())
    lst->append(w);

  for(w = outWindows->first(); w; w = outWindows->next())
    lst->append(w);

  return lst;
}

void
ApplicationWindow::updateRecentProjectsList ()
{
  while ((int) recentProjects.size () > MaxRecentProjects)
    recentProjects.pop_back ();

  recentMenu->clear ();

  for (int i = 0; i < (int) recentProjects.size (); i++)
    recentMenu->insertItem ("&" + QString::number (i + 1) + " " +
			recentProjects[i]);
}

void
ApplicationWindow::translateCurveHor ()
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) w;
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      btnPointer->setOn (true);
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (!g)
    return;

  if (g->isPiePlot ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("This functionality is not available for pie plots!"));

      btnPointer->setOn (true);
      return;
    }
  else if (g->validCurvesDataSize ())
    {
      activeGraph = g;
      btnPointer->setOn (true);
      g->translateCurve (0);
      info->setText (tr ("Double-click on plot to select a data point!"));
      showDisplayBar();
    }
}

void
ApplicationWindow::translateCurveVert ()
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) w;
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      btnPointer->setOn (true);
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (!g)
    return;

  if (g->isPiePlot ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("This functionality is not available for pie plots!"));

      btnPointer->setOn (true);
      return;
    }
  else if (g->validCurvesDataSize ())
    {
      activeGraph = g;
      btnPointer->setOn (true);
      g->translateCurve (1);
      info->setText (tr ("Double-click on plot to select a data point!"));
      showDisplayBar();
    }
}

void
ApplicationWindow::setAscValues ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  ((Table *) ws->activeWindow ())->setAscValues ();
}

void
ApplicationWindow::setRandomValues ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Table"))
    return;

  ((Table *) ws->activeWindow ())->setRandomValues ();
}

void
ApplicationWindow::setXErrCol ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->setPlotDesignation (Table::xErr);
}

void
ApplicationWindow::setYErrCol ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->setPlotDesignation (Table::yErr);
}

void
ApplicationWindow::setXCol ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->setPlotDesignation (Table::X);
}

void
ApplicationWindow::setYCol ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->setPlotDesignation (Table::Y);
}

void
ApplicationWindow::setZCol ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->setPlotDesignation (Table::Z);
}

void
ApplicationWindow::disregardCol ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->setPlotDesignation (Table::None);
}

void
ApplicationWindow::plotBoxDiagram ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->inherits ("Table"))
    return;

  ((Table *) ws->activeWindow ())->plotBoxDiagram ();
}

void
ApplicationWindow::fitMultiPeakGauss ()
{
  fitMultiPeak ((int) MultiPeakFit::Gauss);
}

void
ApplicationWindow::fitMultiPeakLorentz ()
{
  fitMultiPeak ((int) MultiPeakFit::Lorentz);
}

void
ApplicationWindow::fitMultiPeak (int profile)
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA ("MultiLayer"))
    return;

  MultiLayer *plot = (MultiLayer *) w;
  if (plot->isEmpty ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("<h4>There are no plot layers available in this window.</h4>"
			     "<p><h4>Please add a layer and try again!</h4>"));
      btnPointer->setOn (true);
      return;
    }

  Graph *g = (Graph *) plot->activeGraph ();
  if (!g || !g->validCurvesDataSize ())
    return;

  if (g->isPiePlot ())
    {
      QMessageBox::warning (this, tr ("QtiPlot - Warning"),
			    tr
			    ("This functionality is not available for pie plots!"));
      return;
    }
  else
    {
      activeGraph = g;
      bool ok;
      int peaks =
	QInputDialog::getInteger (tr ("QtiPlot - Enter the number of peaks"),
				  tr ("Peaks"), 2, 2, 1000000, 1, &ok, this);
      if (ok && peaks)
	{
	  g->multiPeakFit (this, (int) profile, peaks);
	  info->
	    setText (tr
		     ("Move cursor and click to select a point and double-click/press 'Enter' to set the position of a peak!"));
	  showDisplayBar();
	  connect (g, SIGNAL (showFitResults (const QString &)), this,
		   SLOT (showResults (const QString &)));
	}
    }
}

/*
void
ApplicationWindow::showSupportPage ()
{
  open_browser (this, "http://soft.proindependent.com/contracts.html");
}
*/

void
ApplicationWindow::showDonationsPage ()
{
  open_browser (this, "http://soft.proindependent.com/why_donate.html");
}

/*
void
ApplicationWindow::downloadManual ()
{
  open_browser (this, "http://soft.proindependent.com/manuals.html");
}

void
ApplicationWindow::downloadTranslation ()
{
  open_browser (this, "http://soft.proindependent.com/translations.html");
}
*/

void
ApplicationWindow::showHomePage ()
{
  open_browser (this, "http://soft.proindependent.com/qtiplot.html");
}

bool
ApplicationWindow::open_browser (QWidget * parent, const QString & rUrl)
{
  bool result = true;
  QApplication::setOverrideCursor (Qt::WaitCursor);
#ifdef Q_WS_WIN
  HINSTANCE status =::ShellExecute( parent->winId(), NULL, ( TCHAR * ) qt_winTchar( rUrl, true ), NULL, NULL, SW_SHOW );
  if (32 >= (int)status) {     
    QApplication::restoreOverrideCursor ();
    QMessageBox::information (this, "QtiPlot - Error",
			      tr("Sorry, QtiPlot couldn't start the default browser! Please start a browser manually and visit the following link")
			      + ":\n" + rUrl);
    result = false;
  }

#else
  Q_UNUSED (parent);
  //Try a range of browsers available on UNIX, until we (hopefully) find one that works.  
  //Start with the most popular first.
  QProcess process;
  bool process_started = false;
  process.setArguments (QStringList () << "firefox" << rUrl);
  process_started = process.start ();
  if (!process_started)
    {
      process.setArguments (QStringList () << "mozilla" << rUrl);
      process_started = process.start ();
    }
  if (!process_started)
    {
      process.setArguments (QStringList () << "netscape" << rUrl);
      process_started = process.start ();
    }
  if (!process_started)
    {
      process.setArguments (QStringList () << "konqueror" << rUrl);
      process_started = process.start ();
    }
  result = process_started;
#endif

  QApplication::restoreOverrideCursor ();
  return result;
}

void
ApplicationWindow::showDonationDialog ()
{
  if (askForSupport)
    {
      QString s =
	tr
	("<font size=+2, color = darkBlue><b>QtiPlot is open-source software and its development required hundreds of hours of work.<br><br>\
				   If you like it, you're using it in your work and you would like to see it \
				   constantly improved,<br> please support its author by making a donation.<br><br> \
				   Would you like to make a donation for QtiPlot now?</b></font>");
      switch (QMessageBox::
	      information (this, tr ("Please support QtiPlot!"), s,
			   tr ("Yes, I'd love to!"),
			   tr ("Ask me again later!"),
			   tr ("No, stop bothering me!"), 0, 1))
	{
	case 0:
	  showDonationsPage ();
	  break;

	case 1:
	  break;

	case 2:
	  askForSupport = false;
	  break;
	}
    }
}

void
ApplicationWindow::parseCommandLineArgument (const QString & s, int args)
{
  if (s == "-v" || s == "--version")
    {
      initGlobalConstants ();
      about ();
      exit (0);
    }
  else if (s == "-h" || s == "--help")
    {
      ApplicationWindow *aux = new ApplicationWindow ();
      aux->hideActiveWindow ();
      aux->showStandAloneHelp ();
      delete aux;
      exit (0);
    }
  else if (s.contains ("-lang=") || s.contains ("-l="))
    {
      if (args == 1)
	{
	  init ();
	  applyUserSettings ();
	}

      QString locale = s.mid (s.find ('=') + 1);
      if (locales.contains (locale))
	switchToLanguage (locale);

      if (args == 1)
	{
	  newTable ();
	  setCaption (tr ("QtiPlot - untitled"));
	  showMaximized ();
	  saved = TRUE;
	  actionSaveProject->setEnabled (false);
	}

      if (!locales.contains (locale))
	QMessageBox::critical (0, tr ("QtiPlot - Error"),
			       tr
			       ("<b> %1 </b>: Wrong locale option or no translation available!").
			       arg (locale));
    }
  else
    {
      ApplicationWindow *aux = new ApplicationWindow ();
      aux->hideActiveWindow ();
      QMessageBox::critical (0, tr ("QtiPlot - Error"),
			     tr
			     ("<b> %1 </b>: Unknown command line option or the file doesn't exist!").
			     arg (s));
      delete aux;
      exit (1);
    }
}

void
ApplicationWindow::createLanguagesList ()
{
  appTranslator = new QTranslator (this);
  qtTranslator = new QTranslator (this);
  qApp->installTranslator (appTranslator);
  qApp->installTranslator (qtTranslator);

  QString qmPath = qApp->applicationDirPath () + "/translations";
  QDir dir (qmPath);
  QStringList fileNames = dir.entryList ("qtiplot_*.qm");
  for (int i = 0; i < (int) fileNames.size (); i++)
    {
      QString locale = fileNames[i];
      locale = locale.mid (locale.find ('_') + 1);
      locale.truncate (locale.find ('.'));
      locales.push_back (locale);
    }
  locales.push_back ("en");
  locales.sort ();

  if (appLanguage != "en")
    {
      appTranslator->load ("qtiplot_" + appLanguage, qmPath);
      qtTranslator->load ("qt_" + appLanguage, qmPath + "/qt");
    }
}

void
ApplicationWindow::switchToLanguage (int param)
{
  if (param < (int) locales.size ())
    switchToLanguage (locales[param]);
}

void
ApplicationWindow::switchToLanguage (const QString & locale)
{
  if (!locales.contains (locale) || appLanguage == locale)
    return;

  appLanguage = locale;
  if (locale == "en")
    {
      appTranslator->clear ();
      qtTranslator->clear ();
    }
  else
    {
      QString qmPath = qApp->applicationDirPath () + "/translations";
      appTranslator->load ("qtiplot_" + locale, qmPath);
      qtTranslator->load ("qt_" + locale, qmPath + "/qt");
    }
  insertTranslatedStrings ();
}

QStringList
ApplicationWindow::matrixNames ()
{
  QStringList names;
  QWidgetList *windows = windowsList ();

  for (QWidget *w = windows->first (); w; w = windows->next ())
    if (((myWidget *)w)->isA ("Matrix"))
      names << ((myWidget *)w)->getName();

  delete windows;
  return names;
}

bool
ApplicationWindow::alreadyUsedName (const QString & label)
{
  QWidgetList *windows = windowsList ();
  
  for (QWidget *w = windows->first (); w; w = windows->next ())
    if (((myWidget *)w)->getName() == label)
      {
	delete windows;
	return true;
      }

  delete windows;
  return false;
}

bool
ApplicationWindow::projectHasMatrices ()
{
  QWidgetList *windows = windowsList ();
  bool has = false;
  for (QWidget * w = windows->first (); w; w = windows->next ())
    if((w->isA ("Matrix"))&&(w!=window_to_close))
      {
        has = true;
        break;
      }
  
  delete windows;
  return has;
}

bool
ApplicationWindow::projectHas2DPlots ()
{
  QWidgetList *windows = windowsList ();
  bool hasPlots = false;
  for (QWidget *w = windows->first (); w; w = windows->next ())
    if((w->isA ("MultiLayer"))&&(w!=window_to_close))
      {
	hasPlots = true;
	break;
      }
  
  delete windows;
  return hasPlots;
}

bool
ApplicationWindow::projectHas3DPlots ()
{
  QWidgetList *windows = windowsList ();
  bool hasPlots = false;
  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      if((w->isA ("Graph3D"))&&(w!=window_to_close))
	{
	  hasPlots = true;
	  break;
	}
    }
  delete windows;
  return hasPlots;
}

void
ApplicationWindow::appendProject ()
{
  QString filter = tr ("QtiPlot project") + " (*.qti);;";
  filter += tr ("Compressed QtiPlot project") + " (*.qti.gz);;";
  filter += tr ("Origin project") + " (*.opj *.OPJ);;";

  QString fn = QFileDialog::getOpenFileName (workingDir, filter, this, 0,
					     tr ("QtiPlot - Open Project"), 0,
					     TRUE);

  if (fn.isEmpty ())
    return;

  QFileInfo fi (fn);
  workingDir = fi.dirPath (true);

  if (fn.contains (QRegExp("\\.(qti|qti\\.gz|opj|OPJ)~*$"))) 
    {
      QFileInfo f (fn);
      if (!f.exists ())
	{
	  QMessageBox::critical (this, tr ("QtiPlot - File openning error"),
				 tr ("The file: <b>%1</b> doesn't exist!").
				 arg (fn));
	  return;
	}
    }
  else
    {
      QMessageBox::critical (this, tr ("QtiPlot - File openning error"),
			     tr
			     ("The file: <b>%1</b> is not a QtiPlot or Origin project file!").
			     arg (fn));
      return;
    }

  QApplication::setOverrideCursor (waitCursor);

  if (fn.endsWith (".opj", FALSE))
    {
      QFileInfo fi (fn);
      QString baseName = fi.baseName (true);
      FolderListItem *item = (FolderListItem *) current_folder->folderListItem ();
      QStringList lst = current_folder->subfolders ();
      int n = (int) lst.contains (baseName);
      if (n)
	{                           //avoid identical subfolder names
	  while ((int) lst.contains (baseName + QString::number (n)))
	    n++;
	  baseName += QString::number (n);
	}
      current_folder = new Folder (current_folder, baseName);
      FolderListItem *fli = new FolderListItem (item, current_folder);
      current_folder->setFolderListItem (fli);

      ImportOPJ ( this, fn );
    }
  else
    insertProject( fn, true );

  emit modified();
  
  QApplication::restoreOverrideCursor ();
}

void
ApplicationWindow::saveSubFolder(FolderListItem *item, int level, QIODevice *device)
{
  QTextStream stream(device);
  stream.setEncoding(QTextStream::UnicodeUTF8);

  Folder *folder = item->folder();
  QString text="";
  if(level>0)
    {
      text += "<folder>\t" + folder->folderName () + "\t"
	+ folder->birthDate () + "\t" + folder->modificationDate ();
      if (folder == current_folder)
	text += "\tcurrent\n";
      else
	text += "\n";
    }
  stream << text;

  QPtrList < myWidget > lst = folder->windowsList ();
  myWidget *w;
  for (w = lst.first (); w; w = lst.next ())
    stream << w->saveToString (windowGeometryInfo (w));

  FolderListItem *child = (FolderListItem *) item->firstChild ();
  while (child)
    {
      saveSubFolder( child, level+1, device );
      child = (FolderListItem *)child->nextSibling();
    }
  
  if(level>0)
    stream << "</folder>\n";
}

void
ApplicationWindow::saveFolder (Folder * folder, const QString & fn)
{
  QFile f (fn);
  if (f.exists ())
    {// make byte-copy of current file so that there's always a copy of the data on disk
      QFile backup (fn + "~");
      while (!f.open (IO_ReadOnly) || !backup.open (IO_WriteOnly))
	{
	  if (f.isOpen ())
	    f.close ();
	  if (backup.isOpen ())
	    backup.close ();
	  int choice =
	    QMessageBox::warning (this, tr ("QtiPlot - File Backup Error"),
				  tr
				  ("Cannot make a backup copy of <b>%1</b> (to %2).<br>If you ignore this, you run the risk of <b>data loss</b>.").
				  arg (fn).arg (fn + "~"),
				  QMessageBox::Retry | QMessageBox::Default,
				  QMessageBox::Abort | QMessageBox::Escape,
				  QMessageBox::Ignore);
	  if (choice == QMessageBox::Abort)
	    return;
	  if (choice == QMessageBox::Ignore)
	    break;
	}

      if (f.isOpen () && backup.isOpen ())
	{
	  while (!f.atEnd ())
	    backup.putch (f.getch ());

	  backup.close ();
	  f.close ();
	}
    }

  // compressed file should be generated in temporary folder
  // and then copyed to file.qt.gz
  QString fname;
  bool compress=false;
  if(fn.contains(QRegExp("\\.gz~*$")))
    {
      compress=true;
#ifdef Q_OS_WIN
      fname = getenv("TEMP");
      fname.append("\\dummy.qti");
#else
      fname = "/tmp/dummy.qti";
#endif
    }
  else
    fname = fn;

  QFile f2(fname);
  if (!f2.open (IO_WriteOnly))
    {
      QMessageBox::about (this, tr ("QtiPlot - File Save Error"),
			  tr
			  ("The file: <br><b>%1</b> is opened in read-only mode").
			  arg (fname));
      return;
    }
  QApplication::setOverrideCursor (waitCursor);

  QTextStream t (&f2);
  t.setEncoding (QTextStream::UnicodeUTF8);

  t << "QtiPlot " + QString::number (majVersion) + "." +
    QString::number (minVersion) + "." +
    QString::number (patchVersion) + " project file\n";
  t << "<scripting-lang>\t" + QString (scriptEnv->name ()) + "\n";
  t << "<windows>\t" + QString::number (folder->windowCount(true)) + "\n";

  FolderListItem *item = folder->folderListItem ();
  saveSubFolder( item, 0, &f2 ); 
  
  t<< "<log>\n" + logInfo + "</log>";

  f2.close ();

  QApplication::restoreOverrideCursor ();

  if(compress)
    {
      FILE *in = fopen( fname.local8Bit(),"rb" );
      gzFile out = gzopen( fn.local8Bit(),"wb" );
      gz_compress( in, out );
    }

  QString fn2=fn;
  if(compress) fn2.remove(".gz");
  else fn2.append(".gz");

  if(QFile::exists(fn2)&&
     QMessageBox::question (this, tr ("QtiPlot -- Delete File? "),
			    tr
			    ("A file called: <p><b>%1</b><p> exists.\n"
			     "Do you want to delete it?").arg (fn2),
			    tr("&Yes"),tr("&No"),QString::null,0,1)==0)
    QFile::remove(fn2);
}

void
ApplicationWindow::saveAsProject ()
{
  saveFolderAsProject (current_folder);
}

void
ApplicationWindow::saveFolderAsProject (Folder * f)
{
  QString filter = tr ("QtiPlot project") + " (*.qti);;";
  filter += tr ("Compressed QtiPlot project") + " (*.qti.gz)";

  QString selectedFilter;
  QString fn =
    QFileDialog::getSaveFileName (workingDir, filter, this, "project",
				  tr ("Save Project As"), &selectedFilter,
				  false);
  if (!fn.isEmpty ())
    {
      QFileInfo fi (fn);
      workingDir = fi.dirPath (true);
      //QString filename = fi.fileName ();
      if(!fn.contains(QRegExp("\\.(qti|qti\\.gz)$")))
	fn.append(".qti");
      if(selectedFilter.contains(".gz")&&!fn.endsWith(".gz",true))
	fn.append(".gz");

      if (QFile::exists (fn) &&
	  QMessageBox::question (this, tr ("QtiPlot -- Overwrite File? "),
				 tr
				 ("A file called: <p><b>%1</b><p>already exists.\n"
				  "Do you want to overwrite it?").arg (fn),
				 tr ("&Yes"), tr ("&No"), QString::null, 0,
				 1))
	return;
      else
	saveFolder (f, fn);
    }
}

void
ApplicationWindow::showFolderPopupMenu (QListViewItem * it, const QPoint & p,
					int)
{
  showFolderPopupMenu (it, p, true);
}

//! fromFolders = TRUE means the user clicked right mouse buttom on a list iten from QListView "folderview"
void
ApplicationWindow::showFolderPopupMenu (QListViewItem * it, const QPoint & p,
					bool fromFolders)
{
  // qDebug("showFolderPopupMenu");

  if (!it || folderview->isRenaming ())
    return;

  QPopupMenu cm (this);
  QPopupMenu window (this);
  QPopupMenu viewWindowsMenu (this);
  viewWindowsMenu.setCheckable (true);

  cm.insertItem (tr ("&Find"), this, SLOT (showFindDialogue ()));
  cm.insertSeparator ();
  cm.insertItem (tr ("App&end Project"), this, SLOT (appendProject ()));
  if (((FolderListItem *) it)->folder ()->parent ())
    cm.insertItem (tr ("Save &As Project"), this, SLOT (saveAsProject ()));
  else
    cm.insertItem (tr ("Save Project &As"), this, SLOT (saveProjectAs ()));
  cm.insertSeparator ();

  if (fromFolders && show_windows_policy != HideAll)
    {
      cm.insertItem (tr ("&Show All Windows"), this,
		     SLOT (showAllFolderWindows ()));
      cm.insertItem (tr ("&Hide All Windows"), this,
		     SLOT (hideAllFolderWindows ()));
      cm.insertSeparator ();
    }

  if (((FolderListItem *) it)->folder ()->parent ())
    {
      cm.insertItem (QPixmap (close_xpm), tr ("&Delete Folder"), this,
		     SLOT (deleteFolder ()), Key_F8);
      cm.insertItem (tr ("&Rename"), this, SLOT (startRenameFolder ()),
		     Key_F2);
      cm.insertSeparator ();
    }

  if (fromFolders)
    {
      actionNewTable->addTo (&window);
      actionNewMatrix->addTo (&window);
      actionNewNote->addTo (&window);
      actionNewGraph->addTo (&window);
      actionNewFunctionPlot->addTo (&window);
      actionNewSurfacePlot->addTo (&window);
      cm.insertItem (tr ("New &Window"), &window);
    }

  cm.insertItem (QPixmap (newfolder_xpm), tr ("New Sub-F&older"), this,
		 SLOT (addSubFolder ()), Key_F7);
  cm.insertItem (QPixmap (newfolder_xpm), tr ("New Sibling-Folder"), this,
		 SLOT (addSiblingFolder ()));
  cm.insertSeparator ();

  QStringList lst;
  lst << tr ("&None") << tr ("&Windows in Active Folder") <<
    tr ("Windows in &Active Folder && Subfolders");
  for (int i = 0; i < 3; ++i)
    {
      int id =
	viewWindowsMenu.insertItem (lst[i], this,
				    SLOT (setShowWindowsPolicy (int)));
      viewWindowsMenu.setItemParameter (id, i);
      viewWindowsMenu.setItemChecked (id, show_windows_policy == i);
    }
  cm.insertItem (tr ("&View Windows"), &viewWindowsMenu);
  cm.insertSeparator ();
  cm.insertItem (tr ("&Properties"), this, SLOT (folderProperties ()));
  cm.exec (p);
}

void
ApplicationWindow::setShowWindowsPolicy (int p)
{
  if (show_windows_policy == (ShowWindowsPolicy) p)
    return;

  show_windows_policy = (ShowWindowsPolicy) p;
  if (show_windows_policy == HideAll)
    {
      QWidgetList *lst = windowsList ();
      for (QWidget *w = lst->first (); w; w = lst->next ())
	{
	  hiddenWindows->append (w);
	  w->hide ();
	  setListView (((myWidget *)w)->getName(), tr ("Hidden"));
	}
      delete lst;
    }
  else
    showAllFolderWindows ();
}

void
ApplicationWindow::showFindDialogue ()
{
  findDialog *fd =
    new findDialog (this, 0, TRUE, WStyle_Tool | WDestructiveClose);
  fd->showNormal ();
  fd->setActiveWindow ();
}

void
ApplicationWindow::listViewItemRenamed(QListViewItem * item)
{
  // qDebug("listViewItemRenamed");

  if(item->rtti () == FolderListItem::RTTI)
    startRenameFolder(item);
  else
    renameWindow(item);
}

void
ApplicationWindow::startRenameFolder ()
{
  // qDebug("startRenameFolder 1");

  FolderListItem *fi = current_folder->folderListItem ();
  if (!fi)
    return;

  disconnect (folderview, SIGNAL (currentChanged (QListViewItem *)), this,
	      SLOT (folderItemChanged (QListViewItem *)));
  folderview->setSelectionMode(QListView::NoSelection);
  fi->setRenameEnabled (0, true);
  fi->startRename (0);
}

void
ApplicationWindow::startRenameFolder (QListViewItem * item)
{
  //qDebug("startRenameFolder 2");

  if (!item || item == folderview->firstChild ())
    return;

  disconnect (folderview, SIGNAL (currentChanged (QListViewItem *)), this,
	      SLOT (folderItemChanged (QListViewItem *)));

  folderview->setSelectionMode(QListView::NoSelection);

  if (item->listView () == listview && item->rtti () == FolderListItem::RTTI)
    {
      current_folder = ((FolderListItem *) item)->folder ();
      FolderListItem *it = current_folder->folderListItem ();
      it->setRenameEnabled (0, true);
      it->startRename (0);
    }
  else
    {
      item->setRenameEnabled (0, true);
      item->startRename (0);
    }
}

void
ApplicationWindow::renameFolder (QListViewItem * it, int,
				 const QString & text)
{
  // qDebug("renameFolder");

  if (!it)
    return;
  //
  Folder *folder = ((FolderListItem *)it)->folder();
  Folder *parent = (Folder *) folder->parent();

  if (!parent)			//the parent folder is the project folder (it always exists)
    parent = projectFolder ();

  while (text.isEmpty ())
    {
      QMessageBox::critical (this, tr ("QtiPlot - Error"),
			     tr ("Please enter a valid name!"));
      it->setRenameEnabled (0, true);
      it->startRename (0);
      return;
    }

  QStringList lst = parent->subfolders ();
  lst.remove (folder->folderName ());
  while (lst.contains (text))
    {
      QMessageBox::critical (this, tr ("QtiPlot - Error"),
			     tr ("Name already exists!") + "\n" +
			     tr ("Please choose another name!"));

      it->setRenameEnabled (0, true);
      it->startRename (0);
      return;
    }

  folder->setFolderName (text);
  it->setRenameEnabled (0, false);
  connect (folderview, SIGNAL (currentChanged (QListViewItem *)),
	   this, SLOT (folderItemChanged (QListViewItem *)));
  folderview->setSelectionMode(QListView::Single);
  folderview->setCurrentItem (current_folder->folderListItem ());
  
  emit modified();
}

void
ApplicationWindow::showAllFolderWindows ()
{
  hideFolderWindows();

  QPtrList < myWidget > lst = current_folder->windowsList ();
  myWidget *w, *maxed=0;
  for (w = lst.first (); w; w = lst.next ())
    {				//force show all windows in current folder
      if (w)
	{
	  updateWindowLists (w);
	  switch (w->status ())
	    {
	    case myWidget::Hidden:
	      w->showNormal ();
	      break;

	    case myWidget::Normal:
	      w->showNormal ();
	      break;

	    case myWidget::Minimized:
	      w->showMinimized ();
	      break;

	    case myWidget::Maximized:
	      if(show_windows_policy==SubFolders)
		w->showNormal();
	      else
		maxed = w;
	      // maximized window should be shown last
	      break;
	    }
	}
    }

  if (!current_folder->children ()){
    if(maxed) maxed->showMaximized();
    return;
  }

  FolderListItem *fi = current_folder->folderListItem ();
  FolderListItem *item = (FolderListItem *) fi->firstChild ();
  int initial_depth = item->depth ();
  while (item && item->depth () >= initial_depth)
    {				// show/hide windows in all subfolders
      lst = ((Folder *) item->folder ())->windowsList ();
      for (w = lst.first (); w; w = lst.next ())
	{
	  if (w && show_windows_policy == SubFolders)
	    {
	      updateWindowLists (w);
	      switch (w->status ())
		{
		case myWidget::Hidden:
		  w->showNormal ();
		  break;

		case myWidget::Normal:
		  w->showNormal ();
		  break;

		case myWidget::Minimized:
		  w->showMinimized ();
		  break;

		case myWidget::Maximized:
		  w->showNormal (); 
		  // subfolder windows should not be maximized
		  break;
		}
	    }
	  else
	    w->hide ();
	}

      item = (FolderListItem *) item->itemBelow ();
    }

  if(maxed) maxed->showMaximized();
}

void
ApplicationWindow::hideAllFolderWindows ()
{
  QPtrList < myWidget > lst = current_folder->windowsList ();
  myWidget *w;
  for (w = lst.first (); w; w = lst.next ())
    hideWindow (w);

  if (!current_folder->children ())
    return;

  if (show_windows_policy == SubFolders)
    {
      FolderListItem *fi = current_folder->folderListItem ();
      FolderListItem *item = (FolderListItem *) fi->firstChild ();
      int initial_depth = item->depth ();
      while (item && item->depth () >= initial_depth)
	{
	  lst = item->folder ()->windowsList ();
	  for (w = lst.first (); w; w = lst.next ())
	    hideWindow (w);

	  item = (FolderListItem *) item->itemBelow ();
	}
    }
}

void
ApplicationWindow::minimizeAllFolderWindows ()
{
  QPtrList < myWidget > lst = current_folder->windowsList ();
  myWidget *w;
  for (w = lst.first (); w; w = lst.next ())
    w->showMinimized();

  if (!current_folder->children ())
    return;

  if (show_windows_policy == SubFolders)
    {
      FolderListItem *fi = current_folder->folderListItem ();
      FolderListItem *item = (FolderListItem *) fi->firstChild ();
      int initial_depth = item->depth ();
      while (item && item->depth () >= initial_depth)
	{
	  lst = item->folder ()->windowsList ();
	  for (w = lst.first (); w; w = lst.next ())
	    w->showMinimized();

	  item = (FolderListItem *) item->itemBelow ();
	}
    }
}

void
ApplicationWindow::projectProperties ()
{
  QString s = current_folder->folderName () + "\n\n";
  s += "\n\n\n";
  s += tr ("Type") + ": " + tr ("Project") + "\n\n";
  if (projectname != "untitled")
    {
      s += tr ("Path") + ": " + projectname + "\n\n";

      QFileInfo fi (projectname);
      s +=
	tr ("Size") + ": " + QString::number (fi.size ()) + " " +
	tr ("bytes") + "\n\n";
    }

  QWidgetList *lst = windowsList ();
  s +=
    tr ("Contents") + ": " + QString::number (lst->count ()) + " " +
    tr ("Windows");
  delete lst;

  s +=
    ", " + QString::number (current_folder->subfolders ().count ()) + " " +
    tr ("Folders") + "\n\n";
  s += "\n\n\n";

  if (projectname != "untitled")
    {
      QFileInfo fi (projectname);
      s +=
	tr ("Created") + ": " + fi.created ().toString (Qt::LocalDate) +
	"\n\n";
      s +=
	tr ("Modified") + ": " + fi.lastModified ().toString (Qt::LocalDate) +
	"\n\n";
    }
  else
    s += tr ("Created") + ": " + current_folder->birthDate () + "\n\n";

  QMessageBox *mbox =
    new QMessageBox (tr ("Properties"), s, QMessageBox::NoIcon,
		     QMessageBox::Ok, QMessageBox::NoButton,
		     QMessageBox::NoButton, this);

  mbox->setIconPixmap (QPixmap (qtiplot_logo_xpm));
  mbox->show ();
}

void
ApplicationWindow::folderProperties ()
{
  if (!current_folder->parent ())
    {
      projectProperties ();
      return;
    }

  QString s = current_folder->folderName () + "\n\n";
  s += "\n\n\n";
  s += tr ("Type") + ": " + tr ("Folder") + "\n\n";
  s += tr ("Path") + ": " + current_folder->path () + "\n\n";
  s += tr ("Size") + ": " + current_folder->sizeToString () + "\n\n";
  s +=
    tr ("Contents") + ": " +
    QString::number (current_folder->windowsList ().count ()) + " " +
    tr ("Windows");
  s +=
    ", " + QString::number (current_folder->subfolders ().count ()) + " " +
    tr ("Folders") + "\n\n";
//s += "\n\n\n";
  s += tr ("Created") + ": " + current_folder->birthDate () + "\n\n";
//s += tr("Modified") + ": " + current_folder->modificationDate() + "\n\n";

  QMessageBox *mbox =
    new QMessageBox (tr ("Properties"), s, QMessageBox::NoIcon,
		     QMessageBox::Ok, QMessageBox::NoButton,
		     QMessageBox::NoButton, this);

  mbox->setIconPixmap (QPixmap (folder_open_xpm));
  mbox->show ();
}

void
ApplicationWindow::addFolder(bool sibling)
{
  Folder *p_folder;
  if (sibling)
    p_folder = (Folder *)current_folder->parent();
  else
    p_folder = current_folder;

  // qDebug("addFolder");
  QStringList lst = p_folder->subfolders ();
  QString name = tr ("New Folder");
  lst = lst.grep (name);
  if (!lst.isEmpty ())
    name += " (" + QString::number (lst.size () + 1) + ")";

  Folder *f = new Folder (p_folder, name);
  addFolderListViewItem (f); // add folder to listview

  // addfolder to folder
  FolderListItem *fi =
    new FolderListItem (p_folder->folderListItem (), f);

  if (fi)
    {
      f->setFolderListItem (fi);
      disconnect (folderview, SIGNAL (currentChanged (QListViewItem *)), this,
		  SLOT (folderItemChanged (QListViewItem *)));
      folderview->setSelectionMode(QListView::NoSelection);
      fi->setRenameEnabled (0, true);
      fi->startRename (0);
    }

  //emit modified();
}

void
ApplicationWindow::addSubFolder()
{
  addFolder(false);
}

void
ApplicationWindow::addSiblingFolder()
{
  if( current_folder == projectFolder() )
    return;
  addFolder(true);
}

bool
ApplicationWindow::deleteFolder (Folder * f)
{
  if (confirmCloseFolder
      && QMessageBox::information (this, tr ("QtiPlot - Delete folder?"),
				   tr
				   ("Delete folder '%1' and all the windows it contains?").
				   arg (f->folderName ()), tr ("Yes"),
				   tr ("No"), 0, 0))
    return false;
  else
    {
      FolderListItem *fi = f->folderListItem ();
      QPtrList < myWidget > lst = f->windowsList ();
      myWidget *w;
      for (w = lst.first (); w; w = lst.next ())
	removeWindowFromLists (w);

      if (f->children ())
	{
	  FolderListItem *item = (FolderListItem *) fi->firstChild ();
	  int initial_depth = item->depth ();
	  while (item && item->depth () >= initial_depth)
	    {
	      lst = ((Folder *) item->folder ())->windowsList ();
	      for (w = lst.first (); w; w = lst.next ())
		removeWindowFromLists (w);

	      item = (FolderListItem *) item->itemBelow ();
	    }
	}

      delete f;
      delete fi;

      emit modified();
      return true;
    }
}

void
ApplicationWindow::deleteFolder ()
{
  Folder *parent = (Folder *) current_folder->parent ();
  if (!parent)
    parent = projectFolder ();

  folderview->blockSignals (true);

  if (deleteFolder (current_folder))
    {
      current_folder = parent;
      folderview->setCurrentItem (parent->folderListItem ());
      changeFolder (parent, true);
    }

  folderview->blockSignals (false);
  folderview->setFocus ();
}

void
ApplicationWindow::folderItemDoubleClicked (QListViewItem * it)
{
  if (!it || it->rtti () != FolderListItem::RTTI)
    return;

  FolderListItem *item =
    ((FolderListItem *) it)->folder ()->folderListItem ();
  folderview->setCurrentItem (item);
}

void
ApplicationWindow::folderItemChanged (QListViewItem * it)
{
  // qDebug("folderItemChanged");

  if (!it)
    return;

  Folder *newFolder = ((FolderListItem *) it)->folder ();

  changeFolder (newFolder, false);
  it->setOpen (true);

  folderview->setFocus ();
}

void
ApplicationWindow::hideFolderWindows()
{
  QPtrList < myWidget > lst = current_folder->windowsList ();
  myWidget *w,*maxed=0;
  for (w = lst.first (); w; w = lst.next ())
    {
      if(w->status()==myWidget::Maximized)
	maxed = w;
      else 
	w->hide ();
    }
  if(maxed) maxed->hide();// maximized window must be hidden last.

  aw = 0;

  if (!current_folder->children ())
    return;

  FolderListItem *fi = current_folder->folderListItem ();
  FolderListItem *item = (FolderListItem *) fi->firstChild ();
  int initial_depth;
  if(item)
    initial_depth = item->depth ();
  while (item && item->depth () >= initial_depth)
    {
      lst = item->folder ()->windowsList ();
      for (w = lst.first (); w; w = lst.next ())
	w->hide ();
      item = (FolderListItem *) item->itemBelow ();
    }
}

void
ApplicationWindow::showFolderWindows()
{
  QPtrList < myWidget > lst = current_folder->windowsList ();
  myWidget *w,*maxed=0;
  for (w = lst.first (); w; w = lst.next ())
    {//show only windows in the current folder which are not hidden by the user
      w->blockSignals(true);
      if (!hiddenWindows->containsRef (w) && !outWindows->containsRef (w)
	  && show_windows_policy != HideAll)
	{
	  switch (w->status ())
	    {
	    case myWidget::Normal:
	      w->showNormal ();
	      break;
	    case myWidget::Minimized:
	      w->showMinimized ();
	      break;
	    case myWidget::Maximized:
	      if(show_windows_policy==SubFolders)
		w->showNormal();
	      else
		maxed=w; // maximized window must be maximized last
	      break;
	    }
	}
      else
	{
	  w->setStatus (myWidget::Hidden);
	}
      w->blockSignals(false);
      //addListViewItem (w);
    }

  QObjectList *folderLst = (QObjectList *) current_folder->children ();
  if(folderLst)
    {// Folder has child folders
      FolderListItem *fi = current_folder->folderListItem ();
      FolderListItem *item = (FolderListItem *) fi->firstChild ();
      int initial_depth  = item->depth ();
 
      while (item && item->depth () >= initial_depth)
	{				//show/hide windows in subfolders
	  lst = ((Folder *) item->folder ())->windowsList ();
	  for (w = lst.first (); w; w = lst.next ())
	    {
	      w->blockSignals(true);
	      if (w && !hiddenWindows->containsRef (w)
		  && !outWindows->containsRef (w))
		{
		  if(show_windows_policy==SubFolders)
		    {
		      switch (w->status ())
			{
			case myWidget::Normal:
			  w->showNormal ();
			  break;
			case myWidget::Minimized:
			  w->showMinimized ();
			  break;
			case myWidget::Maximized:
			  w->showNormal ();
			  // subfolder windows should not be maximized
			  break;
			}
		    }
		  else if (w->isVisible ())
		    w->hide ();
		}
	      w->blockSignals(false);
	    }
	  
	  item = (FolderListItem *) item->itemBelow ();
	}
    }

  if(maxed) maxed->showMaximized();
}

void
ApplicationWindow:: refreshListView()
{
  listview->clear ();

  QObjectList *folderLst = (QObjectList *)current_folder->children ();
  if (folderLst)
    {
      Folder *f;
      for (f = (Folder *) folderLst->first (); f;
	   f = (Folder *) folderLst->next ())
	addFolderListViewItem (f);
    }

  QPtrList < myWidget > lst = current_folder->windowsList ();
  myWidget *w;
  for (w = lst.first (); w; w = lst.next ())
    addListViewItem (w);    
}

void
ApplicationWindow::changeFolder (Folder * newFolder, bool force)
{
  //qDebug("changeFolder");
  //qDebug("time = %s",QTime::currentTime().toString("ss.zzz").ascii());

  desactivateFolders ();
  newFolder->folderListItem ()->setActive (true);

  if(current_folder!=newFolder || force)
    {
      hideFolderWindows ();
      current_folder = newFolder;
      showFolderWindows ();
    }

  refreshListView();
}

void
ApplicationWindow::desactivateFolders ()
{
  FolderListItem *item = (FolderListItem *) folderview->firstChild ();
  while (item)
    {
      item->setActive (false);
      item = (FolderListItem *) item->itemBelow ();
    }
}

void
ApplicationWindow::addListViewItem (myWidget * w)
{
  if (!w)
    return;

  WindowListItem *it = new WindowListItem (listview, w);

  if (w->isA ("Matrix"))
    {
      it->setPixmap (0, QPixmap (matrix_xpm));
      it->setText (1, tr ("Matrix"));
    }
  else if (w->inherits ("Table"))
    {
      it->setPixmap (0, QPixmap (worksheet_xpm));
      it->setText (1, tr ("Table"));
    }
  else if (w->isA ("Note"))
    {
      it->setPixmap (0, QPixmap (note_xpm));
      it->setText (1, tr ("Note"));
    }
  else if (w->isA ("MultiLayer"))
    {
      it->setPixmap (0, QPixmap (graph_xpm));
      it->setText (1, tr ("Graph"));
    }
  else if (w->isA ("Graph3D"))
    {
      it->setPixmap (0, QPixmap (trajectory_xpm));
      it->setText (1, tr ("Graph 3D"));
    }

  it->setText (0, w->getName());
  if (w->isHidden ())
    it->setText (2, tr ("Hidden"));
  else
    it->setText (2, w->aspect ());

  it->setText (3, w->sizeToString ());
  it->setText (4, w->birthDate ());
  it->setText (5, w->windowLabel ());
}

void
ApplicationWindow::windowProperties ()
{
  WindowListItem *it = (WindowListItem *) listview->currentItem ();
  myWidget *w = it->window ();
  if (!w)
    return;

  QMessageBox *mbox =
    new QMessageBox (tr ("Properties"), QString::null, QMessageBox::NoIcon,
		     QMessageBox::Ok, QMessageBox::NoButton,
		     QMessageBox::NoButton, this);

  QString s = w->getName() + "\n\n";
  s += "\n\n\n";

  s += tr ("Label") + ": " + ((myWidget *) w)->windowLabel () + "\n\n";

  if (w->isA ("Matrix"))
    {
      mbox->setIconPixmap (QPixmap (matrix_xpm));
      s += tr ("Type") + ": " + tr ("Matrix") + "\n\n";
    }
  else if (w->inherits ("Table"))
    {
      mbox->setIconPixmap (QPixmap (worksheet_xpm));
      s += tr ("Type") + ": " + tr ("Table") + "\n\n";
    }
  else if (w->isA ("Note"))
    {
      mbox->setIconPixmap (QPixmap (note_xpm));
      s += tr ("Type") + ": " + tr ("Note") + "\n\n";
    }
  else if (w->isA ("MultiLayer"))
    {
      mbox->setIconPixmap (QPixmap (graph_xpm));
      s += tr ("Type") + ": " + tr ("Graph") + "\n\n";
    }
  else if (w->isA ("Graph3D"))
    {
      mbox->setIconPixmap (QPixmap (trajectory_xpm));
      s += tr ("Type") + ": " + tr ("Graph 3D") + "\n\n";
    }
  s += tr ("Path") + ": " + current_folder->path () + "\n\n";
  s += tr ("Size") + ": " + w->sizeToString () + "\n\n";
  s += tr ("Created") + ": " + w->birthDate () + "\n\n";
  s += tr ("Status") + ": " + it->text (2) + "\n\n";
  mbox->setText (s);
  mbox->show ();
}

void
ApplicationWindow::addFolderListViewItem (Folder * f)
{
  if (!f)
    return;

  FolderListItem *it = new FolderListItem (listview, f);
  it->setActive (false);
  it->setText (0, f->folderName ());
  it->setText (1, tr ("Folder"));
  it->setText (3, f->sizeToString ());
  it->setText (4, f->birthDate ());
}

void
ApplicationWindow::find (const QString & s, bool windowNames, bool labels,
			 bool folderNames, bool caseSensitive,
			 bool partialMatch, bool subfolders)
{
  if (windowNames || labels)
    {
      myWidget *w =
	current_folder->findWindow (s, windowNames, labels, caseSensitive,
				    partialMatch);
      if (w)
	{
	  activateWindow (w);
	  return;
	}

      if (subfolders)
	{
	  FolderListItem *item =
	    (FolderListItem *) folderview->currentItem ()->firstChild ();
	  while (item)
	    {
	      Folder *f = item->folder ();
	      myWidget *w =
		f->findWindow (s, windowNames, labels, caseSensitive,
			       partialMatch);
	      if (w)
		{
		  folderview->setCurrentItem (f->folderListItem ());
		  activateWindow (w);
		  return;
		}
	      item = (FolderListItem *) item->itemBelow ();
	    }
	}
    }

  if (folderNames)
    {
      Folder *f =
	current_folder->findSubfolder (s, caseSensitive, partialMatch);
      if (f)
	{
	  folderview->setCurrentItem (f->folderListItem ());
	  return;
	}

      if (subfolders)
	{
	  FolderListItem *item =
	    (FolderListItem *) folderview->currentItem ()->firstChild ();
	  while (item)
	    {
	      Folder *f =
		item->folder ()->findSubfolder (s, caseSensitive,
						partialMatch);
	      if (f)
		{
		  folderview->setCurrentItem (f->folderListItem ());
		  return;
		}

	      item = (FolderListItem *) item->itemBelow ();
	    }
	}
    }

  QMessageBox::warning (this, tr ("QtiPlot - No match found"),
			tr ("Sorry, no match found for string: '%1'").
			arg (s));
}

void
ApplicationWindow::dropFolderItems (QListViewItem * dest)
{
  //qDebug("dropFolderItems");

  if (!dest || draggedItems.isEmpty ())
    return;

  Folder *dest_f = ((FolderListItem *) dest)->folder ();

  //qDebug("dest_f = %s",dest_f->folderName().ascii());

  QListViewItem *it;
  QStringList subfolders = dest_f->subfolders ();

  for (it = draggedItems.first (); it; it = draggedItems.next ())
    {
      if (it->rtti () == FolderListItem::RTTI)
	{
	  Folder *f = ((FolderListItem *) it)->folder ();
	  //qDebug("src_f = %s",f->folderName().ascii());
	  FolderListItem *src = f->folderListItem ();
	  if (dest_f == f)
	    {
	      //QMessageBox::critical (this, "QtiPlot - Error",
	      //     tr ("Cannot move an object to itself!"));
	      return;
	    }

	  if (((FolderListItem *) dest)->isChildOf (src))
	    {
	      QMessageBox::critical (this, "QtiPlot - Error",
				     tr("Cannot move a parent folder into a child folder!"));
	      draggedItems.clear ();
	      folderview->setCurrentItem (current_folder->folderListItem ());
	      return;
	    }
	  Folder *parent = (Folder *) f->parent ();
	  if (!parent)
	    parent = projectFolder ();
	  if (dest_f == parent)
	    return;
	}
      else if (dest_f == current_folder)
	return;
    }

  bool folder_moved = false;

  for (it = draggedItems.first (); it; it = draggedItems.next ())
    {
      if (it->rtti () == FolderListItem::RTTI)
	{ // move a folder into another folder
	  Folder *f = ((FolderListItem *) it)->folder ();
	  //src must be a FolderListItem of the folderview
	  FolderListItem *src = f->folderListItem ();
	  if (subfolders.contains (f->folderName ()))
	    {
	      QMessageBox::critical (this,
				     tr ("QtiPlot") + " - " +
				     tr ("Skipped Moving Folder"),
				     tr
				     ("The destination folder already contains a folder called '%1'! Folder skipped!").
				     arg (f->folderName ()));
	    }
	  else
	    {
	      folder_moved = true;
	      //destination must be a FolderListItem of the folderview
	      moveFolder (src, dest_f->folderListItem());
	    }
	}
      else
	{//move window
	  myWidget *w = ((WindowListItem *) it)->window ();
	  if (w)
	    {
	      current_folder->removeWindow (w);
	      w->hide ();
	      dest_f->addWindow (w);
	      w->setFolder(dest_f); // added by S.Yamamoto
	      delete it;
	    }
	}
    }

  draggedItems.clear ();
  if(folder_moved)
    {
      current_folder = dest_f;
      folderview->setCurrentItem (dest_f->folderListItem ());
      changeFolder (dest_f, true);
    }
  else
    folderview->setCurrentItem (current_folder->folderListItem ());

  folderview->setFocus ();

  emit modified();
  //qDebug("end of dropFolderItems");
}


// src and dest must be the FolderListItems of the folderview (not listview)
void
ApplicationWindow::moveFolder (FolderListItem * src, FolderListItem * dest)
{
  //qDebug("moveFolder");
  //qDebug("src  name = %s",src->text(0).ascii());
  //qDebug("dest name = %s",dest->text(0).ascii());

  folderview->blockSignals (true);

  Folder *dest_f = dest->folder ();  
  Folder *src_f = src->folder ();

  dest_f = new Folder (dest_f, src_f->folderName ());
  dest_f->setBirthDate (src_f->birthDate ());
  dest_f->setModificationDate (src_f->modificationDate ());

  // dest must be FolderListItem of the folderview
  FolderListItem *new_item = new FolderListItem (dest, dest_f);
  new_item->setText (0, src_f->folderName ());
  dest_f->setFolderListItem (new_item);

  QPtrList < myWidget > lst = QPtrList < myWidget > (src_f->windowsList ());
  myWidget *w;
  for (w = lst.first (); w; w = lst.next ())
    {
      src_f->removeWindow (w);
      w->hide ();
      dest_f->addWindow (w);
      w->setFolder(dest_f);
    }

  if (src_f->children ())
    {// src folder has child folders
      FolderListItem *item = (FolderListItem *) src->firstChild ();
      FolderListItem *previous_item = new_item;
      int initial_depth = src->depth();
      int previous_depth = initial_depth;
      while (item && item->depth () > initial_depth)
	{
	  src_f = (Folder *) item->folder ();
	  int depth = item->depth();

	  FolderListItem *parent = previous_item;
	  // select parent
	  for(int i = 0; i <= previous_depth - depth; i++)
	    parent = (FolderListItem *) (parent->parent());
	  //

	  dest_f = new Folder ( parent->folder(), src_f->folderName () );
	  dest_f->setBirthDate (src_f->birthDate ());
	  dest_f->setModificationDate (src_f->modificationDate ());

	  new_item = new FolderListItem (parent, dest_f);
	  new_item->setText (0, src_f->folderName ());
	  dest_f->setFolderListItem (new_item);

	  lst = QPtrList < myWidget > (src_f->windowsList ());
	  for (w = lst.first (); w; w = lst.next ())
	    {
	      src_f->removeWindow (w);
	      w->hide ();
	      dest_f->addWindow (w);
	      w->setFolder(dest_f);
	    }

	  previous_item = new_item;
	  previous_depth = item->depth();
	  item = (FolderListItem *) item->itemBelow ();
	}
    }

  src_f = src->folder ();
  delete src_f;
  delete src;
  folderview->blockSignals (false);
  //qDebug("end of moveFolder");
}

/*
void
ApplicationWindow::searchForUpdates ()
{
  version_buffer.open (IO_WriteOnly);

  http.setHost ("soft.proindependent.com");
  http.get ("/version.txt", &version_buffer);
  http.closeConnection ();
}
*/

/*
void
ApplicationWindow::receivedVersionFile (bool error)
{
  if (error)
    {
      QMessageBox::warning (this, tr ("QtiPlot - HTTP Get Version File"),
			    tr
			    ("Error while fetching version file with HTTP: %1.").
			    arg (http.errorString ()));
      return;
    }

  version_buffer.close ();

  if (version_buffer.open (IO_ReadOnly))
    {
      QTextStream t (&version_buffer);
      t.setEncoding (QTextStream::UnicodeUTF8);
      QString version = t.readLine ();
      version_buffer.close ();

      QString currentVersion =
	QString::number (majVersion) + "." + QString::number (minVersion) +
	"." + QString::number (patchVersion) + d_extra_version;

      if (currentVersion != version)
	{
	  if (QMessageBox::question (this, tr ("QtiPlot - Updates Available"),
				     tr
				     ("There is a newer version of QtiPlot (%1) available for download. Would you like to download it?").
				     arg (version),
				     QMessageBox::Yes | QMessageBox::Default,
				     QMessageBox::No | QMessageBox::Escape) ==
	      QMessageBox::Yes)
	    open_browser (this,
			  "http://soft.proindependent.com/download.html");
	}
      else if (!autoSearchUpdatesRequest)
	{
	  QMessageBox::information (this,
				    tr ("QtiPlot - No Updates Available"),
				    tr
				    ("No updates available. Your current version %1 is the last version available!").
				    arg (version));
	}
      autoSearchUpdatesRequest = false;
    }
}
*/

QString
ApplicationWindow::generateUniqueName (const QString & name, bool increment)
{
  //qDebug("generateUniqueName : %s\n",name.ascii());
  int maxind = 0;
  QWidgetList *windows = windowsList ();
  QRegExp rx(name+"(\\d+)");

  for (QWidget * w = windows->first (); w; w = windows->next ())
    {
      QString nm = ((myWidget *)w)->getName();
      if(name == nm)
	{
	  increment = true;
	}
      else if( rx.exactMatch( nm ) )
	{
	  int num = rx.capturedTexts()[1].toInt();
	  if(num>maxind) maxind = num;
	  //qDebug("maxind = %d\n",maxind);
	}
    }
  delete windows;

  QString newName = name;
  if (increment)
    newName += QString::number (maxind+1);

  return newName;
}

void
ApplicationWindow::clearTable ()
{
  Table *t = (Table *) ws->activeWindow ();
  if (!t || !t->isA ("Table"))
    return;

  if (QMessageBox::question (this, tr ("QtiPlot - Warning"),
			     tr
			     ("This will clear the contents of all the data associated with the table. Are you sure?"),
			     tr ("&Yes"), tr ("&No"), QString::null, 0, 1))
    return;
  else
    t->clear ();
}

void
ApplicationWindow::goToRow ()
{
  if (!ws->activeWindow ())
    return;

  if (ws->activeWindow ()->inherits ("Table")
      || ws->activeWindow ()->isA ("Matrix"))
    {
      bool ok;
      int row =
	QInputDialog::getInteger (tr ("QtiPlot - Enter row number"),
				  tr ("Row"),
				  1, 0, 1000000, 1, &ok, this);
      if (!ok)
	return;

      if (ws->activeWindow ()->inherits ("Table"))
	((Table *) ws->activeWindow ())->table ()->ensureCellVisible (row - 1,
								      0);
      else if (ws->activeWindow ()->isA ("Matrix"))
	((Matrix *) ws->activeWindow ())->table ()->ensureCellVisible (row -
								       1, 0);
    }
}

void
ApplicationWindow::showScriptWindow ()
{
  if (!scriptWindow)
    {
      scriptWindow = new ScriptWindow (scriptEnv);
      connect (scriptWindow, SIGNAL (setVisible (bool)),
	       actionShowScriptWindow, SLOT (setOn (bool)));
    }

  if (!scriptWindow->isVisible ())
    {
      scriptWindow->setFocus ();
      scriptWindow->show ();
    }
  else
    scriptWindow->hide ();
}

/*!
  Turns 3D animation on or off
*/
void
ApplicationWindow::toggle3DAnimation (bool on)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    ((Graph3D *) ws->activeWindow ())->animate (on);
}

/*!
  Turns perspective mode on or off
*/
void
ApplicationWindow::togglePerspective (bool on)
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      ((Graph3D *) ws->activeWindow ())->setOrtho (!on);
    }
}

/*!
  Resets rotation of 3D plots to default values
*/
void
ApplicationWindow::resetRotation ()
{
  if (ws->activeWindow () && ws->activeWindow ()->isA ("Graph3D"))
    {
      ((Graph3D *) ws->activeWindow ())->setRotation (30, 0, 15);
    }
}

/*!
  Finds best layout for the 3D plot
*/
void
ApplicationWindow::fitFrameToLayer ()
{
  if (!ws->activeWindow () || !ws->activeWindow ()->isA ("Graph3D"))
    return;

  ((Graph3D *) ws->activeWindow ())->findBestLayout ();
}

void
ApplicationWindow::showCursorInfo(const QString & prestr, 
				  const QString & association,
				  int point)
{//called by the signal cursorInfo in graph.cpp

  //if (association.contains ('=') > 0) // Function Curve
  //  return;

  QStringList cols = QStringList::split (',', association, false);
  QString xcName = cols[0];
  QString ycName = cols[1];

  Table *w = table(ycName);
  if (!w) return;

  int xcol = w->colIndex (xcName);
  int ycol = w->colIndex (ycName);
  int rows = w->tableRows ();

  if (xcol < 0 || ycol < 0)
    return;

  if (w->columnType (ycol) == Table::Numeric)
    {
      int it=-1;
      int r;
      QString xval,yval;
      for (r = 0; r < rows; r++)
        {
          xval = w->text (r, xcol);
          yval = w->text (r, ycol);
          if (!xval.isEmpty () && !yval.isEmpty ())
            {
              it++;
              if(it==point){
                //fprintf(stderr,"r=%d  it=%d  point=%d\n",r,it,point);
                break;
              }
            }
        }
      if(it==point)
	{
	  QString text = prestr;
	  text += ycName;
	  text += "[";
	  text += QString::number (r+1);
	  text += "]: x=";
	  text += xval;
	  text += "; y=";
	  text += yval;
	  info->setText( text );
	}
    }
}

void
ApplicationWindow::resetGraphColors()
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA ("MultiLayer"))
    return;
  MultiLayer *plot = (MultiLayer *) w;
  Graph *g = (Graph *) plot->activeGraph ();
  g->resetColors(false);
}

void
ApplicationWindow::blackenGraphColors()
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA ("MultiLayer"))
    return;
  MultiLayer *plot = (MultiLayer *) w;
  Graph *g = (Graph *) plot->activeGraph ();
  g->resetColors(true);
}

void
ApplicationWindow::resetGraphSymbols()
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA ("MultiLayer"))
    return;
  MultiLayer *plot = (MultiLayer *) w;
  Graph *g = (Graph *) plot->activeGraph ();
  g->resetSymbols();
}

void
ApplicationWindow::resetGraphFonts()
{
  QWidget *w = ws->activeWindow ();
  if (!w || !w->isA ("MultiLayer"))
    return;
  MultiLayer *plot = (MultiLayer *) w;
  Graph *g = (Graph *) plot->activeGraph ();
  g->setFonts(plotTitleFont, plotAxesFont, plotNumbersFont, plotLegendFont);
}

void
ApplicationWindow::setPointerOn(bool on)
{
  btnPointer->setOn(on);
}

void
ApplicationWindow::showDisplayBar()
{
  displayBar->show();
}
